c***********************************************************************
c Loop functions relevant to the NLO and LO matching conditions for the
c SM contribution to B-Bbar mixing. Using the results of Buras, Jamin
c and Weisz Nucl. Phys. B347,491 1990
c***********************************************************************

      Double Precision Function SWW(x)

      Implicit NONE

      Double Precision x

c Eqn (54)
      If (ABS(x-1.D0).lt.1.D-3) Then
         SWW=3.D0/4.D0
      Else
         SWW=x*(1.D0/4.D0+9.D0/(4.D0*(1.D0-x))-3.D0/(2.D0*(1.D0-x)**2)
     &        -3.D0*x**2*LOG(x)/(2.D0*(1.D0-x)**3))
      End If

      Return
      End

c***********************************************************************

      Double Precision Function SWW_Pr(x)

      Implicit NONE

      Double Precision x

c Derivate w.r.t x of Eqn (54)

      If (ABS(x-1.D0).lt.1.D-3) Then
         SWW_Pr=5.D0/8.D0
      Else
         SWW_Pr=(x*(x*(x+3.D0)+18.D0)+4.D0)/(4.D0*(x-1.D0)**3)
     &        -(9.D0*x**2*Log(x))/(2.D0*(x-1.D0)**4)
      End If

      Return
      End

c***********************************************************************

      Double Precision Function BJW_L1(x)

      Implicit NONE

c Inputs
      Double Precision x

c Local
      Double Complex z
      Double Precision tmp1
      Double Precision tmp2

c Functions
      Double Complex DiLog

      z=DCMPLX(x,0.D0)
      tmp1=DBLE(DiLog(1.D0-z))
      tmp2=DBLE(DiLog(1.D0-1.D0/z))

c Equation B.2 of or Buras et. al. calculated using mathematica

      If (ABS(x-1.D0).lt.1.D-3) Then
         BJW_L1=-2.D0
      Else
         BJW_L1=-(54.D0*x**3)/((x-1.D0)**4)*tmp1
     &        -(3.D0*x**5+12.D0*x**4+39.D0*x**3)/((x-1.D0)**4)*tmp2
     &        -(3.D0*x**5+21.D0*x**4+108.D0*x**3-72.D0*x**2+12.D0*x)
     &        /(2.D0*(x-1.D0)**4)*LOG(x)
     &        +(11.D0*x**4+168.D0*x**3-39.D0*x**2+4.D0*x)
     &        /(4.D0*(x-1.D0)**3)
      End If

      Return
      End

c***********************************************************************

      Double Precision Function BJW_L8(x)

      Implicit NONE

c Inputs
      Double Precision x

c Local
      Double Complex z
      Double Precision tmp1
      Double Precision tmp2

c Functions
      Double Complex DiLog

c Parameters
      Double Precision PI
      Parameter(PI=3.1415926535897932384626433832795029D0)

      z=DCMPLX(x,0.D0)
      tmp1=DBLE(DiLog(1.D0-z))
      tmp2=DBLE(DiLog(1.D0-1.D0/z))

c  Equation B.2 of or Buras et. al. calculated using mathematica

      If (ABS(x-1.D0).lt.1.D-3) Then
         BJW_L8=-131.D0/4.D0+8.D0*PI**2/3.D0
      Else
         BJW_L8=(9.D0*x**5-9.D0*x**4+38.D0*x**3-88.D0*x**2
     &        +64.D0*x-16.D0)/(x*(x-1.D0)**4)*tmp1
     &        +(2.D0*x**5-7.D0*x**4+7.D0*x**3-4.D0*x**2)
     &        /((x-1.D0)**4)*tmp2
     &        +(-3.D0*x**4+28.D0*x**3-32.D0*x**2+68.D0*x-32.D0)
     &        /(2.D0*(x-1.D0)**3)*LOG(x)
     &        +(-11.D0*x**3+17.D0*x**2+68.D0*x-64.D0)
     &        /(4.D0*(x-1.D0)**2)
     &        +8.D0*PI**2/(3.D0*x)
      End If

      Return
      End
