c***********************************************************************

      Subroutine bbdelmbsub(debug,genq,iOrd,WilsonC)

c Subroutine that calculates the value of $\Delta M_B$ using formula
c 7.27 of hep-ph/0102316 and the matrix elements (6.9) of hep-ph/0210145
c***********************************************************************

      Implicit NONE


      Include 'bb.h'

c Inputs
      Integer debug
      Integer genq,iOrd

c The (total) Wilson coefficients in the following form.
c WilsonC(1) : VLL
c WilsonC(2) : VRR
c WilsonC(3) : LR1
c WilsonC(4) : LR2
c WilsonC(5) : SLL1
c WilsonC(6) : SLL2
c WilsonC(7) : SRR1
c WilsonC(8) : SRR2
      Double Complex WilsonC(8)
      Double Complex tmpWilc(2,8)

c Local
      Integer i,j

c Functions
      Double Precision delMBout




      Do i=1,2
         Do j=1,8
            tmpWilc(i,j)=DCMPLX(0.D0,0.D0)
         End Do
      End Do

      Do i=1,8
         tmpWilc(genq,i)=WilsonC(i)
      End Do


      delMB(genq)=delMBout(genq,iOrd,tmpWilc)
      delMBSM(genq)=delMBout(genq,iOrd,SMBBWilc)
      delMBDP(genq)=delMBout(genq,iOrd,DPBBWilc)
      delMBCH(genq)=delMBout(genq,iOrd,CHBBWilc)
      delMBNE(genq)=delMBout(genq,iOrd,NINOBBWilc)
      delMBGI(genq)=delMBout(genq,iOrd,GINOBBWilc)
      delMBGN(genq)=delMBout(genq,iOrd,GLNEBBWilc)
      delMBCN(genq)=delMBout(genq,iOrd,CINOBBWilc)
      delMBBSM(genq)=delMBout(genq,iOrd,BSMBBWilc)

      Do i=1,2
         Do j=1,8
            tmpWilc(i,j)=DCMPLX(0.D0,0.D0)
            tmpWilc(i,j)=NINOBBWilc(i,j)
     &           +GINOBBWilc(i,j)
     &           +GLNEBBWilc(i,j)
     &           +CINOBBWilc(i,j)
         End Do
      End Do

      delMBSUBox(genq)=delMBout(genq,iOrd,tmpWilc)


      RatdelMB(genq)=delMB(genq)/delMBSM(genq)

      If (debug.eq.1) Then
         Write(*,*) '***************************************'
         Write(*,*) 'delta M_b'
         Write(*,*) '***************************************'
         Write(*,*) 'SUSY: ', delMB(genq)
         Write(*,*) 'SM: ', delMBSM(genq)
         Write(*,*) 'SUSY/SM: ',RatdelMB(genq)
         Write(*,*) '***************************************'
         Write(*,*) 'NP Contributions to Delta M_B'
         Write(*,*) '***************************************'
         Write(*,*) 'Charged Higgs: ', delMBCH(genq)
         Write(*,*) 'Double Penguin: ', delMBDP(genq)
         Write(*,*) 'Gluino: ', delMBGI(genq)
         Write(*,*) 'Gluino+Neutralino: ', delMBGN(genq)
         Write(*,*) 'Neutralino: ', delMBNE(genq)
         Write(*,*) 'Chargino: ', delMBCN(genq)
         Write(*,*) 'New Physics: ', delMBBSM(genq)
         Write(*,*) 'SUSY Box Diagrams: ', delMBSUBox(genq)
      End If

      Return
      End

c***********************************************************************

      Double Precision Function delMBout(genq,iOrd,InWC)

c Functions that computes $\Delta M_{B_s}$ according to Buras, Jager
c and Urban.
c***********************************************************************

      Implicit NONE

      Include 'const.h'
      Include 'insm.h'
      Include 'vars.h'
      Include 'bb.h'

c Inputs
      Integer genq,iOrd

c The (total) Wilson coefficients in the following form.
c WilsonC(1) : VLL
c WilsonC(2) : VRR
c WilsonC(3) : LR1
c WilsonC(4) : LR2
c WilsonC(5) : SLL1
c WilsonC(6) : SLL2
c WilsonC(7) : SRR1
c WilsonC(8) : SRR2
      Double Complex InWC(2,8)
      Double Complex tmpWC(8)

c The Matrix element 7.27
      Double Complex MatEl

c Local
      Integer i
      Double Precision eta5,aslowt
      Double Precision etaVLL
      Double Precision eta11LR, eta12LR, eta21LR, eta22LR
      Double Precision eta11SLL, eta12SLL, eta21SLL, eta22SLL
      Double Precision BMesFac(5)
      Double Precision prefac

c Functions
      Double Precision MSrmass, alphaStr

c***********************************************************************
c \eta Factors that appear in the Matrix element given by 3.1-3.19
c***********************************************************************

      Do i=1,8
         tmpWC(i)=InWC(genq,i)
      End Do

      eta5=alphaStr(muew,5,4)/alphaStr(mub,5,4)

c      If (iOrd.eq.2.or.iOrd.eq.3.or.iOrd.eq.4) Then
         aslowt=alphaStr(mub,5,4)/(4.D0*PI)
c      Else
c         aslowt=0.D0
c      End If

      etaVLL=eta5**(6.D0/23.D0)
     &     +aslowt*(1.6273D0*(1.D0-eta5)*eta5**(6.D0/23.D0))

      eta11LR=eta5**(3.D0/23.D0)
     &     +aslowt*(0.9250D0*eta5**(-24.D0/23.D0)
     &     +eta5**(3.D0/23.D0)*(-2.0994D0+1.1744D0*eta5))

      eta12LR=aslowt
     &     *1.3875D0*(eta5**(26.D0/23.D0)-eta5**(-24.D0/23.D0))

      eta21LR=2.D0/3.D0*(eta5**(3.D0/23.D0)-eta5**(-24.D0/23.D0))
     &     +aslowt*((-11.7329D0+0.7829D0*eta5)*eta5**(3.D0/23.D0)
     &     +(-5.3048D0+16.2548D0*eta5)*eta5**(-24.D0/23.D0))

      eta22LR=eta5**(-24.D0/23.D0)
     &     +aslowt*((7.9572D0-8.8822D0*eta5)*eta5**(-24.D0/23.D0)
     &     +0.9250D0*eta5**(26.D0/23.D0))

      eta11SLL=(1.0153D0*eta5**(-0.6315D0)-0.0153D0*eta5**(0.7184D0))
     &     +aslowt*((4.8177D0-5.2272D0*eta5)*eta5**(-0.6315D0)
     &     +(0.3371D0+0.0724D0*eta5)*eta5**(0.7184D0))

      eta12SLL=1.9325D0*(eta5**(-0.6315D0)-eta5**(0.7184D0))
     &     +aslowt*((9.1696D0-38.8778D0*eta5)*eta5**(-0.6315D0)
     &     +(42.5021D0-12.7939D0*eta5)*eta5**(0.7184D0))

      eta21SLL=0.0081D0*(eta5**(0.7184D0)-eta5**(-0.6315D0))
     &     +aslowt*((0.0531D0+0.0415D0*eta5)*eta5**(-0.6315D0)
     &     -(0.0566D0+0.0380D0*eta5)*eta5**(0.7184D0))

      eta22SLL=1.0153D0*eta5**(0.7184D0)-0.0153D0*eta5**(-0.6315D0)
     &     +aslowt*((0.1011D0*eta5+0.3083D0*eta5)*eta5**(-0.6315D0)
     &     +(-7.1314D0+6.7219D0*eta5)*eta5**(0.7184D0))

c***********************************************************************
c Hadronic Matrix elements
c***********************************************************************

      prefac=BBm_B(genq)**2
     &     /(MSrmass(mub,5,5,4)+MSrmass(mub,2*genq-1,5,4))**2

      Do i=1,5
         If (i.eq.1) Then
            BMesFac(i)=BBMes(i)
         Else
            BMesFac(i)=prefac*BBMes(i)
         End If
      End Do

c***********************************************************************
c The Matrix element given by equations 7.27-7.32 and 7.34
c***********************************************************************

c Note that there should be an additional factor of the CKM matrix
c present in the formula 7.27
      MatEl=GFermi**2/(48.D0*PI**2)*mw**2*BBm_B(genq)*BBf_B(genq)**2
     &     *ABS(DCONJG(KM(3,3))*KM(3,genq))**2
     &     *(etaVLL*BMesFac(1)*(tmpWC(1)+tmpWC(2))
     &     +(-1.D0/2.D0*eta11LR*BMesFac(2)+3.D0/4.D0*eta21LR*BMesFac(3))
     &     *tmpWC(3)
     &     +(-1.D0/2.D0*eta12LR*BMesFac(2)+3.D0/4.D0*eta22LR*BMesFac(3))
     &     *tmpWC(4)
     &     +(-5.D0/8.D0*eta11SLL*BMesFac(4)
     &     -3.D0/2.D0*eta21SLL*BMesFac(5))
     &     *(tmpWC(5)+tmpWC(7))
     &     +(-5.D0/8.D0*eta12SLL*BMesFac(4)
     &     -3.D0/2.D0*eta22SLL*BMesFac(5))
     &     *(tmpWC(6)+tmpWC(8)))

c Finally simply use eqn 7.26 to obtain delta MB (the numerical factor
c converts to inverse picoseconds)
      delMBout=2.D0*ABS(MatEl)*(1.602176D12/1.054571D0)

      Return
      End
