c***********************************************************************

      Subroutine bbwc(debug,genq,iOrd,WilsonC)

c Subroutine that calculates the contributions the Wilson coefficients 
c relevant to B-Bbar mixing using the matching conditions C6-C12 of
c hep-ph/0104146, eqns. A.11 and A.12 of hep-ph/0210145 and the
c corrected neutral Higgs vertices.
c***********************************************************************

      Implicit NONE

      Include 'insm.h'
      Include 'insoft.h'
      Include 'const.h'
      Include 'vars.h'
      Include 'vertex.h'
      Include 'bb.h'

c Inputs
      Integer debug,genq,iOrd

c Outputs
      Double Complex WilsonC(8)
      
c Local
      Integer i,j,k,l
      Double Precision x
      Double Precision MSH(2), MPH(2),MCHi(2)
      Double Precision mupew(3)
      Double Complex tmp1,tmp2,tmp3,tmp4,tmp5,tmp6,tmp7,tmp8,convfact
      Double Complex SUSYWilcLO(8), SUSYWilcNLO(8)
      Double Complex DPWilcLO(8),DPWilcNLO(8)

c Functions
      Double Precision PV_D0,PV_D00
      Double Precision alphaStr,MSrmass
      Double Precision SWW, SWW_pr, BJW_L1,BJW_L8

      MSH(1)=mHh0
      MSH(2)=mLh0
      MPH(1)=ma0
      MPH(2)=mz
      MCHi(1)=mhc
      MCHi(2)=mw

      Do i=1,3
         mupew(i)=MSrmass(muew,2*i,5,4)
      End Do
      
c Conversion factor between the notation of hep-ph/0102316 and 
c hep-ph/0104146. NOTE I multiply all the results C9-C12 by 
c 128*PI**2/SQRT(2) before applying this factor. Whilst the factor of
c GF^-1 present in C9-C12 is absorbed into convfact

      convfact=1.D0
     &     /(GFermi**2*mw**2*ABS(KM(3,genq)*DCONJG(KM(3,3)))**2)

c***********************************************************************
c SUSY scale contributions
c***********************************************************************
c Neutralinos
c***********************************************************************

      tmp1=DCMPLX(0.D0,0.D0)
      tmp2=DCMPLX(0.D0,0.D0)
      tmp3=DCMPLX(0.D0,0.D0)
      tmp4=DCMPLX(0.D0,0.D0)
      tmp5=DCMPLX(0.D0,0.D0)
      tmp6=DCMPLX(0.D0,0.D0)
      tmp7=DCMPLX(0.D0,0.D0)
      tmp8=DCMPLX(0.D0,0.D0)

      Do i=1,4
         Do j=1,4
            Do k=1,6
               Do l=1,6
c C9a g_R^V > C1VRR
                  tmp1=tmp1-DCONJG(nqsqR(3,j,l))*nqsqR(genq,i,l)
     &                 *(DCONJG(nqsqR(3,i,k))*nqsqR(genq,j,k)
     &                 *PV_D00(MN(i),MN(j),MDoS(k),MDoS(l))
     &                 +DCONJG(nqsqL(3,j,k))*nqsqL(genq,i,k)
     &                 *MN(i)*MN(j)/2.D0
     &                 *PV_D0(MN(i),MN(j),MDoS(k),MDoS(l)))
c C9b g_RR^S > C1SRR
                  tmp2=tmp2+DCONJG(nqsqL(3,j,l))*nqsqR(genq,i,l)
     &                 *DCONJG(nqsqL(3,j,k))*nqsqR(genq,i,k)
     &                 *MN(i)*MN(j)
     &                 *PV_D0(MN(i),MN(j),MDoS(k),MDoS(l))
c C9c g_RR^S' > -1/2 C1SRR+1/8 C2SRR + (E)
                  tmp3=tmp3+DCONJG(nqsqL(3,j,l))*nqsqR(genq,i,l)
     &                 *(DCONJG(nqsqL(3,j,k))*nqsqR(genq,i,k)
     &                 -DCONJG(nqsqL(3,i,k))*nqsqR(genq,j,k))
     &                 *MN(i)*MN(j)
     &                 *PV_D0(MN(i),MN(j),MDoS(k),MDoS(l))
c C9d - Note factor of 2, g_RL^S > C2LR
                  tmp4=tmp4+2.D0*DCONJG(nqsqL(3,j,l))*nqsqR(genq,i,l)
     &                 *(DCONJG(nqsqL(3,j,k))*nqsqR(genq,i,k)
     &                 -DCONJG(nqsqL(3,i,k))*nqsqR(genq,j,k))
     &                 *MN(i)*MN(j)
     &                 *PV_D0(MN(i),MN(j),MDoS(k),MDoS(l))
c C9e g_RL^S > -1/2 C1LR+(E)
                  tmp5=tmp5-DCONJG(nqsqL(3,j,l))*nqsqL(genq,i,l)
     &                 *(DCONJG(nqsqL(3,j,k))*nqsqR(genq,i,k)
     &                 -DCONJG(nqsqL(3,i,k))*nqsqR(genq,j,k))
     &                 *MN(i)*MN(j)
     &                 *PV_D0(MN(i),MN(j),MDoS(k),MDoS(l))
c Now the above operators with the substitution L<->R.
c C9a g_L^V > C1VLL
                  tmp6=tmp6-DCONJG(nqsqL(3,j,l))*nqsqL(genq,i,l)
     &                 *(DCONJG(nqsqL(3,i,k))*nqsqL(genq,j,k)
     &                 *PV_D00(MN(i),MN(j),MDoS(k),MDoS(l))
     &                 +DCONJG(nqsqR(3,j,k))*nqsqR(genq,i,k)
     &                 *MN(i)*MN(j)/2.D0
     &                 *PV_D0(MN(i),MN(j),MDoS(k),MDoS(l)))
c C9b g_LL^S > C1SLL
                  tmp7=tmp7+DCONJG(nqsqR(3,j,l))*nqsqL(genq,i,l)
     &                 *DCONJG(nqsqR(3,j,k))*nqsqL(genq,i,k)
     &                 *MN(i)*MN(j)
     &                 *PV_D0(MN(i),MN(j),MDoS(k),MDoS(l))
c C9c g_LL^S' > -1/2 C1SLL+1/8 C2SLL + (E)
                  tmp8=tmp8+DCONJG(nqsqR(3,j,l))*nqsqL(genq,i,l)
     &                 *(DCONJG(nqsqR(3,j,k))*nqsqL(genq,i,k)
     &                 -DCONJG(nqsqR(3,i,k))*nqsqL(genq,j,k))
     &                 *MN(i)*MN(j)
     &                 *PV_D0(MN(i),MN(j),MDoS(k),MDoS(l))
               End Do
            End Do
         End Do
      End Do

c Additional factors of two correspond to the factors of two present in
c equation 37
      NINOBBWilc(genq,2)=convfact*tmp1/2.D0
      NINOBBWilc(genq,7)=convfact*(tmp2-tmp3/2.D0)/2.D0
      NINOBBWilc(genq,8)=convfact*(tmp3/8.D0)/2.D0
      NINOBBWilc(genq,1)=convfact*tmp6/2.D0
      NINOBBWilc(genq,5)=convfact*(tmp7-tmp8/2.D0)/2.D0
      NINOBBWilc(genq,6)=convfact*(tmp8/8.D0)/2.D0
      NINOBBWilc(genq,4)=convfact*tmp4
      NINOBBWilc(genq,3)=-convfact*tmp5/2.D0

c***********************************************************************
c Charginos
c***********************************************************************

      tmp1=DCMPLX(0.D0,0.D0)
      tmp2=DCMPLX(0.D0,0.D0)
      tmp3=DCMPLX(0.D0,0.D0)
      tmp4=DCMPLX(0.D0,0.D0)
      tmp5=DCMPLX(0.D0,0.D0)
      tmp6=DCMPLX(0.D0,0.D0)
      tmp7=DCMPLX(0.D0,0.D0)
      tmp8=DCMPLX(0.D0,0.D0)

      Do i=1,2
         Do j=1,2
            Do k=1,6
               Do l=1,6
C10a g_R^V > C1VRR
                  tmp1=tmp1-DCONJG(cqsqR(3,j,l))*cqsqR(genq,i,l)
     &                 *DCONJG(cqsqR(3,i,k))*cqsqR(genq,j,k)
     &                 *PV_D00(MC(i),MC(j),MUpS(k),MUpS(l))
C10c g_RR^S' > -1/2 C1SRR+1/8 C2SRR + (E)
                  tmp3=tmp3-DCONJG(cqsqL(3,j,l))*cqsqR(genq,i,l)
     &                 *DCONJG(cqsqL(3,i,k))*cqsqR(genq,j,k)*MC(i)*MC(j)
     &                 *PV_D0(MC(i),MC(j),MUpS(k),MUpS(l))
C10d g_RL^S > C2LR
                  tmp4=tmp4+2.D0*DCONJG(cqsqL(3,j,l))*cqsqR(genq,i,l)
     &                 *DCONJG(cqsqR(3,i,k))*cqsqL(genq,j,k)
     &                 *PV_D00(MC(i),MC(j),MUpS(k),MUpS(l))
C10d g_RL^S > -C1LR/2+(E)
                  tmp5=tmp5-2.D0*DCONJG(cqsqR(3,j,l))*cqsqL(genq,i,l)
     &                 *DCONJG(cqsqL(3,i,k))*cqsqR(genq,j,k)
     &                 *PV_D00(MC(i),MC(j),MUpS(k),MUpS(l))
C10a g_R^V > C1VRR
                  tmp6=tmp6-DCONJG(cqsqR(3,j,l))*cqsqR(genq,i,l)
     &                 *DCONJG(cqsqR(3,i,k))*cqsqR(genq,j,k)
     &                 *PV_D00(MC(i),MC(j),MUpS(k),MUpS(l))
C10c g_RR^S' > -1/2 C1SRR+1/8 C2SRR + (E)
                  tmp8=tmp8-DCONJG(cqsqL(3,j,l))*cqsqR(genq,i,l)
     &                 *DCONJG(cqsqL(3,i,k))*cqsqR(genq,j,k)*MC(i)*MC(j)
     &                 *PV_D0(MC(i),MC(j),MUpS(k),MUpS(l))
               End Do
            End Do
         End Do
      End Do

c Additional factors of two correspond to the factors of two present in
c equation 37
      CINOBBWilc(genq,2)=convfact*tmp1/2.D0
      CINOBBWilc(genq,7)=convfact*(-tmp3/2.D0)/2.D0
      CINOBBWilc(genq,8)=convfact*tmp3/8.D0/2.D0
      CINOBBWilc(genq,1)=convfact*tmp6/2.D0
      CINOBBWilc(genq,5)=convfact*(-tmp8/2.D0)/2.D0
      CINOBBWilc(genq,6)=convfact*tmp8/8.D0/2.D0
      CINOBBWilc(genq,4)=convfact*tmp4
      CINOBBWilc(genq,3)=-convfact*tmp5/2.D0

c***********************************************************************
c Gluinos
c***********************************************************************

      tmp1=DCMPLX(0.D0,0.D0)
      tmp2=DCMPLX(0.D0,0.D0)
      tmp3=DCMPLX(0.D0,0.D0)
      tmp4=DCMPLX(0.D0,0.D0)
      tmp5=DCMPLX(0.D0,0.D0)
      tmp6=DCMPLX(0.D0,0.D0)
      tmp7=DCMPLX(0.D0,0.D0)
      tmp8=DCMPLX(0.D0,0.D0)

      Do i=1,6
         Do j=1,6
c C11a g_R^V->C1VRR
            tmp1=tmp1-DCONJG(gqsqR(3,j))*gqsqR(genq,j)
     &           *DCONJG(gqsqR(3,i))*gqsqR(genq,i)
     &           *(11.D0/18.D0
     &           *PV_D00(MGl,MGl,MDoS(i),MDoS(j))
     &           +1.D0/18.D0*MGl**2
     &           *PV_D0(MGl,MGl,MDoS(i),MDoS(j)))
c C11b g_RR^S->C1SRR
            tmp2=tmp2-DCONJG(gqsqL(3,j))*gqsqR(genq,j)
     &           *DCONJG(gqsqL(3,i))*gqsqR(genq,i)
     &           *17.D0/36.D0*MGl**2
     &           *PV_D0(MGl,MGl,MDoS(i),MDoS(j))
c C11c g_RR^S'-> -1/2 C1SRR+1/8 C2SRR + (E)
            tmp3=tmp3+DCONJG(gqsqL(3,j))*gqsqR(genq,j)
     &           *DCONJG(gqsqL(3,i))*gqsqR(genq,i)
     &           *1.D0/12.D0*MGl**2
     &           *PV_D0(MGl,MGl,MDoS(i),MDoS(j))
c C11d g_RL^S > C2LR
            tmp4=tmp4-((-1.D0/3.D0*DCONJG(gqsqL(3,j))*gqsqL(genq,j)
     &           *DCONJG(gqsqR(3,i))*gqsqR(genq,i)
     &           -11.D0/18.D0*DCONJG(gqsqR(3,j))*gqsqL(genq,j)
     &           *DCONJG(gqsqL(3,i))*gqsqR(genq,i))
     &           *PV_D00(MGl,MGl,MDoS(i),MDoS(j))
     &           +7.D0/12.D0*DCONJG(gqsqR(3,j))*gqsqR(genq,j)
     &           *DCONJG(gqsqL(3,i))*gqsqL(genq,i)*MGl**2
     &           *PV_D0(MGl,MGl,MDoS(i),MDoS(j)))
c C11e g_RL^S > -1/2 C1LR+(E)
            tmp5=tmp5-((-5.D0/6.D0*DCONJG(gqsqR(3,j))*gqsqL(genq,j)
     &           *DCONJG(gqsqL(3,i))*gqsqR(genq,i)
     &           +5.D0/9.D0*DCONJG(gqsqL(3,j))*gqsqL(genq,j)
     &           *DCONJG(gqsqR(3,i))*gqsqR(genq,i))
     &           *PV_D00(MGl,MGl,MDoS(i),MDoS(j))
     &           +1.D0/36.D0*DCONJG(gqsqR(3,j))*gqsqR(genq,j)
     &           *DCONJG(gqsqL(3,i))*gqsqL(genq,i)*MGl**2
     &           *PV_D0(MGl,MGl,MDoS(i),MDoS(j)))

c C11a g_L^V->C1VLL
            tmp6=tmp6-DCONJG(gqsqL(3,j))*gqsqL(genq,j)
     &           *DCONJG(gqsqL(3,i))*gqsqL(genq,i)
     &           *(11.D0/18.D0
     &           *PV_D00(MGl,MGl,MDoS(i),MDoS(j))
     &           +1.D0/18.D0*MGl**2
     &           *PV_D0(MGl,MGl,MDoS(i),MDoS(j)))
c C11b g_LL^S->C1SLL
            tmp7=tmp7-DCONJG(gqsqR(3,j))*gqsqL(genq,j)
     &           *DCONJG(gqsqR(3,i))*gqsqL(genq,i)
     &           *17.D0/36.D0*MGl**2
     &           *PV_D0(MGl,MGl,MDoS(i),MDoS(j))
c C11c g_RR^S'-> -1/2 C1SRR+1/8 C2SRR + (E)
            tmp8=tmp8+DCONJG(gqsqR(3,j))*gqsqL(genq,j)
     &           *DCONJG(gqsqR(3,i))*gqsqL(genq,i)
     &           *1.D0/12.D0*MGl**2
     &           *PV_D0(MGl,MGl,MDoS(i),MDoS(j))
         End Do
      End Do

      GINOBBWilc(genq,2)=convfact*tmp1/2.D0
      GINOBBWilc(genq,7)=convfact*(tmp2-tmp3/2.D0)/2.D0
      GINOBBWilc(genq,8)=convfact*(tmp3/8.D0)/2.D0
      GINOBBWilc(genq,1)=convfact*tmp6/2.D0
      GINOBBWilc(genq,5)=convfact*(tmp7-tmp8/2.D0)/2.D0
      GINOBBWilc(genq,6)=convfact*(tmp8/8.D0)/2.D0
      GINOBBWilc(genq,4)=convfact*tmp4
      GINOBBWilc(genq,3)=-convfact*tmp5/2.D0

c***********************************************************************
c Neutralino / Gluino
c***********************************************************************

      tmp1=DCMPLX(0.D0,0.D0)
      tmp2=DCMPLX(0.D0,0.D0)
      tmp3=DCMPLX(0.D0,0.D0)
      tmp4=DCMPLX(0.D0,0.D0)
      tmp5=DCMPLX(0.D0,0.D0)
      tmp6=DCMPLX(0.D0,0.D0)
      tmp7=DCMPLX(0.D0,0.D0)
      tmp8=DCMPLX(0.D0,0.D0)

      Do i=1,4
         Do j=1,6
            Do k=1,6
C12a
               tmp1=tmp1-2.D0/3.D0*DCONJG(nqsqR(3,i,k))*gqsqR(genq,k)
     &              *DCONJG(gqsqR(3,j))*nqsqR(genq,i,j)
     &              *PV_D00(MN(i),MGl,MDoS(j),MDoS(k))
     &              -1.D0/6.D0*(DCONJG(nqsqR(3,i,k))*gqsqR(genq,k)
     &              *DCONJG(nqsqR(3,i,j))*gqsqR(genq,j)
     &              +DCONJG(gqsqR(3,k))*nqsqR(genq,i,k)
     &              *DCONJG(gqsqR(3,j))*nqsqR(genq,i,j))*MGl*MN(i)
     &              *PV_D0(MN(i),MGl,MDoS(j),MDoS(k))
C12b
               tmp2=tmp2-(DCONJG(nqsqL(3,i,k))*gqsqR(genq,k)
     &              *DCONJG(gqsqL(3,j))*nqsqR(genq,i,j)
     &              -1.D0/3.D0*(DCONJG(nqsqL(3,i,k))*gqsqR(genq,k)
     &              *DCONJG(nqsqL(3,i,j))*gqsqR(genq,j)
     &              +DCONJG(gqsqL(3,k))*nqsqR(genq,i,k)
     &              *DCONJG(gqsqL(3,j))*nqsqR(genq,i,j)))*MGl*MN(i)
     &              *PV_D0(MN(i),MGl,MDoS(j),MDoS(k))
C12c
               tmp3=tmp3+1.D0/3.D0*(DCONJG(nqsqL(3,i,k))*gqsqR(genq,k)
     &              *DCONJG(gqsqL(3,j))*nqsqR(genq,i,j)
     &              +DCONJG(nqsqL(3,i,k))*gqsqR(genq,k)
     &              *DCONJG(nqsqL(3,i,j))*gqsqR(genq,j)
     &              +DCONJG(gqsqL(3,k))*nqsqR(genq,i,k)
     &              *DCONJG(gqsqL(3,j))*nqsqR(genq,i,j))*MGl*MN(i)
     &              *PV_D0(MN(i),MGl,MDoS(j),MDoS(k))
C12d
               tmp4=tmp4-(1.D0/3.D0*(DCONJG(nqsqR(3,i,k))*gqsqL(genq,k)
     &              +DCONJG(gqsqR(3,k))*nqsqL(genq,i,k))
     &              *(DCONJG(gqsqL(3,j)*nqsqR(genq,i,j))
     &              +DCONJG(nqsqL(3,i,j))*gqsqR(genq,j))
     &              +(DCONJG(nqsqL(3,i,k))*gqsqL(genq,k)
     &              *DCONJG(nqsqR(3,i,j))*gqsqR(genq,j)
     &              +DCONJG(gqsqL(3,k))*nqsqL(genq,i,k)
     &              *DCONJG(gqsqR(3,j))*nqsqR(genq,i,j)))
     &              *PV_D00(MN(i),MGl,MDoS(j),MDoS(k))
     &              -1.D0/2.D0*(DCONJG(nqsqR(3,i,k))*gqsqR(genq,k)
     &              *nqsqL(3,i,j)*DCONJG(gqsqL(genq,j))
     &              +DCONJG(gqsqR(3,k))*nqsqR(genq,i,k)
     &              *gqsqL(3,j)*DCONJG(nqsqL(genq,i,j)))*MGl*MN(i)
     &              *PV_D0(MN(i),MGl,MDoS(j),MDoS(k))
C12e
               tmp5=tmp5+((DCONJG(nqsqR(3,i,k))*gqsqL(genq,k)
     &              +DCONJG(gqsqR(3,k))*nqsqL(genq,i,k))
     &              *(DCONJG(gqsqL(3,j)*nqsqR(genq,i,j))
     &              +DCONJG(nqsqL(3,i,j))*gqsqR(genq,j))
     &              +1.D0/3.D0*(DCONJG(nqsqL(3,i,k))*gqsqL(genq,k)
     &              *DCONJG(nqsqR(3,i,j))*gqsqR(genq,j)
     &              +DCONJG(gqsqL(3,k))*nqsqL(genq,i,k)
     &              *DCONJG(gqsqR(3,j))*nqsqR(genq,i,j)))
     &              *PV_D00(MN(i),MGl,MDoS(j),MDoS(k))
     &              +1.D0/6.D0*(DCONJG(nqsqR(3,i,k))*gqsqR(genq,k)
     &              *nqsqL(3,i,j)*DCONJG(gqsqL(genq,j))
     &              +DCONJG(gqsqR(3,k))*nqsqR(genq,i,k)
     &              *gqsqL(3,j)*DCONJG(nqsqL(genq,i,j)))*MGl*MN(i)
     &              *PV_D0(MN(i),MGl,MDoS(j),MDoS(k))
C12a
               tmp6=tmp6-2.D0/3.D0*DCONJG(nqsqL(3,i,k))*gqsqL(genq,k)
     &              *DCONJG(gqsqL(3,j))*nqsqL(genq,i,j)
     &              *PV_D00(MN(i),MGl,MDoS(j),MDoS(k))
     &              -1.D0/6.D0*(DCONJG(nqsqL(3,i,k))*gqsqL(genq,k)
     &              *DCONJG(nqsqL(3,i,j))*gqsqL(genq,j)
     &              +DCONJG(gqsqL(3,k))*nqsqL(genq,i,k)
     &              *DCONJG(gqsqL(3,j))*nqsqL(genq,i,j))*MGl*MN(i)
     &              *PV_D0(MN(i),MGl,MDoS(j),MDoS(k))
C12b
               tmp7=tmp7-(DCONJG(nqsqR(3,i,k))*gqsqL(genq,k)
     &              *DCONJG(gqsqR(3,j))*nqsqL(genq,i,j)
     &              -1.D0/3.D0*(DCONJG(nqsqR(3,i,k))*gqsqL(genq,k)
     &              *DCONJG(nqsqR(3,i,j))*gqsqL(genq,j)
     &              +DCONJG(gqsqR(3,k))*nqsqL(genq,i,k)
     &              *DCONJG(gqsqR(3,j))*nqsqL(genq,i,j)))*MGl*MN(i)
     &              *PV_D0(MN(i),MGl,MDoS(j),MDoS(k))
C12c
               tmp8=tmp8+1.D0/3.D0*(DCONJG(nqsqR(3,i,k))*gqsqL(genq,k)
     &              *DCONJG(gqsqR(3,j))*nqsqL(genq,i,j)
     &              +DCONJG(nqsqR(3,i,k))*gqsqL(genq,k)
     &              *DCONJG(nqsqR(3,i,j))*gqsqL(genq,j)
     &              +DCONJG(gqsqR(3,k))*nqsqL(genq,i,k)
     &              *DCONJG(gqsqR(3,j))*nqsqL(genq,i,j))*MGl*MN(i)
     &              *PV_D0(MN(i),MGl,MDoS(j),MDoS(k))
            End Do
         End Do
      End Do

      GLNEBBWilc(genq,2)=convfact*tmp1/2.D0
      GLNEBBWilc(genq,7)=convfact*(tmp2-tmp3/2.D0)/2.D0
      GLNEBBWilc(genq,8)=convfact*(tmp3/8.D0)/2.D0
      GLNEBBWilc(genq,1)=convfact*tmp6/2.D0
      GLNEBBWilc(genq,5)=convfact*(tmp7-tmp8/2.D0)/2.D0
      GLNEBBWilc(genq,6)=convfact*(tmp8/8.D0)/2.D0
      GLNEBBWilc(genq,4)=convfact*tmp4
      GLNEBBWilc(genq,3)=-convfact*tmp5/2.D0

c***********************************************************************
c RGE Evolution down to the electroweak scale
c***********************************************************************

      Do i=1,8
         SUSYWilcLO(i)=DCMPLX(0.D0,0.D0)
         SUSYWilcNLO(i)=DCMPLX(0.D0,0.D0)
      End Do

      Do i=1,8
         SUSYWilcLO(i)=NINOBBWilc(genq,i)+CINOBBWilc(genq,i)
     &        +GINOBBWilc(genq,i)+GLNEBBWilc(genq,i)
      End Do

      If (debug.eq.1) Then
         Write(*,*) '***************************************'
         Write(*,*) 'LO SUSY Contributions before evolution'
         Write(*,*) 'VLL :',SUSYWilcLO(1)
         Write(*,*) 'VRR :',SUSYWilcLO(2)
         Write(*,*) 'LR1 :',SUSYWilcLO(3)
         Write(*,*) 'LR2 :',SUSYWilcLO(4)
         Write(*,*) 'SLL1 :',SUSYWilcLO(5)
         Write(*,*) 'SLL2 :',SUSYWilcLO(6)
         Write(*,*) 'SRR1 :',SUSYWilcLO(7)
         Write(*,*) 'SRR2 :',SUSYWilcLO(8)
         Write(*,*) '***************************************'
         Write(*,*) 'LO SUSY Contributions before evolution'
         Write(*,*) 'VLL :',SUSYWilcNLO(1)
         Write(*,*) 'VRR :',SUSYWilcNLO(2)
         Write(*,*) 'LR1 :',SUSYWilcNLO(3)
         Write(*,*) 'LR2 :',SUSYWilcNLO(4)
         Write(*,*) 'SLL1 :',SUSYWilcNLO(5)
         Write(*,*) 'SLL2 :',SUSYWilcNLO(6)
         Write(*,*) 'SRR1 :',SUSYWilcNLO(7)
         Write(*,*) 'SRR2 :',SUSYWilcNLO(8)
      End If

      Call BBwcev6to5(debug,SUSYWilcLO,SUSYWilcNLO,mususy,muew)

      If (debug.eq.1) Then
         Write(*,*) '***************************************'
         Write(*,*) 'LO SUSY Contributions after evolution'
         Write(*,*) 'VLL :',SUSYWilcLO(1)
         Write(*,*) 'VRR :',SUSYWilcLO(2)
         Write(*,*) 'LR1 :',SUSYWilcLO(3)
         Write(*,*) 'LR2 :',SUSYWilcLO(4)
         Write(*,*) 'SLL1 :',SUSYWilcLO(5)
         Write(*,*) 'SLL2 :',SUSYWilcLO(6)
         Write(*,*) 'SRR1 :',SUSYWilcLO(7)
         Write(*,*) 'SRR2 :',SUSYWilcLO(8)
         Write(*,*) '***************************************'
         Write(*,*) 'NLO SUSY Contributions after evolution'
         Write(*,*) 'VLL :',SUSYWilcNLO(1)
         Write(*,*) 'VRR :',SUSYWilcNLO(2)
         Write(*,*) 'LR1 :',SUSYWilcNLO(3)
         Write(*,*) 'LR2 :',SUSYWilcNLO(4)
         Write(*,*) 'SLL1 :',SUSYWilcNLO(5)
         Write(*,*) 'SLL2 :',SUSYWilcNLO(6)
         Write(*,*) 'SRR1 :',SUSYWilcNLO(7)
         Write(*,*) 'SRR2 :',SUSYWilcNLO(8)
      End If

c***********************************************************************
c Electroweak Scale Contributions
c***********************************************************************
c Double Penguins
c***********************************************************************

      Do i=1,8
         DPBBWilc(genq,i)=DCMPLX(0.D0,0.D0)
      End Do

      Do i=1,2
c SLL1
         DPBBWilc(genq,5)=DPBBWilc(genq,5)-16.D0*PI**2*convfact
     &        *SHqqLCor(i,2,3)**2/(2.D0*MSH(i)**2)
c SRR1
         DPBBWilc(genq,7)=DPBBWilc(genq,7)-16.D0*PI**2*convfact
     &        *SHqqRCor(i,2,3)**2/(2.D0*MSH(i)**2)
c LR2
         DPBBWilc(genq,4)=DPBBWilc(genq,4)-16.D0*PI**2*convfact
     &        *SHqqLCor(i,2,3)*SHqqRCor(i,2,3)/MSH(i)**2
c SLL1
         DPBBWilc(genq,5)=DPBBWilc(genq,5)-16.D0*PI**2*convfact
     &        *PHqqLCor(i,2,3)**2/(2.D0*MPH(i)**2)
c SRR1
         DPBBWilc(genq,7)=DPBBWilc(genq,7)-16.D0*PI**2*convfact
     &        *PHqqRCor(i,2,3)**2/(2.D0*MPH(i)**2)
c LR2
         DPBBWilc(genq,4)=DPBBWilc(genq,4)-16.D0*PI**2*convfact
     &        *PHqqLCor(i,2,3)*PHqqRCor(i,2,3)/MPH(i)**2
      End Do

      Do i=1,8
         DPWilcLO(i)=DPBBWilc(genq,i)
         DPWilcNLO(i)=DCMPLX(0.D0,0.D0)
      End Do

c***********************************************************************
c Charged Higgs / Mixed Standard Model
c***********************************************************************
c In this case I use the matching conditions of hep-ph/0210145 the
c effective Lagrangian in that paper is the same as that of 
c hep-ph/0102316.
c Concerning the loop functions D0 and D2, in my notation they are
c equivalent to D0=-PV_D0, D2=-4*PV_D00.


      tmp1=DCMPLX(0.D0,0.D0)
      tmp2=DCMPLX(0.D0,0.D0)
      tmp3=DCMPLX(0.D0,0.D0)
      tmp4=DCMPLX(0.D0,0.D0)
      tmp5=DCMPLX(0.D0,0.D0)
      tmp6=DCMPLX(0.D0,0.D0)
      tmp7=DCMPLX(0.D0,0.D0)
      Do i=1,3
         Do j=1,3
c A.11a
            tmp1=tmp1+DCONJG(WqqL(3,i))*WqqL(genq,j)
     &           *DCONJG(CHqqL(1,3,j))*CHqqL(1,genq,i)
     &           *mupew(j)*mupew(i)
     &           *PV_D0(mw,mhc,mupew(i),mupew(j))
     &           -1.D0/2.D0*DCONJG(CHqqL(1,3,i))*CHqqL(1,genq,i)
     &           *DCONJG(CHqqL(1,3,j))*CHqqL(1,genq,j)
     &           *PV_D00(mhc,mhc,mupew(i),mupew(j))
     &           -DCONJG(CHqqL(2,3,i))*CHqqL(1,genq,i)
     &           *DCONJG(CHqqL(1,3,j))*CHqqL(2,genq,j)
     &           *PV_D00(mw,mhc,mupew(i),mupew(j))
            If (iOrd.eq.1.or.iOrd.eq.3.or.iOrd.eq.4) Then
               tmp1=tmp1+DCONJG(WqqL(3,i))*WqqL(genq,j)
     &              *DCONJG(delCHqqL(1,3,j))*CHqqL(1,genq,i)
     &              *mupew(j)*mupew(i)
     &              *PV_D0(mw,mhc,mupew(i),mupew(j))
     &              -1.D0/2.D0*DCONJG(delCHqqL(1,3,i))*CHqqL(1,genq,i)
     &              *DCONJG(delCHqqL(1,3,j))*CHqqL(1,genq,j)
     &              *PV_D00(mhc,mhc,mupew(i),mupew(j))
     &              -DCONJG(delCHqqL(2,3,i))*CHqqL(1,genq,i)
     &              *DCONJG(delCHqqL(1,3,j))*CHqqL(2,genq,j)
     &              *PV_D00(mw,mhc,mupew(i),mupew(j))
               tmp1=tmp1+DCONJG(WqqL(3,i))*WqqL(genq,j)
     &              *DCONJG(CHqqL(1,3,j))*delCHqqL(1,genq,i)
     &              *mupew(j)*mupew(i)
     &              *PV_D0(mw,mhc,mupew(i),mupew(j))
     &              -1.D0/2.D0*DCONJG(CHqqL(1,3,i))*delCHqqL(1,genq,i)
     &              *DCONJG(CHqqL(1,3,j))*delCHqqL(1,genq,j)
     &              *PV_D00(mhc,mhc,mupew(i),mupew(j))
     &              -DCONJG(CHqqL(2,3,i))*delCHqqL(1,genq,i)
     &              *DCONJG(CHqqL(1,3,j))*delCHqqL(2,genq,j)
     &              *PV_D00(mw,mhc,mupew(i),mupew(j))
            End If
            Do k=1,2
               Do l=1,2
c A.11b
                  tmp2=tmp2-1.D0/2.D0
     &                 *DCONJG(CHqqR(l,3,i))*CHqqR(k,genq,i)
     &                 *DCONJG(CHqqR(k,3,j))*CHqqR(l,genq,j)
     &                 *PV_D00(MCHi(k),MCHi(l),mupew(i),mupew(j))
                  If (iOrd.eq.1.or.iOrd.eq.3.or.iOrd.eq.4) Then
                     tmp2=tmp2-1.D0/2.D0
     &                    *DCONJG(delCHqqR(l,3,i))*CHqqR(k,genq,i)
     &                    *DCONJG(delCHqqR(k,3,j))*CHqqR(l,genq,j)
     &                    *PV_D00(MCHi(k),MCHi(l),mupew(i),mupew(j))
                     tmp2=tmp2-1.D0/2.D0
     &                    *DCONJG(CHqqR(l,3,i))*delCHqqR(k,genq,i)
     &                    *DCONJG(CHqqR(k,3,j))*delCHqqR(l,genq,j)
     &                    *PV_D00(MCHi(k),MCHi(l),mupew(i),mupew(j))
                  End If
c A.11c
                  tmp3=tmp3-1.D0/2.D0
     &                 *DCONJG(CHqqR(l,3,i))*CHqqL(k,genq,i)
     &                 *DCONJG(CHqqR(k,3,j))*CHqqL(l,genq,j)
     &                 *mupew(j)*mupew(i)
     &                 *PV_D0(MCHi(k),MCHi(l),mupew(i),mupew(j))
                  If (iOrd.eq.1.or.iOrd.eq.3.or.iOrd.eq.4) Then
                     tmp3=tmp3-1.D0/2.D0
     &                    *DCONJG(delCHqqR(l,3,i))*CHqqL(k,genq,i)
     &                    *DCONJG(delCHqqR(k,3,j))*CHqqL(l,genq,j)
     &                    *mupew(j)*mupew(i)
     &                    *PV_D0(MCHi(k),MCHi(l),mupew(i),mupew(j))
                     tmp3=tmp3-1.D0/2.D0
     &                    *DCONJG(CHqqR(l,3,i))*delCHqqL(k,genq,i)
     &                    *DCONJG(CHqqR(k,3,j))*delCHqqL(l,genq,j)
     &                    *mupew(j)*mupew(i)
     &                    *PV_D0(MCHi(k),MCHi(l),mupew(i),mupew(j))
                  End If
c A.11e
                  tmp4=tmp4-1.D0/2.D0
     &                 *DCONJG(CHqqL(l,3,i))*CHqqR(k,genq,i)
     &                 *DCONJG(CHqqL(k,3,j))*CHqqR(l,genq,j)
     &                 *mupew(j)*mupew(i)
     &                 *PV_D0(MCHi(k),MCHi(l),mupew(i),mupew(j))
                  If (iOrd.eq.1.or.iOrd.eq.3.or.iOrd.eq.4) Then
                     tmp4=tmp4-1.D0/2.D0
     &                    *DCONJG(delCHqqL(l,3,i))*CHqqR(k,genq,i)
     &                    *DCONJG(delCHqqL(k,3,j))*CHqqR(l,genq,j)
     &                    *mupew(j)*mupew(i)
     &                    *PV_D0(MCHi(k),MCHi(l),mupew(i),mupew(j))
                     tmp4=tmp4-1.D0/2.D0
     &                    *DCONJG(CHqqL(l,3,i))*delCHqqR(k,genq,i)
     &                    *DCONJG(CHqqL(k,3,j))*delCHqqR(l,genq,j)
     &                    *mupew(j)*mupew(i)
     &                    *PV_D0(MCHi(k),MCHi(l),mupew(i),mupew(j))
                  End If
c A.11g
                  tmp5=tmp5-DCONJG(CHqqL(l,3,i))*CHqqL(k,genq,i)
     &                 *DCONJG(CHqqR(k,3,j))*CHqqR(l,genq,j)
     &                 *PV_D00(MCHi(k),MCHi(l),mupew(i),mupew(j))
                  If (iOrd.eq.1.or.iOrd.eq.3.or.iOrd.eq.4) Then
                     tmp5=tmp5-DCONJG(delCHqqL(l,3,i))*CHqqL(k,genq,i)
     &                    *DCONJG(delCHqqR(k,3,j))*CHqqR(l,genq,j)
     &                    *PV_D00(MCHi(k),MCHi(l),mupew(i),mupew(j))
                     tmp5=tmp5-DCONJG(CHqqL(l,3,i))*delCHqqL(k,genq,i)
     &                    *DCONJG(CHqqR(k,3,j))*delCHqqR(l,genq,j)
     &                    *PV_D00(MCHi(k),MCHi(l),mupew(i),mupew(j))
                  End If
c A.11h (pt1)
                  tmp6=tmp6-DCONJG(CHqqR(l,3,i))*CHqqL(k,genq,i)
     &                 *DCONJG(CHqqL(k,3,j))*CHqqR(l,genq,j)
     &                 *mupew(j)*mupew(i)
     &                 *PV_D0(MCHi(k),MCHi(l),mupew(i),mupew(j))
                  If (iOrd.eq.1.or.iOrd.eq.3.or.iOrd.eq.4) Then
                     tmp6=tmp6-DCONJG(delCHqqR(l,3,i))*CHqqL(k,genq,i)
     &                    *DCONJG(delCHqqL(k,3,j))*CHqqR(l,genq,j)
     &                    *mupew(j)*mupew(i)
     &                    *PV_D0(MCHi(k),MCHi(l),mupew(i),mupew(j))
                     tmp6=tmp6-DCONJG(CHqqR(l,3,i))*delCHqqL(k,genq,i)
     &                    *DCONJG(CHqqL(k,3,j))*delCHqqR(l,genq,j)
     &                    *mupew(j)*mupew(i)
     &                    *PV_D0(MCHi(k),MCHi(l),mupew(i),mupew(j))
                  End If
               End Do
            End Do
            tmp7=tmp7+1.D0/4.D0*DCONJG(WqqL(3,i))*WqqL(genq,j)
     &           *DCONJG(CHqqR(1,3,j))*CHqqR(1,genq,i)
     &           *PV_D00(mw,mhc,mupew(i),mupew(j))
            If (iOrd.eq.1.or.iOrd.eq.3.or.iOrd.eq.4.or.iOrd.eq.4) Then
               tmp7=tmp7+1.D0/4.D0*DCONJG(WqqL(3,i))*WqqL(genq,j)
     &              *DCONJG(delCHqqR(1,3,j))*CHqqR(1,genq,i)
     &              *PV_D00(mw,mhc,mupew(i),mupew(j))
               tmp7=tmp7+1.D0/4.D0*DCONJG(WqqL(3,i))*WqqL(genq,j)
     &              *DCONJG(CHqqR(1,3,j))*delCHqqR(1,genq,i)
     &              *PV_D00(mw,mhc,mupew(i),mupew(j))
            End If
         End Do
      End Do

      CHBBWilc(genq,1)=convfact*tmp1
      CHBBWilc(genq,2)=convfact*tmp2
      CHBBWilc(genq,3)=convfact*tmp5
      CHBBWilc(genq,4)=convfact*(tmp6+tmp7)
      CHBBWilc(genq,5)=convfact*tmp3
      CHBBWilc(genq,6)=DCMPLX(0.D0,0.D0)
      CHBBWilc(genq,7)=convfact*tmp4
      CHBBWilc(genq,8)=DCMPLX(0.D0,0.D0)

c***********************************************************************
c Standard Model
c***********************************************************************
c I use the NLO matching conditions of Buras, Jamin and Weisz 
c Nucl. Phys. B347,491 1990

      x=MSrmass(muew,6,5,4)**2/mw**2

      SMBBWilc(genq,1)=4.D0*SWW(x)
      tmp1=SMBBWilc(genq,1)

      SMBBWilc(genq,1)=SMBBWilc(genq,1)
     &     +4.D0*alphaStr(muew,5,4)/(4.D0*PI)
     &     *(4.D0/3.D0*(BJW_L1(x)+6.D0*x*LOG(muew**2/mw**2)*SWW_Pr(x)
     &     +3.D0*SWW(x))
     &     +1.D0/3.D0*(BJW_L8(x)+6.D0*LOG(muew**2/mw**2)*SWW(x)
     &     +5.D0*SWW(x)))

      SMBBWilc(genq,2)=DCMPLX(0.D0,0.D0)
      SMBBWilc(genq,3)=DCMPLX(0.D0,0.D0)
      SMBBWilc(genq,4)=DCMPLX(0.D0,0.D0)
      SMBBWilc(genq,5)=DCMPLX(0.D0,0.D0)
      SMBBWilc(genq,6)=DCMPLX(0.D0,0.D0)
      SMBBWilc(genq,7)=DCMPLX(0.D0,0.D0)
      SMBBWilc(genq,8)=DCMPLX(0.D0,0.D0)

c***********************************************************************
c Debug Writes

      If (debug.eq.1) Then
         Write(*,*) '***************************************'
         Write(*,*) 'Standard Model Contributions'
         Write(*,*) 'CVLL :',SMBBWilc(genq,1)
         Write(*,*) 'CVRR :',SMBBWilc(genq,2)
         Write(*,*) 'CLR1 :',SMBBWilc(genq,3)
         Write(*,*) 'CLR2 :',SMBBWilc(genq,4)
         Write(*,*) 'CSLL1 :',SMBBWilc(genq,5)
         Write(*,*) 'CSLL2 :',SMBBWilc(genq,6)
         Write(*,*) 'CSRR1 :',SMBBWilc(genq,7)
         Write(*,*) 'CSRR2 :',SMBBWilc(genq,8)
         Write(*,*) '***************************************'
         Write(*,*) 'Charged Higgs Contributions'
         Write(*,*) 'CVLL :',CHBBWilc(genq,1)
         Write(*,*) 'CVRR :',CHBBWilc(genq,2)
         Write(*,*) 'CLR1 :',CHBBWilc(genq,3)
         Write(*,*) 'CLR2 :',CHBBWilc(genq,4)
         Write(*,*) 'CSLL1 :',CHBBWilc(genq,5)
         Write(*,*) 'CSLL2 :',CHBBWilc(genq,6)
         Write(*,*) 'CSRR1 :',CHBBWilc(genq,7)
         Write(*,*) 'CSRR2 :',CHBBWilc(genq,8)
         Write(*,*) '***************************************'
         Write(*,*) 'Gluino Contributions'
         Write(*,*) 'CVLL :',GINOBBWilc(genq,1)
         Write(*,*) 'CVRR :',GINOBBWilc(genq,2)
         Write(*,*) 'CLR1 :',GINOBBWilc(genq,3)
         Write(*,*) 'CLR2 :',GINOBBWilc(genq,4)
         Write(*,*) 'CSLL1 :',GINOBBWilc(genq,5)
         Write(*,*) 'CSLL2 :',GINOBBWilc(genq,6)
         Write(*,*) 'CSRR1 :',GINOBBWilc(genq,7)
         Write(*,*) 'CSRR2 :',GINOBBWilc(genq,8)
         Write(*,*) '***************************************'
         Write(*,*) 'Neutralino Contributions'
         Write(*,*) 'CVLL :',NINOBBWilc(genq,1)
         Write(*,*) 'CVRR :',NINOBBWilc(genq,2)
         Write(*,*) 'CLR1 :',NINOBBWilc(genq,3)
         Write(*,*) 'CLR2 :',NINOBBWilc(genq,4)
         Write(*,*) 'CSLL1 :',NINOBBWilc(genq,5)
         Write(*,*) 'CSLL2 :',NINOBBWilc(genq,6)
         Write(*,*) 'CSRR1 :',NINOBBWilc(genq,7)
         Write(*,*) 'CSRR2 :',NINOBBWilc(genq,8)
         Write(*,*) '***************************************'
         Write(*,*) 'Chargino Contributions'
         Write(*,*) 'CVLL :',CINOBBWilc(genq,1)
         Write(*,*) 'CVRR :',CINOBBWilc(genq,2)
         Write(*,*) 'CLR1 :',CINOBBWilc(genq,3)
         Write(*,*) 'CLR2 :',CINOBBWilc(genq,4)
         Write(*,*) 'CSLL1 :',CINOBBWilc(genq,5)
         Write(*,*) 'CSLL2 :',CINOBBWilc(genq,6)
         Write(*,*) 'CSRR1 :',CINOBBWilc(genq,7)
         Write(*,*) 'CSRR2 :',CINOBBWilc(genq,8)
         Write(*,*) '***************************************'
         Write(*,*) 'Mixed Neutralino-Gluino Contributions'
         Write(*,*) 'CVLL :',GLNEBBWilc(genq,1)
         Write(*,*) 'CVRR :',GLNEBBWilc(genq,2)
         Write(*,*) 'CLR1 :',GLNEBBWilc(genq,3)
         Write(*,*) 'CLR2 :',GLNEBBWilc(genq,4)
         Write(*,*) 'CSLL1 :',GLNEBBWilc(genq,5)
         Write(*,*) 'CSLL2 :',GLNEBBWilc(genq,6)
         Write(*,*) 'CSRR1 :',GLNEBBWilc(genq,7)
         Write(*,*) 'CSRR2 :',GLNEBBWilc(genq,8)
         Write(*,*) '***************************************'
         Write(*,*) 'Double Penguin Contributions'
         Write(*,*) 'CVLL :',DPBBWilc(genq,1)
         Write(*,*) 'CVRR :',DPBBWilc(genq,2)
         Write(*,*) 'CLR1 :',DPBBWilc(genq,3)
         Write(*,*) 'CLR2 :',DPBBWilc(genq,4)
         Write(*,*) 'CSLL1 :',DPBBWilc(genq,5)
         Write(*,*) 'CSLL2 :',DPBBWilc(genq,6)
         Write(*,*) 'CSRR1 :',DPBBWilc(genq,7)
         Write(*,*) 'CSRR2 :',DPBBWilc(genq,8)
      End If

c***********************************************************************
c Total Wilson coefficients
c***********************************************************************

      Do i=1,8
         WilsonC(i)=SMBBWilc(genq,i)+SUSYWilcLO(i)+DPWilcLO(i)
     &        +CHBBWilc(genq,i)
         If (iOrd.eq.2.or.iOrd.eq.3.or.iOrd.eq.4) Then
            WilsonC(i)=WilsonC(i)+alphaStr(muew,5,4)/(4.D0*PI)
     &           *(SUSYWilcNLO(i)+DPWilcNLO(i))
         End If
      End Do

      Do i=1,8
         BSMBBWilc(genq,i)=SUSYWilcLO(i)+DPWilcLO(i)
     &        +CHBBWilc(genq,i)
         If (iOrd.eq.2.or.iOrd.eq.3.or.iOrd.eq.4) Then
            BSMBBWilc(genq,i)=BSMBBWilc(genq,i)
     &           +alphaStr(muew,5,4)/(4.D0*PI)*(SUSYWilcNLO(i)
     &           +DPWilcNLO(i))
         End If
      End Do

      Return
      End
