c***********************************************************************

      Subroutine BBwcev6to5(debug,WilsonCLO,WilsonCNLO,mu_high,mu_low)

c Subroutine that given the relevant Wilson coefficients evolves them
c from the energy scale mu0 to the energy scale mu1 using the RGEs
c with 6  active flavours.
c***********************************************************************

      Implicit NONE

      Include 'vars.h'

c Inputs
      Integer debug

c The start point and end point for the RG evolution
      Double Precision mu_high, mu_low

c The array WilsonC0 contains the LO Wilson coefficients at the scale
c mu_high on output it contains the LO Wilson coefficients at the 
c scale mu_low. The array Wilson C1 contains the NLO contributions 
c to the Wilson coefficients and hence only undergoes LO evolution. On
c output it contains the NLO contributions after evolution.
c They should have the following form :    
c WilsonC(1) : VLL
c WilsonC(2) : VRR
c WilsonC(3) : LR1
c WilsonC(4) : LR2
c WilsonC(5) : SLL1
c WilsonC(6) : SLL2
c WilsonC(7) : SRR1
c WilsonC(8) : SRR2
      Double Complex WilsonCLO(8),WilsonCNLO(8)

c Local
      Integer i,j
      Double Precision Uevol0(8,8),Uevol1(8,8)
      Double Precision eta6
      Double Complex tmp1,tmp2, WilsonCLO_0(8),WilsonCNLO_0(8)


c Functions
      Double Precision alphaStr

c Expressions for U_6 are taken from Buras et. al. hep-ph/0102316

      eta6=alphaStr(mu_high,6,4)/alphaStr(mu_low,6,4)
      
      Do i=1,8
         Do j=1,8
            Uevol0(i,j)=0.D0
            Uevol1(i,j)=0.D0
         End Do
      End Do

c VLL and VRR
      Uevol0(1,1)=eta6**(6.D0/21.D0)
      Uevol0(2,2)=eta6**(6.D0/21.D0)

c LR1 and LR2
      Uevol0(3,3)=eta6**(3.D0/21.D0)
      Uevol0(3,4)=0.D0
      Uevol0(4,3)=2.D0/3.D0*(eta6**(3.D0/21.D0)-eta6**(-24.D0/21.D0))
      Uevol0(4,4)=eta6**(-24.D0/21.D0)

c SLL1 and SLL2
      Uevol0(5,5)=1.0153D0*eta6**(-0.6916D0)-0.0153D0*eta6**(0.7869D0)
      Uevol0(5,6)=1.9325D0*(eta6**(-0.6916D0)-eta6**(0.7869D0))
      Uevol0(6,5)=0.0081D0*(eta6**(0.7869D0)-eta6**(-0.6916D0))
      Uevol0(6,6)=1.0153D0*eta6**(0.7869D0)-0.0153D0*eta6**(-0.6916D0)

c SRR1 and SRR2
      Uevol0(7,7)=1.0153D0*eta6**(-0.6916D0)-0.0153D0*eta6**(0.7869D0)
      Uevol0(7,8)=1.9325D0*(eta6**(-0.6916D0)-eta6**(0.7869D0))
      Uevol0(8,7)=0.0081D0*(eta6**(0.7869D0)-eta6**(-0.6916D0))
      Uevol0(8,8)=1.0153D0*eta6**(0.7869D0)-0.0153D0*eta6**(-0.6916D0)

c VLL and VRR
      Uevol1(1,1)=1.3707D0*(1-eta6)*eta6**(6.D0/21.D0)
      Uevol1(2,2)=1.3707D0*(1-eta6)*eta6**(6.D0/21.D0)

c LR1 and LR2
      Uevol1(3,3)=eta6**(3.D0/21.D0)
      Uevol1(3,4)=1.3828D0*(eta6**(24.D0/21.D0)-eta6**(-23.D0/21.D0))
      Uevol1(4,3)=eta6**(3.D0/21.D0)*(-10.1463D0+0.850D0*eta6)
     &     +eta6**(-24.D0/21.D0)*(-6.4603D0+15.4715D0*eta6)
      Uevol1(4,4)=0.9219D0*eta6**(24.D0/21.D0)
     &     +eta6**(-24.D0/21.D0)*(9.6904D0-10.6122D0*eta6)

c SLL1 and SLL2
      Uevol1(5,5)=eta6**(-0.6916D0)*(5.6478D0-6.0350D0*eta6)
     &     +eta6**(0.7869D0)*(0.372D0+0.0600D0*eta6)
      Uevol1(5,6)=eta6**(-0.6916D0)*(5.6478D0-6.0350D0*eta6)
     &     +eta6**(0.7869D0)*(41.2556D0-14.0841D0*eta6)
      Uevol1(6,5)=eta6**(0.7869D0)*(-0.0618D0-0.0315D0*eta6)
     &     +eta6**(-0.6916D0)*(0.0454D0+0.0479D0*eta6)
      Uevol1(6,6)=eta6**(-0.6916D0)*(0.0865D0+0.3007D0*eta6)
     &     +eta6**(0.7869D0)*(-7.7870D0+7.3999D0*eta6)

c SRR1 and SRR2
      Uevol1(7,7)=eta6**(-0.6916D0)*(5.6478D0-6.0350D0*eta6)
     &     +eta6**(0.7869D0)*(0.372D0+0.0600D0*eta6)
      Uevol1(7,8)=eta6**(-0.6916D0)*(5.6478D0-6.0350D0*eta6)
     &     +eta6**(0.7869D0)*(41.2556D0-14.0841D0*eta6)
      Uevol1(8,7)=eta6**(0.7869D0)*(-0.0618D0-0.0315D0*eta6)
     &     +eta6**(-0.6916D0)*(0.0454D0+0.0479D0*eta6)
      Uevol1(8,8)=eta6**(-0.6916D0)*(0.0865D0+0.3007D0*eta6)
     &     +eta6**(0.7869D0)*(-7.7870D0+7.3999D0*eta6)

      Do i=1,8
         WilsonCLO_0(i)=WilsonCLO(i)
         WilsonCNLO_0(i)=WilsonCNLO(i)
      End Do

      Do i=1,8
         tmp1=DCMPLX(0.D0,0.D0)
         tmp2=DCMPLX(0.D0,0.D0)
         Do j=1,8
            tmp1=tmp1+Uevol0(i,j)*WilsonCLO_0(j)
            tmp2=tmp2+Uevol0(i,j)*WilsonCNLO_0(j)
     &           +Uevol1(i,j)*WilsonCLO_0(j)
         End Do
         WilsonCLO(i)=tmp1
         WilsoncNLO(i)=tmp2
      End Do

      Return
      End
