c*************************************************************************

      Subroutine CKMcalc(debug,Vus,Vcb,Vub_ov_Vcb,delta)
c Subroutine that calculates the CKM matrix based on an input of 3 matrix
c elements and the complex phase associated with the matrix.
c Based on set_CKM .f by Ken-ichi Okumura
c*************************************************************************


      Implicit NONE

c Input 
      Integer debug
      Double Precision Vus,Vcb,Vub_ov_Vcb, delta

c Output
      Double Complex KM(3,3)
      Common/CKMMat/KM

c Local variables
      Integer i,j
      Double Precision th13,th23,th12
      Double Precision rho,eta

c*************************************************************************

c Calculate the various angles etc

      th13=DASIN(Vcb*Vub_ov_Vcb)
      th23=DASIN(Vcb/DCOS(th13))
      th12=DASIN(Vus/DCOS(th13))

      Call calc3x3(th12,th13,th23,delta,KM)

      If (debug.eq.1) Then
         Do i=1,3
            Do j=1,3
               Write(*,'(A,I2,A,I2,A,G15.7,G15.7)') 'KM(',i,',',j,')='
     &           ,KM(i,j)
            End do
         End do
         rho = -DReal(KM(1,1)*DConjg(KM(1,3))/KM(2,1)/DConjg(KM(2,3)))
         eta = -DImag(KM(1,1)*DConjg(KM(1,3))/KM(2,1)/DConjg(KM(2,3)))
         Write(*,*)'rho=',rho
         Write(*,*)'eta=',eta
      End If
      Return
      End

c*************************************************************************

      Subroutine CKMWolf(debug,lam,A,rhob,etab)
c Subroutine that calculates the CKM matrix based on the Wolfenstein
c parameterisation.
c*************************************************************************

      Implicit NONE

c Input
      Integer debug
      Double Precision lam,A,rhob,etab

c Output
      Double Complex KM(3,3)
      Common/CKMMat/KM

c Local
      Integer i,j
      Double Precision rho, eta

      rho=rhob/(1.D0-lam**2/2.D0)
      eta=etab/(1.D0-lam**2/2.D0)

c Equation 8 hep-ph/0304132

      KM(1,1)=1.D0-1.D0/2.D0*lam**2-1.D0/8.D0*lam**4
      KM(1,2)=lam
      KM(1,3)=A*lam**3*(rho-DCMPLX(0.D0,1.D0)*eta)
      KM(2,1)=-lam+1.D0/2.D0*A**2*lam**5
     &     *(1.D0-2.D0*(rho+DCMPLX(0.D0,1.D0)*eta))
      KM(2,2)=1.D0-1.D0/2.D0*lam**2-1.D0/8.D0*lam**4*(1.D0+4.D0*A**2)
      KM(2,3)=A*lam**2
      KM(3,1)=A*lam**3*(1.D0-rhob-DCMPLX(0.D0,1.D0)*etab)
      KM(3,2)=-A*lam**2
     &     +1.D0/2.D0*A*lam**4*(1.D0-2.D0*(rho+DCMPLX(0.D0,1.D0)*eta))
      KM(3,3)=1-1.D0/2.D0*A**2*lam**4

      If (debug.eq.1) Then
         Do i=1,3
            Do j=1,3
               Write(*,'(A,I2,A,I2,A,G15.7,G15.7)') 'KM(',i,',',j,')='
     &           ,KM(i,j)
            End do
         End do
         rho = -DReal(KM(1,1)*DConjg(KM(1,3))/KM(2,1)/DConjg(KM(2,3)))
         eta = -DImag(KM(1,1)*DConjg(KM(1,3))/KM(2,1)/DConjg(KM(2,3)))
         Write(*,*)'rho=',rho
         Write(*,*)'eta=',eta
      End If

      Return
      End
