c**********************************************************************

      Subroutine MassMatrix(debug,type,dim,MMatrix)

c Subroutine that determines the mass matrix given the two soft
c square terms, the type (i.e up squark, down squark, selectrons etc),
c and outputs the matrix via MMatrix
c type=1 Down squarks
c type=2 Up squarks
c type=3 Selectrons
c type=4 Sneutrinos
c**********************************************************************

      Implicit NONE

      Include 'insoft.h'
      Include 'vars.h'
      Include 'insm.h'
      Include 'const.h'
      
      Integer debug,type,dim

      Double Complex MMatrix(dim,dim),tmp1,tmp2,tmp3

      Double Precision beta,gw

      Integer i,j,k,l

      Double Precision alpha2
      

      gw=DSQRT(alpha2(mususy)*4.D0*PI)
      beta=ATAN(tanb)

      Do i=1,dim
         Do j=1,dim
            MMatrix(i,j)=DCMPLX(0.D0,0.D0)
         End Do
      End Do

c Down type
      If (type.eq.1) Then
         Do i=1,3
            Do j=1,3
               tmp1=DCMPLX(0.D0,0.D0)
               tmp2=DCMPLX(0.D0,0.D0)
               tmp3=DCMPLX(0.D0,0.D0)
               Do k=1,3
                  tmp1=tmp1+DCONJG(mdbare(k,i))*mdbare(k,j)
                  tmp2=tmp2+mdbare(i,k)*DCONJG(mdbare(j,k))
               End Do
               tmp1=tmp1+MSoftQuL(i,j)-1.D0/6.D0*mz**2*COS(2.D0*beta)
     &              *(3.D0-2.D0*sw2)*Unit3(i,j)
               tmp2=tmp2+MSoftDoR(i,j)-1.D0/3.D0*mz**2*COS(2.D0*beta)
     &              *sw2*Unit3(i,j)
               tmp3=tmp3+SQRT(2.D0)*COS(beta)*mw
     &              *DCONJG(ADo(i,j))/gw
               tmp3=tmp3-DCONJG(mdbare(j,i))*muH*tanb
               MMatrix(i,j)=tmp1
               MMatrix(i+3,j+3)=tmp2
               MMatrix(i,j+3)=tmp3
               MMatrix(j+3,i)=DCONJG(MMatrix(i,j+3))
            End Do
         End Do
c Up Type
      Else If(type.eq.2) Then
         Do i=1,3
            Do j=1,3
               tmp1=DCMPLX(0.D0,0.D0)
               tmp2=DCMPLX(0.D0,0.D0)
               tmp3=DCMPLX(0.D0,0.D0)
               Do k=1,3
                  Do l=1,3
                    tmp1=tmp1
     &                    +DCONJG(KMbare(k,i))*MSoftQuL(k,l)*KMbare(l,j)
                  End Do
                  tmp1=tmp1+DCONJG(mubare(k,i))*mubare(k,j)
                  tmp2=tmp2+mubare(i,k)*DCONJG(mubare(j,k))
               End Do
               tmp1=tmp1+1.D0/6.D0*mz**2*COS(2.D0*beta)
     &              *(3.D0-4.D0*sw2)*Unit3(i,j)
               tmp2=tmp2+MSoftUpR(i,j)+2.D0/3.D0*mz**2*COS(2.D0*beta)
     &              *sw2*Unit3(i,j)
               tmp3=tmp3+SQRT(2.D0)*SIN(beta)*mw
     &              *DCONJG(AUp(i,j))/gw
               tmp3=tmp3-DCONJG(mubare(j,i))*muH/tanb
               MMatrix(i,j)=tmp1
               MMatrix(i+3,j+3)=tmp2
               MMatrix(i,j+3)=tmp3
               MMatrix(j+3,i)=DCONJG(MMatrix(i,j+3))
            End Do
         End Do
c Electron type
      Else If(type.eq.3) Then
         Do i=1,3
            Do j=1,3
               MMatrix(i,j)=MSoftLeL(i,j)+m_el(j)**2*Unit3(i,j)
     &              -1.D0/2.D0*mz**2*Cos(2.D0*beta)*sw2
     &              *Unit3(i,j)
               MMatrix(i,j+3)=SQRT(2.D0)*COS(beta)*mw
     &              *DCONJG(ALe(i,j))/gw
               MMatrix(i,j+3)=MMatrix(i,j+3)-m_el(j)*Unit3(i,j)*muH
     &              *tanb
               MMatrix(j+3,i)=DCONJG(MMatrix(i,j+3))
               MMatrix(i+3,j+3)=MSoftElR(i,j)+m_el(j)**2*Unit3(i,j)
     &              -mz**2*Cos(2.D0*beta)*sw2*Unit3(i,j)
            End Do
         End Do
c Neutrino type
      Else If(type.eq.4) Then
         Do i=1,3
            Do j=1,3
               MMatrix(i,j)=MSoftLeL(i,j)+1.D0/2.D0*mz**2*COS(2.D0*beta)
     &              *Unit3(i,j)
            End Do
         End Do
      End If

      If (debug.eq.1) Then
         Do i=1,dim
            Do j=1,dim
               Write(*,'(A,I2,A,I2,A,G15.7,G15.7)') 
     &              'M(',i,',',j,')=',MMatrix(i,j)
            End Do
         End Do
      End If

      Return
      End
