c**********************************************************************
      Subroutine vertices(debug,iOrd,iEWcor)

c Subroutine that calculates the vertices used in calculating Wilson
c coefficients. I use the Feynman rules of hep-ph/9511250 with the
c following differences in notation.
c \Gamma_{uL,R}  -->  \Gamma_{uL,R}^{*}
c A_u            -->  -A_u
c Yd             -->  -Yd
c v_i/SQRT{2}    -->  v_i
c Z_{+}^{ij}     -->  V^{ji*}
c Z_{-}^{ij}     -->  U^{ji}
c Z_N            -->  N^{ji*}
c Z_{\nu}        -->  Z_{\nu}^{*}
c**********************************************************************

      Implicit NONE

c Inputs
      Integer debug,iOrd,iEWcor

      Include 'insm.h'
      Include 'const.h'
      Include 'insoft.h'
      Include 'vars.h'
      Include 'vertex.h'

c Local
      Integer i,j,k,l,m,n,p
      Double Precision beta,thw,gsSU,gEM,gwEW,gwSU,gwSQ,gwQH,gsQG
      Double Precision ordfac
      Double Precision ZP(2,2), ZS(2,2)
      Double Complex eps2(2,2),tmp1,tmp2,tmp3,tmp4,tmp
      Double Complex SDLeft(3,3),SDRight(3,3),SULeft(3,3),SURight(3,3)

c Functions 
      Double Precision alphaStr,alphaEM,alpha2,MSrmass

c**********************************************************************
c Conversions between DR-bar and MS-bar quantities
c**********************************************************************

      If (iOrd.eq.2.or.iOrd.eq.3.or.iOrd.eq.4) Then
         ordfac=1.D0
      Else
         ordfac=0.D0
      End If

c NOTE in the conversion between MS-bar and DR-bar quantities I only
c retain O(alphaS) terms
      beta=ATAN(tanb)
      thw=ASIN(DSQRT(sw2))
c Gluon MS-bar coupling 
      gsSU=DSQRT(alphaStr(mususy,6,4)*4.D0*PI)

c Gluino MS-bar coupling C(f)=4/3 C(G)=3
      gsQG=DSQRT(alphaStr(mususy,6,4)*4.D0*PI)
     &     *(1.D0+3.D0*ordfac*alphaStr(mususy,6,4)/(6.D0*PI)
     &     -ordfac*alphaStr(mususy,6,4)/(6.D0*PI))

c EM coupling
      gEM=DSQRT(alphaEM(muew)*4.D0*PI)

c SU(2)_L MS-bar coupling at mususy
      gwSU=DSQRT(alpha2(mususy)*4.D0*PI)

c The gauge coupling that appears in the gaugino interactions involing
c squarks 
      gwSQ=DSQRT(alpha2(mususy)*4.D0*PI)
     &     *(1.D0-ordfac*alphaStr(mususy,6,4)/(6.D0*PI))

c The gauge coupling that appears in the Yukawa interation involving
c squarks and Higgsinos
      gwQH=DSQRT(alpha2(mususy)*4.D0*PI)
     &     *(1.D0-ordfac*alphaStr(mususy,6,4)/(2.D0*PI))

c SU(2)_L MS-bar coupling at muew
      gwEW=DSQRT(alpha2(muew)*4.D0*PI)

c**********************************************************************
c Some useful matrices
c**********************************************************************

c Higgs Matrices
      ZS(1,1)=COS(alpha_eff)
      ZS(1,2)=-SIN(alpha_eff)
      ZS(2,1)=SIN(alpha_eff)
      ZS(2,2)=COS(alpha_eff)

      ZP(1,1)=SIN(beta)
      ZP(1,2)=-COS(beta)
      ZP(2,1)=COS(beta)
      ZP(2,2)=SIN(beta)

c Antisymmetric 2x2 tensor
      eps2(1,1)=DCMPLX(0.D0,0.D0)
      eps2(1,2)=DCMPLX(-1.D0,0.D0)
      eps2(2,1)=DCMPLX(1.D0,0.D0)
      eps2(2,2)=DCMPLX(0.D0,0.D0)

c**********************************************************************
c The matrices S_L and S_R that appear when the heavy particles are
c integrated out.
c**********************************************************************

      If (iOrd.eq.1.or.iOrd.eq.3.or.iOrd.eq.4) Then
         Call c_Sigmas(debug,iEWcor)
         Do i=1,3
            Do j=1,3
               SDLeft(i,j)=Unit3(i,j)+1.D0/2.D0*SigmadVL(i,j)
               SDRight(i,j)=Unit3(i,j)+1.D0/2.D0*SigmadVR(i,j)
               SULeft(i,j)=Unit3(i,j)+1.D0/2.D0*SigmauVL(i,j)
               SURight(i,j)=Unit3(i,j)+1.D0/2.D0*SigmauVR(i,j)
            End Do
         End Do
      Else
         Do i=1,3
            Do j=1,3
               SDLeft(i,j)=Unit3(i,j)
               SDRight(i,j)=Unit3(i,j)
               SULeft(i,j)=Unit3(i,j)
               SURight(i,j)=Unit3(i,j)
            End Do
         End Do
      End If

c***********************************************************************
c Down-Quark-Squark-Gaugino/Higgsino Vertices
c***********************************************************************

c Gluino Vertices

c Note that colour algebra factors need to be included when calculating
c Wilson coefficients

      Do i=1,3
         Do j=1,6
            tmp1=DCMPLX(0.D0,0.D0)
            tmp2=DCMPLX(0.D0,0.D0)
            Do k=1,3
               tmp1=tmp1-DSQRT(2.D0)*gsQG*ZSD(j,k)*SDLeft(k,i)
               tmp2=tmp2+DSQRT(2.D0)*gsQG*ZSD(j,k+3)*SDRight(k,i)
            End Do
            gqsqL(i,j)=tmp1
            gqsqR(i,j)=tmp2
         End Do
      End Do

c Chargino Vertices
      Do i=1,3
         Do j=1,2
            Do k=1,6
               tmp1=DCMPLX(0.D0,0.D0)
               tmp2=DCMPLX(0.D0,0.D0)
               tmp3=DCMPLX(0.D0,0.D0)
               Do l=1,3
                  Do m=1,3
                     Do n=1,3
                        tmp1=tmp1+gwQH*ZSU(k,m+3)*mubare(m,l)
     &                       *KMbare(l,n)*SDLeft(n,i)*DCONJG(ZCV(j,2))
     &                       /(DSQRT(2.D0)*mw*SIN(beta))
                        tmp2=tmp2+gwQH
     &                       *ZSU(k,m)*KMbare(m,l)*DCONJG(mdbare(n,l))
     &                       *SDRight(n,i)*DCONJG(ZCU(j,2))
     &                       /(DSQRT(2.D0)*mw*COS(beta))
                     End Do
                     tmp3=tmp3-gwSQ*ZSU(k,l)*DCONJG(ZCV(j,1))
     &                    *KMbare(l,m)*SDLeft(m,i)
                  End Do
               End Do
               cqsqL(i,j,k)=tmp1+tmp3
               cqsqR(i,j,k)=tmp2
            End Do
         End Do
      End Do

c Neutralino Vertices
      Do i=1,3
         Do j=1,4
            Do k=1,6
               tmp1=DCMPLX(0.D0,0.D0)
               tmp2=DCMPLX(0.D0,0.D0)
               tmp3=DCMPLX(0.D0,0.D0)
               tmp4=DCMPLX(0.D0,0.D0)
               Do l=1,3
                  Do m=1,3
                     tmp1=tmp1-gwQH/(DSQRT(2.D0)*mw*COS(beta))
     &                    *ZSD(k,l+3)*mdbare(l,m)*SDLeft(m,i)
     &                    *DCONJG(ZN(j,3))
                     tmp2=tmp2-gwQH/(DSQRT(2.D0)*mw*COS(beta))*ZSD(k,l)
     &                    *DCONJG(mdbare(m,l))*SDRight(m,i)*ZN(j,3)
                  End Do
                  tmp3=tmp3-gwSQ/(DSQRT(2.D0)*COS(thw))*ZSD(k,l)
     &                 *SDLeft(l,i)*(1.D0/3.D0*DCONJG(ZN(j,1))*SIN(thw)
     &                 -DCONJG(ZN(j,2))*COS(thw))
                  tmp4=tmp4-DSQRT(2.D0)*gwSQ*TAN(thw)/3.D0
     &                 *ZSD(k,l+3)*SDRight(l,i)*ZN(j,1)
               End Do
               nqsqL(i,j,k)=tmp1+tmp3
               nqsqR(i,j,k)=tmp2+tmp4
            End Do
         End Do
      End Do

c***********************************************************************
c Debug Writes
c***********************************************************************

      If (debug.eq.1) Then
         Write(*,*) '**********************************************'
         Write(*,*) 'Down Quark Vertices'
         Write(*,*) '**********************************************'

         Do i=1,3
            Do j=1,6
               Write(*,'(A,I2,A,I2,A,G15.7,G15.7)') 
     &              'gqsqL(',i,',',j,')=',gqsqL(i,j)
               Write(*,'(A,I2,A,I2,A,G15.7,G15.7)') 
     &              'gqsqR(',i,',',j,')=',gqsqR(i,j)
            End Do
         End Do

         Do i=1,3
            Do j=1,2
               Do k=1,6
                  Write(*,'(A,I2,A,I2,A,I2,A,G15.7,G15.7)') 
     &                 'cqsqL(',i,',',j,',',k,')=',cqsqL(i,j,k)
                  Write(*,'(A,I2,A,I2,A,I2,A,G15.7,G15.7)') 
     &                 'cqsqR(',i,',',j,',',k,')=',cqsqR(i,j,k)
               End Do
            End Do
         End Do

         Do i=1,3
            Do j=1,4
               Do k=1,6
                  Write(*,'(A,I2,A,I2,A,I2,A,G15.7,G15.7)') 
     &                 'nqsqL(',i,',',j,',',k,')=',nqsqL(i,j,k)
                  Write(*,'(A,I2,A,I2,A,I2,A,G15.7,G15.7)') 
     &                 'nqsqR(',i,',',j,',',k,')=',nqsqR(i,j,k)
               End Do
            End Do
         End Do
      End If

c***********************************************************************
c Up-Quark-Squark-Gaugino/Higgsino Vertices
c***********************************************************************

c Gluino Vertices

c Note that colour algebra factors need to be included when calculating
c Wilson coefficients

      Do i=1,3
         Do j=1,6
            tmp1=DCMPLX(0.D0,0.D0)
            tmp2=DCMPLX(0.D0,0.D0)
            Do k=1,3
               tmp1=tmp1-DSQRT(2.D0)*gsQG*ZSU(j,k)*SULeft(k,i)
               tmp2=tmp2+DSQRT(2.D0)*gsQG*ZSU(j,k+3)*SURight(k,i)
            End Do
            gqsqLup(i,j)=tmp1
            gqsqRup(i,j)=tmp2
         End Do
      End Do

c Chargino Vertices
      Do i=1,3
         Do j=1,2
            Do k=1,6
               tmp1=DCMPLX(0.D0,0.D0)
               tmp2=DCMPLX(0.D0,0.D0)
               tmp3=DCMPLX(0.D0,0.D0)
               Do l=1,3
                  Do m=1,3
                     Do n=1,3
                        tmp1=tmp1+gwQH*ZSD(k,m+3)*mdbare(m,l)
     &                       *DCONJG(KMbare(n,l))*ZCU(j,2)*SULeft(n,i)
     &                       /(DSQRT(2.D0)*mw*COS(beta))
                        tmp2=tmp2+gwQH*ZSD(k,m)*DCONJG(KMbare(l,m))
     &                       *DCONJG(mubare(n,l))*ZCV(j,2)*SURight(n,i)
     &                       /(DSQRT(2.D0)*mw*SIN(beta))
                     End Do
                     tmp3=tmp3-gwSQ*ZSD(k,l)*ZCU(j,1)
     &                    *DCONJG(KMbare(i,l))*SULeft(m,i)
                  End Do
               End Do
               cqsqLup(i,j,k)=tmp1+tmp3
               cqsqRup(i,j,k)=tmp2
            End Do
         End Do
      End Do

c Neutralino Vertices
      Do i=1,3
         Do j=1,4
            Do k=1,6
               tmp1=DCMPLX(0.D0,0.D0)
               tmp2=DCMPLX(0.D0,0.D0)
               tmp3=DCMPLX(0.D0,0.D0)
               tmp4=DCMPLX(0.D0,0.D0)
               Do l=1,3
                  Do m=1,3
                     tmp1=tmp1-gwQH/(DSQRT(2.D0)*mw*SIN(beta))
     &                    *ZSU(k,l+3)
     &                    *mubare(l,m)*SULeft(m,i)*DCONJG(ZN(j,4))
                     tmp2=tmp2-gwQH/(DSQRT(2.D0)*mw*SIN(beta))*ZSU(k,l)
     &                    *DCONJG(mubare(m,l))*SURight(m,i)*ZN(j,4)
                  End Do
                  tmp3=tmp3-gwSQ/(DSQRT(2.D0)*COS(thw))*ZSU(k,l)
     &                 *SULeft(l,i)*(1.D0/3.D0*DCONJG(ZN(j,1))*SIN(thw)
     &                 +DCONJG(ZN(j,2))*COS(thw))
                  tmp4=tmp4-2.D0*DSQRT(2.D0)
     &                 *gwSQ*TAN(thw)/3.D0*ZSU(k,l+3)*SURight(l,i)
     &                 *ZN(j,1)
               End Do
               nqsqLup(i,j,k)=tmp1+tmp3
               nqsqRup(i,j,k)=tmp2+tmp4
            End Do
         End Do
      End Do

c***********************************************************************
c Debug Writes
c***********************************************************************
      If (debug.eq.1) Then
         Write(*,*) '**********************************************'
         Write(*,*) 'Up Quark Vertices'
         Write(*,*) '**********************************************'

         Do i=1,3
            Do j=1,6
               Write(*,'(A,I2,A,I2,A,G15.7,G15.7)') 
     &              'gqsqLup(',i,',',j,')=',gqsqLup(i,j)
               Write(*,'(A,I2,A,I2,A,G15.7,G15.7)') 
     &              'gqsqRup(',i,',',j,')=',gqsqRup(i,j)
            End Do
         End Do

         Do i=1,3
            Do j=1,2
               Do k=1,6
                  Write(*,'(A,I2,A,I2,A,I2,A,G15.7,G15.7)') 
     &                 'cqsqLup(',i,',',j,',',k,')=',cqsqLup(i,j,k)
                  Write(*,'(A,I2,A,I2,A,I2,A,G15.7,G15.7)') 
     &                 'cqsqRup(',i,',',j,',',k,')=',cqsqRup(i,j,k)
               End Do
            End Do
         End Do

         Do i=1,3
            Do j=1,4
               Do k=1,6
                  Write(*,'(A,I2,A,I2,A,I2,A,G15.7,G15.7)') 
     &                 'nqsqLup(',i,',',j,',',k,')=',nqsqLup(i,j,k)
                  Write(*,'(A,I2,A,I2,A,I2,A,G15.7,G15.7)') 
     &                 'nqsqRup(',i,',',j,',',k,')=',nqsqRup(i,j,k)
               End Do
            End Do
         End Do
      End If

c***********************************************************************
c Lepton-Slepton-Gaugino/Higgsino Vertices
c***********************************************************************

c Chargino Vertices

      Do i=1,3
         Do j=1,2
            Do k=1,3
               tmp=DCMPLX(0.D0,0.D0)
               tmp=-gwSU*DCONJG(ZCV(j,1))*ZSN(k,i)
               clslL(i,j,k)=tmp
               tmp=DCMPLX(0.D0,0.D0)
               tmp=gwSU*m_el(i)/(2.D0*mw*COS(beta))*DCONJG(ZCU(j,2))
     &              *ZSN(k,i)
               clslR(i,j,k)=tmp
            End Do
         End Do
      End Do

c Neutralino Vertices

      Do i=1,3
         Do j=1,4
            Do k=1,6
               tmp=DCMPLX(0.D0,0.D0)
               tmp=tmp+gwSU/(DSQRT(2.D0)*COS(thw))*ZSE(k,i)
     &              *(DCONJG(ZN(j,1))*SIN(thw)+DCONJG(ZN(j,2))*COS(thw))
     &              -gwSU*m_el(i)/(2.D0*mw*COS(beta))*ZSE(k,i+3)
     &              *DCONJG(ZN(j,3))
               nlslL(i,j,k)=tmp
               tmp=DCMPLX(0.D0,0.D0)
               tmp=tmp-gwSU*TAN(thw)*DSQRT(2.D0)*ZSE(k,i+3)*ZN(j,1)
     &              -gwSU*m_el(i)/(2.D0*mw*COS(beta))*ZSE(k,i)*ZN(j,3)
               nlslR(i,j,k)=tmp
            End Do
         End Do
      End Do

c***********************************************************************
c Debug Writes
c***********************************************************************

      If (debug.eq.1) Then
         Write(*,*) '**********************************************'
         Write(*,*) 'Lepton Vertices'
         Write(*,*) '**********************************************'

         Do i=1,3
            Do j=1,2
               Do k=1,3
                  Write(*,'(A,I2,A,I2,A,I2,A,G15.7,G15.7)') 
     &                 'clslL(',i,',',j,',',k,')=',clslL(i,j,k)
                  Write(*,'(A,I2,A,I2,A,I2,A,G15.7,G15.7)') 
     &                 'clslR(',i,',',j,',',k,')=',clslR(i,j,k)
               End Do
            End Do
         End Do

         Do i=1,3
            Do j=1,4
               Do k=1,6
                  Write(*,'(A,I2,A,I2,A,I2,A,G15.7,G15.7)') 
     &                 'nlslL(',i,',',j,',',k,')=',nlslL(i,j,k)
                  Write(*,'(A,I2,A,I2,A,I2,A,G15.7,G15.7)') 
     &                 'nlslR(',i,',',j,',',k,')=',nlslR(i,j,k)
               End Do
            End Do
         End Do
      End If

c***********************************************************************
c Neutral Higgs Vertices
c***********************************************************************

c Higgs - Down Type Squark vertices
c Scalar
      Do i=1,2
         Do j=1,6
            Do k=1,6
               tmp1=DCMPLX(0.D0,0.D0)
               tmp2=DCMPLX(0.D0,0.D0)
               tmp3=DCMPLX(0.D0,0.D0)
               tmp4=DCMPLX(0.D0,0.D0)
               Do l=1,3
                  Do m=1,3
                     Do n=1,3
                        tmp1=tmp1-gwSU/(mw*COS(beta))*ZS(1,i)
     &                    *(ZSD(k,l)*DCONJG(mdbare(m,l))
     &                    *mdbare(m,n)*DCONJG(ZSD(j,n))
     &                    +ZSD(k,l+3)*mdbare(l,m)
     &                    *DCONJG(mdbare(n,m))*DCONJG(ZSD(j,n+3)))
                     End Do
                     tmp2=tmp2-1.D0/DSQRT(2.D0)*ZS(1,i)*
     &                    (DCONJG(ADo(l,m))*ZSD(k,l)*DCONJG(ZSD(j,m+3))
     &                    +ADo(l,m)*ZSD(k,m+3)*DCONJG(ZSD(j,l)))
                     tmp3=tmp3+gwSU/(2.D0*mw*COS(beta))*ZS(2,i)
     &                    *(DCONJG(muH)*ZSD(k,l+3)*mdbare(l,m)
     &                    *CONJG(ZSD(j,m))
     &                    +muH*ZSD(k,m)*DCONJG(mdbare(l,m))
     &                    *CONJG(ZSD(j,l+3)))
                  End Do
                  tmp4=tmp4+gwSU*mw*TAN(thw)**2/(3.D0)
     &                 *(COS(beta)*ZS(1,i)-SIN(beta)*ZS(2,i))
     &                 *(Unit6(j,k)-((3.D0-4.D0*sw2)/(2.D0*sw2))
     &                 *DCONJG(ZSD(j,l))*ZSD(k,l))
               End Do
               SHdsqsq(i,j,k)=tmp2+tmp3
     &              +tmp4
     &              +tmp1
            End Do
         End Do
      End Do

c Pseudoscalar
      Do i=1,2
         Do j=1,6
            Do k=1,6
               tmp1=DCMPLX(0.D0,0.D0)
               tmp2=DCMPLX(0.D0,0.D0)
               Do l=1,3
                  Do m=1,3
                     tmp1=tmp1+1.D0/DSQRT(2.D0)*ZP(1,i)*
     &                    (DCONJG(ADo(l,m))*ZSD(k,l)*DCONJG(ZSD(j,m+3))
     &                    -ADo(l,m)*ZSD(k,m+3)*DCONJG(ZSD(j,l)))
                     tmp2=tmp2-gwSU/(2.D0*mw*COS(beta))*ZP(2,i)
     &                    *(DCONJG(muH)*ZSD(k,l+3)*mdbare(l,m)
     &                    *DCONJG(ZSD(j,m))
     &                    -muH*ZSD(k,m)*DCONJG(mdbare(l,m))
     &                    *DCONJG(ZSD(j,l+3)))
                  End Do
               End Do
               PHdsqsq(i,j,k)=(tmp1+tmp2)*DCMPLX(0.D0,1.D0)
            End Do
         End Do
      End Do

c Higgs Up type Squark Vertices
c Scalar
      Do i=1,2
         Do j=1,6
            Do k=1,6
               tmp1=DCMPLX(0.D0,0.D0)
               tmp2=DCMPLX(0.D0,0.D0)
               tmp3=DCMPLX(0.D0,0.D0)
               tmp4=DCMPLX(0.D0,0.D0)
               Do l=1,3
                  Do m=1,3
                     Do n=1,3
                        tmp1=tmp1-gwSU/(mw*SIN(beta))*ZS(2,i)
     &                       *(ZSU(k,l)*DCONJG(mubare(m,l))
     &                       *mubare(m,n)*DCONJG(ZSU(j,n))
     &                       +ZSU(k,l+3)*mubare(l,m)
     &                       *DCONJG(mubare(n,m))*DCONJG(ZSU(j,n+3)))
                     End Do
                     tmp2=tmp2-1.D0/DSQRT(2.D0)*ZS(2,i)*
     &                    (DCONJG(AUp(l,m))*ZSU(k,l)*DCONJG(ZSU(j,m+3))
     &                    +AUp(l,m)*ZSU(k,m+3)*DCONJG(ZSU(j,l)))
                     tmp3=tmp3+gwSU/(2.D0*mw*SIN(beta))*ZS(1,i)
     &                    *(DCONJG(muH)*ZSU(k,l+3)*mubare(l,m)
     &                    *DCONJG(ZSU(j,m))
     &                    +muH*ZSU(k,m)*DCONJG(mubare(l,m))
     &                    *DCONJG(ZSU(j,l+3)))
                  End Do
                  tmp4=tmp4-2.D0*gwSU*mw*TAN(thw)**2/(3.D0)
     &                 *(COS(beta)*ZS(1,i)-SIN(beta)*ZS(2,i))
     &                 *(Unit6(j,k)+((3.D0-8.D0*sw2)/(4.D0*sw2))
     &                 *DCONJG(ZSU(j,l))*ZSU(k,l))
               End Do
               SHusqsq(i,j,k)=tmp2+tmp3
     &              +tmp4
     &              +tmp1
            End Do
         End Do
      End Do

c Pseudoscalar
      Do i=1,2
         Do j=1,6
            Do k=1,6
               tmp1=DCMPLX(0.D0,0.D0)
               tmp2=DCMPLX(0.D0,0.D0)
               Do l=1,3
                  Do m=1,3
                     tmp1=tmp1+1.D0/DSQRT(2.D0)*ZP(1,i)*
     &                    (DCONJG(AUp(l,m))*ZSU(k,l)*DCONJG(ZSU(j,m+3))
     &                    -AUp(l,m)*ZSU(k,m+3)*DCONJG(ZSU(j,l)))
                     tmp2=tmp2-gwSU/(2.D0*mw*SIN(beta))*ZP(1,i)
     &                    *(DCONJG(muH)*ZSU(k,l+3)*mubare(l,m)
     &                    *DCONJG(ZSU(j,m))
     &                    -muH*ZSU(k,m)*DCONJG(mubare(l,m))
     &                    *DCONJG(ZSU(j,l+3)))
                  End Do
               End Do
               PHusqsq(i,j,k)=(tmp1+tmp2)*DCMPLX(0.D0,1.D0)
            End Do
         End Do
      End Do

c Higgs Chargino Chargino Vertices

c Scalar
      Do i=1,2
         Do j=1,2
            Do k=1,2
               SHchchL(i,j,k)=-gwSU/DSQRT(2.D0)
     &              *(ZS(1,i)*ZCU(j,2)*DCONJG(ZCV(k,1))
     &              +ZS(2,i)*ZCU(j,1)*DCONJG(ZCV(k,2)))
               SHchchR(i,j,k)=-gwSU/DSQRT(2.D0)
     &              *(ZS(1,i)*DCONJG(ZCU(k,2))*ZCV(j,1)
     &              +ZS(2,i)*DCONJG(ZCU(k,1))*ZCV(j,2))
            End Do
         End Do
      End Do

c Pseudoscalar
      Do i=1,2
         Do j=1,2
            Do k=1,2
               PHchchL(i,j,k)=gwSU/DSQRT(2.D0)
     &              *(ZP(1,i)*ZCU(j,2)*DCONJG(ZCV(k,1))
     &              +ZP(2,i)*ZCU(j,1)*DCONJG(ZCV(k,2)))
     &              *DCMPLX(0.D0,1.D0)
               PHchchR(i,j,k)=-gwSU/DSQRT(2.D0)
     &              *(ZP(1,i)*DCONJG(ZCU(k,2))*ZCV(j,1)
     &              +ZP(2,i)*DCONJG(ZCU(k,1))*ZCV(j,2))
     &              *DCMPLX(0.D0,1.D0)
            End Do
         End Do
      End Do

c Higgs Neutralino Neutralino Vertices

c Scalar
      Do i=1,2
         Do j=1,4
            Do k=1,4
               tmp=DCMPLX(0.D0,0.D0)
               tmp=tmp+gwSU/(2.D0*COS(thw))*
     &              ((ZS(1,i)*DCONJG(ZN(j,3))-ZS(2,i)*DCONJG(ZN(j,4)))
     &              *(DCONJG(ZN(k,1))*SIN(thw)-DCONJG(ZN(k,2))*COS(thw))
     &              +(ZS(1,i)*DCONJG(ZN(k,3))-ZS(2,i)*DCONJG(ZN(k,4)))
     &              *(DCONJG(ZN(j,1))*SIN(thw)
     &              -DCONJG(ZN(j,2))*COS(thw)))
               SHneneL(i,j,k)=tmp
               tmp=DCMPLX(0.D0,0.D0)
               tmp=tmp+gwSU/(2.D0*COS(thw))*
     &              ((ZS(1,i)*ZN(k,3)-ZS(2,i)*ZN(k,4))
     &              *(ZN(j,1)*SIN(thw)-ZN(j,2)*COS(thw))
     &              +(ZS(1,i)*ZN(j,3)-ZS(2,i)*ZN(j,4))
     &              *(ZN(k,1)*SIN(thw)-ZN(k,2)*COS(thw)))
               SHneneR(i,j,k)=tmp
            End Do
         End Do
      End Do

c Pseudoscalar
      Do i=1,2
         Do j=1,4
            Do k=1,4
               tmp=DCMPLX(0.D0,0.D0)
               tmp=tmp-gwSU/(2.D0*COS(thw))*
     &              ((ZP(1,i)*DCONJG(ZN(j,3))-ZP(2,i)*DCONJG(ZN(j,4)))
     &              *(DCONJG(ZN(k,1))*SIN(thw)-DCONJG(ZN(k,2))*COS(thw))
     &              +(ZP(1,i)*DCONJG(ZN(k,3))-ZP(2,i)*DCONJG(ZN(k,4)))
     &              *(DCONJG(ZN(j,1))*SIN(thw)
     &              -DCONJG(ZN(j,2))*COS(thw)))
               PHneneL(i,j,k)=tmp*DCMPLX(0.D0,1.D0)
               tmp=DCMPLX(0.D0,0.D0)
               tmp=tmp+gwSU/(2.D0*COS(thw))*
     &              ((ZP(1,i)*ZN(k,3)-ZP(2,i)*ZN(k,4))
     &              *(ZN(j,1)*SIN(thw)-ZN(j,2)*COS(thw))
     &              +(ZP(1,i)*ZN(j,3)-ZP(2,i)*ZN(j,4))
     &              *(ZN(k,1)*SIN(thw)-ZN(k,2)*COS(thw)))
               PHneneR(i,j,k)=tmp*DCMPLX(0.D0,1.D0)
            End Do
         End Do
      End Do

c Higgs-Charged Higgs-Charged Higgs Vertices

      Do i=1,2
         Do j=1,2
            Do k=1,2
               tmp=DCMPLX(0.D0,0.D0)
               tmp=tmp-gwEW/(2.D0*COS(thw))*mz
     &              *(ZP(1,j)*ZP(1,k)-ZP(2,j)*ZP(2,k))
     &              *(COS(beta)*ZS(1,i)-SIN(beta)*ZS(2,i))
     &              -gwEW*mw
     &              *((ZS(1,i)*ZP(2,k)-ZS(2,i)*ZP(1,k))*Unit2(1,j)
     &              +(ZS(1,i)*ZP(2,j)-ZS(2,i)*ZP(1,j))*Unit2(1,k))
               SHCHi(i,j,k)=tmp
            End Do
         End Do
      End Do

      Do i=1,2
         Do j=1,2
            Do k=1,2
               tmp=DCMPLX(0.D0,0.D0)
               tmp=tmp-gwEW*mw/(2.D0)*Eps2(j,k)*Unit2(1,i)
               PHCHi(i,j,k)=tmp*DCMPLX(0.D0,1.D0)
            End Do
         End Do
      End Do

c Higgs quark vertices

      Do i=1,2
         Do j=1,3
            SHqqL(i,j)=-gwEW/(2.D0*mw*COS(beta))*ZS(1,i)
     &           *MSrmass(muew,2*j-1,5,4)
            SHqqR(i,j)=-gwEW/(2.D0*mw*COS(beta))*ZS(1,i)
     &           *MSrmass(muew,2*j-1,5,4)
            SHuqqL(i,j)=-gwEW/(2.D0*mw*SIN(beta))*ZS(1,i)
     &           *MSrmass(muew,2*j,5,4)
            SHuqqR(i,j)=-gwEW/(2.D0*mw*SIN(beta))*ZS(1,i)
     &           *MSrmass(muew,2*j,5,4)
         End Do
      End Do

      Do i=1,2
         Do j=1,3
            PHqqL(i,j)=-gwEW/(2.D0*mw*COS(beta))*ZP(1,i)
     &           *MSrmass(muew,2*j-1,5,4)
     &           *DCMPLX(0.D0,1.D0)
            PHqqR(i,j)=gwEW/(2.D0*mw*COS(beta))*ZP(1,i)
     &           *MSrmass(muew,2*j-1,5,4)
     &           *DCMPLX(0.D0,1.D0)
            PHuqqL(i,j)=-gwEW/(2.D0*mw*SIN(beta))*ZP(1,i)
     &           *MSrmass(muew,2*j,5,4)
     &           *DCMPLX(0.D0,1.D0)
            PHuqqR(i,j)=gwEW/(2.D0*mw*SIN(beta))*ZP(1,i)
     &           *MSrmass(muew,2*j,5,4)
     &           *DCMPLX(0.D0,1.D0)
         End Do
      End Do

c Effective Higgs Quark Quark Vertices

      Call NHiggsVertex(debug,iEWcor)

c Higgs Lepton Lepton Vertices
      Do i=1,2
         Do j=1,3
            SHll(i,j)=-gEM/(2.D0*COS(beta)*mw*SIN(thw))*m_el(j)*ZS(1,i)
            PHll(i,j)=gEM/(2.D0*COS(beta)*mw*SIN(thw))*m_el(j)*ZP(1,i)
     &           *DCMPLX(0D0,1D0)
         End Do
      End Do

c***********************************************************************
c Debug Writes
c***********************************************************************

      If (debug.eq.1) Then

         Write(*,*) '**********************************************'
         Write(*,*) 'Neutral Higgs Vertices'
         Write(*,*) '**********************************************'

         Do i=1,2
            Do j=1,6
               Do k=1,6
                  Write(*,'(A,I2,A,I2,A,I2,A,G15.7,G15.7)') 
     &                 'SHdsqsq(',i,',',j,',',k,')=',SHdsqsq(i,j,k)
               End Do
            End Do
         End Do

         Do i=1,2
            Do j=1,6
               Do k=1,6
                  Write(*,'(A,I2,A,I2,A,I2,A,G15.7,G15.7)') 
     &                 'PHdsqsq(',i,',',j,',',k,')=',PHdsqsq(i,j,k)
               End Do
            End Do
         End Do

         Do i=1,2
            Do j=1,6
               Do k=1,6
                  Write(*,'(A,I2,A,I2,A,I2,A,G15.7,G15.7)') 
     &                 'SHusqsq(',i,',',j,',',k,')=',SHusqsq(i,j,k)
               End Do
            End Do
         End Do

         Do i=1,2
            Do j=1,6
               Do k=1,6
                  Write(*,'(A,I2,A,I2,A,I2,A,G15.7,G15.7)') 
     &                 'PHusqsq(',i,',',j,',',k,')=',PHusqsq(i,j,k)
               End Do
            End Do
         End Do

         Do i=1,2
            Do j=1,2
               Do k=1,2
                  Write(*,'(A,I2,A,I2,A,I2,A,G15.7,G15.7)') 
     &                 'SHchchL(',i,',',j,',',k,')=',SHchchL(i,j,k)
                  Write(*,'(A,I2,A,I2,A,I2,A,G15.7,G15.7)') 
     &                 'SHchchR(',i,',',j,',',k,')=',SHchchR(i,j,k)
               End Do
            End Do
         End Do

         Do i=1,2
            Do j=1,2
               Do k=1,2
                  Write(*,'(A,I2,A,I2,A,I2,A,G15.7,G15.7)') 
     &                 'PHchchL(',i,',',j,',',k,')=',PHchchL(i,j,k)
                  Write(*,'(A,I2,A,I2,A,I2,A,G15.7,G15.7)') 
     &                 'PHchchR(',i,',',j,',',k,')=',PHchchR(i,j,k)
               End Do
            End Do
         End Do


         Do i=1,2
            Do j=1,4
               Do k=1,4
                  Write(*,'(A,I2,A,I2,A,I2,A,G15.7,G15.7)') 
     &                 'SHneneL(',i,',',j,',',k,')=',SHneneL(i,j,k)
                  Write(*,'(A,I2,A,I2,A,I2,A,G15.7,G15.7)') 
     &                 'SHneneR(',i,',',j,',',k,')=',SHneneR(i,j,k)
               End Do
            End Do
         End Do

         Do i=1,2
            Do j=1,4
               Do k=1,4
                  Write(*,'(A,I2,A,I2,A,I2,A,G15.7,G15.7)') 
     &                 'PHneneL(',i,',',j,',',k,')=',PHneneL(i,j,k)
                  Write(*,'(A,I2,A,I2,A,I2,A,G15.7,G15.7)') 
     &                 'PHneneR(',i,',',j,',',k,')=',PHneneR(i,j,k)
               End Do
            End Do
         End Do


         Do i=1,2
            Do j=1,3
               Write(*,'(A,I2,A,I2,A,G15.7,G15.7)') 
     &              'SHqqL(',i,',',j,')=',SHqqL(i,j)
               Write(*,'(A,I2,A,I2,A,G15.7,G15.7)') 
     &              'SHqqR(',i,',',j,')=',SHqqR(i,j)
            End Do
         End Do

         Do i=1,2
            Do j=1,3
               Write(*,'(A,I2,A,I2,A,G15.7,G15.7)') 
     &              'PHqqL(',i,',',j,')=',PHqqL(i,j)
               Write(*,'(A,I2,A,I2,A,G15.7,G15.7)') 
     &              'PHqqR(',i,',',j,')=',PHqqR(i,j)
            End Do
         End Do

         Do i=1,2
            Do j=1,3
               Do k=1,3
                  Write(*,'(A,I2,A,I2,A,I2,A,G15.7,G15.7)') 
     &                 'SHqqLCor(',i,',',j,',',k,')=',SHqqLCor(i,j,k)
                  Write(*,'(A,I2,A,I2,A,I2,A,G15.7,G15.7)') 
     &                 'SHqqRCor(',i,',',j,',',k,')=',SHqqRCor(i,j,k)
               End Do
            End Do
         End Do

         Do i=1,2
            Do j=1,3
               Do k=1,3
                  Write(*,'(A,I2,A,I2,A,I2,A,G15.7,G15.7)') 
     &                 'PHqqLCor(',i,',',j,',',k,')=',PHqqLCor(i,j,k)
                  Write(*,'(A,I2,A,I2,A,I2,A,G15.7,G15.7)') 
     &                 'PHqqRCor(',i,',',j,',',k,')=',PHqqRCor(i,j,k)
               End Do
            End Do
         End Do


         Do i=1,2
            Do j=1,3
               Write(*,'(A,I2,A,I2,A,G15.7,G15.7)') 
     &              'SHll(',i,',',j,')=',SHll(i,j)
               Write(*,'(A,I2,A,I2,A,G15.7,G15.7)') 
     &              'PHll(',i,',',j,')=',PHll(i,j)
            End Do
         End Do
      End If

c***********************************************************************
c Z Boson Vertices
c***********************************************************************

c Z Down Squark Vertices

      Do i=1,6
         Do j=1,6
            tmp=DCMPLX(0.D0,0.D0)
            Do k=1,3
               tmp=tmp+gwSU/(2.D0*COS(thw))*(ZSD(j,k)*DCONJG(ZSD(i,k)))
            End Do
            tmp=tmp-gwSU/(2.D0*COS(thw))*(2.D0/3.D0*sw2)*Unit6(i,j)
            Zdsqsq(i,j)=tmp
         End Do
      End Do

c Z Up Squark Vertices

      Do i=1,6
         Do j=1,6
            tmp=DCMPLX(0.D0,0.D0)
            Do k=1,3
               tmp=tmp-gwSU/(2.D0*COS(thw))*(ZSU(j,k)*DCONJG(ZSU(i,k)))
            End Do
            tmp=tmp+gwSU/(2.D0*COS(thw))*(4.D0/3.D0*sw2)*Unit6(i,j)
            Zusqsq(i,j)=tmp
         End Do
      End Do

c Z Chargino Vertices

      Do i=1,2
         Do j=1,2
            tmp=DCMPLX(0.D0,0.D0)
            tmp=tmp+gwSU/(2.D0*COS(thw))*(ZCV(i,1)*DCONJG(ZCV(j,1))
     &           +(COS(thw)**2-SIN(thw)**2)*Unit2(i,j))
            ZchchR(i,j)=tmp
            tmp=DCMPLX(0.D0,0.D0)
            tmp=tmp+gwSU/(2.D0*COS(thw))*(ZCU(i,1)*DCONJG(ZCU(j,1))
     &           +(COS(thw)**2-SIN(thw)**2)*Unit2(i,j))
            ZchchL(i,j)=tmp
         End Do
      End Do

c Z Neutralino Vertices

      Do i=1,4
         Do j=1,4
            tmp=DCMPLX(0.D0,0.D0)
            tmp=tmp+gwSU/(2.D0*COS(thw))*(ZN(j,4)*DCONJG(ZN(i,4))
     &           -ZN(j,3)*DCONJG(ZN(i,3)))
            ZneneL(i,j)=tmp
            tmp=DCMPLX(0.D0,0.D0)
            tmp=tmp-gwSU/(2.D0*COS(thw))*(DCONJG(ZN(j,4))*ZN(i,4)
     &           -DCONJG(ZN(j,3))*ZN(i,3))
            ZneneR(i,j)=tmp
         End Do
      End Do

c Z Charged Higgs Vertex

      Do i=1,2
         tmp=DCMPLX(0.D0,0.D0)
         tmp=tmp+gwEW/(2.D0*COS(thw))*(COS(thw)**2-SIN(thw)**2)
         ZCHi(i)=tmp
      End Do

c Z Quark Vertices
      
      Do i=1,3
         ZdqqL(i)=gwEW/(2.D0*COS(thw))*(1.D0-2.D0/3.D0*sw2)
         ZdqqR(i)=-gwEW/(2.D0*COS(thw))*(2.D0/3.D0*sw2)
         ZuqqL(i)=-gwEW/(2.D0*COS(thw))*(1.D0-4.D0/3.D0*sw2)
         ZuqqR(i)=gwEW/(2.D0*COS(thw))*(4.D0/3.D0*sw2)
      End Do

c Effective Z-quark-quark vertex

      Call ZVertex(debug,iEWcor)

c Z Lepton Vertex

      Do i=1,3
         ZleleA(i)=-gwEW/(4.D0*COS(thw))
      End Do

c***********************************************************************
c Debug Writes
c***********************************************************************

      If (debug.eq.1) Then

         Write(*,*) '**********************************************'
         Write(*,*) 'Z Boson Vertices'
         Write(*,*) '**********************************************'

         Do i=1,6
            Do j=1,6
               Write(*,'(A,I2,A,I2,A,G15.7,G15.7)') 
     &              'Zdsqsq(',i,',',j,')=',Zdsqsq(i,j)
            End Do
         End Do

         Do i=1,6
            Do j=1,6
               Write(*,'(A,I2,A,I2,A,G15.7,G15.7)') 
     &              'Zusqsq(',i,',',j,')=',Zusqsq(i,j)
            End Do
         End Do

         Do i=1,2
            Do j=1,2
               Write(*,'(A,I2,A,I2,A,G15.7,G15.7)') 
     &              'ZchchL(',i,',',j,')=',ZchchL(i,j)
               Write(*,'(A,I2,A,I2,A,G15.7,G15.7)') 
     &              'ZchchR(',i,',',j,')=',ZchchR(i,j)
            End Do
         End Do

         Do i=1,4
            Do j=1,4
               Write(*,'(A,I2,A,I2,A,G15.7,G15.7)') 
     &              'ZneneL(',i,',',j,')=',ZneneL(i,j)
               Write(*,'(A,I2,A,I2,A,G15.7,G15.7)') 
     &              'ZnenR(',i,',',j,')=',ZneneR(i,j)
            End Do
         End Do

         Do i=1,3
            Do j=1,3
               Write(*,'(A,I2,A,I2,A,G15.7,G15.7)') 
     &              'ZqqLCor(',i,',',j,')=',ZdqqLCor(i,j)
               Write(*,'(A,I2,A,I2,A,G15.7,G15.7)') 
     &              'ZqqRCor(',i,',',j,')=',ZdqqRCor(i,j)
            End Do
         End Do

         Do i=1,3
            Write(*,'(A,I2,A,G15.7,G15.7)') 
     &           'ZdqqL(',i,')=',ZdqqL(i)
            Write(*,'(A,I2,A,G15.7,G15.7)') 
     &           'ZdqqR(',i,')=',ZdqqR(i)
         End Do

         Do i=1,3
            Write(*,'(A,I2,A,G15.7,G15.7)') 
     &           'ZuqqL(',i,')=',ZuqqL(i)
            Write(*,'(A,I2,A,G15.7,G15.7)') 
     &           'ZuqqR(',i,')=',ZuqqR(i)
         End Do


         Do i=1,3
            Write(*,'(A,I2,A,G15.7,G15.7)') 
     &           'ZleleA(',i,')=',ZleleA(i)
         End Do
      End If

c***********************************************************************
c Charged Higgs
c***********************************************************************
      
c Charged Higgs down-squark up-squark

      Do i=1,2
         Do j=1,6
            Do k=1,6
               tmp1=DCMPLX(0.D0,0.D0)
               tmp2=DCMPLX(0.D0,0.D0)
               tmp3=DCMPLX(0.D0,0.D0)
               tmp4=DCMPLX(0.D0,0.D0)
               Do l=1,3
                  Do m=1,3
                     Do n=1,3
                        Do p=1,3
                           tmp3=tmp3+gwSU/(SQRT(2.D0)*mw*COS(beta))
     &                          *(ZSU(k,l)*KMbare(l,m)
     &                          *DCONJG(mdbare(n,m))*mdbare(n,p)
     &                          *DCONJG(ZSD(j,p)))*ZP(1,i)
     &                          +gwSU/(SQRT(2.D0)*mw*SIN(beta))
     &                          *(ZSU(k,l)*DCONJG(mubare(m,l))
     &                          *mubare(m,n)*KMbare(n,p)
     &                          *DCONJG(ZSD(j,p)))*ZP(2,i)
     &                          +gwSU*Unit2(1,i)
     &                          /(SQRT(2.D0)*mw*COS(beta)*SIN(beta))
     &                          *(ZSU(k,l+3)*mubare(l,m)*KMbare(m,n)
     &                          *DCONJG(mdbare(p,n))*DCONJG(ZSD(j,p+3)))
                        End Do
                        tmp1=tmp1+ZP(1,i)
     &                       *(DCONJG(muH)
     &                       *gwSU/(DSQRT(2.D0)*mw*SIN(beta))
     &                       *ZSU(k,l+3)*mubare(l,m)
     &                       *KMbare(m,n)*DCONJG(ZSD(j,n))
     &                       +ZSU(k,l)*KMbare(l,m)
     &                       *DCONJG(ADo(m,n))*DCONJG(ZSD(j,n+3)))
     &                       +ZP(2,i)
     &                       *(muH*gwSU/(DSQRT(2.D0)*mw*COS(beta))
     &                       *ZSU(k,l)*KMbare(l,m)
     &                       *DCONJG(mdbare(n,m))*DCONJG(ZSD(j,n+3))
     &                       +ZSU(k,l+3)*AUp(m,l)
     &                       *KMbare(m,n)*DCONJG(ZSD(j,n)))
                     End Do
                     tmp4=tmp4-gwSU*mw/(DSQRT(2.D0)*SIN(thw))
     &                    *(COS(beta)*ZP(1,i)+SIN(beta)*ZP(2,i))
     &                    *ZSU(k,l)*KMbare(l,m)*DCONJG(ZSD(j,m))
                  End Do
               End Do
               CHdsqusq(i,j,k)=tmp1+tmp3+tmp4
           End Do
         End Do
      End Do

c Charged Higgs-neutralino-chargino

      Do i=1,2
         Do j=1,2
            Do k=1,4
               CHchneL(i,j,k)=gwSU/(COS(thw))*ZP(1,i)*(1.D0/DSQRT(2.D0)
     &              *ZCU(j,2)*(DCONJG(ZN(k,1))*SIN(thw)
     &              +DCONJG(ZN(k,2))*COS(thw))-ZCU(j,1)*DCONJG(ZN(k,3)))
               CHchneR(i,j,k)=-gwSU/(COS(thw))*ZP(2,i)*(1.D0/DSQRT(2.D0)
     &              *ZCV(j,2)*(ZN(k,1)*SIN(thw)
     &              +ZN(k,2)*COS(thw))+ZCV(j,1)*ZN(k,4))

            End Do
         End Do
      End Do

c Charged Higgs quark quark vertex

      Do i=1,2
         Do j=1,3
            Do k=1,3
               CHqqL(i,j,k)=gwEW*ZP(2,i)/(DSQRT(2.D0)*mw*SIN(beta))
     &              *KMbare(k,j)*MSrmass(muew,2*k,5,4)
               CHqqR(i,j,k)=gwEW*ZP(1,i)/(DSQRT(2.D0)*mw*COS(beta))
     &              *KMbare(k,j)*MSrmass(muew,2*j-1,5,4)
            End Do
         End Do
      End Do

c Charged Higgs neutrino lepton vertex

      Do i=1,2
         Do j=1,3
            CHnulR(i,j)=gwEW*ZP(2,i)/(DSQRT(2.D0)*mw*COS(beta))*m_el(j)
         End Do
      End Do

c Charged Higgs effective vertex

      Call CHiggsVertex(debug,iEWcor)

c***********************************************************************
c Debug Writes
c***********************************************************************

      If (debug.eq.1) Then

         Write(*,*) '**********************************************'
         Write(*,*) 'Charged Higgs Vertices'
         Write(*,*) '**********************************************'

         Do i=1,2
            Do j=1,6
               Do k=1,6
                  Write(*,'(A,I2,A,I2,A,I2,A,G15.7,G15.7)') 
     &                 'CHdsqusq(',i,',',j,',',k,')=',CHdsqusq(i,j,k)
               End Do
            End Do
         End Do

         Do i=1,2
            Do j=1,2
               Do k=1,4
                  Write(*,'(A,I2,A,I2,A,I2,A,G15.7,G15.7)') 
     &                 'CHchneL(',i,',',j,',',k,')=',CHchneL(i,j,k)
                  Write(*,'(A,I2,A,I2,A,I2,A,G15.7,G15.7)') 
     &                 'CHchneR(',i,',',j,',',k,')=',CHchneR(i,j,k)
               End Do
            End Do
         End Do

         Do i=1,2
            Do j=1,3
               Do k=1,3
                  Write(*,'(A,I2,A,I2,A,I2,A,G15.7,G15.7)') 
     &                 'CHqqL(',i,',',j,',',k,')=',CHqqL(i,j,k)
                  Write(*,'(A,I2,A,I2,A,I2,A,G15.7,G15.7)') 
     &                 'CHqqR(',i,',',j,',',k,')=',CHqqR(i,j,k)
               End Do
            End Do
         End Do

         Do i=1,2
            Do j=1,3
               Do k=1,3
                  Write(*,'(A,I2,A,I2,A,I2,A,G15.7,G15.7)') 
     &                 'CHqqLCor(',i,',',j,',',k,')=',CHqqLCor(i,j,k)
                  Write(*,'(A,I2,A,I2,A,I2,A,G15.7,G15.7)') 
     &                 'CHqqRCor(',i,',',j,',',k,')=',CHqqRCor(i,j,k)
               End Do
            End Do
         End Do

         Do i=1,2
            Do j=1,3
               Do k=1,3
                  Write(*,'(A,I2,A,I2,A,I2,A,G15.7,G15.7)') 
     &                 'delCHqqL(',i,',',j,',',k,')=',delCHqqL(i,j,k)
                  Write(*,'(A,I2,A,I2,A,I2,A,G15.7,G15.7)') 
     &                 'delCHqqR(',i,',',j,',',k,')=',delCHqqR(i,j,k)
               End Do
            End Do
         End Do
      End If

c***********************************************************************
c W Boson Vertices
c***********************************************************************

c W-Boson Chargino Neutralino vertex
      Do i=1,2
         Do j=1,4
            WChNeL(i,j)=gwSU*(DCONJG(ZN(j,2))*ZCV(i,1)
     &           -1.D0/DSQRT(2.D0)*DCONJG(ZN(j,4))*ZCV(i,2))
            WChNeR(i,j)=gwSU*(ZN(j,2)*ZCU(i,1)
     &           -1.D0/DSQRT(2.D0)*ZN(j,4)*ZCU(i,2))
         End Do
      End Do

c W-Boson Squark Squark vertex
      Do i=1,6
         Do j=1,6
            tmp=DCMPLX(0.D0,0.D0)
            Do k=1,3
               Do l=1,3
                  tmp=tmp-gwSU/DSQRT(2.D0)
     &                 *DCONJG(ZSD(i,k))*ZSU(j,l)*KMbare(l,k)
               End Do
            End Do
            Wdsqusq(i,j)=tmp
         End Do
      End Do

c W-Boson Quark Quark Vertex
      Do i=1,3
         Do j=1,3
            WqqL(i,j)=-KM(j,i)*gwEW/(DSQRT(2.D0))
         End Do
      End Do

c W-Boson neutralino lepton vertex
      Do i=1,3
            WnulL(i)=-gwEW/(DSQRT(2.D0))
      End Do

c Effective right handed coupling
      Call WVertex(debug,iEWcor)

c***********************************************************************
c Debug Writes
c***********************************************************************

      If (debug.eq.1) Then

         Write(*,*) '**********************************************'
         Write(*,*) 'W Boson Vertices'
         Write(*,*) '**********************************************'

         Do i=1,2
            Do j=1,4
               Write(*,'(A,I2,A,I2,A,G15.7,G15.7)') 
     &              'WChNeL(',i,',',j,')=',WChNeL(i,j)
               Write(*,'(A,I2,A,I2,A,G15.7,G15.7)') 
     &              'WChNeR(',i,',',j,')=',WChNeR(i,j)
            End Do
         End Do

         Do i=1,6
            Do j=1,6
               Write(*,'(A,I2,A,I2,A,G15.7,G15.7)') 
     &              'Wdsqusq(',i,',',j,')=',Wdsqusq(i,j)
            End Do
         End Do

         Do i=1,3
            Do j=1,3
               Write(*,'(A,I2,A,I2,A,G15.7,G15.7)') 
     &              'WqqL(',i,',',j,')=',WqqL(i,j)
               Write(*,'(A,I2,A,I2,A,G15.7,G15.7)') 
     &              'WqqRCor(',i,',',j,')=',WqqRCor(i,j)
            End Do
         End Do
      End If

      Return
      End
      
