c***********************************************************************

      Subroutine BDmain(debug,iresum,iEWcor,iOrd,isoft,genq,bderr)



c Subroutine that calls the various subroutines used to calculate the
c branching ratio.

      Implicit NONE

      Integer debug,iresum,iEWcor,iOrd,isoft,genq,bderr

c isoft
c 0 - Common blocks already filled, bare basis scale soft terms
c 1 - Common blocks already filled, physical basis soft terms

c iresum
c Whether resummation should be applied for the up, down and CKM sectors
c if set to 0 resummation is skipped, whilst if set to a value above 0
c it denotes the number of iterations used in resummation.

c iEWcor
c Whether the additional corrections due to charginos and neutralinos
c are included in the resummation procedure and the calculations 
c concerning the effective vertices.

c iOrd
c The order of the calculation with the following options
c 0 - LO only no effective theory corrections to vertices
c 1 - LO matching conditions with effective theory corrections to
c     vertices
c 2 - NLO matching conditions with no effective theory corrections to
c     vertices
c 3 - NLO + Effective theory corrections.
c 4 - Effective theory corrections for SUSY, NLO + Eff for EW

c init
c Used in the calls to the various decay subroutines, if set to 1
c relevant parameters are read from the corresponding .in files.
c Typically unless SM parameters are being drastically changed this only
c needs to be done once.

c genq
c B_d decays or B_s decays (irrelavent for B->s\gamma)

c Output
c bderr
c Whether an error has occured
c 0 - No error
c 1 - Error


c***********************************************************************
c Check parameters etc have been entered correctly


      If (debug.eq.1) Call BDinpchk(debug)




c***********************************************************************
c Resummation procedure and general setup
c***********************************************************************

c Calculate slepton mixing
      Call slepmass(debug)


c Calculate down and up sector mass resummation.
      Call barepara(debug,iresum,iEWcor,isoft,iOrd,bderr)
     


c Calculate the vertices used in the calculation
      Call vertices(debug,iOrd,iEWcor)


      
c***********************************************************************
c The various decays
c***********************************************************************

c B_q->mumu
c      Call bsmmain(debug,genq,iOrd)

c B_q-\bar{B}_q mixing
      Call bbmain(debug,genq,iOrd)

c Bs->X_s \gamma
c      Call bsgmain(debug,iOrd)

      Return
      End
