c***********************************************************************
c Subroutines that calculate the various effective vertices and
c contributions to bare masses used when calculating various resummation
c effects and bare quantities. Broadly speaking I follow the methods
c of Buras et. al. (section 2) hep-ph/0207241 generalized to GFM and
c Okumura and Roszkowski hep-ph/0308102 taking into account additional
c electroweak effects as well as those due to the strong interation.
c It is relatively easy to remove the contributions due to EW effects
c simply comment out the lines in the relevant subroutines.
c***********************************************************************

      Subroutine c_deltamd(debug,deltamd,iEWcor)

c Subroutine that determines the radiative corrections to md using
c formula 2.5 of hep-ph/0210145
c***********************************************************************

      Implicit NONE

      Include 'vars.h'
      Include 'vertex.h'

c Inputs
      Integer debug,iEWcor

c Outputs
      Double Complex deltamd(3,3)

c Local
      Double Complex tmp
      Integer i,j,k

c Now put them all together to yield deltamd
      Do i=1,3
         Do j=1,3
            tmp=DCMPLX(0.D0,0.D0)
            Do k=1,3
               tmp=tmp
     &              +1.D0/2.D0*SigmadVR(i,k)*mdbare(k,j)
     &              +1.D0/2.D0*mdbare(i,k)*SigmadVL(k,j)
            End Do
            deltamd(i,j)=SigmadmL(i,j)+tmp
         End Do
      End Do

      If (debug.eq.1) Then
         Write(*,*) '***************************************'
         Write(*,*) 'mdbare'
         Do i=1,3
            Do j=1,3
               Write(*,*) mdbare(i,j)
            End Do
         End Do
         Write(*,*) '***************************************'
         Write(*,*) 'SigmadVL'
         Do i=1,3
            Do j=1,3
               Write(*,*) SigmadVL(i,j)
            End Do
         End Do
         Write(*,*) '***************************************'
         Write(*,*) 'SigmadVR'
         Do i=1,3
            Do j=1,3
               Write(*,*) SigmadVR(i,j)
            End Do
         End Do
         Write(*,*) '**************************************'
         Write(*,*) 'SigmadmL'
         Do i=1,3
            Do j=1,3
               Write(*,*) SigmadmL(i,j)
            End Do
         End Do
         Write(*,*) '***************************************'
         Write(*,*) 'deltamd'
         Do i=1,3
            Do j=1,3
               Write(*,*) deltamd(i,j)
            End Do
         End Do
      End If

      Return
      End

c***********************************************************************

      Subroutine c_deltamu(debug,deltamu,iEWcor)

c Subroutine that determines the radiative corrections to mu using
c formula 2.5 of hep-ph/0210145
c***********************************************************************

      Implicit NONE

      Include 'vars.h'
      Include 'vertex.h'

c Inputs
      Integer debug,iEWcor

c Outputs
      Double Complex deltamu(3,3)

c Local
      Integer i,j,k
      Double Complex tmp

c Now put them all together to yield deltamu
      Do i=1,3
         Do j=1,3
            tmp=DCMPLX(0.D0,0.D0)
            Do k=1,3
               tmp=tmp+1.D0/2.D0*SigmauVR(i,k)*mubare(k,j)
     &              +1.D0/2.D0*mubare(i,k)*SigmauVL(k,j)
            End Do
            deltamu(i,j)=SigmaumL(i,j)+tmp
         End Do
      End Do

      If (debug.eq.1) Then
         Write(*,*) '***************************************'
         Write(*,*) 'mubare'
         Do i=1,3
            Do j=1,3
               Write(*,*) mubare(i,j)
            End Do
         End Do
         Write(*,*) '***************************************'
         Write(*,*) 'SigmauVL'
         Do i=1,3
            Do j=1,3
               Write(*,*) SigmauVL(i,j)
            End Do
         End Do
         Write(*,*) '***************************************'
         Write(*,*) 'SigmauVR'
         Do i=1,3
            Do j=1,3
               Write(*,*) SigmauVR(i,j)
            End Do
         End Do
         Write(*,*) '***************************************'
         Write(*,*) 'SigmaumL'
         Do i=1,3
            Do j=1,3
               Write(*,*) SigmaumL(i,j)
            End Do
         End Do
         Write(*,*) '***************************************'
         Write(*,*) 'deltamu'
         Do i=1,3
            Do j=1,3
               Write(*,*) deltamu(i,j)
            End Do
         End Do
      End If

      Return
      End

c***********************************************************************

      Subroutine bareCKM(debug,deltaKM,iEWcor)

c Subroutine that calculates the radiative corrections to the W-boson
c vertex determining the physical and bare CKM matrices used in
c calculations.
c***********************************************************************

      Implicit NONE

      Include 'const.h'
      Include 'insoft.h'
      Include 'insm.h'
      Include 'vars.h'
      Include 'vertex.h'

c Inputs
      Integer debug,iEWcor

c Outputs
      Double Complex deltaKM(3,3)

c Local
      Integer i,j,k,l,m
      Double Precision gw
      Double Complex deltaF_W(3,3)
      Double Complex tmpGl,tmpNe,tmpNC,tmpCN,tmpSE

c Functions
      Double Precision alpha2,PV_C00,PV_C0

      gw=DSQRT(alpha2(MGl)*4.D0*PI)

c Down quark index
      Do i=1,3                  
c Up quark index
         Do j=1,3
            tmpGl=DCMPLX(0.D0,0.D0)
            tmpNe=DCMPLX(0.D0,0.D0)
            tmpNC=DCMPLX(0.D0,0.D0)
            tmpCN=DCMPLX(0.D0,0.D0)
c Down squark index
            Do k=1,6
c Up squark index
               Do l=1,6
                  tmpGl=tmpGl+2.D0/3.D0*DCONJG(gqsqLup(j,l))*gqsqL(i,k)
     &                 *Wdsqusq(k,l)
     &                 *PV_C00(MGl,MDoS(k),MUpS(l),mususy)
c Neutralino index
                  Do m=1,4
                     tmpNe=tmpNe+DCONJG(nqsqLup(j,m,l))*nqsqL(i,m,k)
     &                    *Wdsqusq(k,l)*2.D0
     &                    *PV_C00(MN(m),MDoS(k),MUpS(l),mususy)
                  End Do
               End Do
c Neutralino index
               Do l=1,4
c Chargino index
                  Do m=1,2
                     tmpNC=tmpNC-MN(l)*MC(m)
     &                    *DCONJG(cqsqLup(j,m,k))*nqsqL(i,l,k)
     &                    *WChNeL(m,l)
     &                    *PV_C0(MDoS(k),MN(l),MC(m))
     &                    +2.D0*DCONJG(cqsqLup(j,m,k))*nqsqL(i,l,k)
     &                    *WChNeR(m,l)
     &                    *(PV_C00(MDoS(k),MN(l),MC(m),mususy)
     &                    -1.D0/4.D0)
                     tmpCN=tmpCN+MN(l)*MC(m)
     &                    *DCONJG(nqsqLup(j,l,k))*cqsqL(i,m,k)
     &                    *WChNeR(m,l)
     &                    *PV_C0(MDoS(k),MN(l),MC(m))
     &                    -2.D0*DCONJG(nqsqLup(j,l,k))*cqsqL(i,m,k)
     &                    *WChNeL(m,l)
     &                    *(PV_C00(MDoS(k),MN(l),MC(m),mususy)
     &                    -1.D0/4.D0)
                  End Do
               End Do
            End Do
            If (iEWcor.eq.1) Then
               deltaF_W(i,j)=1.D0/(16.D0*PI**2)*(tmpGl
     &              +tmpNe+tmpNC+tmpCN
     &              )
            Else
               deltaF_W(i,j)=1.D0/(16.D0*PI**2)*(tmpGl)
            End If
         End Do
      End Do

      Do i=1,3
         Do j=1,3
            tmpSE=DCMPLX(0.D0,0.D0)
            Do k=1,3
               tmpSE=tmpSE+1.D0/2.D0*SigmauVL(i,k)*KMbare(k,j)
     &              +1.D0/2.D0*KMbare(i,k)*SigmadVL(k,j)
            End Do
            deltaKM(i,j)=gw/(SQRT(2.D0))*tmpSE-deltaF_W(j,i)
         End Do
      End Do

      Return
      End

c***********************************************************************

      Subroutine c_Sigmas(debug,iEWcor)

c Subroutine that calculates the down quark self energies
c NOTE - This subroutine requires DoBareVer to be called before it
c is run.
c***********************************************************************

      Implicit NONE

      Include 'vars.h'

c Inputs
      Integer debug,iEWcor

      Call c_SigmadVL(debug,iEWcor)
      Call c_SigmadVR(debug,iEWcor)
      Call c_SigmauVL(debug,iEWcor)
      Call c_SigmauVR(debug,iEWcor)
      Call c_SigmadmL(debug,iEWcor)
      Call c_SigmaumL(debug,iEWcor)

      Return
      End

c***********************************************************************

      Subroutine c_SigmadVL(debug,iEWcor)

c Subroutine that calculates the down quark self energies
c NOTE - This subroutine requires DoBareVer to be called before it
c is run.
c***********************************************************************

      Implicit NONE

      Include 'const.h'
      Include 'insoft.h'
      Include 'insm.h'
      Include 'vars.h'
      Include 'vertex.h'

c Inputs
      Integer debug,iEWcor

c Local
      Integer i,j,k,l
      Double Complex tmpGl,tmpCh,tmpNe

c Functions
      Double Precision PV_B1

      Do i=1,3
         Do j=1,3
            tmpGl=DCMPLX(0.D0,0.D0)
            tmpCh=DCMPLX(0.D0,0.D0)
            tmpNe=DCMPLX(0.D0,0.D0)
            Do k=1,6
               tmpGl=tmpGl+4.D0/3.D0*DCONJG(gqsqL(i,k))*gqsqL(j,k)
     &              *PV_B1(MGl,MDoS(k),mususy)
            End Do
            Do k=1,6
               Do l=1,4
                  tmpNe=tmpNe+DCONJG(nqsqL(i,l,k))*nqsqL(j,l,k)
     &                 *PV_B1(MN(l),MDoS(k),mususy)
               End Do
            End Do
            Do k=1,6
               Do l=1,2
                  tmpCh=tmpCh+DCONJG(cqsqL(i,l,k))*cqsqL(j,l,k)
     &                 *PV_B1(MC(l),MUpS(k),mususy)
               End Do
            End Do
            If (iEWcor.eq.1) Then
               SigmadVL(i,j)=1.D0/(16.D0*PI**2)*(tmpGl+tmpNe+tmpCh)
            Else
               SigmadVL(i,j)=1.D0/(16.D0*PI**2)*(tmpGl)
            End If
         End Do
      End Do

      Return
      End

c***********************************************************************

      Subroutine c_SigmadVR(debug,iEWcor)

c Subroutine that calculates the down quark self energies
c NOTE - This subroutine requires DoBareVer to be called before it
c is run.
c***********************************************************************

      Implicit NONE

      Include 'const.h'
      Include 'insoft.h'
      Include 'insm.h'
      Include 'vars.h'
      Include 'vertex.h'

c Inputs
      Integer debug,iEWcor

c Local
      Integer i,j,k,l
      Double Complex tmpGl,tmpCh,tmpNe

c Functions
      Double Precision PV_B1

      Do i=1,3
         Do j=1,3
            tmpGl=DCMPLX(0.D0,0.D0)
            tmpCh=DCMPLX(0.D0,0.D0)
            tmpNe=DCMPLX(0.D0,0.D0)
            Do k=1,6
               tmpGl=tmpGl+4.D0/3.D0*DCONJG(gqsqR(i,k))*gqsqR(j,k)
     &              *PV_B1(MGl,MDoS(k),mususy)
            End Do
            Do k=1,6
               Do l=1,4
                  tmpNe=tmpNe+DCONJG(nqsqR(i,l,k))*nqsqR(j,l,k)
     &                 *PV_B1(MN(l),MDoS(k),mususy)
               End Do
            End Do
            Do k=1,6
               Do l=1,2
                  tmpCh=tmpCh+DCONJG(cqsqR(i,l,k))*cqsqR(j,l,k)
     &                 *PV_B1(MC(l),MUpS(k),mususy)
               End Do
            End Do
            If (iEWcor.eq.1) Then
               SigmadVR(i,j)=1.D0/(16.D0*PI**2)*(tmpGl+tmpNe+tmpCh)
            Else
               SigmadVR(i,j)=1.D0/(16.D0*PI**2)*(tmpGl)
            End If
         End Do
      End Do

      Return
      End

c***********************************************************************

      Subroutine c_SigmadmL(debug,iEWcor)

c Subroutine that calculates the down quark self energies
c NOTE - This subroutine requires doBareVer to be called before it
c is run.
c***********************************************************************

      Implicit NONE

      Include 'const.h'
      Include 'insoft.h'
      Include 'insm.h'
      Include 'vars.h'
      Include 'vertex.h'

c Inputs
      Integer debug,iEWcor

c Local
      Integer i,j,k,l
      Double Complex tmpGl,tmpCh,tmpNe

c Functions
      Double Precision PV_B0

      Do i=1,3
         Do j=1,3
            tmpGl=DCMPLX(0.D0,0.D0)
            tmpCh=DCMPLX(0.D0,0.D0)
            tmpNe=DCMPLX(0.D0,0.D0)
            Do k=1,6
               tmpGl=tmpGl-4.D0/3.D0*MGl*DCONJG(gqsqR(i,k))*gqsqL(j,k)
     &              *PV_B0(MGl,MDoS(k),mususy)
            End Do
            Do k=1,6
               Do l=1,4
                  tmpNe=tmpNe-MN(l)*DCONJG(nqsqR(i,l,k))*nqsqL(j,l,k)
     &                 *PV_B0(MN(l),MDoS(k),mususy)
               End Do
            End Do
            Do k=1,6
               Do l=1,2
                  tmpCh=tmpCh-MC(l)*DCONJG(cqsqR(i,l,k))*cqsqL(j,l,k)
     &                 *PV_B0(MC(l),MUpS(k),mususy)
               End Do
            End Do
            If (iEWcor.eq.1) Then
               SigmadmL(i,j)=1.D0/(16.D0*PI**2)*(tmpGl+tmpCh+tmpNe)
            Else
               SigmadmL(i,j)=1.D0/(16.D0*PI**2)*(tmpGl)
            End If
         End Do
      End Do

      Return
      End

c***********************************************************************

      Subroutine c_SigmauVL(debug,iEWcor)

c Subroutine that calculates the up quark self energies
c NOTE - This subroutine requires UpBareVer to be called before it
c is run.
c***********************************************************************

      Implicit NONE

      Include 'const.h'
      Include 'insoft.h'
      Include 'insm.h'
      Include 'vars.h'
      Include 'vertex.h'

c Inputs
      Integer debug,iEWcor

c Local
      Integer i,j,k,l
      Double Complex tmpGl,tmpCh,tmpNe

c Functions
      Double Precision PV_B1

      Do i=1,3
         Do j=1,3
            tmpGl=DCMPLX(0.D0,0.D0)
            tmpCh=DCMPLX(0.D0,0.D0)
            tmpNe=DCMPLX(0.D0,0.D0)
            Do k=1,6
               tmpGl=tmpGl+4.D0/3.D0*DCONJG(gqsqLup(i,k))*gqsqLup(j,k)
     &              *PV_B1(MGl,MUpS(k),mususy)
            End Do
            Do k=1,6
               Do l=1,4
                  tmpNe=tmpNe+DCONJG(nqsqLup(i,l,k))*nqsqLup(j,l,k)
     &                 *PV_B1(MN(l),MUpS(k),mususy)
               End Do
            End Do
            Do k=1,6
               Do l=1,2
                  tmpCh=tmpCh+DCONJG(cqsqLup(i,l,k))*cqsqLup(j,l,k)
     &                 *PV_B1(MC(l),MDoS(k),mususy)
               End Do
            End Do
            If (iEWcor.eq.1) Then
               SigmauVL(i,j)=1.D0/(16.D0*PI**2)*(tmpGl+tmpNe+tmpCh)
            Else
               SigmauVL(i,j)=1.D0/(16.D0*PI**2)*(tmpGl)
            End If
         End Do
      End Do

      Return
      End

c***********************************************************************

      Subroutine c_SigmauVR(debug,iEWcor)

c Subroutine that calculates the up quark self energies
c NOTE - This subroutine requires UpBareVer to be called before it
c is run.
c***********************************************************************

      Implicit NONE

      Include 'const.h'
      Include 'insoft.h'
      Include 'insm.h'
      Include 'vars.h'
      Include 'vertex.h'

c Inputs
      Integer debug,iEWcor

c Local
      Integer i,j,k,l
      Double Complex tmpGl,tmpCh,tmpNe

c Functions
      Double Precision PV_B1

      Do i=1,3
         Do j=1,3
            tmpGl=DCMPLX(0.D0,0.D0)
            tmpCh=DCMPLX(0.D0,0.D0)
            tmpNe=DCMPLX(0.D0,0.D0)
            Do k=1,6
               tmpGl=tmpGl+4.D0/3.D0*DCONJG(gqsqRup(i,k))*gqsqRup(j,k)
     &              *PV_B1(MGl,MUpS(k),mususy)
            End Do
            Do k=1,6
               Do l=1,4
                  tmpNe=tmpNe+DCONJG(nqsqRup(i,l,k))*nqsqRup(j,l,k)
     &                 *PV_B1(MN(l),MUpS(k),mususy)
               End Do
            End Do
            Do k=1,6
               Do l=1,2
                  tmpCh=tmpCh+DCONJG(cqsqRup(i,l,k))*cqsqRup(j,l,k)
     &                 *PV_B1(MC(l),MDoS(k),mususy)
               End Do
            End Do
            If (iEWcor.eq.1) Then
               SigmauVR(i,j)=1.D0/(16.D0*PI**2)*(tmpGl+tmpNe+tmpCh)
            Else
               SigmauVR(i,j)=1.D0/(16.D0*PI**2)*(tmpGl)
            End If
         End Do
      End Do

      Return
      End

c***********************************************************************

      Subroutine c_SigmaumL(debug,iEWcor)

c Subroutine that calculates the up quark self energies
c NOTE - This subroutine requires UpBareVer to be called before it
c is run.
c***********************************************************************

      Implicit NONE

      Include 'const.h'
      Include 'insoft.h'
      Include 'insm.h'
      Include 'vars.h'
      Include 'vertex.h'

c Inputs
      Integer debug,iEWcor

c Local
      Integer i,j,k,l
      Double Complex tmpGl,tmpCh,tmpNe

c Functions
      Double Precision PV_B0

      Do i=1,3
         Do j=1,3
            tmpGl=DCMPLX(0.D0,0.D0)
            tmpCh=DCMPLX(0.D0,0.D0)
            tmpNe=DCMPLX(0.D0,0.D0)
            Do k=1,6
               tmpGl=tmpGl-4.D0/3.D0*MGl
     &              *DCONJG(gqsqRup(i,k))*gqsqLup(j,k)
     &              *PV_B0(MGl,MUpS(k),mususy)
            End Do
            Do k=1,6
               Do l=1,4
                  tmpNe=tmpNe-MN(l)
     &                 *DCONJG(nqsqRup(i,l,k))*nqsqLup(j,l,k)
     &                 *PV_B0(MN(l),MUpS(k),mususy)
               End Do
            End Do
            Do k=1,6
               Do l=1,2
                  tmpCh=tmpCh-MC(l)
     &                 *DCONJG(cqsqRup(i,l,k))*cqsqLup(j,l,k)
     &                 *PV_B0(MC(l),MDoS(k),mususy)
               End Do
            End Do
            If (iEWcor.eq.1) Then
               SigmaumL(i,j)=1.D0/(16.D0*PI**2)*(tmpGl+tmpNe+tmpCh)
            Else
               SigmaumL(i,j)=1.D0/(16.D0*PI**2)*(tmpGl)
            End If
         End Do
      End Do

      Return
      End

c***********************************************************************

      Subroutine NHiggsVertex(debug,iEWcor)

c Subroutine that determines the various Supersymmetric corrections to
c the Neutral Higgs vertices
c***********************************************************************

      Implicit NONE

      Include 'const.h'
      Include 'insoft.h'
      Include 'insm.h'
      Include 'vars.h'
      Include 'vertex.h'

c Input
      Integer debug,iEWcor

c Local
      Integer i,j,k,l,m,n
      Double Precision gw,beta
      Double Precision ZS(2,2)
      Double Complex ZP(2,2)
      Double Complex tmpGl,tmpNe1,tmpCh1,tmpNe2,tmpCh2
      Double Complex deltaF_HL(2,3,3),deltaF_HR(2,3,3)
      Double Complex deltaF_PL(2,3,3),deltaF_PR(2,3,3)

c Functions
      Double Precision PV_C0, PV_C00, MSrmass, alpha2

      gw=DSQRT(alpha2(mususy)*4.D0*PI)
 
      beta=ATAN(tanb)

      ZS(1,1)=COS(alpha_eff)
      ZS(1,2)=-SIN(alpha_eff)
      ZS(2,1)=SIN(alpha_eff)
      ZS(2,2)=COS(alpha_eff)

      ZP(1,1)=SIN(beta)
      ZP(1,2)=-COS(beta)
      ZP(2,1)=COS(beta)
      ZP(2,2)=SIN(beta)

      Do i=1,2
         Do j=1,2
            ZP(i,j)=ZP(i,j)*DCMPLX(0.D0,1.D0)
         End Do
      End Do

      Call c_SigmadmL(debug,iEWcor)

      Do i=1,2
         Do j=1,3
            Do k=1,3
               tmpNe1=DCMPLX(0.D0,0.D0)
               tmpCh1=DCMPLX(0.D0,0.D0)
               tmpGl=DCMPLX(0.D0,0.D0)
               tmpNe2=DCMPLX(0.D0,0.D0)
               tmpCh2=DCMPLX(0.D0,0.D0)
               Do l=1,6
                  Do m=1,4
                     Do n=1,4
                        tmpNe1=tmpNe1-DCONJG(nqsqR(k,n,l))*nqsqL(j,m,l)
     &                       *SHneneL(i,m,n)*MN(m)*MN(n)
     &                       *PV_C0(MDoS(l),MN(m),MN(n))
     &                       +DCONJG(nqsqR(k,n,l))*nqsqL(j,m,l)
     &                       *SHneneR(i,m,n)
     &                       *4.D0*(PV_C00(MDoS(l),MN(m),MN(n),mususy)
     &                       -1.D0/8.D0)
                     End Do
                  End Do
                  Do m=1,2
                     Do n=1,2
                        tmpCh1=tmpCh1-DCONJG(cqsqR(k,n,l))*cqsqL(j,m,l)
     &                       *SHchchL(i,m,n)*MC(m)*MC(n)
     &                       *PV_C0(MUpS(l),MC(m),MC(n))
     &                       +DCONJG(cqsqR(k,n,l))*cqsqL(j,m,l)
     &                       *SHchchR(i,m,n)
     &                       *4.D0*(PV_C00(MUpS(l),MC(m),MC(n),mususy)
     &                       -1.D0/8.D0)
                     End Do
                  End Do
                  Do m=1,6
                     tmpGl=tmpGl-4.D0/3.D0*DCONJG(gqsqR(k,m))*gqsqL(j,l)
     &                    *SHdsqsq(i,l,m)
     &                    *MGl*PV_C0(MGl,MDoS(l),MDoS(m))
                     Do n=1,4
                        tmpNe2=tmpNe2-DCONJG(nqsqR(k,n,m))*nqsqL(j,n,l)
     &                       *SHdsqsq(i,l,m)*MN(n)
     &                       *PV_C0(MN(n),MDoS(l),MDoS(m))
                     End Do
                     Do n=1,2
                        tmpCh2=tmpCh2-DCONJG(cqsqR(k,n,m))*cqsqL(j,n,l)
     &                       *SHusqsq(i,l,m)*MC(n)
     &                       *PV_C0(MC(n),MUpS(l),MUpS(m))
                     End Do
                  End Do
               End Do
               If (iEWcor.eq.1) Then
                  deltaF_HL(i,j,k)=1.D0/(16.D0*PI**2)
     &                 *(tmpGl+tmpNe1+tmpCh1+tmpNe2+tmpCh2)
               Else
                  deltaF_HL(i,j,k)=1.D0/(16.D0*PI**2)
     &                 *(tmpGl)
               End If
            End Do
         End Do
      End Do

      Do i=1,2
         Do j=1,3
            Do k=1,3
               tmpNe1=DCMPLX(0.D0,0.D0)
               tmpCh1=DCMPLX(0.D0,0.D0)
               tmpGl=DCMPLX(0.D0,0.D0)
               tmpNe2=DCMPLX(0.D0,0.D0)
               tmpCh2=DCMPLX(0.D0,0.D0)
               Do l=1,6
                  Do m=1,4
                     Do n=1,4
                        tmpNe1=tmpNe1-DCONJG(nqsqL(k,n,l))*nqsqR(j,m,l)
     &                       *SHneneR(i,m,n)*MN(m)*MN(n)
     &                       *PV_C0(MDoS(l),MN(m),MN(n))
     &                       +DCONJG(nqsqL(k,n,l))*nqsqR(j,m,l)
     &                       *SHneneL(i,m,n)
     &                       *4.D0*(PV_C00(MDoS(l),MN(m),MN(n),mususy)
     &                       -1.D0/8.D0)
                     End Do
                  End Do
                  Do m=1,2
                     Do n=1,2
                        tmpCh1=tmpCh1-DCONJG(cqsqL(k,n,l))*cqsqR(j,m,l)
     &                       *SHchchR(i,m,n)*MC(m)*MC(n)
     &                       *PV_C0(MUpS(l),MC(m),MC(n))
     &                       +DCONJG(cqsqL(k,n,l))*cqsqR(j,m,l)
     &                       *SHchchL(i,m,n)
     &                       *4.D0*(PV_C00(MUpS(l),MC(m),MC(n),mususy)
     &                       -1.D0/8.D0)
                     End Do
                  End Do
                  Do m=1,6
                     tmpGl=tmpGl-4.D0/3.D0*DCONJG(gqsqL(k,m))*gqsqR(j,l)
     &                    *SHdsqsq(i,l,m)
     &                    *MGl*PV_C0(MGl,MDoS(l),MDoS(m))
                     Do n=1,4
                        tmpNe2=tmpNe2-DCONJG(nqsqL(k,n,m))*nqsqR(j,n,l)
     &                       *SHdsqsq(i,l,m)*MN(n)
     &                       *PV_C0(MN(n),MDoS(l),MDoS(m))
                     End Do
                     Do n=1,2
                        tmpCh2=tmpCh2-DCONJG(cqsqL(k,n,m))*cqsqR(j,n,l)
     &                       *SHusqsq(i,l,m)*MC(n)
     &                       *PV_C0(MC(n),MUpS(l),MUpS(m))
                     End Do
                  End Do
               End Do
               If (iEWcor.eq.1) Then
                  deltaF_HR(i,j,k)=1.D0/(16.D0*PI**2)
     &                 *(tmpGl+tmpNe1+tmpCh1+tmpNe2+tmpCh2)
               Else
                  deltaF_HR(i,j,k)=1.D0/(16.D0*PI**2)
     &                 *(tmpGl)
               End If
            End Do
         End Do
      End Do

      Do i=1,2
         Do j=1,3
            Do k=1,3
               SHqqLCor(i,j,k)=-gw/(2.D0*mw*COS(beta))*ZS(1,i)
     &              *(MSrmass(mususy,2*j-1,5,4)*Unit3(k,j)
     &              -SigmadmL(k,j))+deltaF_HL(i,j,k)
               SHqqRCor(i,j,k)=-gw/(2.D0*mw*COS(beta))*ZS(1,i)
     &              *(MSrmass(mususy,2*j-1,5,4)*Unit3(k,j)
     &              -DCONJG(SigmadmL(j,k)))+deltaF_HR(i,j,k)
            End Do
         End Do
      End Do

      Do i=1,2
         Do j=1,3
            Do k=1,3
               tmpNe1=DCMPLX(0.D0,0.D0)
               tmpCh1=DCMPLX(0.D0,0.D0)
               tmpGl=DCMPLX(0.D0,0.D0)
               tmpNe2=DCMPLX(0.D0,0.D0)
               tmpCh2=DCMPLX(0.D0,0.D0)
               Do l=1,6
                  Do m=1,4
                     Do n=1,4
                        tmpNe1=tmpNe1-DCONJG(nqsqR(k,n,l))*nqsqL(j,m,l)
     &                       *PHneneL(i,m,n)*MN(m)*MN(n)
     &                       *PV_C0(MDoS(l),MN(m),MN(n))
     &                       +DCONJG(nqsqR(k,n,l))*nqsqL(j,m,l)
     &                       *PHneneR(i,m,n)
     &                       *4.D0*(PV_C00(MDoS(l),MN(m),MN(n),mususy)
     &                       -1.D0/8.D0)
                     End Do
                  End Do
                  Do m=1,2
                     Do n=1,2
                        tmpCh1=tmpCh1-DCONJG(cqsqR(k,n,l))*cqsqL(j,m,l)
     &                       *PHchchL(i,m,n)*MC(m)*MC(n)
     &                       *PV_C0(MUpS(l),MC(m),MC(n))
     &                       +DCONJG(cqsqR(k,n,l))*cqsqL(j,m,l)
     &                       *PHchchR(i,m,n)
     &                       *4.D0*(PV_C00(MUpS(l),MC(m),MC(n),mususy)
     &                       -1.D0/8.D0)
                     End Do
                  End Do
                  Do m=1,6
                     tmpGl=tmpGl-4.D0/3.D0*DCONJG(gqsqR(k,m))*gqsqL(j,l)
     &                    *PHdsqsq(i,l,m)
     &                    *MGl*PV_C0(MGl,MDoS(l),MDoS(m))
                     Do n=1,4
                        tmpNe2=tmpNe2-DCONJG(nqsqR(k,n,m))*nqsqL(j,n,l)
     &                       *PHdsqsq(i,l,m)*MN(n)
     &                       *PV_C0(MN(n),MDoS(l),MDoS(m))
                     End Do
                     Do n=1,2
                        tmpCh2=tmpCh2-DCONJG(cqsqR(k,n,m))*cqsqL(j,n,l)
     &                       *PHusqsq(i,l,m)*MC(n)
     &                       *PV_C0(MC(n),MUpS(l),MUpS(m))
                     End Do
                  End Do
               End Do
               If (iEWcor.eq.1) Then
                  deltaF_PL(i,j,k)=1.D0/(16.D0*PI**2)
     &                 *(tmpGl+tmpNe1+tmpCh1+tmpNe2+tmpCh2)
               Else
                  deltaF_PL(i,j,k)=1.D0/(16.D0*PI**2)
     &                 *(tmpGl)
               End If
            End Do
         End Do
      End Do

      Do i=1,2
         Do j=1,3
            Do k=1,3
               tmpNe1=DCMPLX(0.D0,0.D0)
               tmpCh1=DCMPLX(0.D0,0.D0)
               tmpGl=DCMPLX(0.D0,0.D0)
               tmpNe2=DCMPLX(0.D0,0.D0)
               tmpCh2=DCMPLX(0.D0,0.D0)
               Do l=1,6
                  Do m=1,4
                     Do n=1,4
                        tmpNe1=tmpNe1-DCONJG(nqsqL(k,n,l))*nqsqR(j,m,l)
     &                       *PHneneR(i,m,n)*MN(m)*MN(n)
     &                       *PV_C0(MDoS(l),MN(m),MN(n))
     &                       +DCONJG(nqsqL(k,n,l))*nqsqR(j,m,l)
     &                       *PHneneL(i,m,n)
     &                       *4.D0*(PV_C00(MDoS(l),MN(m),MN(n),mususy)
     &                       -1.D0/8.D0)
                     End Do
                  End Do
                  Do m=1,2
                     Do n=1,2
                        tmpCh1=tmpCh1-DCONJG(cqsqL(k,n,l))*cqsqR(j,m,l)
     &                       *PHchchR(i,m,n)*MC(m)*MC(n)
     &                       *PV_C0(MUpS(l),MC(m),MC(n))
     &                       +DCONJG(cqsqL(k,n,l))*cqsqR(j,m,l)
     &                       *PHchchL(i,m,n)
     &                       *4.D0*(PV_C00(MUpS(l),MC(m),MC(n),mususy)
     &                       -1.D0/8.D0)
                     End Do
                  End Do
                  Do m=1,6
                     tmpGl=tmpGl-4.D0/3.D0*DCONJG(gqsqL(k,m))*gqsqR(j,l)
     &                    *PHdsqsq(i,l,m)
     &                    *MGl*PV_C0(MGl,MDoS(l),MDoS(m))
                     Do n=1,4
                        tmpNe2=tmpNe2-DCONJG(nqsqL(k,n,m))*nqsqR(j,n,l)
     &                       *PHdsqsq(i,l,m)*MN(n)
     &                       *PV_C0(MN(n),MDoS(l),MDoS(m))
                     End Do
                     Do n=1,2
                        tmpCh2=tmpCh2-DCONJG(cqsqL(k,n,m))*cqsqR(j,n,l)
     &                       *PHusqsq(i,l,m)*MC(n)
     &                       *PV_C0(MC(n),MUpS(l),MUpS(m))
                     End Do
                  End Do
               End Do
               If (iEWcor.eq.1) Then
                  deltaF_PR(i,j,k)=1.D0/(16.D0*PI**2)
     &                 *(tmpGl+tmpNe1+tmpCh1+tmpNe2+tmpCh2)
               Else
                  deltaF_PR(i,j,k)=1.D0/(16.D0*PI**2)
     &                 *(tmpGl)
               End If
            End Do
         End Do
      End Do

      Do i=1,2
         Do j=1,3
            Do k=1,3
               PHqqLCor(i,j,k)=-gw/(2.D0*mw*COS(beta))*ZP(1,i)
     &              *(MSrmass(mususy,2*j-1,5,4)*Unit3(k,j)
     &              -SigmadmL(k,j))+deltaF_PL(i,j,k)
               PHqqRCor(i,j,k)=-gw/(2.D0*mw*COS(beta))*DCONJG(ZP(1,i))
     &              *(MSrmass(mususy,2*j-1,5,4)*Unit3(k,j)
     &              -DCONJG(SigmadmL(j,k)))+deltaF_PR(i,j,k)
            End Do
         End Do
      End Do

      Return
      End

c***********************************************************************

      Subroutine CHiggsVertex(debug,iEWcor)

c Subroutine that determines the various Supersymmetric corrections to
c the Charged Higgs vertices
c***********************************************************************

      Implicit NONE

      Include 'const.h'
      Include 'insoft.h'
      Include 'insm.h'
      Include 'vars.h'
      Include 'vertex.h'

c Input
      Integer debug,iEWcor

c Local
      Integer i,j,k,l,m,n
      Double Precision gw,beta,ZP(2,2)
      Double Complex tmpGl,tmpNe,tmpCN,tmpNC,tmp1,tmp2,tmp3,tmp4
      Double Complex deltaF_CL(2,3,3),deltaF_CR(2,3,3)

c Functions
      Double Precision PV_C0, PV_C00, MSrmass, alpha2

      gw=DSQRT(alpha2(muew)*4.D0*PI)

      beta=ATAN(tanb)

      ZP(1,1)=SIN(beta)
      ZP(1,2)=-COS(beta)
      ZP(2,1)=COS(beta)
      ZP(2,2)=SIN(beta)

      Call c_SigmadmL(debug,iEWcor)
      Call c_SigmaumL(debug,iEWcor)
      Call c_SigmauVL(debug,iEWcor)
      Call c_SigmadVL(debug,iEWcor)
      
      Do i=1,2
         Do j=1,3
            Do k=1,3
               tmpCN=DCMPLX(0.D0,0.D0)
               tmpNC=DCMPLX(0.D0,0.D0)
               tmpGl=DCMPLX(0.D0,0.D0)
               tmpNe=DCMPLX(0.D0,0.D0)
               Do l=1,6
                  Do m=1,2
                     Do n=1,4
                        tmpNC=tmpNC-DCONJG(cqsqRup(k,m,l))*nqsqL(j,n,l)
     &                       *CHchneL(i,m,n)*MC(m)*MN(n)
     &                       *PV_C0(MDoS(l),MC(m),MN(n))
     &                       -DCONJG(cqsqRup(k,m,l))*nqsqL(j,n,l)
     &                       *CHchneR(i,m,n)
     &                       *4.D0*(PV_C00(MDoS(l),MC(m),MN(n),mususy)
     &                       -1.D0/8.D0)
                        tmpCN=tmpCN-DCONJG(nqsqRup(k,n,l))*cqsqL(j,m,l)
     &                       *CHchneL(i,m,n)*MC(m)*MN(n)
     &                       *PV_C0(MUpS(l),MC(m),MN(n))
     &                       -DCONJG(nqsqRup(k,n,l))*cqsqL(j,m,l)
     &                       *CHchneR(i,m,n)
     &                       *4.D0*(PV_C00(MUpS(l),MC(m),MN(n),mususy)
     &                       -1.D0/8.D0)
                     End Do
                  End Do
                  Do m=1,6
                     tmpGl=tmpGl
     &                    -4.D0/3.D0*DCONJG(gqsqRup(k,m))*gqsqL(j,l)
     &                    *CHdsqusq(i,l,m)
     &                    *MGl*PV_C0(MGl,MDoS(l),MUpS(m))
                     Do n=1,4
                        tmpNe=tmpNe
     &                       -DCONJG(nqsqRup(k,n,m))*nqsqL(j,n,l)
     &                       *CHdsqusq(i,l,m)*MN(n)
     &                       *PV_C0(MN(n),MDoS(l),MUpS(m))
                     End Do
                  End Do
               End Do
               If (iEWcor.eq.1) Then
                  deltaF_CL(i,j,k)=1.D0/(16.D0*PI**2)
     &                 *(tmpGl+tmpCN+tmpNC+tmpNe)
               Else
                  deltaF_CL(i,j,k)=1.D0/(16.D0*PI**2)
     &                 *(tmpGl)
               End If
            End Do
         End Do
      End Do

      Do i=1,2
         Do j=1,3
            Do k=1,3
               tmpCN=DCMPLX(0.D0,0.D0)
               tmpNC=DCMPLX(0.D0,0.D0)
               tmpGl=DCMPLX(0.D0,0.D0)
               tmpNe=DCMPLX(0.D0,0.D0)
               Do l=1,6
                  Do m=1,2
                     Do n=1,4
                        tmpNC=tmpNC-DCONJG(cqsqLup(k,m,l))*nqsqR(j,n,l)
     &                       *CHchneR(i,m,n)*MC(m)*MN(n)
     &                       *PV_C0(MDoS(l),MC(m),MN(n))
     &                       -DCONJG(cqsqLup(k,m,l))*nqsqR(j,n,l)
     &                       *CHchneL(i,m,n)
     &                       *4.D0*(PV_C00(MDoS(l),MC(m),MN(n),mususy)
     &                       -1.D0/8.D0)
                        tmpCN=tmpCN+DCONJG(nqsqLup(k,n,l))*cqsqR(j,m,l)
     &                       *CHchneR(i,m,n)*MC(m)*MN(n)
     &                       *PV_C0(MUpS(l),MC(m),MN(n))
     &                       -DCONJG(nqsqLup(k,n,l))*cqsqR(j,m,l)
     &                       *CHchneL(i,m,n)
     &                       *4.D0*(PV_C00(MUpS(l),MC(m),MN(n),mususy)
     &                       -1.D0/8.D0)
                     End Do
                  End Do
                  Do m=1,6
                     tmpGl=tmpGl
     &                    -4.D0/3.D0*DCONJG(gqsqLup(k,m))*gqsqR(j,l)
     &                    *CHdsqusq(i,l,m)
     &                    *MGl*PV_C0(MGl,MDoS(l),MUpS(m))
                     Do n=1,4
                        tmpNe=tmpNe
     &                       -DCONJG(nqsqLup(k,n,m))*nqsqR(j,n,l)
     &                       *CHdsqusq(i,l,m)*MN(n)
     &                       *PV_C0(MN(n),MDoS(l),MUpS(m))
                     End Do
                  End Do
               End Do
               If (iEWcor.eq.1) Then
                  deltaF_CR(i,j,k)=1.D0/(16.D0*PI**2)
     &                 *(tmpGl+tmpCN+tmpNC+tmpNe)
               Else
                  deltaF_CR(i,j,k)=1.D0/(16.D0*PI**2)
     &                 *(tmpGl)
               End If
            End Do
         End Do
      End Do

      Do i=1,2
         Do j=1,3
            Do k=1,3
               tmp1=DCMPLX(0.D0,0.D0)
               tmp2=DCMPLX(0.D0,0.D0)
               tmp3=DCMPLX(0.D0,0.D0)
               tmp4=DCMPLX(0.D0,0.D0)
               Do l=1,3
                  Do m=1,3
                     tmp1=tmp1-1.D0/2.D0
     &                    *mubare(k,l)*SigmauVL(l,m)*KMbare(m,j)
     &                    +1.D0/2.D0
     &                    *mubare(k,l)*KMbare(l,m)*SigmadVL(m,j)
                     tmp2=tmp2-1.D0/2.D0
     &                    *KMbare(k,l)*SigmadVL(l,m)*DCONJG(mdbare(j,m))
     &                    +1.D0/2.D0
     &                    *SigmauVL(k,l)*KMbare(l,m)*DCONJG(mdbare(j,m))
                  End Do
                  tmp1=tmp1-SigmaumL(k,l)*KMbare(l,j)
                  tmp3=tmp3+MSrmass(muew,2*k,5,4)*Unit3(k,l)*KMbare(l,j)
                  tmp2=tmp2-KMbare(k,l)*DCONJG(SigmadmL(j,l))
                  tmp4=tmp4+KMbare(k,l)*MSrmass(muew,2*j-1,5,4)
     &                 *Unit3(l,j)
               End Do

               CHqqLCor(i,j,k)=gw/(SQRT(2.D0)*mw*SIN(beta))*ZP(2,i)
     &              *(tmp1+tmp3)
     &              +deltaF_CL(i,j,k)
               CHqqRCor(i,j,k)=gw/(SQRT(2.D0)*mw*COS(beta))*ZP(1,i)
     &              *(tmp2+tmp4)
     &              +deltaF_CR(i,j,k)
               delCHqqL(i,j,k)=gw/(SQRT(2.D0)*mw*SIN(beta))*ZP(2,i)*tmp1
     &              +deltaF_CL(i,j,k)
               delCHqqR(i,j,k)=gw/(SQRT(2.D0)*mw*COS(beta))*ZP(1,i)*tmp2
     &              +deltaF_CR(i,j,k)
            End Do
         End Do
      End Do

      Return
      End

c***********************************************************************

      Subroutine ZVertex(debug,iEWcor)

c Subroutine that determines the various Supersymmetric corrections to
c the Z boson vertex
c***********************************************************************

      Implicit NONE

      Include 'const.h'
      Include 'insoft.h'
      Include 'insm.h'
      Include 'vars.h'
      Include 'vertex.h'

c Input
      Integer debug,iEWcor

c Local
      Integer i,j,k,l,m
      Double Precision gw,thw
      Double Complex tmpGl,tmpNe1,tmpCh1,tmpNe2,tmpCh2
      Double Complex deltaF_ZL(3,3),deltaF_ZR(3,3)

c Functions
      Double Precision PV_C0, PV_C00, alpha2

      gw=DSQRT(alpha2(muew)*4.D0*PI)
      thw=ASIN(SQRT(sw2))

      Call c_SigmadVR(debug,iEWcor)
      Call c_SigmadVL(debug,iEWcor)

      Do i=1,3
         Do j=1,3
            tmpGl=DCMPLX(0.D0,0.D0)
            tmpNe1=DCMPLX(0.D0,0.D0)
            tmpNe2=DCMPLX(0.D0,0.D0)
            tmpCh1=DCMPLX(0.D0,0.D0)
            tmpCh2=DCMPLX(0.D0,0.D0)
            Do k=1,6
               Do l=1,6
                  tmpGl=tmpGl+8.D0/3.D0*DCONJG(gqsqL(j,l))*gqsqL(i,k)
     &                 *Zdsqsq(k,l)
     &                 *PV_C00(MGl,MDoS(k),MDoS(l),mususy)
                  Do m=1,4
                     tmpNe1=tmpNe1+DCONJG(nqsqL(j,m,l))*nqsqL(i,m,k)
     &                    *Zdsqsq(k,l)
     &                    *2.D0*PV_C00(MN(m),MDoS(k),MDoS(l),mususy)
                  End Do
                  Do m=1,2
                     tmpCh1=tmpCh1+DCONJG(cqsqL(j,m,l))*cqsqL(i,m,k)
     &                    *Zusqsq(k,l)
     &                    *2.D0*PV_C00(MC(m),MUpS(k),MUpS(l),mususy)
                  End do
               End Do
               Do l=1,4
                  Do m=1,4
                     tmpNe2=tmpNe2-DCONJG(nqsqL(j,m,k))*nqsqL(i,l,k)
     &                    *ZneneL(l,m)*MN(l)*MN(m)
     &                    *PV_C0(MDoS(k),MN(l),MN(m))
     &                    +2.D0*DCONJG(nqsqL(j,m,k))*nqsqL(i,l,k)
     &                    *ZneneR(l,m)
     &                    *(PV_C00(MDoS(k),MN(l),MN(m),mususy)
     &                    -1.D0/4.D0)
                  End Do
               End Do
               Do l=1,2
                  Do m=1,2
                     tmpCh2=tmpCh2-DCONJG(cqsqL(j,m,k))*cqsqL(i,l,k)
     &                    *ZchchL(l,m)*MC(l)*MC(m)
     &                    *PV_C0(MUpS(k),MC(l),MC(m))
     &                    +2.D0*DCONJG(cqsqL(j,m,k))*cqsqL(i,l,k)
     &                    *ZchchR(l,m)
     &                    *(PV_C00(MUpS(k),MC(l),MC(m),mususy)
     &                    -1.D0/4.D0)
                  End Do
               End Do
            End Do
            If (iEWcor.eq.1) Then
               deltaF_ZL(i,j)=1.D0/(16.D0*PI**2)*(tmpGl+tmpNe1+tmpCh1
     &              +tmpNe2+tmpCh2)
            Else
               deltaF_ZL(i,j)=1.D0/(16.D0*PI**2)*(tmpGl)
            End If
         End Do
      End Do

      Do i=1,3
         Do j=1,3
            tmpGl=DCMPLX(0.D0,0.D0)
            tmpNe1=DCMPLX(0.D0,0.D0)
            tmpNe2=DCMPLX(0.D0,0.D0)
            tmpCh1=DCMPLX(0.D0,0.D0)
            tmpCh2=DCMPLX(0.D0,0.D0)
            Do k=1,6
               Do l=1,6
                  tmpGl=tmpGl+8.D0/3.D0*DCONJG(gqsqR(j,l))*gqsqR(i,k)
     &                 *Zdsqsq(k,l)
     &                 *PV_C00(MGl,MDoS(k),MDoS(l),mususy)
                  Do m=1,4
                     tmpNe1=tmpNe1+DCONJG(nqsqR(j,m,l))*nqsqR(i,m,k)
     &                    *Zdsqsq(k,l)
     &                    *2.D0*PV_C00(MN(m),MDoS(k),MDoS(l),mususy)
                  End Do
                  Do m=1,2
                     tmpCh1=tmpCh1+DCONJG(cqsqR(j,m,l))*cqsqR(i,m,k)
     &                    *Zusqsq(k,l)
     &                    *2.D0*PV_C00(MC(m),MUpS(k),MUpS(l),mususy)
                  End do
               End Do
               Do l=1,4
                  Do m=1,4
                     tmpNe2=tmpNe2-DCONJG(nqsqR(j,m,k))*nqsqR(i,l,k)
     &                    *ZneneR(l,m)*MN(l)*MN(m)
     &                    *PV_C0(MDoS(k),MN(l),MN(m))
     &                    +2.D0*DCONJG(nqsqR(j,m,k))*nqsqR(i,l,k)
     &                    *ZneneL(l,m)
     &                    *(PV_C00(MDoS(k),MN(l),MN(m),mususy)
     &                    -1.D0/4.D0)
                  End Do
               End Do
               Do l=1,2
                  Do m=1,2
                     tmpCh2=tmpCh2-DCONJG(cqsqR(j,m,k))*cqsqR(i,l,k)
     &                    *ZchchR(l,m)*MC(l)*MC(m)
     &                    *PV_C0(MUpS(k),MC(l),MC(m))
     &                    +2.D0*DCONJG(cqsqR(j,m,k))*cqsqR(i,l,k)
     &                    *ZchchL(l,m)
     &                    *(PV_C00(MUpS(k),MC(l),MC(m),mususy)
     &                    -1.D0/4.D0)
                  End Do
               End Do
            End Do
            If (iEWcor.eq.1) Then
               deltaF_ZR(i,j)=1.D0/(16.D0*PI**2)*(tmpGl+tmpNe1+tmpCh1
     &              +tmpNe2+tmpCh2)
            Else
               deltaF_ZR(i,j)=1.D0/(16.D0*PI**2)*(tmpGl)
            End If
         End Do
      End Do

      Do i=1,3
         Do j=1,3
            ZdqqLCor(i,j)=gw/(2.D0*COS(thw))*(1.D0-2.D0/3.D0*sw2)
     &           *(Unit3(i,j)+SigmadVL(j,i))+deltaF_ZL(i,j)
            ZdqqRCor(i,j)=-gw/(2.D0*COS(thw))*(2.D0/3.D0*sw2)
     &           *(Unit3(i,j)+SigmadVR(j,i))+deltaF_ZR(i,j)
         End Do
      End Do

      Return
      End

c***********************************************************************

      Subroutine WVertex(debug,iEWcor)

c Subroutine that calculates the radiative corrections to the V+A
c W-boson vertex 
c***********************************************************************

      Implicit NONE

      Include 'const.h'
      Include 'insoft.h'
      Include 'insm.h'
      Include 'vars.h'
      Include 'vertex.h'

c Inputs
      Integer debug,iEWcor

c Local
      Integer i,j,k,l,m
      Double Complex tmpGl,tmpNe,tmpNC,tmpCN

c Functions
      Double Precision PV_C00,PV_C0

c Down quark index
      Do i=1,3
c Up quark index
         Do j=1,3
            tmpGl=DCMPLX(0.D0,0.D0)
            tmpNe=DCMPLX(0.D0,0.D0)
            tmpNC=DCMPLX(0.D0,0.D0)
            tmpCN=DCMPLX(0.D0,0.D0)
c Down squark index
            Do k=1,6
c Up squark index
               Do l=1,6
                  tmpGl=tmpGl-4.D0/3.D0*DCONJG(gqsqRup(j,l))*gqsqR(i,k)
     &                 *Wdsqusq(k,l)
     &                 *PV_C00(MGl,MDoS(k),MUpS(l),mususy)
c Neutralino index
                  Do m=1,4
                     tmpNe=tmpNe-DCONJG(nqsqRup(j,m,l))*nqsqR(i,m,k)
     &                    *Wdsqusq(k,l)
     &                    *PV_C00(MN(m),MDoS(k),MUpS(l),mususy)
                  End Do
               End Do
c Neutralino index
               Do l=1,4
c Chargino index
                  Do m=1,2
                     tmpNC=tmpNC+MN(l)*MC(2)
     &                    *DCONJG(cqsqRup(j,m,k))*nqsqR(i,l,k)
     &                    *WChNeR(m,l)
     &                    *PV_C0(MDoS(k),MN(l),MC(m))
     &                    -2.D0*DCONJG(cqsqRup(j,m,k))*nqsqR(i,l,k)
     &                    *WChNeL(m,l)
     &                    *(PV_C00(MDoS(k),MN(l),MC(m),mususy)
     &                    -1.D0/4.D0)
                     tmpCN=tmpCN-MN(l)*MC(2)
     &                    *DCONJG(nqsqRup(j,l,k))*cqsqR(i,m,k)
     &                    *WChNeL(m,l)
     &                    *PV_C0(MDoS(k),MN(l),MC(m))
     &                    +2.D0*DCONJG(nqsqRup(j,l,k))*cqsqR(i,m,k)
     &                    *WChNeR(m,l)
     &                    *(PV_C00(MDoS(k),MN(l),MC(m),mususy)
     &                    -1.D0/4.D0)
                  End Do
               End Do
            End Do
            If (iEWcor.eq.1) Then
               WqqRCor(i,j)=-1.D0/(16.D0*PI**2)
     &              *(tmpNC+tmpCN+tmpGl+tmpNe)
            Else
               WqqRCor(i,j)=-1.D0/(16.D0*PI**2)
     &              *(tmpGl)
            End If
         End Do
      End Do

      Return
      End
