      Subroutine init(opt,higgsmodel)
c...set parameters for calculating parameters 
c...used in calculating b->s gamma branching ratio 
c...opt - character string specifying choice to be made
c...higgsmodel - character string specifying "two-higgs 
c...  doublet" models 
c...author: roberto ruiz
  
      implicit none
      include 'dssusy.h'
      include 'sheffaccom.h'
    
      real*8 alfs,mqcd
      character*(*) opt,higgsmodel

c ----- local parameters ------------

      integer i,j
      real*8 mtilde1, mtilde2, tantht, mas_st1, mas_st2 
     &   , mas_sb1 , mas_sb2, r_sb1g, r_sb2g, r_st1g, r_st2g
     &   , epsilonb_1, epsilonb_2, mas_lch, z_eff, aux, yukt_susy
      real*8 L, realr2, imagr2
      real*8  a_i(8), e_i(8), f_i(8), h_i(8), g_i(8)
      real*8 deltabox_SM, deltabox_H, deltabox_MSSM, deltabox
      real*8 aab, gboxh, fpbox,  gpbox 
      real*8 ZRi3
      parameter (ZRi3 = 1.2021d0) !Riemman Z-function in 3.
      complex*16 CI
      parameter (CI = (0.0d0,1.0d0))

      real*8 H_2
            


c -- About b-scale (mub); I set to be pole mb and mt -- 
c -- Just I need pole masses for top and bottom to keep
c independence with the original DS. the rest pole masses
c are set as mass().

c --- parameters coming from experiments(details in  pag 11 of 
c hep-ph/9612313)

      alpha_em = 1.0d0/137.036d0    
      alphae_w = 1.0d0/130.3d0
      CKMfactor = 0.9676d0 !module of V_ts^*V_tb/V_cb !.95
      delta = 0.90d0 !It's a safer quantity(=0.99 "inclusive" process)
      rbce = 0.1045d0 !to be fixed hep-ph/9612313 (changed to Gambino)
      
       
      lambda = 0.12d0 ! (mB*^2-mB^2)/4: B-meson mass splitting


c--- Parameters used as arguments in auxiliar functions 

      eta = alfs(mass(kw))/alfs(mass(kb))
      x= (mqcd(kt,mass(kw))/mass(kw))**2.0d0
      y= (mqcd(kt,mass(kw))/mass(khc))**2.0d0
      z = 0.29d0**2.0d0 ! z=(mass(kc)/mass(kb))**2.0d0
      u = (mass(kw)/mass(khc))**2.0d0

      mass(kc)=sqrt(z)*mass(kb) !from experimental value of 'z'

c------ Parameters used in calculating Wilson coeff. for MSSM

c      eta_susy = runalphas(ksqu(1))/runalphas(kw) !to decide susy scale
c      eta_susy =  0.72040270d0
c

      mtilde1 = 0.0d0
      mtilde2 = 0.0d0
      do i=1,2
       mtilde1 = mtilde1 
     &   + (mass(ksqu(i))+mass(ksqd(i)))/4.0d0
       mtilde2 = mtilde2
     &   + (mass(ksqu(i+3))+mass(ksqd(i+3)))/4.0d0
      enddo
      mtilde = (mtilde1+mtilde2)/2.0d0  

      aux = g2weak/dsqrt(2.d0)/mass(kw)

      yukt_susy= aux*mqcd(kqu(i),mtilde)/sinbe

       
      eta_susy = alfs(mtilde)/alfs(mass(kw)) !to decide susy scale


      sintht = squrmx(3,3)
      costht = squlmx(3,3)            
      tantht = sintht/costht


      mas_st2 = dmin1(mass(ksqu(3)),mass(ksqu(6)))
      mas_st1 = dmax1(mass(ksqu(3)),mass(ksqu(6)))
       

      do i=1,2
        Y_i(i) = costht/dsqrt(2.0d0)/sinbe/mass(kw)
     &          *mqcd(kt,mtilde)*chavmx(i,2) !improved version
c     &          *mqcd(kt,mass(kw))*chavmx(i,2)  !to check               
        t1(i) = chavmx(i,1)*costht-tantht*Y_i(i)
        t2(i) = chavmx(i,1)*sintht+Y_i(i)
        z_i(i) = (mtilde/mass(kcha(i)))**2.0d0
        y1(i) = (mas_st1/mass(kcha(i)))**2.0d0
        y2(i) = (mas_st2/mass(kcha(i)))**2.0d0        
      enddo
      

        
      do i=1,2
        r_wc(i) = (mass(kw)/mass(kcha(i)))**2.0d0
      enddo 
      r_wsq = (mass(kw)/mtilde)**2.0d0
      r_wst1 = (mass(kw)/mas_st1)**2.0d0
      r_wst2 = (mass(kw)/mas_st2)**2.0d0                   
       

c ---- parameter to include corrections from SUSY
      
      mas_sb2 = dmin1(mass(ksqd(3)),mass(ksqd(6)))
      mas_sb1 = dmax1(mass(ksqd(3)),mass(ksqd(6)))


      r_sb1g = (mas_sb1/mass(kgluin))**2.0d0
      r_sb2g = (mas_sb2/mass(kgluin))**2.0d0
      r_st1g = (mas_st1/mass(kgluin))**2.0d0
      r_st2g = (mas_st2/mass(kgluin))**2.0d0 
      

      epsilonb_1 = 
     &  - 2.0d0/3.0d0*alfs(mtilde)/pi*mu/mass(kgluin)
     &          *H_2(r_sb1g,r_sb2g)



      epsilonb_2 =0.0d0
      do i=1,2
        epsilonb_2 = epsilonb_2
     &   + 1.0d0/mass(kcha(i))*chaumx(i,2)*chavmx(i,2)
     &        *H_2(y1(i),y2(i))
      enddo



      epsilonb = epsilonb_1
     &  - 1.0d0/16.0d0/pi**2.0d0*yukt_susy**2.0d0*Asoftu(3) 
     &         *epsilonb_2

      K_tanbe = 1.0d0/(1.0d0+epsilonb*tanbe)

c      write(*,*) 'epsilonb',epsilonb


      do i=1,2
       t2_correc(i) = chavmx(i,1)*sintht*(K_tanbe+epsilonb*tanbe)
     &   + Y_i(i)*(K_tanbe+epsilonb*tanbe*mqcd(kt,mass(kw))
     &                                       /mqcd(kt,mtilde))
      enddo


c --- decision about frame to use in SUSY contributions

      mas_lch = dmin1(mass(kcha(1)),mass(kcha(2)))
 
      If(2.5d0*mas_lch .le. mass(kgluin) 
     &      .and. 2.5d0*mas_st2 .le. mass(kgluin)
     &      .and. 2.5d0*mas_lch .le. mtilde 
     &      .and. 2.5d0*mas_st2 .le. mtilde) then
        scenario1 = .false.
        scenario2 = .true.
      else
        scenario1 = .true.
        scenario2 = .false.
      endif

c --- Parameters useful to rescale the Wilson coeff from weak to
c b scale. They are taken from hep-ph/9612313.     


   
      a_i(1) = 14.0d0/23.0d0
      a_i(2) = 16.0d0/23.0d0
      a_i(3) = 6.0d0/23.0d0
      a_i(4) = -12.0d0/23.0d0
      a_i(5) = 0.4086d0
      a_i(6) = -0.4230d0
      a_i(7) = -0.8994d0
      a_i(8) = 0.1456d0

      e_i(1) = 4661194.0d0/816831.0d0
      e_i(2) = -8516.0d0/2217.0d0
      e_i(3) = 0.0d0
      e_i(4) = 0.0d0
      e_i(5) = -1.9043d0
      e_i(6) = -0.1008d0
      e_i(7) = 0.1216d0
      e_i(8) = 0.0183d0

      f_i(1) = -17.3023d0
      f_i(2) = 8.5027d0
      f_i(3) = 4.5508d0
      f_i(4) = 0.7519d0
      f_i(5) = 2.0040d0
      f_i(6) = 0.7476d0
      f_i(7) = -0.5385d0
      f_i(8) = 0.0914d0

      g_i(1) = 14.8088d0
      g_i(2) = -10.8090d0
      g_i(3) = -0.8740d0
      g_i(4) = 0.4218d0
      g_i(5) = -2.9347d0
      g_i(6) = 0.3971d0
      g_i(7) = 0.16d0
      g_i(8) = 0.0225d0

      h_i(1) = 626126.0d0/272277.0d0
      h_i(2) = -56281.0d0/51730.0d0
      h_i(3) = -3.0d0/7.0d0
      h_i(4) = -1.0d0/14.0d0
      h_i(5) = -0.6494d0
      h_i(6) = -0.0380d0
      h_i(7) = -0.0186d0
      h_i(8) = -0.0057d0

c ----- parameters useful to calculate "D" coeff. in b-> s gamma
c branche ratio formula. See hep-ph/9612313.   

      gamma0(1,1) = -4.0d0
      gamma0(1,2) = 8.0d0/3.0d0
      gamma0(1,3) = 0.0d0
      gamma0(1,4) = -2.0d0/9.0d0
      gamma0(1,5) = 0.0d0
      gamma0(1,6) = 0.0d0
      gamma0(1,7) = -208.0d0/243.0d0
      gamma0(1,8) = 173.0d0/162.0d0
      gamma0(2,1) = 12.0d0
      gamma0(2,2) = 0.0d0
      gamma0(2,3) = 0.0d0
      gamma0(2,4) = 4.0d0/3.0d0
      gamma0(2,5) = 0.0d0
      gamma0(2,6) = 0.0d0
      gamma0(2,7) = 416.0d0/81.0d0
      gamma0(2,8) = 70.0d0/27.0d0
      gamma0(3,1) = 0.0d0
      gamma0(3,2) = 0.0d0    
      gamma0(3,3) = 0.0d0
      gamma0(3,4) = -52.0d0/3.0d0
      gamma0(3,5) = 0.0d0
      gamma0(3,6) = 2.0d0
      gamma0(3,7) = -176.0d0/81.0d0
      gamma0(3,8) = 14.0d0/27.0d0
      gamma0(4,1) = 0.0d0    
      gamma0(4,2) = 0.0d0
      gamma0(4,3) = -40.0d0/9.0d0
      gamma0(4,4) = -100.0d0/9.0d0
      gamma0(4,5) = 4.0d0/9.0d0
      gamma0(4,6) = 5.0d0/6.0d0
      gamma0(4,7) = -152.0d0/243.0d0
      gamma0(4,8) = -587.0d0/162.0d0
      gamma0(5,1) = 0.0d0
      gamma0(5,2) = 0.0d0    
      gamma0(5,3) = 0.0d0     
      gamma0(5,4) = -256.0d0/3.0d0
      gamma0(5,5) = 0.0d0
      gamma0(5,6) = 20.0d0
      gamma0(5,7) = -6272.0d0/81.0d0
      gamma0(5,8) = 6596.0d0/27.0d0
      gamma0(6,1) = 0.0d0
      gamma0(6,2) = 0.0d0
      gamma0(6,3) = -256.0d0/9.0d0
      gamma0(6,4) = 56.0d0/9.0d0
      gamma0(6,5) = 40.0d0/9.0d0
      gamma0(6,6) = -2.0d0/3.0d0
      gamma0(6,7) = 4624.0d0/243.0d0
      gamma0(6,8) = 4772.0d0/81.0d0
      gamma0(7,1) = 0.0d0
      gamma0(7,2) = 0.0d0    
      gamma0(7,3) = 0.0d0
      gamma0(7,4) = 0.0d0
      gamma0(7,5) = 0.0d0    
      gamma0(7,6) = 0.0d0     
      gamma0(7,7) = 32.0d0/3.0d0
      gamma0(7,8) = 0.0d0
      gamma0(8,1) = 0.0d0
      gamma0(8,2) = 0.0d0    
      gamma0(8,3) = 0.0d0
      gamma0(8,4) = 0.0d0
      gamma0(8,5) = 0.0d0    
      gamma0(8,6) = 0.0d0     
      gamma0(7,7) = -32.0d0/9.0d0
      gamma0(7,8) = 28.0d0/3.0d0


c------------------------------------------
c I use z = (mc_MS/mb)^2 around  0.22^2. To include quark masses
c effects. Look at hep/-ph/0104034. 
      
      z_eff = 0.22d0**2.0d0  !0.29d0**2.0d0 First estimation
      L = dlog(z_eff)
      realr2 = 2.0d0/243.0d0*(-833.0d0+144.0d0*pi**2*z_eff**1.5d0
     &       + (1728.0d0-180.0d0*pi**2.0d0-1296.0d0*ZRi3
     &           + (1296.0d0-324.0d0*pi**2.0d0)*L
     &           + 108.0d0*L**2.0d0+ 36.0d0*L**3.0d0)*z_eff
     &       + (648.0d0+72.0d0*pi**2.0d0
     &           + (432.0d0-216.0d0*pi**2.0d0)*L
     &           + 36*L**3.0d0)*z_eff**2.0d0
     &       + (-54.0d0-84.0d0*pi**2.0d0+1092.0d0*L
     &           - 756.0d0*L**2.0d0)*z_eff**3.0d0)
      imagr2 = 16.0d0*pi/81.0d0*(-5
     &       + (45.0d0-3.0d0*pi**2.0d0+9.0d0*L
     &           + 9.0d0*L**2.0d0)*z_eff
     &       + (-3.0d0*pi**2.0d0+9.0d0*L**2.0d0)*z_eff**2.0d0
     &       + (28.0d0-12.0d0*L)*z_eff**3.0d0)
      r(2) = realr2+CI*imagr2
      r(1) = -1.0d0/6.0d0*r(2)
      r(7) = -10.0d0/3.0d0-8.0d0/9.0d0*pi**2.0d0
      r(8) = -4.0d0/27.0d0*(-33.0d0+2*pi**2.0d0-6*CI*pi)
      do i=3,6
       r(i) = (0.0d0,0.0d0)   
      enddo
    
c ----- parameters useful to calculate "A" coeff. in b-> s gamma
c branche ratio formula. It's model dependent. I have them  set
c to a "total" inclusive decay (hep-ph/9710335) which corresponds
c to 'default' option (i.e. delta = 0.9). Also I add results for
c delta = 0.99. For calculations I use hep-ph/9612313 and
c hep-ph/0104034 
    
      do i=1,8
       do j=1,8
         f_ij(i,j)=0.0d0
       enddo
      enddo
      if(opt.eq.'default') then
        f_ij(2,2) = 0.0543037d0
        f_ij(2,7) = -0.0204210d0 !corresponding to z=0.22**2
        f_ij(7,7) = 3.206d0
        f_ij(7,8) = 0.3873d0
        f_ij(8,8) = 1.3174d0 !These are z independent 
      else if (opt.eq.'inclusive') then !delta = 0.99(total inclusive)
        f_ij(2,2) = 0.03402d0
        f_ij(2,7) = 0.018175d0
        f_ij(7,7) = 3.42106d0
        f_ij(7,8) = 0.38966d0
        f_ij(8,8) = 3.21616d0
      else
          write (*,*) 
     &           'acinit: unrecognized model option ',opt
          stop
      endif   

      f_ij(1,1) = f_ij(2,2)/36.0d0 
      f_ij(1,2) = -f_ij(2,2)/3.0d0
      f_ij(1,3) = -0.0035d0
      f_ij(1,4) = 0.0006d0
      f_ij(1,5) = -0.0459d0
      f_ij(1,6) = -0.06d0
      f_ij(1,7) = -f_ij(2,7)/6.0d0
      f_ij(1,8) = f_ij(2,8)/18.0d0
      f_ij(2,3) = 0.0210d0
      f_ij(2,4) = -0.0035d0
      f_ij(2,5) = 0.2754d0
      f_ij(2,6) = 0.3599d0
      f_ij(2,8) = -f_ij(2,7)/3.0d0
      f_ij(3,3) = 0.0140d0
      f_ij(3,4) = -0.0047d0
      f_ij(3,5) = 0.327d0
      f_ij(3,6) = 0.0666d0
      f_ij(3,7) = 0.0421d0
      f_ij(3,8) = -0.0140d0
      f_ij(4,4) = 0.0088d0
      f_ij(4,5) = -0.0546d0
      f_ij(4,6) = 0.1570d0
      f_ij(4,7) = -0.0070d0
      f_ij(4,8) = 0.0023d0
      f_ij(5,5) = 1.9369d0
      f_ij(5,6) = 0.9506d0
      f_ij(5,7) = 0.5926d0
      f_ij(5,8) = -0.1975d0
      f_ij(6,6) = 0.9305d0
      f_ij(6,7) = 0.0002d0
      f_ij(6,8) = -0.0001d0

c----------------------------------------

      term1 = 0.0d0
      term2 = 0.0d0
      term3 = 0.0d0
      term4 = 0.0d0
      do i=1,8
        term1 = term1 + h_i(i)*eta**a_i(i)
        term2 = term2 + e_i(i)*eta**(a_i(i)+1.0d0)
        term3 = term3 + g_i(i)*eta**(a_i(i)+1.0d0)
        term4 = term4 + f_i(i)*eta**a_i(i)
      enddo


    
c ---- part to define the "higss-doublet" model to use (hep-ph/9710335). 

      if(higgsmodel.eq.'model1') then
        Au = 1/tanbe
        Ad = Au
      else if(higgsmodel.eq.'model2') then      
        Au = 1/tanbe
        Ad = -1.0d0/Au
      else
          write (*,*) 
     &           'acinit: unrecognized model option ',higgsmodel
            stop
      endif   

c the ratio R_delta for the B-Bbar mixing and epsilon.

      deltabox_SM = aab(x)                !SM contribution

      deltabox_H = Au**4.0d0*y*gboxh(y)/4.d0 
     &    + 2.0d0*Au**2.0d0*x*(fpbox(x,x/y)+ gpbox(x,x/y)/4.d0)  !2HDM

c       print*,"d",deltabox_SM

      deltabox_MSSM=0.d0
      do i=1,2
        do j=1,2
          deltabox_MSSM=deltabox_MSSM+ t2(j)**2*t2(i)**2
     &        *(mass(kw)**2/mass(kcha(j))**2)/x       
     &          *gpbox(y2(j),(mass(kcha(i))**2/mass(kcha(j))**2))
        enddo
      enddo

      deltabox=deltabox_SM+deltabox_H+ deltabox_MSSM


      bbox= deltabox/aab(x)


      return
      end










