      subroutine bsmm(brbsmm)
c_________________________________________________________________
c branching ratio of B_s -> mu+ mu- decay
c common:
c   'dssusy.h' - file with susy common blocks
c   /wilcoeff/ - wilson coefficients
c output:
c   brbsmm : branching ratio
c author: Y.G.Kim 
c   formula given in hep-ph/0104284
c ! caution : the convention of chargino mixing matrix
c             is different between darksusy's 
c             and the above paper's.  U,V_{dark} = U,V_{above}^*  
c------------------------------------------------------------------
      implicit none
      include 'dssusy.h'
      include 'sheffaccom.h'
      real*8 Gf,alp,mbs,tbs,fbs,CKMfact
      complex*16 ii,FS,FP,FA
      complex*16 c_s,cp_s,c_p,cp_p,c_10
      common /wilcoeff/c_s,cp_s,c_p,cp_p,c_10
      real*8 brbsmm

      ii = cmplx(0.0d0,1.0d0)

      Gf = 1.16639D-5
      alp = alphem
      mbs = 5.3696d0
      tbs = (1.493d-12)*(1.51927d24)
      fbs = 0.23d0
      CKMfact = 0.04d0**2


      call wilson_bsmm


      c_s = c_s * K_tanbe * K_tanbe !bottom-Yukawa corrected
      c_p = c_p * K_tanbe * K_tanbe

      cp_s = cp_s * K_tanbe * K_tanbe
      cp_p = cp_p * K_tanbe * K_tanbe


      FS = -ii/2.0d0*mbs**2*fbs*
     &     (c_s*mass(kb)-cp_s*mass(ks))/
     &     (mass(kb)+mass(ks))
      FP = -ii/2.0d0*mbs**2*fbs*
     &     (c_p*mass(kb)-cp_p*mass(ks))/
     &     (mass(kb)+mass(ks))
      FA = -ii/2.0d0*fbs*c_10


      brbsmm = Gf**2*alp**2*mbs*tbs*CKMfact/
     &         (16.d0*pi**3)*
     &         dsqrt(1.d0-4.0d0*mass(kl(2))**2/mbs**2)*
     &         ( (1.0d0-4.0d0*mass(kl(2))**2/mbs**2)*
     &     cdabs(FS)**2+cdabs(FP+2.0d0*mass(kl(2))*FA)**2 )


      return
      end
c
c
      subroutine wilson_bsmm
c....................................................
c
c....................................................
      implicit none
      include 'dssusy.h'
      integer i,j,k,m,n,a,b
      complex*16 ii,lambda(3,3)
      complex*16 c_s,cp_s,c_p,cp_p,c_10
      common /wilcoeff/c_s,cp_s,c_p,cp_p,c_10
      real*8 s12,s23,s13,delta,c12,c23,c13
      complex*16 Vud,Vus,Vub,Vcd,Vcs,Vcb,Vtd,Vts,Vtb
      real*8 xt,Yt0,Yt
      real*8 r,x,y,z,yai,yaj,ybi
      real*8 box_pre,pen_pre,cnt_pre
      real*8 dabkm,dij
      real*8 D1,D2,D3
      complex*16 Gai,Gaimn,Gaimnd
      complex*16 c_shiggs,c_phiggs,cp_shiggs,cp_phiggs
      complex*16 c_sbox,c_pbox,cp_sbox,cp_pbox
      complex*16 c_spen,c_ppen,cp_spen,cp_ppen
      complex*16 c_scnt,c_pcnt,cp_scnt,cp_pcnt,c_cnt,cp_cnt
c
      ii = cmplx(0.d0,1.d0)
      s12 = 0.221d0
      s23 = 0.04d0
      s13 = 0.0035d0
      delta = 1.08d0
      c12 = sqrt(1.d0-s12**2)
      c23 = sqrt(1.d0-s23**2)
      c13 = sqrt(1.d0-s13**2)
      Vud = c12*c13
      Vus = s12*c13
      Vub = s13*exp(-ii*delta)
      Vcd = -s12*c23-c12*s23*s13*exp(ii*delta)
      Vcs = c12*c23-s12*s23*s13*exp(ii*delta)
      Vcb = s23*c13
      Vtd = s12*s23-c12*c23*s13*exp(ii*delta)
      Vts = -c12*s23-s12*c23*s13*exp(ii*delta)
      Vtb = c23*c13
      lambda(1,1) = Vub*conjg(Vus)/(Vtb*conjg(Vts)) 
      lambda(1,2) = Vub*conjg(Vcs)/(Vtb*conjg(Vts)) 
      lambda(1,3) = Vub*conjg(Vts)/(Vtb*conjg(Vts)) 
      lambda(2,1) = Vcb*conjg(Vus)/(Vtb*conjg(Vts)) 
      lambda(2,2) = Vcb*conjg(Vcs)/(Vtb*conjg(Vts)) 
      lambda(2,3) = Vcb*conjg(Vts)/(Vtb*conjg(Vts)) 
      lambda(3,1) = Vtb*conjg(Vus)/(Vtb*conjg(Vts)) 
      lambda(3,2) = Vtb*conjg(Vcs)/(Vtb*conjg(Vts)) 
      lambda(3,3) = Vtb*conjg(Vts)/(Vtb*conjg(Vts)) 

      xt = 167.d0**2/mass(kw)**2
c      xt = runmass(kt)**2/mass(kw)**2
      Yt0 = -xt/8.d0*( (xt-4.d0)/(xt-1.d0)+
     &                3.d0*xt/(xt-1.d0)**2*dlog(xt) )
      Yt = 1.012d0*Yt0   ! from hep-ph/0101336
      
      c_10 = Yt/s2thw

c
c Two-Higgs doublet model
c
      r = mass(khc)**2/runmass(kt)**2
      c_shiggs = mass(kl(2))*tanbe**2/(4.0d0*mass(kw)**2*s2thw)*
     &           dlog(r)/(1.0d0-r)
      c_phiggs = - c_shiggs

      cp_shiggs = cmplx(0.d0,0.d0)
      cp_phiggs = cmplx(0.d0,0.d0)

c      print*,'c_shiggs=',c_shiggs
c      print*,'c_phiggs=',c_phiggs

c
c SUSY with MFV
c

c box diagram

      c_sbox = cmplx(0.d0,0.d0)
      c_pbox = cmplx(0.d0,0.d0)
      cp_sbox = cmplx(0.d0,0.d0)
      cp_pbox = cmplx(0.d0,0.d0)


 
      do 60 i=1,2
       do 50 j=1,2
        do 40 a=1,6
c         do 30 k=1,3  ! only for ksnumu
          do 20 m=1,3
           do 10 n=1,3

       x = mass(ksnumu)**2/mass(kcha(i))**2
       y = mass(ksqu(a))**2/mass(kcha(i))**2
       z = mass(kcha(j))**2/mass(kcha(i))**2


       Gai = 1.d0/(2.d0*dsqrt(2.d0)*s2thw)*
     &       ( dsqrt(2.d0)*mass(kw)*
     &         conjg(chavmx(i,1))*conjg(squlmx(a,n))-
     &         runmass(kqu(n))*conjg(chavmx(i,2))*conjg(squrmx(a,n)) )
       Gaimn = Gai*lambda(m,n)
c       Gaimnd = Gai*conjg(lambda(n,m))


       c_sbox = c_sbox + 1.d0/mass(kcha(i))**2*
     &       squlmx(a,m)*conjg(chaumx(j,2))*Gaimn*
     &      ( y*chaumx(j,2)*chavmx(i,1)+
     &        mass(kcha(j))/mass(kcha(i))*
     &        conjg(chaumx(i,2))*conjg(chavmx(j,1)) )*
     &        D1(x,y,z) 
c       cp_sbox = cp_sbox + 1.d0/mass(kcha(i))**2*
c     &       squlmx(a,m)*conjg(chaumx(j,2))*Gaimnd*
c     &      ( y*chaumx(j,2)*chavmx(i,1)+
c     &        mass(kcha(j))/mass(kcha(i))*
c     &        conjg(chaumx(i,2))*conjg(chavmx(j,1)) )*
c     &        D1(x,y,z) 
      
       c_pbox = c_pbox + 1.d0/mass(kcha(i))**2*
     &       squlmx(a,m)*conjg(chaumx(j,2))*Gaimn*
     &      ( y*chaumx(j,2)*chavmx(i,1)-
     &        mass(kcha(j))/mass(kcha(i))*
     &        conjg(chaumx(i,2))*conjg(chavmx(j,1)) )*
     &        D1(x,y,z) 
c       cp_pbox = cp_pbox + 1.d0/mass(kcha(i))**2*
c     &       squlmx(a,m)*conjg(chaumx(j,2))*Gaimnd*
c     &      ( y*chaumx(j,2)*chavmx(i,1)-
c     &        mass(kcha(j))/mass(kcha(i))*
c     &        conjg(chaumx(i,2))*conjg(chavmx(j,1)) )*
c     &        D1(x,y,z) 
      
 10         continue       
 20        continue       
c 30       continue       
 40      continue       
 50     continue       
 60    continue       
 
      box_pre = mass(kmu)*tanbe**2/(2.d0*mass(kw))
      c_sbox = -box_pre*c_sbox
      c_pbox =  box_pre*c_pbox
      cp_sbox = -box_pre*cp_sbox
      cp_pbox =  box_pre*cp_pbox

c      print*,'c_sbox=',c_sbox
c      print*,'c_pbox=',c_pbox
c      print*,'cp_sbox=',cp_sbox
c      print*,'cp_pbox=',cp_pbox

c penguin diagram

      c_spen = cmplx(0.d0,0.d0)
      c_ppen = cmplx(0.d0,0.d0)
      cp_spen = cmplx(0.d0,0.d0)
      cp_ppen = cmplx(0.d0,0.d0)

      do 170 i=1,2
       do 160 j=1,2
        do 150 a=1,6
         do 140 b=1,6
          do 130 k=1,3  
           do 120 m=1,3
            do 110 n=1,3

       yai = mass(ksqu(a))**2/mass(kcha(i))**2
       yaj = mass(ksqu(a))**2/mass(kcha(j))**2
       ybi = mass(ksqu(b))**2/mass(kcha(i))**2
       z   = mass(kcha(i))**2/mass(kcha(j))**2

       dabkm = 0.d0
       if(a.eq.b .and. k.eq.m) dabkm = 1.d0
       dij = 0.d0
       if(i.eq.j) dij = 1.d0

       Gai = 1.d0/(2.d0*dsqrt(2.d0)*s2thw)*
     &       ( dsqrt(2.d0)*mass(kw)*
     &         conjg(chavmx(i,1))*conjg(squlmx(a,n))-
     &         runmass(kqu(n))*conjg(chavmx(i,2))*conjg(squrmx(a,n)) )
       Gaimn = Gai*lambda(m,n)
c       Gaimnd = Gai*conjg(lambda(n,m))

       c_spen = c_spen + Gaimn*squlmx(b,m)*conjg(chaumx(j,2))*
     &       ( mass(kw)*(yaj*chaumx(j,2)*chavmx(i,1)+
     &         mass(kcha(i))/mass(kcha(j))*
     &         conjg(chaumx(i,2))*conjg(chavmx(j,1)) )*
     &      D2(yaj,z)*dabkm - runmass(kqu(k))/dsqrt(2.d0)/mass(kcha(i))*
     &       ( mu*squrmx(a,k)*conjg(squlmx(b,k)) + 
     &         mu*squlmx(a,k)*conjg(squrmx(b,k)) )*
     &        D2(yai,ybi)*dij ) 
c       cp_spen = cp_spen + Gaimnd*squlmx(b,m)*conjg(chaumx(j,2))*
c     &       ( mass(kw)*(yaj*chaumx(j,2)*chavmx(i,1)+
c     &         mass(kcha(i))/mass(kcha(j))*
c     &         conjg(chaumx(i,2))*conjg(chavmx(j,1)) )*
c     &        D2(yaj,z)*dabkm - mass(kqu(k))/dsqrt(2.d0)/mass(kcha(i))*
c     &       ( mu*squrmx(a,k)*conjg(squlmx(b,k)) + 
c     &         mu*squlmx(a,k)*conjg(squrmx(b,k)) )*
c     &        D2(yai,ybi)*dij ) 
      
       c_ppen = c_ppen + Gaimn*squlmx(b,m)*conjg(chaumx(j,2))*
     &       ( mass(kw)*(yaj*chaumx(j,2)*chavmx(i,1)-
     &         mass(kcha(i))/mass(kcha(j))*
     &         conjg(chaumx(i,2))*conjg(chavmx(j,1)) )*
     &      D2(yaj,z)*dabkm - runmass(kqu(k))/dsqrt(2.d0)/mass(kcha(i))*
     &       ( mu*squrmx(a,k)*conjg(squlmx(b,k)) - 
     &         mu*squlmx(a,k)*conjg(squrmx(b,k)) )*
     &        D2(yai,ybi)*dij ) 
c       cp_ppen = cp_ppen + Gaimn*squlmx(b,m)*conjg(chaumx(j,2))*
c     &       ( mass(kw)*(yaj*chaumx(j,2)*chavmx(i,1)-
c     &         mass(kcha(i))/mass(kcha(j))*
c     &         conjg(chaumx(i,2))*conjg(chavmx(j,1)) )*
c     &        D2(yaj,z)*dabkm - mass(kqu(k))/dsqrt(2.d0)/mass(kcha(i))*
c     &       ( mu*squrmx(a,k)*conjg(squlmx(b,k)) - 
c     &         mu*squlmx(a,k)*conjg(squrmx(b,k)) )*
c     &        D2(yai,ybi)*dij ) 

 110         continue 
 120        continue 
 130       continue 
 140      continue 
 150     continue 
 160    continue 
 170   continue 

      pen_pre = mass(kmu)*tanbe**2/mass(kw)**2/
     &          (mass(khc)**2-mass(kw)**2)
      c_spen =  pen_pre*c_spen
      c_ppen = -pen_pre*c_ppen
      cp_spen =  pen_pre*cp_spen
      cp_ppen = -pen_pre*cp_ppen

c      print*,'c_spen=',c_spen
c      print*,'c_ppen=',c_ppen
c      print*,'cp_spen=',cp_spen
c      print*,'cp_ppen=',cp_ppen

c counter term

      c_cnt = cmplx(0.d0,0.d0)
      c_scnt = cmplx(0.d0,0.d0)
      c_pcnt = cmplx(0.d0,0.d0)

      cp_cnt = cmplx(0.d0,0.d0)
      cp_scnt = cmplx(0.d0,0.d0)
      cp_pcnt = cmplx(0.d0,0.d0)

      do 240 i=1,2     
       do 230 a=1,6
        do 220 m=1,3
         do 210 n=1,3

       y = mass(ksqu(a))**2/mass(kcha(i))**2

       Gai = 1.d0/(2.d0*dsqrt(2.d0)*s2thw)*
     &       ( dsqrt(2.d0)*mass(kw)*
     &         conjg(chavmx(i,1))*conjg(squlmx(a,n))-
     &         runmass(kqu(n))*conjg(chavmx(i,2))*conjg(squrmx(a,n)) )
       Gaimn = Gai*lambda(m,n)
c       Gaimnd = Gai*conjg(lambda(n,m))

      c_cnt = c_cnt + mass(kcha(i))*D3(y)*
     &         conjg(chaumx(i,2))*squlmx(a,m)*Gaimn
c      cp_cnt = cp_cnt + mass(kcha(i))*D3(y)*
c     &         conjg(chaumx(i,2))*squlmx(a,m)*Gaimnd

 210     continue
 220    continue
 230   continue
 240  continue

      cnt_pre = mass(kmu)*tanbe**3/
     &         dsqrt(2.d0)/mass(kw)**2/(mass(khc)**2-mass(kw)**2)
      c_scnt = -cnt_pre*c_cnt  
      c_pcnt =  cnt_pre*c_cnt  
      cp_scnt = -cnt_pre*cp_cnt  
      cp_pcnt =  cnt_pre*cp_cnt  

c      print*,'c_scnt=',c_scnt
c      print*,'c_pcnt=',c_pcnt
c      print*,'cp_scnt=',cp_scnt
c      print*,'cp_pcnt=',cp_pcnt

c
c summation
c

      c_s = c_shiggs + c_sbox + c_spen + c_scnt
      c_p = c_phiggs + c_pbox + c_ppen + c_pcnt


c      print*,'cp_sbox,cp_pbox=',cp_sbox,cp_pbox
c      print*,'cp_spen,cp_ppen=',cp_spen,cp_ppen
c      print*,'cp_scnt,cp_pcnt=',cp_scnt,cp_pcnt
      
c      cp_s = cp_scnt
c      cp_p = cp_pcnt
c      cp_s = conjg(cp_s)
c      cp_p = -conjg(cp_p)

      cp_s = cmplx(0.d0,0.d0)
      cp_p = cmplx(0.d0,0.d0)

      return
      end

c
c......................................................
c
      real*8 function D1(x,y,z)
      real*8 x,y,z
     
      if(z.gt.0.999d0 .and. z.lt.1.001d0) then 
        D1 = x*dlog(x)/(1.0d0-x)/(x-y)/(x-z)+
     &       y*dlog(y)/(1.0d0-y)/(y-x)/(y-z)+
     &       (-1.d0-0.5d0*(z-1.d0))/(z-y)/(z-x)
      else
        D1 = x*dlog(x)/(1.0d0-x)/(x-y)/(x-z)+
     &       y*dlog(y)/(1.0d0-y)/(y-x)/(y-z)+
     &       z*dlog(z)/(1.0d0-z)/(z-y)/(z-x)
      endif

      return
      end
c
c
      real*8 function D2(x,y)
      real*8 x,y
    
      if(x.eq.y) then
        D2 = (1.d0-x+dlog(x))/(1.d0-x)**2
      elseif(y.gt.0.999d0 .and. y.lt.1.001d0) then 
        D2 = x*dlog(x)/(1.0d0-x)/(x-y)+
     &       (-1.d0-0.5d0*(z-1.d0))/(y-x)
      else
        D2 = x*dlog(x)/(1.0d0-x)/(x-y)+
     &       y*dlog(y)/(1.0d0-y)/(y-x)
      endif

      return
      end
c
c
      real*8 function D3(x)
      real*8 x
     
      if(x.gt.0.999d0 .and. x.lt.1.001d0) then 
        D3 = -1.d0-0.5d0*(x-1.d0)
      else
        D3 = x*dlog(x)/(1.0d0-x)
      endif

      return
      end



      
