      subroutine sheffddneunuc(sigsip,sigsin,sigsdp,sigsdn)  

c_______________________________________________________________________
c  neutralino nucleon cross section.
c  common:
c    'dssusy.h' - file with susy common blocks
c    'dsddloops.h' - file with common blocks used in calculating
c     netralino-proton cross section including QCD corrections.
c  output:
c    sigsip, sigsin : proton and neutron spin-independent cross sections
c    sigsdp, sigsdn : proton and neutron spin-dependent cross sections
c    units: cm^2
c  author: roberto ruiz
c     13-sep-94 no drees-nojiri twist-2 terms
c     22-apr-95 important bug corrected [ft -> ft mp/mq]
c=======================================================================
      implicit none
      include 'dssusy.h'
      include 'dsprep.h'
      include 'dsddcom.h'
      include 'sheffdd.h' !new include file


c ------- local variables ----------------------

      integer i,a,r
      real*8 gev2cm2,mp,mn,mq,fkin_p,fkin_n,f1q,df1q,fp,
     &     fn,du,dd,ds,dsabsq
      real*8 f_TG_p, f_TG_n   
      complex*16 c_sf_pls(6,6), d_sf_pls(6,6) ! comb. of couplings. 

      real*8 fp_mp, fp_mn, sum_gq_p , sum_gq_n
      real*8 B_d_i(6), B_s_i(6), B_1d_i(6), B_1s_i(6), B_2s_i(6)
      real*8 B_1d_t2g,  B_1s_t2g,  B_2s_t2g  !just to check

c ------ temp. parameters are introducce in a new ".h" file  --------

      real*8 sigsip,sigsin,sigsdp,sigsdn

      parameter (gev2cm2 = (197.327053d-16)**2)
      parameter (mp=0.93827231d0, mn=0.93956563d0)
   
cccccccccccccccccccccccccccc
c
c  new variables:
c  - 'i' means differents quark species (i=1..6).   
c  - 'r' means differents neutral scalar higgs (r=1,2).
c  - 'a' means differents squarks corresponding to one quark (a=1,6).
c  - 'kq' array with code for quarks.
c  - 'ksq' 2X2 dimensional array with code for squarks.
c  - 'kh' array with code for charge neutral scalar higgses.
c  - 'g_b, T_sq, B_s, B_1s, B_2s, B_d, B_1d' scalars from 1-loop correc.
c  - 'fp_mp' is an auxiliar function to calculate fp.
cccccccccccccccccccccccccccc
c
c  new arrays and functions:
c  - 'c_sf_pls,d_sf_pls' are auxiliar combination of 
c       neutralinno-fermion-sfermion couplings.
c  - 'f_q_h(i)' is the effective coupling for higgs ech. 
c  - 'f_q_sq(i)' is the effective coupling for sfermion exch.
c  - 'f_q(i)' is the sumation of f_q_h(i) and f_q_sq(i).
c  - 'B_s_i, B_1s_i, B_2s_i, B_d_i, B_1d_i' arrays from 1-loop correc. 

ccccccccccccccccccccccccccc


      real*8 msq,qN



      call sheffddinit ! init. functions used here


      ftp(10) = ftp_s
      ftn(10) = ftn_s !nuisance parameters


c-------------- auxiliar combinations for sfermion exchange --

      do i = 1,6    
        do a = 1,6
          c_sf_pls(i,a) = cdabs(gl(ksq(i,a),kn(kln),kq(i)))**2.0d0
     &                  + cdabs(gr(ksq(i,a),kn(kln),kq(i)))**2.0d0
          d_sf_pls(i,a)
     &       = gl(ksq(i,a),kn(kln),kq(i))
     &                *dconjg(gr(ksq(i,a),kn(kln),kq(i)))
     &        + dconjg(gl(ksq(i,a),kn(kln),kq(i)))
     &                 *gr(ksq(i,a),kn(kln),kq(i))

        enddo
      enddo



c------------------------ f_q parameter ------------------------

      do i=1,6
        f_q_h(i) = 0.0d0
        f_q_sq(i) = 0.0d0
         do r=1,2   
          f_q_h(i)= f_q_h(i)
     &      +gr(kh(r),kn(kln),kn(kln))
     &           *gl(kh(r),kq(i),kq(i))/mass(kh(r))**2.0d0

c ------- temp. parameter to check --------------------------------

          f_q_hr(i,r) = +gr(kh(r),kn(kln),kn(kln))
     &           *gl(kh(r),kq(i),kq(i))/mass(kh(r))**2.0d0 

c -----------------------------------------------------------------

        enddo

        do a=1,6
          f_q_sq(i)= f_q_sq(i)
     &      +d_sf_pls(i,a)/(mass(ksq(i,a))**2.0d0
     &                            -(mx+mass(kq(i)))**2.0d0)
        enddo

        do r=1,2
            f_q_hr(i,r) = f_q_hr(i,r)/2.0d0
        enddo
        f_q_h(i) = f_q_h(i)/2.0d0           
        f_q_sq(i) = - f_q_sq(i)/2.0d0/4.0d0
        f_q(i) =  f_q_h(i) +f_q_sq(i) !test
      enddo
c comment about it

c------------------------- g_q --------------------------------

      do i=1,6
        g_q(i) = 0.0d0
        if (i.eq.3) then
          continue
        else
          do a=1,6
           g_q(i) = g_q(i)
     &      +c_sf_pls(i,a)/(mass(ksq(i,a))**2.0d0
     &                  -(mx+mass(kq(i)))**2.0d0)**2.0d0
          enddo
        endif
        g_q(i) = - g_q(i)/2.0d0/4.0d0
      enddo 
c comment about it
      if(mass(ksq(6,6))**2.0d0.lt.2.0d0*(mx+5.0d0)**2.0d0) then
           g_q(6) = 0.0d0
      endif   
     

c comment about it

c------------------ g_b -----------------------------------------

      T_sq = 0.0d0
      do r=1,2
        do i=1,6
          do a=1,6
            T_sq = T_sq + 1.0d0/mass(kh(r))/mass(kh(r))/mass(ksq(i,a))
     &        /mass(ksq(i,a))*gr(kh(r),kn(kln),kn(kln))*g_sfsfh(i,a,r)
          enddo
        enddo
      enddo 
      T_sq = T_sq/96.0d0/2.0d0/pi
      
c
      do i=1,6
        B_d_i(i) = 0.0d0
        B_s_i(i) = 0.0d0
        B_1d_i(i) = 0.0d0
        B_1s_i(i) = 0.0d0
        B_2s_i(i) = 0.0d0

c---- first option: passing masses ---------
c        do a=1,6
c          B_d_i(i) = d_sf_pls(i,a)
c     &          *I1(mass(ksq(i,a))**2.0d0,mass(Kq(i))**2.0d0,mx**2.0d0)
c          B_s_i(i) = c_sf_pls(i,a)
c     &          *I2(mass(ksq(i,a))**2.0d0,mass(Kq(i))**2.0d0,mx**2.0d0)     
c          B_1d_i(i) = d_sf_pls(i,a)*mass(Kq(i))
c     &          *I3(mass(ksq(i,a))**2.0d0,mass(Kq(i))**2.0d0,mx**2.0d0)
c          B_1s_i(i) = c_sf_pls(i,a)
c     &          *I4(mass(ksq(i,a))**2.0d0,mass(Kq(i))**2.0d0,mx**2.0d0)
c          B_2s_i(i) = c_sf_pls(i,a)
c     &          *I5(mass(ksq(i,a))**2.0d0,mass(Kq(i))**2.0d0,mx**2.0d0)
c         enddo
c----- second option: passing labels --------

        do a=1,6
          B_d_i(i) = B_d_i(i)+d_sf_pls(i,a)*I1(ksq(i,a),Kq(i))
          B_s_i(i) = B_s_i(i)+c_sf_pls(i,a)*I2(ksq(i,a),Kq(i))
          B_1d_i(i) = B_1d_i(i)+d_sf_pls(i,a)*I3(ksq(i,a),Kq(i))
          B_1s_i(i) = B_1s_i(i)+c_sf_pls(i,a)*I4(ksq(i,a),Kq(i))
          B_2s_i(i) = B_2s_i(i)+c_sf_pls(i,a)*I5(ksq(i,a),Kq(i))
        enddo

c---------------------------------------------

        B_d_i(i) = B_d_i(i)*mass(Kq(i))/2.0d0/32.0d0/pi
        B_s_i(i) = B_s_i(i)*mx/2.0d0/32.0d0/pi         
        B_1d_i(i) = B_1d_i(i)*mass(Kq(i))/2.0d0/12.0d0/pi  
        B_1s_i(i) = B_1s_i(i)*mx/2.0d0/12.0d0/pi
        B_2s_i(i) = B_2s_i(i)/2.0d0/48.0d0/pi
      enddo
c
      if(mass(ksq(6,6))**2.0d0.gt.2.0d0*(mx+5)**2.0d0) then 
        B_1d_t2g = B_1d_i(3)
        B_1s_t2g = B_1s_i(3)    
        B_2s_t2g = B_2s_i(3)
      else 
        B_1d_t2g = B_1d_i(3) + B_1d_i(6)
        B_1s_t2g = B_1s_i(3) + B_1s_i(6)   
        B_2s_t2g = B_2s_i(3)+ B_2s_i(6)
      endif
    
c
      B_d = B_d_i(2) + B_d_i(3) + B_d_i(6) 
      B_s = B_s_i(2) + B_s_i(3) + B_s_i(6)
      B_1d = B_1d_i(2) + B_1d_i(3) + B_1d_i(6)
      B_1s = B_1s_i(2) + B_1s_i(3) + B_1s_i(6)
      B_2s = B_2s_i(2) + B_2s_i(3) + B_2s_i(6) !just heavy quarks
c      
      g_b = - T_sq + B_d + B_s - mx/2.0d0*B_2s
     &                                -mx**2.0d0/4*(B_1d+B_1s)

c

! treatment of twist-2 like interactions

      msq=0.d0

      do a=1,6 
         msq=msq+mass(ksqu(a))+mass(ksqd(a)) 
      enddo

      msq=(msq/12.d0)**2.d0


      mu0=sqrt(abs(msq-mx**2.d0))



c      5, 4, 3, 2, 1, 0, -1, ......, -5)
c     (b, c, s, d, u, g, u_bar, ..., b_bar),
c
      
      sum_gq_p = g_q(1)*(qN(1,mu0)+qN(-1,mu0))+2.d0*g_q(2)*qN(4,mu0)
     &      +g_q(4)*(qN(2,mu0)+qN(-2,mu0))+2.d0*g_q(5)*qN(3,mu0)
     &      +2.d0*g_q(6)*qN(5,mu0)
 
c the neutron case is obtained by interchanging up with down quarks 
c and interchanging anti-up with anti-down antiquarks

      sum_gq_n = g_q(1)*(qN(4,mu0)+qN(-4,mu0))+2.d0*g_q(2)*qN(4,mu0)
     &      +g_q(4)*(qN(1,mu0)+qN(-1,mu0))+2.d0*g_q(5)*qN(3,mu0)
     &      +2.d0*g_q(6)*qN(5,mu0)
      

c      print*,sum_gq_p,sum_gq_n


c----------- I leave neutron case for later. Let's consider proton
c and neutron as nucleon with different third comp. of isospin --

      f_TG_p = 1.0d0-(ftp(7)+ftp(8)+ftp(10))
      f_TG_n = 1.0d0-(ftn(7)+ftn(8)+ftn(10))


c
      fp_mp = ftp(7)/mass(kq(1))*(f_q(1)-mass(kq(1))*mx*g_q(1)/2.0d0)
     &      + ftp(8)/mass(kq(4))*(f_q(4)-mass(kq(4))*mx*g_q(4)/2.0d0)
     &      + ftp(10)/mass(kq(5))*(f_q(5)-mass(kq(5))*mx*g_q(5)/2.0d0)
     &      + 2.0d0/27.0d0*f_TG_p*(f_q_h(2)/mass(kq(2))
     &                   +f_q_h(6)/mass(kq(6))+f_q_h(3)/mass(kq(3)))
     &      -3.0d0/2.0d0*mx*sum_gq_p
     &      -8.0d0*pi/9.0d0*g_b*f_TG_p
     &      +3.0d0/2.0d0*alph3mz*mx !alpha_s_mu0
     &            *(B_2s_t2g+mx/2.0d0*(B_1d_t2g+B_1s_t2g))*qN(0,mu0) !G_2

c  
      fp_mn = ftn(7)/mass(kq(1))*(f_q(1)-mass(kq(1))*mx*g_q(1)/2.0d0)
     &      + ftn(8)/mass(kq(4))*(f_q(4)-mass(kq(4))*mx*g_q(4)/2.0d0)
     &      + ftn(10)/mass(kq(5))*(f_q(5)-mass(kq(5))*mx*g_q(5)/2.0d0)
     &      + 2.0d0/27.0d0*f_TG_n*(f_q_h(2)/mass(kq(2))
     &                   + f_q_h(6)/mass(kq(6))+ f_q_h(3)/mass(kq(3)))
     &      -3.0d0/2.0d0*mx*sum_gq_n
     &      -8.0d0*pi/9.0d0*g_b*f_TG_n
     &      +3.0d0/2.0d0*alph3mz*mx
     &            *(B_2s_t2g+mx/2.0d0*(B_1d_t2g+B_1s_t2g))*qN(0,mu0) !G_2      



c----------- The formula is given in Baer et. al. (hep-ph/9706509)
c and parameters used are defined in Drees et. al. (Phys.Rev)---   

c

      fp = fp_mp*mp
      fn = fp_mn*mn
 
c-------- spin-ind. neutralino-nucleon cs --------------------------

      fkin_p = 4.0d0/pi*(mp*mx/(mp+mx))**2.0d0
      fkin_n = 4.0d0/pi*(mn*mx/(mn+mx))**2.0d0

      sigsip = fkin_p*fp**2.0d0  ! GeV-2 to pb
      sigsin = fkin_n*fn**2.0d0 !(197.327053d-16)**2   
  
     
      sigsip = sigsip*gev2cm2
      sigsin = sigsin*gev2cm2


c --------- spin-dep. neutralino-nucleon cs ------------------------ 

      du = 0.5*gr(kz,kn(kln),kn(kln))*(gl(kz,ku,ku)-gr(kz,ku,ku))/
     &     mass(kz)**2
      dd = 0.5*gr(kz,kn(kln),kn(kln))*(gl(kz,kd,kd)-gr(kz,kd,kd))/
     &     mass(kz)**2
      ds = 0.5*gr(kz,kn(kln),kn(kln))*(gl(kz,ks,ks)-gr(kz,ks,ks))/
     &     mass(kz)**2
      do i=1,6
        du = du
     &     + 0.125*(dsabsq(gl(ksqu(i),kn(kln),ku))+
     &              dsabsq(gr(ksqu(i),kn(kln),ku)))/
     &      (mass(ksqu(i))**2)
c     &     (mass(ksqu(i))**2-(mx+mass(ku))**2)
        dd = dd
     &     + 0.125*(dsabsq(gl(ksqd(i),kn(kln),kd))+
     &              dsabsq(gr(ksqd(i),kn(kln),kd)))/
     &      (mass(ksqd(i))**2)
c     &     (mass(ksqd(i))**2-(mx+mass(kd))**2)
        ds = ds
     &     + 0.125*(dsabsq(gl(ksqd(i),kn(kln),ks))+
     &              dsabsq(gr(ksqd(i),kn(kln),ks)))/
     &      (mass(ksqu(i))**2)
c     &     (mass(ksqu(i))**2-(mx+mass(ks))**2)
      enddo

      sigsdp = fkin_p*(3./4.)*(du*delu+dd*deld+ds*dels)**2 ! in gev^-2
      sigsdn = fkin_n*(3./4.)*(du*deld+dd*delu+ds*dels)**2 ! in gev^-2

      sigsdp=sigsdp*gev2cm2
      sigsdn=sigsdn*gev2cm2


      return
      end
