**************************************************************
*** SUBROUTINE dsaschizero                                 ***
*** computes dW_{ij}/dcostheta                             ***
*** sfermion(i) + neutralino(j) -> gamma/gluon + fermion   ***
*** ampl2 obtained with sum over physical polarizations    ***
***                                                        *** 
*** input askin variables: p12,costheta                    ***
*** AUTHOR: Piero Ullio, ullio@sissa.it                    ***
*** Date: 02-06-13                                         ***
**************************************************************

      SUBROUTINE dsaschizero(kp1,kp2,kp3,kp4,result)
      implicit none
      include 'dssusy.h'
      include 'dsascom.h'
      include 'dsidtag.h'
      integer kp1,kp2,kp3,kp4,kgb,kfer,kfers,ksfert,ksfer1,
     &  ksfer2,kfercheck,kchiu,iaux1,iaux2
      character*3 type
      real*8 result,colfact
      double complex dsasdepro,ampl2,den
      double complex A0L,A0R,A1L,A1R,A2L1,A2R1,A2L2,A2R2,
     & A2L3,A2R3,A0Ls,A0Rs,A1Ls,A1Rs,A2L1s,A2R1s,A2L2s,A2R2s,
     & A2L3s,A2R3s
      real*8 e1p1,e1p2,k1p1,k1p2,p1p2,mass2b,mass4b
      integer i
*****
c....check if the final state is kinematically allowed
      if((Svar-(mass(kp3)+mass(kp4))**2)/Svar.lt.thstep) then
        result=0.d0
        return
      endif      
***** check initial state is ok   
      if(iifam(1).eq.0) then   
        write(*,*) 'dsaschizero called with particles'   
        write(*,*) 'in the wrong order:'  
        write(*,*) pname(kp1),pname(kp2)  
        stop  
      endif    
*****
***** set type and check kp2:
      if(kp2.ge.kn(1).and.kp2.le.kn(4)) then
        type='neu' 
      elseif(kp2.ge.kcha(1).and.kp2.le.kcha(2)) then     
        type='cha' 
      else
        write(*,*) 'dsaschizero called with kp2 ='   
        write(*,*) pname(kp2)  
        write(*,*) 'instead of a neutralino or a chargino'
        stop
      endif    
*****
***** set particles in final state and check kp3:
      kgb=kp3
      kfer=kp4
      if(kgb.ne.kgamma.and.kgb.ne.kgluon) then
        write(*,*) 'dsaschizero called with kp3 ='   
        write(*,*) pname(kgb)  
        write(*,*) 'instead of a gauge boson'
        stop
      endif 
*****
      call dsasfercode('same',iifam(1),kfercheck) 
      iaux1=iifam(1)/10
      iaux2=iifam(1)-iaux1*10
*****
      colfact=1.d0
      if(iifam(1).ge.ivfam(ksu1).and.iifam(1).le.ivfam(ksb2))
     &  colfact=3.d0 
*****
***** non equal particle final state: 
      s34=1.d0
***** masses in final state:
      mass3=mass(kgb)
      mass4=mass(kfer)
***** define the kinematic variables
      call dsaskinset2
*****
      p1p2=(Svar-(mass1**2+mass2**2))/2.d0
      k1p1=ep1*ek3-p12*k34*costheta
      k1p2=ep2*ek3+p12*k34*costheta
      mass2b=mass2
      mass4b=mass4
      ampl2=(0.d0,0.d0)
*****
*****
***** sum over two polarizations
      do i=1,2
      if(i.eq.1) then   
***** first polarization:
        e1p1=p12*dsqrt(1.d0-costheta**2)
        e1p2=-p12*dsqrt(1.d0-costheta**2)
***** second polarization:
      elseif(i.eq.2) then  
        e1p1=0.d0
        e1p2=0.d0
      else
        write(*,*) 'wrong polarization index'
        stop
      endif  
*****
***** initialization  
      A0L=(0.d0,0.d0)
      A0R=(0.d0,0.d0)
      A1L=(0.d0,0.d0)
      A1R=(0.d0,0.d0)
      A2L1=(0.d0,0.d0)
      A2R1=(0.d0,0.d0)
      A2L2=(0.d0,0.d0)
      A2R2=(0.d0,0.d0)
      A2L3=(0.d0,0.d0)
      A2R3=(0.d0,0.d0)
*****
*****
***** 
***** the sixth and seventh cases    
***** sfermion(i) + neutralino(j) -> photon + fermion
***** (not allowed for neutrinos)
*****
      if(type.eq.'neu'.and.kgb.eq.kgamma.and.
     &  kfercheck.eq.kfer.and.dabs(echarg(kfer)).gt.1.d-16) then 
*****
***** f in s-channel ******************************************
        kfers=kfer
        den=dsasdepro(Svar,kfers)
        A2L1=A2L1-den
     &         *gr(kgb,kfer,kfers)*gl(kp1,kfers,kp2)
        A2R1=A2R1-den
     &         *gl(kgb,kfer,kfers)*gr(kp1,kfers,kp2)
        A2L2=A2L2-den
     &         *gr(kgb,kfer,kfers)*gl(kp1,kfers,kp2)
        A2R2=A2R2-den
     &         *gl(kgb,kfer,kfers)*gr(kp1,kfers,kp2)
        A1L=A1L-den
     &      *(mass(kfers)*gl(kgb,kfer,kfers)*gl(kp1,kfers,kp2))
        A1R=A1R-den
     &      *(mass(kfers)*gr(kgb,kfer,kfers)*gr(kp1,kfers,kp2))
*****
        call dsassfercode('same',iifam(1),ksfer1,ksfer2)
***** ksfer1 in t-channel *************************************
        ksfert=ksfer1
        den=dsasdepro(Tvar,ksfert)
        A0L=A0L-e1p1*den
     &      *2.d0*gl(ksfert,kfer,kp2)*gl(kgb,ksfert,kp1) 
        A0R=A0R-e1p1*den
     &      *2.d0*gr(ksfert,kfer,kp2)*gl(kgb,ksfert,kp1) 
***** ksfer2 in t-channel *************************************
        ksfert=ksfer2
        den=dsasdepro(Tvar,ksfert)
        A0L=A0L-e1p1*den
     &      *2.d0*gl(ksfert,kfer,kp2)*gl(kgb,ksfert,kp1) 
        A0R=A0R-e1p1*den
     &      *2.d0*gr(ksfert,kfer,kp2)*gl(kgb,ksfert,kp1) 
        goto 100
      endif  
*****
*****
*****
***** the nineth case
***** squark(i) + neutralino(j) 
***** -> gluon + quark
      if(type.eq.'neu'.and.kgb.eq.kgluon.and.
     &   iifam(1).ge.ivfam(ksu1).and.iifam(1).le.ivfam(ksb2).and.
     &   kfercheck.eq.kfer) then
        colfact=4.d0
*****
***** f in s-channel ******************************************
        kfers=kfer
        den=dsasdepro(Svar,kfers)
        A2L1=A2L1-den
     &         *gr(kgb,kfer,kfers)*gl(kp1,kfers,kp2)
        A2R1=A2R1-den
     &         *gl(kgb,kfer,kfers)*gr(kp1,kfers,kp2)
        A2L2=A2L2-den
     &         *gr(kgb,kfer,kfers)*gl(kp1,kfers,kp2)
        A2R2=A2R2-den
     &         *gl(kgb,kfer,kfers)*gr(kp1,kfers,kp2)
        A1L=A1L-den
     &      *(mass(kfers)*gl(kgb,kfer,kfers)*gl(kp1,kfers,kp2))
        A1R=A1R-den
     &      *(mass(kfers)*gr(kgb,kfer,kfers)*gr(kp1,kfers,kp2))
*****
        call dsassfercode('same',iifam(1),ksfer1,ksfer2)
***** ksfer1 in t-channel *************************************
        ksfert=ksfer1
        den=dsasdepro(Tvar,ksfert)
        A0L=A0L-e1p1*den
     &      *2.d0*gl(ksfert,kfer,kp2)*gl(kgb,ksfert,kp1) 
        A0R=A0R-e1p1*den
     &      *2.d0*gr(ksfert,kfer,kp2)*gl(kgb,ksfert,kp1) 
***** ksfer2 in t-channel *************************************
        ksfert=ksfer2
        den=dsasdepro(Tvar,ksfert)
        A0L=A0L-e1p1*den
     &      *2.d0*gl(ksfert,kfer,kp2)*gl(kgb,ksfert,kp1) 
        A0R=A0R-e1p1*den
     &      *2.d0*gr(ksfert,kfer,kp2)*gl(kgb,ksfert,kp1) 
        goto 100
      endif  
*****
*****
***** 
***** the eighth case    
***** down-type-sfermion(i) + chargino^+(j) 
***** -> photon + up-type-fermion
      if(type.eq.'cha'.and.kgb.eq.kgamma.and.iaux2.eq.2.and.
     &       kfercheck.ne.kfer) then 
*****
***** f in s-channel ******************************************
        if(dabs(echarg(kfer)).gt.1.d-16) then
        kfers=kfer
        den=dsasdepro(Svar,kfers)
        A2L1=A2L1-den
     &         *gr(kgb,kfer,kfers)*gl(kp1,kfers,kp2)
        A2R1=A2R1-den
     &         *gl(kgb,kfer,kfers)*gr(kp1,kfers,kp2)
        A2L2=A2L2-den
     &         *gr(kgb,kfer,kfers)*gl(kp1,kfers,kp2)
        A2R2=A2R2-den
     &         *gl(kgb,kfer,kfers)*gr(kp1,kfers,kp2)
        A1L=A1L-den
     &      *(mass(kfers)*gl(kgb,kfer,kfers)*gl(kp1,kfers,kp2))
        A1R=A1R-den
     &      *(mass(kfers)*gr(kgb,kfer,kfers)*gr(kp1,kfers,kp2))
        endif 
*****
        call dsassfercode('same',iifam(1),ksfer1,ksfer2)
***** ksfer1 in t-channel *************************************
        ksfert=ksfer1
        den=dsasdepro(Tvar,ksfert)
        A0L=A0L-e1p1*den
     &      *2.d0*gl(ksfert,kfer,kp2)*gl(kgb,ksfert,kp1) 
        A0R=A0R-e1p1*den
     &      *2.d0*gr(ksfert,kfer,kp2)*gl(kgb,ksfert,kp1) 
***** ksfer2 in t-channel *************************************
        ksfert=ksfer2
        den=dsasdepro(Tvar,ksfert)
        A0L=A0L-e1p1*den
     &      *2.d0*gl(ksfert,kfer,kp2)*gl(kgb,ksfert,kp1) 
        A0R=A0R-e1p1*den
     &      *2.d0*gr(ksfert,kfer,kp2)*gl(kgb,ksfert,kp1) 
*****
***** kcha(1) in u-channel ************************************
        kchiu=kcha1
        den=dsasdepro(Uvar,kchiu)
        A0L=A0L-2.d0*e1p2*den*gl(kp1,kfer,kchiu)*gl(kgb,kchiu,kp2)
        A0R=A0R-2.d0*e1p2*den*gr(kp1,kfer,kchiu)*gr(kgb,kchiu,kp2)
        A1L=A1L-den*gr(kp1,kfer,kchiu)*gl(kgb,kchiu,kp2)
     &       *mass(kchiu)
        A1R=A1R-den*gl(kp1,kfer,kchiu)*gr(kgb,kchiu,kp2)
     &       *mass(kchiu)
        A2L2=A2L2+den*gl(kp1,kfer,kchiu)*gl(kgb,kchiu,kp2)
        A2R2=A2R2+den*gr(kp1,kfer,kchiu)*gr(kgb,kchiu,kp2)
        A2L3=A2L3-den*gl(kp1,kfer,kchiu)*gl(kgb,kchiu,kp2)
        A2R3=A2R3-den*gr(kp1,kfer,kchiu)*gr(kgb,kchiu,kp2)
***** kcha(2) in u-channel ************************************
        kchiu=kcha2
        den=dsasdepro(Uvar,kchiu)
        A0L=A0L-2.d0*e1p2*den*gl(kp1,kfer,kchiu)*gl(kgb,kchiu,kp2)
        A0R=A0R-2.d0*e1p2*den*gr(kp1,kfer,kchiu)*gr(kgb,kchiu,kp2)
        A1L=A1L-den*gr(kp1,kfer,kchiu)*gl(kgb,kchiu,kp2)
     &       *mass(kchiu)
        A1R=A1R-den*gl(kp1,kfer,kchiu)*gr(kgb,kchiu,kp2)
     &       *mass(kchiu)
        A2L2=A2L2+den*gl(kp1,kfer,kchiu)*gl(kgb,kchiu,kp2)
        A2R2=A2R2+den*gr(kp1,kfer,kchiu)*gr(kgb,kchiu,kp2)
        A2L3=A2L3-den*gl(kp1,kfer,kchiu)*gl(kgb,kchiu,kp2)
        A2R3=A2R3-den*gr(kp1,kfer,kchiu)*gr(kgb,kchiu,kp2)
*****
        goto 100
      endif
*****
*****
*****
***** the tenth case    
***** down-type-squark(i) + chargino^+(j) 
***** -> gluon + up-type-quark
      if(type.eq.'cha'.and.kgb.eq.kgluon.and.
     &   iifam(1).ge.ivfam(ksu1).and.iifam(1).le.ivfam(ksb2).and.
     &   iaux2.eq.2.and.kfercheck.ne.kfer) then 
***** set the color factors
        colfact=4.d0
*****
***** f in s-channel ******************************************
        kfers=kfer
        den=dsasdepro(Svar,kfers)
        A2L1=A2L1-den
     &         *gr(kgb,kfer,kfers)*gl(kp1,kfers,kp2)
        A2R1=A2R1-den
     &         *gl(kgb,kfer,kfers)*gr(kp1,kfers,kp2)
        A2L2=A2L2-den
     &         *gr(kgb,kfer,kfers)*gl(kp1,kfers,kp2)
        A2R2=A2R2-den
     &         *gl(kgb,kfer,kfers)*gr(kp1,kfers,kp2)
        A1L=A1L-den
     &      *(mass(kfers)*gl(kgb,kfer,kfers)*gl(kp1,kfers,kp2))
        A1R=A1R-den
     &      *(mass(kfers)*gr(kgb,kfer,kfers)*gr(kp1,kfers,kp2))
*****
        call dsassfercode('same',iifam(1),ksfer1,ksfer2)
***** ksfer1 in t-channel *************************************
        ksfert=ksfer1
        den=dsasdepro(Tvar,ksfert)
        A0L=A0L-e1p1*den
     &      *2.d0*gl(ksfert,kfer,kp2)*gl(kgb,ksfert,kp1) 
        A0R=A0R-e1p1*den
     &      *2.d0*gr(ksfert,kfer,kp2)*gl(kgb,ksfert,kp1) 
***** ksfer2 in t-channel *************************************
        ksfert=ksfer2
        den=dsasdepro(Tvar,ksfert)
        A0L=A0L-e1p1*den
     &      *2.d0*gl(ksfert,kfer,kp2)*gl(kgb,ksfert,kp1) 
        A0R=A0R-e1p1*den
     &      *2.d0*gr(ksfert,kfer,kp2)*gl(kgb,ksfert,kp1) 
        goto 100
      endif    
*****
*****
*****
***** the fifth case 
***** anti-up-type-sfermion(i) + \chi^+(j) 
***** -> photon + anti-down-type-fermion
      if(type.eq.'cha'.and.kgb.eq.kgamma.and.iaux2.eq.1.and.
     &   kfercheck.ne.kfer) then
*****
***** f in s-channel ******************************************
        kfers=kfer
        den=dsasdepro(Svar,kfers)
        A2L1=A2L1+den
     &         *gl(kp2,kfers,kp1)*gl(kgb,kfers,kfer)
        A2R1=A2R1+den
     &         *gr(kp2,kfers,kp1)*gr(kgb,kfers,kfer)
        A2L2=A2L2+den
     &         *gl(kp2,kfers,kp1)*gl(kgb,kfers,kfer)
        A2R2=A2R2+den
     &         *gr(kp2,kfers,kp1)*gr(kgb,kfers,kfer)
        A1L=A1L+den
     &      *(mass(kfers)*gl(kgb,kfer,kfers)*gr(kp1,kfers,kp2))
        A1R=A1R+den
     &      *(mass(kfers)*gr(kgb,kfer,kfers)*gl(kp1,kfers,kp2))
*****
        call dsassfercode('same',iifam(1),ksfer1,ksfer2)
        if(ksfer2.ne.0) then
***** ksfer1 in t-channel *************************************
          ksfert=ksfer1
          den=dsasdepro(Tvar,ksfert)
          A0L=A0L+e1p1*den
     &      *2.d0*gl(kp2,kfer,ksfert)*gl(kgb,kp1,ksfert) 
          A0R=A0R+e1p1*den
     &      *2.d0*gr(kp2,kfer,ksfert)*gl(kgb,kp1,ksfert) 
***** ksfer2 in t-channel *************************************
          ksfert=ksfer2
          den=dsasdepro(Tvar,ksfert)
          A0L=A0L+e1p1*den
     &      *2.d0*gl(kp2,kfer,ksfert)*gl(kgb,kp1,ksfert) 
          A0R=A0R+e1p1*den
     &      *2.d0*gr(kp2,kfer,ksfert)*gl(kgb,kp1,ksfert) 
        endif
*****
***** kcha(1) in u-channel ************************************
        kchiu=kcha1
        den=dsasdepro(Uvar,kchiu)
        A0L=A0L-2.d0*e1p2*den*gl(kchiu,kfer,kp1)*gl(kgb,kchiu,kp2)
        A0R=A0R-2.d0*e1p2*den*gr(kchiu,kfer,kp1)*gr(kgb,kchiu,kp2)
        A1L=A1L-den*gr(kchiu,kfer,kp1)*gl(kgb,kchiu,kp2)
     &       *mass(kchiu)
        A1R=A1R-den*gl(kchiu,kfer,kp1)*gr(kgb,kchiu,kp2)
     &       *mass(kchiu)
        A2L2=A2L2+den*gl(kchiu,kfer,kp1)*gl(kgb,kchiu,kp2)
        A2R2=A2R2+den*gr(kchiu,kfer,kp1)*gr(kgb,kchiu,kp2)
        A2L3=A2L3-den*gl(kchiu,kfer,kp1)*gl(kgb,kchiu,kp2)
        A2R3=A2R3-den*gr(kchiu,kfer,kp1)*gr(kgb,kchiu,kp2)
***** kcha(2) in u-channel ************************************
        kchiu=kcha2
        den=dsasdepro(Uvar,kchiu)
        A0L=A0L-2.d0*e1p2*den*gl(kchiu,kfer,kp1)*gl(kgb,kchiu,kp2)
        A0R=A0R-2.d0*e1p2*den*gr(kchiu,kfer,kp1)*gr(kgb,kchiu,kp2)
        A1L=A1L-den*gr(kchiu,kfer,kp1)*gl(kgb,kchiu,kp2)
     &       *mass(kchiu)
        A1R=A1R-den*gl(kchiu,kfer,kp1)*gr(kgb,kchiu,kp2)
     &       *mass(kchiu)
        A2L2=A2L2+den*gl(kchiu,kfer,kp1)*gl(kgb,kchiu,kp2)
        A2R2=A2R2+den*gr(kchiu,kfer,kp1)*gr(kgb,kchiu,kp2)
        A2L3=A2L3-den*gl(kchiu,kfer,kp1)*gl(kgb,kchiu,kp2)
        A2R3=A2R3-den*gr(kchiu,kfer,kp1)*gr(kgb,kchiu,kp2)
        goto 100
      endif
*****
*****
*****
***** the sixth case 
***** anti-up-type-squark(i) + \chi^+(j) 
***** -> gluon + anti-down-type-quark
      if(type.eq.'cha'.and.kgb.eq.kgluon.and.
     &   iifam(1).ge.ivfam(ksu1).and.iifam(1).le.ivfam(ksb2).and.     
     &   iaux2.eq.1.and.kfercheck.ne.kfer) then
***** set the color factors
        colfact=4.d0
*****
***** f in s-channel ******************************************
        kfers=kfer
        den=dsasdepro(Svar,kfers)
        A2L1=A2L1+den
     &         *gl(kp2,kfers,kp1)*gl(kgb,kfers,kfer)
        A2R1=A2R1+den
     &         *gr(kp2,kfers,kp1)*gr(kgb,kfers,kfer)
        A2L2=A2L2+den
     &         *gl(kp2,kfers,kp1)*gl(kgb,kfers,kfer)
        A2R2=A2R2+den
     &         *gr(kp2,kfers,kp1)*gr(kgb,kfers,kfer)
        A1L=A1L+den
     &      *(mass(kfers)*gl(kgb,kfer,kfers)*gr(kp1,kfers,kp2))
        A1R=A1R+den
     &      *(mass(kfers)*gr(kgb,kfer,kfers)*gl(kp1,kfers,kp2))
*****
        call dsassfercode('same',iifam(1),ksfer1,ksfer2)
***** ksfer1 in t-channel *************************************
        ksfert=ksfer1
        den=dsasdepro(Tvar,ksfert)
        A0L=A0L+e1p1*den
     &      *2.d0*gl(kp2,kfer,ksfert)*gl(kgb,kp1,ksfert) 
        A0R=A0R+e1p1*den
     &      *2.d0*gr(kp2,kfer,ksfert)*gl(kgb,kp1,ksfert) 
***** ksfer2 in t-channel *************************************
        ksfert=ksfer2
        den=dsasdepro(Tvar,ksfert)
        A0L=A0L+e1p1*den
     &      *2.d0*gl(kp2,kfer,ksfert)*gl(kgb,kp1,ksfert) 
        A0R=A0R+e1p1*den
     &      *2.d0*gr(kp2,kfer,ksfert)*gl(kgb,kp1,ksfert) 
        goto 100
      endif
*****
*****
*****


 100  continue

      A0Ls=dconjg(A0L)
      A0Rs=dconjg(A0R)
      A1Ls=dconjg(A1L)
      A1Rs=dconjg(A1R)
      A2L1s=dconjg(A2L1)
      A2L2s=dconjg(A2L2)
      A2L3s=dconjg(A2L3)
      A2R1s=dconjg(A2R1)
      A2R2s=dconjg(A2R2)
      A2R3s=dconjg(A2R3)



*****
***** amplitude squared *************************************** 
*****

      ampl2=ampl2
     &  - 2.d0*mass2b*mass4b*mass1**2*A2L1*A2R1s
     &  - 2.d0*mass2b*mass4b*mass1**2*A2L1s*A2R1
     &  - 2.d0*mass2b*mass4b*A1L*A1Rs
     &  - 2.d0*mass2b*mass4b*A1R*A1Ls
     &  + 2.d0*mass2b*mass4b*A0L*A0Rs
     &  + 2.d0*mass2b*mass4b*A0L*e1p1*A2R1s
     &  - 2.d0*mass2b*mass4b*A0L*e1p1*A2R2s
     &  + 2.d0*mass2b*mass4b*A0R*A0Ls
     &  + 2.d0*mass2b*mass4b*A0R*e1p1*A2L1s
     &  - 2.d0*mass2b*mass4b*A0R*e1p1*A2L2s
     &  + 2.d0*mass2b*mass4b*A0Ls*e1p1*A2R1
     &  - 2.d0*mass2b*mass4b*A0Ls*e1p1*A2R2
     &  + 2.d0*mass2b*mass4b*A0Rs*e1p1*A2L1
     &  - 2.d0*mass2b*mass4b*A0Rs*e1p1*A2L2
     &  - 2.d0*mass2b*mass4b*p1p2*A2L1*A2R2s
     &  - 2.d0*mass2b*mass4b*p1p2*A2L1s*A2R2
     &  - 2.d0*mass2b*mass4b*p1p2*A2L2*A2R1s
     &  - 2.d0*mass2b*mass4b*p1p2*A2L2s*A2R1
     &  - 2.d0*mass2b*mass4b*k1p1*A2L1*A2R3s
     &  - 2.d0*mass2b*mass4b*k1p1*A2L1s*A2R3
     &  - 2.d0*mass2b*mass4b*k1p1*A2L3*A2R1s
     &  - 2.d0*mass2b*mass4b*k1p1*A2L3s*A2R1
     &  - 2.d0*mass2b*mass4b*k1p2*A2L2*A2R3s
     &  - 2.d0*mass2b*mass4b*k1p2*A2L2s*A2R3
     &  - 2.d0*mass2b*mass4b*k1p2*A2L3*A2R2s
     &  - 2.d0*mass2b*mass4b*k1p2*A2L3s*A2R2
     &  + 2.d0*mass2b*mass1**2*A1L*A2R1s
     &  + 2.d0*mass2b*mass1**2*A1R*A2L1s
     &  + 2.d0*mass2b*mass1**2*A1Ls*A2R1
     &  + 2.d0*mass2b*mass1**2*A1Rs*A2L1
     &  + 2.d0*mass2b*A1L*p1p2*A2R1s
     &  + 2.d0*mass2b*A1L*p1p2*A2R2s
     &  - 2.d0*mass2b*A1L*k1p1*A2R1s
     &  + 2.d0*mass2b*A1L*k1p1*A2R3s
     &  - 2.d0*mass2b*A1L*k1p2*A2R2s
     &  + 2.d0*mass2b*A1L*k1p2*A2R3s
     &  + 2.d0*mass2b*A1R*p1p2*A2L1s
     &  + 2.d0*mass2b*A1R*p1p2*A2L2s
     &  - 2.d0*mass2b*A1R*k1p1*A2L1s
     &  + 2.d0*mass2b*A1R*k1p1*A2L3s
     &  - 2.d0*mass2b*A1R*k1p2*A2L2s
     &  + 2.d0*mass2b*A1R*k1p2*A2L3s
     &  + 2.d0*mass2b*A1Ls*p1p2*A2R1
     &  + 2.d0*mass2b*A1Ls*p1p2*A2R2
     &  - 2.d0*mass2b*A1Ls*k1p1*A2R1
     &  + 2.d0*mass2b*A1Ls*k1p1*A2R3
     &  - 2.d0*mass2b*A1Ls*k1p2*A2R2
     &  + 2.d0*mass2b*A1Ls*k1p2*A2R3
     &  + 2.d0*mass2b*A1Rs*p1p2*A2L1
     &  + 2.d0*mass2b*A1Rs*p1p2*A2L2
     &  - 2.d0*mass2b*A1Rs*k1p1*A2L1
     &  + 2.d0*mass2b*A1Rs*k1p1*A2L3
     &  - 2.d0*mass2b*A1Rs*k1p2*A2L2
     &  + 2.d0*mass2b*A1Rs*k1p2*A2L3
     &  - 2.d0*mass2b**2*mass4b*A1L*A2L2s
     &  - 2.d0*mass2b**2*mass4b*A1R*A2R2s
     &  - 2.d0*mass2b**2*mass4b*A1Ls*A2L2
     &  - 2.d0*mass2b**2*mass4b*A1Rs*A2R2
     &  - 2.d0*mass2b**2*mass1**2*A2L1*A2L1s
     &  + 2.d0*mass2b**2*mass1**2*A2L1*A2L2s
     &  + 2.d0*mass2b**2*mass1**2*A2L1s*A2L2
     &  - 2.d0*mass2b**2*mass1**2*A2R1*A2R1s
     &  + 2.d0*mass2b**2*mass1**2*A2R1*A2R2s
     &  + 2.d0*mass2b**2*mass1**2*A2R1s*A2R2
     &  + 2.d0*mass2b**2*A1L*A1Ls
     &  + 2.d0*mass2b**2*A1R*A1Rs
     &  + 2.d0*mass2b**2*A0L*A0Ls
     &  + 2.d0*mass2b**2*A0L*e1p1*A2L1s
     &  + 2.d0*mass2b**2*A0R*A0Rs
     &  + 2.d0*mass2b**2*A0R*e1p1*A2R1s
     &  + 2.d0*mass2b**2*A0Ls*e1p1*A2L1
     &  + 2.d0*mass2b**2*A0Rs*e1p1*A2R1
     &  + 2.d0*mass2b**2*p1p2*A2L1*A2L2s
     &  + 2.d0*mass2b**2*p1p2*A2L1s*A2L2
     &  + 2.d0*mass2b**2*p1p2*A2L2*A2L2s
     &  + 2.d0*mass2b**2*p1p2*A2R1*A2R2s
     &  + 2.d0*mass2b**2*p1p2*A2R1s*A2R2
     &  + 2.d0*mass2b**2*p1p2*A2R2*A2R2s
     &  - 2.d0*mass2b**2*k1p1*A2L1*A2L2s
     &  - 2.d0*mass2b**2*k1p1*A2L1*A2L3s
     &  - 2.d0*mass2b**2*k1p1*A2L1s*A2L2
     &  - 2.d0*mass2b**2*k1p1*A2L1s*A2L3
     &  + 2.d0*mass2b**2*k1p1*A2L2*A2L3s
     &  + 2.d0*mass2b**2*k1p1*A2L2s*A2L3
     &  - 2.d0*mass2b**2*k1p1*A2R1*A2R2s
     &  - 2.d0*mass2b**2*k1p1*A2R1*A2R3s
     &  - 2.d0*mass2b**2*k1p1*A2R1s*A2R2
     &  - 2.d0*mass2b**2*k1p1*A2R1s*A2R3
     &  + 2.d0*mass2b**2*k1p1*A2R2*A2R3s
     &  + 2.d0*mass2b**2*k1p1*A2R2s*A2R3
     &  - 2.d0*mass2b**2*k1p2*A2L2*A2L2s
     &  + 2.d0*mass2b**2*k1p2*A2L2*A2L3s
     &  + 2.d0*mass2b**2*k1p2*A2L2s*A2L3
     &  - 2.d0*mass2b**2*k1p2*A2R2*A2R2s
     &  + 2.d0*mass2b**2*k1p2*A2R2*A2R3s
     &  + 2.d0*mass2b**2*k1p2*A2R2s*A2R3
     &  - 2.d0*mass2b**3*mass4b*A2L2*A2R2s
     &  - 2.d0*mass2b**3*mass4b*A2L2s*A2R2
     &  + 2.d0*mass2b**3*A1L*A2R2s
     &  + 2.d0*mass2b**3*A1R*A2L2s
     &  + 2.d0*mass2b**3*A1Ls*A2R2
     &  + 2.d0*mass2b**3*A1Rs*A2L2
     &  + 2.d0*mass2b**4*A2L2*A2L2s
     &  + 2.d0*mass2b**4*A2R2*A2R2s
     &  - 2.d0*mass4b*A1L*A0Ls*e1p1
     &  - 2.d0*mass4b*A1L*p1p2*A2L1s
     &  - 2.d0*mass4b*A1L*k1p2*A2L3s
     &  - 2.d0*mass4b*A1R*A0Rs*e1p1
     &  - 2.d0*mass4b*A1R*p1p2*A2R1s
     &  - 2.d0*mass4b*A1R*k1p2*A2R3s
     &  - 2.d0*mass4b*A1Ls*A0L*e1p1
     &  - 2.d0*mass4b*A1Ls*p1p2*A2L1
     &  - 2.d0*mass4b*A1Ls*k1p2*A2L3
     &  - 2.d0*mass4b*A1Rs*A0R*e1p1
     &  - 2.d0*mass4b*A1Rs*p1p2*A2R1
     &  - 2.d0*mass4b*A1Rs*k1p2*A2R3
     &  + 2.d0*mass1**2*A0L*e1p1*A2L1s
     &  + 2.d0*mass1**2*A0R*e1p1*A2R1s
     &  + 2.d0*mass1**2*A0Ls*e1p1*A2L1
     &  + 2.d0*mass1**2*A0Rs*e1p1*A2R1
     &  + 2.d0*mass1**2*p1p2*A2L1*A2L1s
     &  + 2.d0*mass1**2*p1p2*A2R1*A2R1s
     &  + 2.d0*mass1**2*k1p2*A2L1*A2L1s
     &  + 2.d0*mass1**2*k1p2*A2L1*A2L3s
     &  + 2.d0*mass1**2*k1p2*A2L1s*A2L3
     &  + 2.d0*mass1**2*k1p2*A2R1*A2R1s
     &  + 2.d0*mass1**2*k1p2*A2R1*A2R3s
     &  + 2.d0*mass1**2*k1p2*A2R1s*A2R3
     &  + 2.d0*A1L*A1Ls*p1p2
     &  - 2.d0*A1L*A1Ls*k1p2
     &  + 2.d0*A1R*A1Rs*p1p2
     &  - 2.d0*A1R*A1Rs*k1p2
     &  + 2.d0*A0L*A0Ls*p1p2
     &  - 2.d0*A0L*A0Ls*k1p2
     &  + 4.d0*A0L*p1p2*e1p1*A2L1s
     &  - 2.d0*A0L*e1p1*k1p1*A2L1s
     &  + 2.d0*A0L*e1p1*k1p1*A2L3s
     &  - 2.d0*A0L*e1p1*k1p2*A2L1s
     &  + 2.d0*A0L*e1p1*k1p2*A2L3s
     &  + 2.d0*A0R*A0Rs*p1p2
     &  - 2.d0*A0R*A0Rs*k1p2
     &  + 4.d0*A0R*p1p2*e1p1*A2R1s
     &  - 2.d0*A0R*e1p1*k1p1*A2R1s
     &  + 2.d0*A0R*e1p1*k1p1*A2R3s
     &  - 2.d0*A0R*e1p1*k1p2*A2R1s
     &  + 2.d0*A0R*e1p1*k1p2*A2R3s
     &  + 4.d0*A0Ls*p1p2*e1p1*A2L1
     &  - 2.d0*A0Ls*e1p1*k1p1*A2L1
     &  + 2.d0*A0Ls*e1p1*k1p1*A2L3
     &  - 2.d0*A0Ls*e1p1*k1p2*A2L1
     &  + 2.d0*A0Ls*e1p1*k1p2*A2L3
     &  + 4.d0*A0Rs*p1p2*e1p1*A2R1
     &  - 2.d0*A0Rs*e1p1*k1p1*A2R1
     &  + 2.d0*A0Rs*e1p1*k1p1*A2R3
     &  - 2.d0*A0Rs*e1p1*k1p2*A2R1
     &  + 2.d0*A0Rs*e1p1*k1p2*A2R3
     &  - 4.d0*p1p2*k1p1*A2L1*A2L1s
     &  - 4.d0*p1p2*k1p1*A2R1*A2R1s
     &  + 4.d0*p1p2*k1p2*A2L1*A2L3s
     &  + 4.d0*p1p2*k1p2*A2L1s*A2L3
     &  + 4.d0*p1p2*k1p2*A2R1*A2R3s
     &  + 4.d0*p1p2*k1p2*A2R1s*A2R3
     &  + 4.d0*p1p2**2*A2L1*A2L1s
     &  + 4.d0*p1p2**2*A2R1*A2R1s
     &  + 4.d0*k1p1*k1p2*A2L3*A2L3s
     &  + 4.d0*k1p1*k1p2*A2R3*A2R3s
     &  + 4.d0*k1p2**2*A2L3*A2L3s
     &  + 4.d0*k1p2**2*A2R3*A2R3s
*****
      enddo
*****
      if (dble(ampl2).lt.0.0d0) then     
        write(*,*) ' '
        write(*,*) 
     &  'ERROR IN dsaschizero with negative cross section:'
        write(*,*) 'Model: ',idtag
        write(*,*) 'kp1=',kp1,'  ',pname(kp1)
        write(*,*) 'kp2=',kp2,'  ',pname(kp2)
        write(*,*) 'kp3=',kp3,'  ',pname(kp3)
        write(*,*) 'kp4=',kp4,'  ',pname(kp4)
        write(*,*) 'amplitude sqaured =',ampl2
        write(*,*) 'mass1=',mass1
        write(*,*) 'mass2=',mass2
        write(*,*) 'mass3=',mass3
        write(*,*) 'mass4=',mass4
        write(*,*) 's=',Svar
        write(*,*) 't=',Tvar
        write(*,*) 'u=',Uvar
        write(*,*) 'p12=',p12
        write(*,*) 'costheta=',costheta
        ampl2=dcmplx(0.0d0,0.0d0)
      endif
      result=dble(ampl2)*k34/(8.d0*pi*gg1*gg2*s34*dsqrt(Svar))
     &       *colfact
      return
      end
