************************************************************************
*** SUBROUTINE dsasdwdcossfchi                                       ***
*** computes dW_{ij}/dcostheta                                       ***
*** for sfermion(1) + neutralino(2) (or chargino(2))                 ***
*** plus sfermion(1) + neutralino(2) (or chargino(2))                ***
***                                                                  *** 
*** AUTHOR: Piero Ullio, ullio@sissa.it                              ***
*** Date: 01-11-04                                                   ***
*** Modified: Joakim Edsjo, Mia Schelke                              ***
***   to include gluon final states, 2002-03-21                      ***
*** Modified: Piero Ullio                                            ***
***   to switch to ampl2 with physical polarizations, 02-07-01       ***
************************************************************************

      real*8 function dsasdwdcossfchi(p,costhe,kp1,kp2)
      implicit none
      include 'dssusy.h'
      include 'dsascom.h'
      include 'dsandwcom.h'
      include 'dsidtag.h'
      real*8 p,costhe
      integer kp1,kp2,kp3,kp4,itmp,iaux,i
      integer kfers,kferd
      real*8 w,par
      logical wok
************************************************************************
      if (p.lt.0.0d0) then
         dsasdwdcossfchi=0.0d0
      return
      endif

***** tollerance on the position of a threshold
      thstep=1.d-15


************************************************************************
*****
***** set the askin variables
      p12=p
      costheta=costhe
***** masses in initial state:
      mass1=mass(kp1)
      mass2=mass(kp2)
***** kinematic variables:
      call dsaskinset1
***** family indices:
      iifam(1)=ivfam(kp1)
      iifam(2)=0
***** internal degrees of freedom of the initial particles:
***** note: here we do not include particle anti-particle degrees of
*****       freedom. Only spin and colour are included here.
*****       The particle anti-particle degrees of freedom are
*****       taken into account when w is summed below.
      gg1=1.d0
      if(iifam(1).ge.ivfam(ksu1).and.iifam(1).le.ivfam(ksb2)) then
        gg1=3.d0
      endif
      gg2=2.d0  ! both for neutralinos and charginos since particle
                ! antiparticle states are not included here.
***** 
*****    2 cases:
***** 1) sfermion + neutralino
      if(kp2.ge.kn1.and.kp2.le.kn4) then
        do i=1,54
          prtial(i)=0.0d0
        enddo
        call dsasfercode('same',iifam(1),kfers)
        call dsasfercode('diff',iifam(1),kferd)
        itmp=iifam(1)/10
        iaux=iifam(1)-itmp*10
******************************************************** sf chi^0 -> Z f
        kp3=kz
        kp4=kfers
        call dsaschicasea(kp1,kp2,kp3,kp4,par)
        prtial(1)=par
**************************************************** sf chi^0 -> gamma f
        kp3=kgamma
        kp4=kfers
        call dsaschizero(kp1,kp2,kp3,kp4,par)
        prtial(2)=par
******************************************************** sf chi^0 -> h f
        kp3=kh2
        kp4=kfers
        call dsaschicasec(kp1,kp2,kp3,kp4,par)
        prtial(3)=par
******************************************************** sf chi^0 -> H f
        kp3=kh1
        kp4=kfers
        call dsaschicasec(kp1,kp2,kp3,kp4,par)
        prtial(4)=par
******************************************************** sf chi^0 -> a f
        kp3=kh3
        kp4=kfers
        call dsaschicasec(kp1,kp2,kp3,kp4,par)
        prtial(5)=par
**************************************************** sf chi^0 -> gluon f
        if(iifam(1)/10.ne.ivfam(knue)/10
     &     .and.iifam(1)/10.ne.ivfam(knumu)/10
     &     .and.iifam(1)/10.ne.ivfam(knutau)/10) then
          kp3=kgluon
          kp4=kfers
          call dsaschizero(kp1,kp2,kp3,kp4,par)
          prtial(8)=par
        else
          prtial(8)=0.0d0
        endif
********
        if(iaux.eq.1) then
*********************************************** sfu^* chi^0 -> W^- fdbar
          kp3=kw
          kp4=kferd
          call dsaschicaseb(kp1,kp2,kp3,kp4,par)
          prtial(6)=par
*********************************************** sfu^* chi^0 -> h^- fdbar
          kp3=khc
          kp4=kferd
          call dsaschicased(kp1,kp2,kp3,kp4,par)
          prtial(7)=par
        elseif(iaux.eq.2) then  
**************************************************** sfd chi^0 -> W^- fu
          kp3=kw
          kp4=kferd
          call dsaschicasea(kp1,kp2,kp3,kp4,par)
          prtial(6)=par
**************************************************** sfd chi^0 -> h^- fu
          kp3=khc
          kp4=kferd
          call dsaschicasec(kp1,kp2,kp3,kp4,par)
          prtial(7)=par
        else
          write(*,*) 'wrong iaux value =',iaux
          write(*,*) 'in dsasdwdcossfchi. program stopped.'
          stop
        endif
**************************************************** sum partial results
        w=0.d0
        w=w+prtial(1)            ! sf chi^0 -> Z f
        w=w+prtial(2)            ! sf chi^0 -> gamma f
        w=w+prtial(3)            ! sf chi^0 -> h f
        w=w+prtial(4)            ! sf chi^0 -> H f
        w=w+prtial(5)            ! sf chi^0 -> a f
        w=w+prtial(6)            ! sfu^* chi^0 -> W^- fdbar
                                 ! or  sfd chi^0 -> W^- + fu
        w=w+prtial(7)            ! sfu^* chi^0 -> h^- fdbar
                                 ! or  sfd chi^0 -> h^- fu
        w=w+prtial(8)            ! sf chi^0 -> gluon f
        dsasdwdcossfchi = w      ! The weighting factor is 1

************************************************************************
*****
***** 2) sfermion + chargino
      elseif(kp2.ge.kcha1.and.kp2.le.kcha2) then
        do i=1,54
          prtial(i)=0.0d0
        enddo
        call dsasfercode('same',iifam(1),kfers)
        call dsasfercode('diff',iifam(1),kferd)
        itmp=iifam(1)/10
        iaux=iifam(1)-itmp*10
********
        if(iaux.eq.1) then
************************************************* sfu^* chi^+ -> Z fdbar
          kp3=kz
          kp4=kferd
          call dsaschicaseb(kp1,kp2,kp3,kp4,par)
          prtial(1)=par
********************************************* sfu^* chi^+ -> gamma fdbar
          kp3=kgamma
          kp4=kferd
          call dsaschizero(kp1,kp2,kp3,kp4,par)
          prtial(2)=par
************************************************* sfu^* chi^+ -> h fdbar
          kp3=kh2
          kp4=kferd
          call dsaschicased(kp1,kp2,kp3,kp4,par)
          prtial(3)=par
************************************************* sfu^* chi^+ -> H fdbar
          kp3=kh1
          kp4=kferd
          call dsaschicased(kp1,kp2,kp3,kp4,par)
          prtial(4)=par
************************************************* sfu^* chi^+ -> a fdbar
          kp3=kh3
          kp4=kferd
          call dsaschicased(kp1,kp2,kp3,kp4,par)
          prtial(5)=par
**************************************************** sfu chi^+ -> W^+ fu
          kp3=kw
          kp4=kfers
          call dsaschicasea(kp1,kp2,kp3,kp4,par)
          prtial(6)=par
*********************************************** sfu^* chi^+ -> W^+ fubar
          kp3=kw
          kp4=kfers
          call dsaschicaseb(kp1,kp2,kp3,kp4,par)
          prtial(7)=par
**************************************************** sfu chi^+ -> H^+ fu
          kp3=khc
          kp4=kfers
          call dsaschicasec(kp1,kp2,kp3,kp4,par)
          prtial(8)=par
*********************************************** sfu^* chi^+ -> H^+ fubar
          kp3=khc
          kp4=kfers
          call dsaschicased(kp1,kp2,kp3,kp4,par)
          prtial(9)=par
*********************************************** sfu^* chi^+ -> g fdbar
          if(iifam(1)/10.ne.ivfam(knue)/10
     &       .and.iifam(1)/10.ne.ivfam(knumu)/10
     &       .and.iifam(1)/10.ne.ivfam(knutau)/10) then
            kp3=kgluon
            kp4=kferd
            call dsaschizero(kp1,kp2,kp3,kp4,par)
            prtial(10)=par
          else
            prtial(10)=0.0d0
          endif
**************************************************** sum partial results
          w=0.d0
          w=w+prtial(1)            ! sfu^* chi^+ -> Z fdbar
          w=w+prtial(2)            ! sfu^* chi^+ -> gamma fdbar
          w=w+prtial(3)            ! sfu^* chi^+ -> h fdbar
          w=w+prtial(4)            ! sfu^* chi^+ -> H fdbar
          w=w+prtial(5)            ! sfu^* chi^+ -> a fdbar
          w=w+prtial(6)            ! sfu chi^+ -> W^+ fu
          w=w+prtial(7)            ! sfu^* chi^+ -> W^+ fubar
          w=w+prtial(8)            ! sfu chi^+ -> H^+ fu
          w=w+prtial(9)            ! sfu^* chi^+ -> H^+ fubar
          w=w+prtial(10)           ! sfu^* chi^+ -> g fdbar
          dsasdwdcossfchi = w*0.5d0 ! 0.5d0 <- we should return weff for sf chi+ ann
                                    ! with sf and chi+ combined part. and anti-particle states
********
        elseif(iaux.eq.2) then
****************************************************** sfd chi^+ -> Z fu
          kp3=kz
          kp4=kferd
          call dsaschicasea(kp1,kp2,kp3,kp4,par)
          prtial(1)=par
************************************************** sfd chi^+ -> gamma fu
          kp3=kgamma
          kp4=kferd
          call dsaschizero(kp1,kp2,kp3,kp4,par)
          prtial(2)=par
****************************************************** sfd chi^+ -> h fu
          kp3=kh2
          kp4=kferd
          call dsaschicasec(kp1,kp2,kp3,kp4,par)
          prtial(3)=par
****************************************************** sfd chi^+ -> H fu
          kp3=kh1
          kp4=kferd
          call dsaschicasec(kp1,kp2,kp3,kp4,par)
          prtial(4)=par
****************************************************** sfd chi^+ -> a fu
          kp3=kh3
          kp4=kferd
          call dsaschicasec(kp1,kp2,kp3,kp4,par)
          prtial(5)=par
**************************************************** sfd chi^+ -> W^+ fd
          kp3=kw
          kp4=kfers
          call dsaschicasea(kp1,kp2,kp3,kp4,par)
          prtial(6)=par
**************************************************** sfd chi^+ -> h^+ fd
          kp3=khc
          kp4=kfers
          call dsaschicasec(kp1,kp2,kp3,kp4,par)
          prtial(7)=par
*********************************************** sfd^* chi^+ -> W^+ fdbar
          kp3=kw
          kp4=kfers
          call dsaschicaseb(kp1,kp2,kp3,kp4,par)
          prtial(8)=par
*********************************************** sfd^* chi^+ -> h^+ fdbar
          kp3=khc
          kp4=kfers
          call dsaschicased(kp1,kp2,kp3,kp4,par)
          prtial(9)=par
*********************************************** sfd chi^+ -> g fu
          if(iifam(1)/10.ne.ivfam(knue)/10
     &       .and.iifam(1)/10.ne.ivfam(knumu)/10
     &       .and.iifam(1)/10.ne.ivfam(knutau)/10) then
            kp3=kgluon
            kp4=kferd
            call dsaschizero(kp1,kp2,kp3,kp4,par)
            prtial(10)=par
          else
            prtial(10)=0.0d0
          endif
**************************************************** sum partial results
          w=0.d0
          w=w+prtial(1)            ! sfd chi^+ -> Z fu
          w=w+prtial(2)            ! sfd chi^+ -> gamma fu
          w=w+prtial(3)            ! sfd chi^+ -> h fu
          w=w+prtial(4)            ! sfd chi^+ -> H fu
          w=w+prtial(5)            ! sfd chi^+ -> a fu
          w=w+prtial(6)            ! sfd chi^+ -> W^+ fd
          w=w+prtial(7)            ! sfd chi^+ -> h^+ fd
          w=w+prtial(8)            ! sfd^* chi^+ -> W^+ fdbar
          w=w+prtial(9)            ! sfd^* chi^+ -> h^+ fdbar
          w=w+prtial(10)           ! sfd chi^+ -> g fu
          dsasdwdcossfchi = w*0.5d0 ! 0.5d0 <- we should return weff for sf chi+ ann
                                    ! with sf and chi+ combined part. and anti-particle states
********
        else
          write(*,*) 'wrong iaux value =',iaux
          write(*,*) 'in dsasdwdcossfchi. program stopped.'
          stop
        endif
      else 
        write(*,*) 'in dsasdwdcossfchi wrong particle in initial state:'
        write(*,*) pname(kp2),'instead of neutralino or chargino.'
        write(*,*) 'program stopped.'
        stop
      endif 


      wok=.true.
      do i=1,10
        if (prtial(i).lt.0.0d0) wok=.false.
      enddo

      if (.not.wok) then
        write(*,*) 'Model: ',idtag
        write(*,*) 'dsasdwdcossfchi called with kp1=',kp1,' kp2=',kp2
        write(*,*) '  p=',p,' costh=',costhe,' w=',w
        do i=1,10
          write(*,*) 'i=',i,' prtial=',prtial(i)
        enddo
c        stop
      endif
  
      if (w.lt.0.0d0) w=abs(w)
c      if (w.lt.0.0d0) stop

      return
      end








