**************************************************************
*** SUBROUTINE dsasfercol                                  ***
*** computes dW_{ij}/dcostheta                             ***
*** sfermion(i) + antisfermion(j)                          ***
*** -> fermion(k1) + antifermion(k2)                       ***
*** version to be used if i or j are both squarks          ***
***                                                        *** 
*** input askin variables: p12,costheta                    ***
*** kpk1 and kpk2 are the fermion code                     ***
*** AUTHOR: Piero Ullio, ullio@sissa.it                    ***
*** Date: 01-08-10                                         ***
**************************************************************

      SUBROUTINE dsasfercol(kpi,kpj,kpk1,kpk2,result)
      implicit none
      include 'dssusy.h'
      include 'dsascom.h'
      integer kpi,kpj,kpk1,kpk2
      integer iii,jjj,kfercol,ick1,ick2
      real*8 result
      real*8 ampl2
c....check if the final state is kinematically allowed
      if((Svar-(mass(kpk1)+mass(kpk2))**2)/Svar.lt.thstep) then
        result=0.d0
        return
      endif      
***** define family indices
      iifam(3)=ivfam(kpk1)
      iifam(4)=ivfam(kpk2)
***** non equal particle final state: 
      s34=1.d0
***** fermion type in final state:
      q3=2
      q4=1
***** masses in final state:
      mass3=mass(kpk1)
      mass4=mass(kpk2)
***** define the kinematic variables
      call dsaskinset2
      call dsaskinset3
***** initialize ASx and ASy to zero
      do jjj=1,6
      do iii=1,4
      ASxplc(jjj,iii)=(0.d0,0.d0)
      ASxprc(jjj,iii)=(0.d0,0.d0)
      enddo
      ASylc(jjj)=(0.d0,0.d0)
      ASyrc(jjj)=(0.d0,0.d0)
      enddo
***** then fill them in:
*****
*************** case 1 ****************************************
*****
***** i. s.: particle - antiparticle **************************
      if(iifam(1).eq.iifam(2)) then
***** f. s.: particle - antiparticle? *************************
        if(iifam(3).ne.iifam(4)) then 
          write(*,*) 'dsasfercol called with wrong particle'
          write(*,*) 'in the final or initial state :'
          write(*,*) pname(kpi),pname(kpj)
     &               ,'and ',pname(kpk1),pname(kpk2)
          stop        
        endif  
***** Z in s-channel **+***************************************
        call dsasscscsSVffbcol(kpi,kpj,kpk1,kpk2,kz)
***** if kpk1 is not a neutrino: ******************************
        if(kpk1.ne.knue.and.kpk1.ne.knumu
     &     .and.kpk1.ne.knutau) then
***** H01 in s-channel ****************************************
          call dsasscscsSHffbcol(kpi,kpj,kpk1,kpk2,kh1)
***** H02 in s-channel ****************************************
          call dsasscscsSHffbcol(kpi,kpj,kpk1,kpk2,kh2)
***** H03 in s-channel ****************************************
          call dsasscscsSHffbcol(kpi,kpj,kpk1,kpk2,kh3)
***** gamma in s-channel **************************************
          call dsasscscsSVffbcol(kpi,kpj,kpk1,kpk2,kgamma)
        endif  
***** t-channels: *********************************************
        if(iifam(1).eq.iifam(3)) then
***** neutralinos in t-channel **+*****************************
          do iii=1,4
            call dsasscscsTCffbcol(kpi,kpj,kpk1,kpk2,kn(iii))
          enddo
        endif  
        if(iifam(1).eq.(iifam(3)-1).or.iifam(1).eq.(iifam(3)+1))
     &    then      
***** charginos in t-channel **********************************
          do iii=1,2
            call dsasscscsTCffbcol(kpi,kpj,kpk1,kpk2,kcha(iii))
          enddo  
        endif
***** gluon in s-channel **************************************
        call dsasfercode('same',iifam(1),kfercol)
        if(ncolor(kfercol).gt.2.d0.and.ncolor(kpk1).gt.2.d0) 
     &    then
          call dsasscscsSVffbcol(kpi,kpj,kpk1,kpk2,kgluon)
        endif  
***** gluino in t-channel *************************************
        if(iifam(1).eq.iifam(3).and.ncolor(kpk1).gt.2.d0) then
          call dsasscscsTCffbcol(kpi,kpj,kpk1,kpk2,kgluin)
        endif
*****
*************** case 2 ****************************************
*****
***** i. s.: particle - antiparticle in same doublet **********
      elseif(iifam(1).eq.(iifam(2)-1)
     &       .or.iifam(1).eq.(iifam(2)+1)) then  
***** f. s.: particle - antiparticle in same doublet? *********
        if(iifam(3).ne.(iifam(4)-1)
     &     .and.iifam(3).ne.(iifam(4)+1)) then  
          write(*,*) 'dsasfercol called with wrong particle'
          write(*,*) 'in the final or initial state :'
          write(*,*) pname(kpi),pname(kpj)
     &               ,'and ',pname(kpk1),pname(kpk2)
          stop        
        endif  
***** W in s-channel **+***************************************
        call dsasscscsSVffbcol(kpi,kpj,kpk1,kpk2,kw)  
***** H^+ in s-channel ****************************************
        call dsasscscsSHffbcol(kpi,kpj,kpk1,kpk2,khc)
        if(iifam(1).eq.iifam(3).and.iifam(2).eq.iifam(4)) then  
***** neutralinos in t-channel ********************************
          do iii=1,4
            call dsasscscsTCffbcol(kpi,kpj,kpk1,kpk2,kn(iii))
          enddo
        endif  
***** gluino in t-channel *************************************
        if(iifam(1).eq.iifam(3).and.iifam(2).eq.iifam(4)
     &     .and.ncolor(kpk1).gt.2.d0) then
          call dsasscscsTCffbcol(kpi,kpj,kpk1,kpk2,kgluin)
        endif
*****
*************** case 3 ****************************************
*****
***** i. s.: particle - antiparticle in different doublet *****
      else  
***** f. s.: same families as i. s.? **************************
        ick1=iifam(1)/10-iifam(3)/10
        ick2=iifam(2)/10-iifam(4)/10
        if(ick1.ne.0.or.ick2.ne.0) then  
          write(*,*) 'dsasfercol called with wrong particle'
          write(*,*) 'in the final or initial state :'
          write(*,*) pname(kpi),pname(kpj)
     &               ,'and ',pname(kpk1),pname(kpk2)
          stop        
        endif  
        if(iifam(1).eq.iifam(3).and.iifam(2).eq.iifam(4)) then  
***** neutralinos in t-channel ********************************
          do iii=1,4
            call dsasscscsTCffbcol(kpi,kpj,kpk1,kpk2,kn(iii))
          enddo
***** gluino in t-channel *************************************
          if(ncolor(kpk1).gt.2.d0.and.ncolor(kpk2).gt.2.d0) then
            call dsasscscsTCffbcol(kpi,kpj,kpk1,kpk2,kgluin)
          endif
        else  
***** charginos in t-channel **********************************
          do iii=1,2
            call dsasscscsTCffbcol(kpi,kpj,kpk1,kpk2,kcha(iii))
          enddo
        endif
      endif  
*****
*****
***** compute the amplitude squared 
      call dsasffcol(ampl2)
***** computes dW_{ij}/dcostheta
      result=ampl2*k34/(8.d0*pi*gg1*gg2*s34*dsqrt(Svar))
      return
      end

