**************************************************************
*** SUBROUTINE dsasgbhb                                    ***
*** computes dW_{ij}/dcostheta                             ***
*** up-sfermion(i) + down-antisfermion(j) ->               ***
***    gauge boson + Higgs boson                           ***
*** ampl2 obtained summing over physical polarizations     ***
***                                                        *** 
*** input askin variables: p12,costheta                    ***
***    iifam(1),iifam(2),mass1,mass2                       ***
*** AUTHOR: Piero Ullio, ullio@sissa.it                    ***
*** Date: 02-06-13                                         ***
*** This routine has been compared with the routine of     ***
*** Mia Schelke and the agreement is perfect, except in    ***
*** the low-p limit (due to widths in propagators)         ***
*** as expected.                                           ***
**************************************************************

      SUBROUTINE dsasgbhb(kp1,kp2,kp3,kp4,result)
      implicit none
      include 'dssusy.h'
      include 'dsascom.h'
      integer kp1,kp2,kp3,kp4
      integer ksfer1,ksfer2,ksfer3,ksfer4,kgb,khb,iaux1,iaux2
      integer khs,kgs,kfer
      real*8 ampl2,result,dsabsq,colfact
      double complex dsasdepro,A1,A2,Aaux
      character*4 type


c....check if the final state is kinematically allowed
      if((Svar-(mass(kp3)+mass(kp4))**2)/Svar.lt.thstep) then
        result=0.d0
        return
      endif      

***** check initial state is ok and set type  
      if(iifam(1).ne.0.and.iifam(2).ne.0) then
         call dsassfercode('same',iifam(1),ksfer1,ksfer2)  
         call dsassfercode('diff',iifam(1),ksfer3,ksfer4)           
         if(kp2.eq.ksfer1.or.kp2.eq.ksfer2) then
            type='same'
         elseif(kp2.eq.ksfer3.or.kp2.eq.ksfer4) then
            type='diff'
         endif
      else
        write(*,*) 'dsasgbhb called with wrong particles'   
        write(*,*) 'in the initial state :'  
        write(*,*) pname(kp1),pname(kp2)  
        stop  
      endif    

***** set and check particles in the final state:
      kgb=kp3
      khb=kp4
      if(kgb.ne.kz.and.kgb.ne.kw.and.
     &   kgb.ne.kgamma.and.kgb.ne.kgluon) then
        write(*,*) 'dsasgbhb called with kp3 ='   
        write(*,*) pname(kgb)  
        write(*,*) 'instead of a gauge boson'
        stop
      endif 
      if(khb.ne.kh1.and.khb.ne.kh2.and.khb.ne.kh3.and.
     &   khb.ne.khc) then
         write(*,*) 'dsasgbhb called with kp4 ='   
         write(*,*) pname(khb)  
         write(*,*) 'instead of a higgs-boson'
        stop
      endif 

      colfact=1.d0
      if(iifam(1).ge.ivfam(ksu1).and.iifam(1).le.ivfam(ksb2))
     &  colfact=3.d0

***** non equal particle final state: 
      s34=1.d0
***** masses in final state:
      mass3=mass(kp3)
      mass4=mass(kp4)
***** define the kinematic variables  
      call dsaskinset2
*****
      A1=(0.d0,0.d0)
      A2=(0.d0,0.d0)

***** now look at the different initial/final state cases
***** first define some numbers that can be used to check
***** if the sfermions are up- or down-type
      iaux1=iifam(1)/10
      iaux2=iifam(1)-iaux1*10
***** iaux2 will be 1 when kp1 is an up-type-sfermions
***** and 2 when it is a down-type-sfermions
*****       
***** 
***** the first case    
***** up-type-sfermion(i) + anti-down-type-sfermion(j) 
***** -> Z-boson + H^+
      if(type.eq.'diff'.and.iaux2.eq.1.and.kgb.eq.kz.and.
     &   khb.eq.khc) then 
***** ksfer1 in t-channel *************************************
        A1=A1-2.d0*dsasdepro(Tvar,ksfer1)*gl(khb,kp2,ksfer1)
     &          *gl(kgb,ksfer1,kp1)
        if(ksfer2.ne.0) then
***** ksfer2 in t-channel *************************************
          A1=A1-2.d0*dsasdepro(Tvar,ksfer2)*gl(khb,kp2,ksfer2)
     &          *gl(kgb,ksfer2,kp1)
        endif
***** ksfer3 in u-channel *************************************
        A2=A2+2.d0*dsasdepro(Uvar,ksfer3)*gl(khb,ksfer3,kp1)
     &          *gl(kgb,kp2,ksfer3)
***** ksfer4 in u-channel *************************************
        A2=A2+2.d0*dsasdepro(Uvar,ksfer4)*gl(khb,ksfer4,kp1)
     &          *gl(kgb,kp2,ksfer4)
***** h^+ in s-channel ****************************************
        khs=khc
        A1=A1-2.d0*dsasdepro(Svar,khs)*gl(khs,kp2,kp1)
     &          *gl(kgb,khb,khs)
        A2=A2-2.d0*dsasdepro(Svar,khs)*gl(khs,kp2,kp1)
     &          *gl(kgb,khb,khs)
        goto 600
      endif
*****
***** the second case
***** up-type-sfermion(i) + anti-down-type-sfermion(j) 
***** -> photon + H^+
      if(type.eq.'diff'.and.iaux2.eq.1.and.
     &   kgb.eq.kgamma.and.khb.eq.khc) then 
        if(ksfer2.ne.0) then
***** ksfer1 in t-channel *************************************
          A1=A1-2.d0*dsasdepro(Tvar,ksfer1)*gl(khb,kp2,ksfer1)
     &          *gl(kgb,ksfer1,kp1)
***** ksfer2 in t-channel *************************************
          A1=A1-2.d0*dsasdepro(Tvar,ksfer2)*gl(khb,kp2,ksfer2)
     &          *gl(kgb,ksfer2,kp1)
        endif
***** ksfer3 in u-channel *************************************
        A2=A2+2.d0*dsasdepro(Uvar,ksfer3)*gl(khb,ksfer3,kp1)
     &          *gl(kgb,kp2,ksfer3)
***** ksfer4 in u-channel *************************************
        A2=A2+2.d0*dsasdepro(Uvar,ksfer4)*gl(khb,ksfer4,kp1)
     &          *gl(kgb,kp2,ksfer4)
***** h^+ in s-channel ****************************************
        khs=khc
        A1=A1-2.d0*dsasdepro(Svar,khs)*gl(khs,kp2,kp1)
     &          *gl(kgb,khb,khs)
        A2=A2-2.d0*dsasdepro(Svar,khs)*gl(khs,kp2,kp1)
     &          *gl(kgb,khb,khs)
        goto 600
      endif
*****
***** the third case
***** up-type-sfermion(i) + anti-down-type-sfermion(j) 
***** -> W^+ + H^0_1/H^0_2
      if(type.eq.'diff'.and.iaux2.eq.1.and.
     &   kgb.eq.kw.and.khb.ne.kh3.and.khb.ne.khc) then 
***** ksfer3 in t-channel *************************************
        A1=A1-2.d0*dsasdepro(Tvar,ksfer3)*gl(khb,kp2,ksfer3)
     &          *gl(kgb,ksfer3,kp1)
***** ksfer4 in t-channel *************************************
        A1=A1-2.d0*dsasdepro(Tvar,ksfer4)*gl(khb,kp2,ksfer4)
     &          *gl(kgb,ksfer4,kp1)
***** ksfer1 in u-channel *************************************
        A2=A2+2.d0*dsasdepro(Uvar,ksfer1)*gl(khb,ksfer1,kp1)
     &          *gl(kgb,kp2,ksfer1)
        if(ksfer2.ne.0) then
***** ksfer2 in u-channel *************************************
          A2=A2+2.d0*dsasdepro(Uvar,ksfer2)*gl(khb,ksfer2,kp1)
     &          *gl(kgb,kp2,ksfer2)
        endif  
***** h^+ in s-channel ****************************************
        khs=khc
        A1=A1-2.d0*dsasdepro(Svar,khs)*gl(khs,kp2,kp1)
     &          *gl(kgb,khb,khs)
        A2=A2-2.d0*dsasdepro(Svar,khs)*gl(khs,kp2,kp1)
     &          *gl(kgb,khb,khs)
***** W^+ in s-channel ****************************************
        kgs=kw
        Aaux=gl(kgs,kp2,kp1)*mass(kw)*gl(khb,kgb,kgs)
     &        *dsasdepro(Svar,kgs)
        A1=A1+(1.d0-(mass1**2-mass2**2)/mass(kgs)**2)*Aaux
        A2=A2+(-1.d0-(mass1**2-mass2**2)/mass(kgs)**2)*Aaux
        goto 600
      endif
***** the fourth case
***** up-type-sfermion(i) + anti-down-type-sfermion(j) 
***** -> W^+ + H^0_3
      if(type.eq.'diff'.and.iaux2.eq.1.and.
     &   kgb.eq.kw.and.khb.eq.kh3) then 
***** ksfer3 in t-channel *************************************
        A1=A1-2.d0*dsasdepro(Tvar,ksfer3)*gl(khb,kp2,ksfer3)
     &          *gl(kgb,ksfer3,kp1)
***** ksfer4 in t-channel *************************************
        A1=A1-2.d0*dsasdepro(Tvar,ksfer4)*gl(khb,kp2,ksfer4)
     &          *gl(kgb,ksfer4,kp1)
***** in the u-channel (2 sfermions or none):  
***** (H^0_3 can not couple to a pair of sneutrinos as far as there 
*****  are no right-handed sneutrinos)
*****
        if(ksfer2.ne.0) then
***** ksfer1 in u-channel *************************************
          A2=A2+2.d0*dsasdepro(Uvar,ksfer1)*gl(khb,ksfer1,kp1)
     &          *gl(kgb,kp2,ksfer1)
***** ksfer2 in u-channel *************************************
          A2=A2+2.d0*dsasdepro(Uvar,ksfer2)*gl(khb,ksfer2,kp1)
     &          *gl(kgb,kp2,ksfer2)
        endif  
***** h^+ in s-channel ****************************************
        khs=khc
        A1=A1-2.d0*dsasdepro(Svar,khs)*gl(khs,kp2,kp1)
     &          *gl(kgb,khb,khs)
        A2=A2-2.d0*dsasdepro(Svar,khs)*gl(khs,kp2,kp1)
     &          *gl(kgb,khb,khs)
        goto 600
      endif
*****
***** the fifth case
***** up-type-sfermion(i) + anti-up-type-sfermion(j) 
***** -> W^- + H^+
      if(type.eq.'same'.and.iaux2.eq.1.and.
     &   kgb.eq.kw.and.khb.eq.khc) then 
***** ksfer3 in u-channel *************************************
        A2=A2+2.d0*dsasdepro(Uvar,ksfer3)*gl(khb,kp1,ksfer3)
     &          *gl(kgb,ksfer3,kp2)
***** ksfer4 in u-channel *************************************
        A2=A2+2.d0*dsasdepro(Uvar,ksfer4)*gl(khb,kp1,ksfer4)
     &          *gl(kgb,ksfer4,kp2)
***** h1 in s-channel  ****************************************
        khs=kh1
        A1=A1-2.d0*dsasdepro(Svar,khs)*gl(khs,kp2,kp1)
     &          *gl(kgb,khb,khs)
        A2=A2-2.d0*dsasdepro(Svar,khs)*gl(khs,kp2,kp1)
     &          *gl(kgb,khb,khs)
***** h2 in s-channel  ****************************************
        khs=kh2
        A1=A1-2.d0*dsasdepro(Svar,khs)*gl(khs,kp2,kp1)
     &          *gl(kgb,khb,khs)
        A2=A2-2.d0*dsasdepro(Svar,khs)*gl(khs,kp2,kp1)
     &          *gl(kgb,khb,khs)
        call dsasfercode('same',iifam(1),kfer)         
***** kfer is the fermion corresponding to the sfermion
        if(dabs(echarg(kfer)).gt.1.d-16) then
***** h3 in s-channel  ****************************************
          khs=kh3
          A1=A1-2.d0*dsasdepro(Svar,khs)*gl(khs,kp2,kp1)
     &          *gl(kgb,khb,khs)
          A2=A2-2.d0*dsasdepro(Svar,khs)*gl(khs,kp2,kp1)
     &          *gl(kgb,khb,khs)
        endif  
        goto 600
      endif
***** 
***** the sixth case 
***** down-type-sfermion(i) + anti-down-type-sfermion(j) 
***** -> W^- + H^+
      if(type.eq.'same'.and.iaux2.eq.2.and.
     &   kgb.eq.kw.and.khb.eq.khc) then 
***** ksfer3 in t-channel *************************************
        A1=A1-2.d0*dsasdepro(Tvar,ksfer3)*gl(khb,kp2,ksfer3)
     &          *gl(kgb,ksfer3,kp1)
***** ksfer4 in t-channel *************************************
        if(ksfer4.ne.0) then
          A1=A1-2.d0*dsasdepro(Tvar,ksfer4)*gl(khb,kp2,ksfer4)
     &          *gl(kgb,ksfer4,kp1)
        endif  
***** h1 in s-channel  ****************************************
        khs=kh1
        A1=A1-2.d0*dsasdepro(Svar,khs)*gl(khs,kp2,kp1)
     &          *gl(kgb,khb,khs)
        A2=A2-2.d0*dsasdepro(Svar,khs)*gl(khs,kp2,kp1)
     &          *gl(kgb,khb,khs)
***** h2 in s-channel  ****************************************
        khs=kh2
        A1=A1-2.d0*dsasdepro(Svar,khs)*gl(khs,kp2,kp1)
     &          *gl(kgb,khb,khs)
        A2=A2-2.d0*dsasdepro(Svar,khs)*gl(khs,kp2,kp1)
     &          *gl(kgb,khb,khs)
***** h3 in s-channel  ****************************************
        khs=kh3
        A1=A1-2.d0*dsasdepro(Svar,khs)*gl(khs,kp2,kp1)
     &        *gl(kgb,khb,khs)
        A2=A2-2.d0*dsasdepro(Svar,khs)*gl(khs,kp2,kp1)
     &        *gl(kgb,khb,khs)
        goto 600
      endif
*****
***** the seventh case
***** sfermion(i) + anti-sfermion(j) 
***** -> photon + H^0_1/H^0_2/H^0_3
      if(type.eq.'same'.and.kgb.eq.kgamma.and.
     &       (khb.eq.kh1.or.khb.eq.kh2.or.khb.eq.kh3)) then 
        if(ksfer2.ne.0) then
***** ksfer1 in t-channel *************************************
          A1=A1-2.d0*dsasdepro(Tvar,ksfer1)*gl(khb,kp2,ksfer1)
     &          *gl(kgb,ksfer1,kp1)
***** ksfer2 in t-channel *************************************
          A1=A1-2.d0*dsasdepro(Tvar,ksfer2)*gl(khb,kp2,ksfer2)
     &          *gl(kgb,ksfer2,kp1)
***** ksfer1 in u-channel *************************************
          A2=A2+2.d0*dsasdepro(Uvar,ksfer1)*gl(khb,ksfer1,kp1)
     &          *gl(kgb,kp2,ksfer1)
***** ksfer2 in u-channel *************************************
          A2=A2+2.d0*dsasdepro(Uvar,ksfer2)*gl(khb,ksfer2,kp1)
     &          *gl(kgb,kp2,ksfer2)
        endif
        goto 600
      endif
***** 
***** the eighth case    
***** sfermion(i) + anti-sfermion(j) 
***** -> Z-boson + H^0_1/H^0_2
      if(type.eq.'same'.and.kgb.eq.kz.and.
     &       (khb.eq.kh1.or.khb.eq.kh2)) then 
***** ksfer1 in t-channel *************************************
        A1=A1-2.d0*dsasdepro(Tvar,ksfer1)*gl(khb,kp2,ksfer1)
     &          *gl(kgb,ksfer1,kp1)
        if(ksfer2.ne.0) then
***** ksfer2 in t-channel *************************************
          A1=A1-2.d0*dsasdepro(Tvar,ksfer2)*gl(khb,kp2,ksfer2)
     &          *gl(kgb,ksfer2,kp1)
        endif  
***** ksfer1 in u-channel *************************************
        A2=A2+2.d0*dsasdepro(Uvar,ksfer1)*gl(khb,ksfer1,kp1)
     &          *gl(kgb,kp2,ksfer1)
        if(ksfer2.ne.0) then
***** ksfer2 in u-channel *************************************
          A2=A2+2.d0*dsasdepro(Uvar,ksfer2)*gl(khb,ksfer2,kp1)
     &          *gl(kgb,kp2,ksfer2)
        endif
***** in the s-channel with higgs exchange:
        call dsasfercode('same',iifam(1),kfer)         
***** kfer is the fermion corresponding to the sfermion
***** (H^0_3 can not couple to a pair of sneutrinos as far as there 
*****  are no right-handed sneutrinos)
        if(dabs(echarg(kfer)).gt.1.d-16) then
***** h3 in s-channel  ****************************************
          khs=kh3
          A1=A1-2.d0*dsasdepro(Svar,khs)*gl(khs,kp2,kp1)
     &        *gl(kgb,khb,khs)
          A2=A2-2.d0*dsasdepro(Svar,khs)*gl(khs,kp2,kp1)
     &        *gl(kgb,khb,khs)
        endif
***** Z^0 in s-channel ****************************************
        kgs=kz
        Aaux=gl(kgs,kp2,kp1)*mass(kw)*gl(khb,kgb,kgs)
     &        *dsasdepro(Svar,kgs)
        A1=A1+(1.d0-(mass1**2-mass2**2)/mass(kgs)**2)*Aaux
        A2=A2+(-1.d0-(mass1**2-mass2**2)/mass(kgs)**2)*Aaux
        goto 600
      endif
***** the ninth case    
***** sfermion(i) + anti-sfermion(j) 
***** -> Z-boson + H^0_3
      if(type.eq.'same'.and.kgb.eq.kz.and.khb.eq.kh3) then 
        if(ksfer2.ne.0) then
***** ksfer1 in t-channel *************************************
          A1=A1-2.d0*dsasdepro(Tvar,ksfer1)*gl(khb,kp2,ksfer1)
     &          *gl(kgb,ksfer1,kp1)
***** ksfer2 in t-channel *************************************
          A1=A1-2.d0*dsasdepro(Tvar,ksfer2)*gl(khb,kp2,ksfer2)
     &          *gl(kgb,ksfer2,kp1)
***** ksfer1 in u-channel *************************************
          A2=A2+2.d0*dsasdepro(Uvar,ksfer1)*gl(khb,ksfer1,kp1)
     &          *gl(kgb,kp2,ksfer1)
***** ksfer2 in u-channel *************************************
          A2=A2+2.d0*dsasdepro(Uvar,ksfer2)*gl(khb,ksfer2,kp1)
     &          *gl(kgb,kp2,ksfer2)
        endif
***** h1 in s-channel  ****************************************
        khs=kh1
        A1=A1-2.d0*dsasdepro(Svar,khs)*gl(khs,kp2,kp1)
     &          *gl(kgb,khb,khs)
        A2=A2-2.d0*dsasdepro(Svar,khs)*gl(khs,kp2,kp1)
     &          *gl(kgb,khb,khs)
***** h2 in s-channel  ****************************************
        khs=kh2
        A1=A1-2.d0*dsasdepro(Svar,khs)*gl(khs,kp2,kp1)
     &          *gl(kgb,khb,khs)
        A2=A2-2.d0*dsasdepro(Svar,khs)*gl(khs,kp2,kp1)
     &          *gl(kgb,khb,khs)
        goto 600
      endif
***** the tenth case    
***** squark(i) + anti-squark(j) 
***** -> gluon + H^0_{1,2,3}
      if(iifam(1).ge.ivfam(ksu1).and.iifam(1).le.ivfam(ksb2).and.
     &   type.eq.'same'.and.kgb.eq.kgluon.and.
     &   (khb.eq.kh1.or.khb.eq.kh2.or.khb.eq.kh3)) then
        colfact=4.d0
***** ksfer1 in t-channel *************************************
        A1=A1-2.d0*dsasdepro(Tvar,ksfer1)*gl(khb,kp2,ksfer1)
     &          *gl(kgb,ksfer1,kp1)
***** ksfer2 in t-channel *************************************
        A1=A1-2.d0*dsasdepro(Tvar,ksfer2)*gl(khb,kp2,ksfer2)
     &          *gl(kgb,ksfer2,kp1)
***** ksfer1 in u-channel *************************************
        A2=A2+2.d0*dsasdepro(Uvar,ksfer1)*gl(khb,ksfer1,kp1)
     &          *gl(kgb,kp2,ksfer1)
***** ksfer2 in u-channel *************************************
        A2=A2+2.d0*dsasdepro(Uvar,ksfer2)*gl(khb,ksfer2,kp1)
     &          *gl(kgb,kp2,ksfer2)
        goto 600
      endif
*****
***** the eleventh case
***** up-type-squark(i) + anti-down-type-squark(j) 
***** -> gluon + H^+
      if(iifam(1).ge.ivfam(ksu1).and.iifam(1).le.ivfam(ksb2).and.
     &   type.eq.'diff'.and.iaux2.eq.1.and.
     &   kgb.eq.kgluon.and.khb.eq.khc) then
        colfact=4.d0
***** ksfer1 in t-channel *************************************
        A1=A1-2.d0*dsasdepro(Tvar,ksfer1)*gl(khb,kp2,ksfer1)
     &          *gl(kgb,ksfer1,kp1)
***** ksfer2 in t-channel *************************************
        A1=A1-2.d0*dsasdepro(Tvar,ksfer2)*gl(khb,kp2,ksfer2)
     &          *gl(kgb,ksfer2,kp1)
***** ksfer3 in u-channel *************************************
        A2=A2+2.d0*dsasdepro(Uvar,ksfer3)*gl(khb,ksfer3,kp1)
     &          *gl(kgb,kp2,ksfer3)
***** ksfer4 in u-channel *************************************
        A2=A2+2.d0*dsasdepro(Uvar,ksfer4)*gl(khb,ksfer4,kp1)
     &          *gl(kgb,kp2,ksfer4)
        goto 600
      endif
*****


 600  continue
***** amplitude squared  **************************************
***** ampl2 obtained summing over physical polarizations    ***
      ampl2=p12**2*(1.d0-costheta**2)*dsabsq(A1-A2)
      if(dabs(mass3).gt.1.d-15) then
        Aaux=k34/mass3*(ep1*A1+ep2*A2)
     &       -costheta*ek3/mass3*p12*(A1-A2)
        ampl2=ampl2+dsabsq(Aaux)
      endif  
*****
      result=ampl2*k34/(8.d0*pi*gg1*gg2*s34*dsqrt(Svar))*colfact
      return
      end
