      subroutine dshmset(c)
****************************************************************
*** subroutine dshmset:                                      ***
*** initialize the density profile and or the small clump    ***
*** probability distribution                                 ***
*** type of halo:                                            ***
***   hclumpy=1 smooth, hclumpy=2 clumpy                     ***
***                                                          ***
*** a few sample cases are given; specified values of the    ***
*** local halo density 'rho0' and of the length scale        ***
*** parameter 'a' should be considered just indicative       ***
***                                                          ***
*** author: piero ullio (piero@tapir.caltech.edu)            ***
*** date: 00-07-13                                           ***
*** small modif: paolo gondolo 00-07-19                      ***
*** mod: 03-11-19 je, 04-01-13 pu                            ***
****************************************************************
      implicit none
      include 'dshmcom.h'
      include 'dssusy.h'
      character*(*) c
      real*8 dshmaxirho

c...Set default flags
      udfload=.true.  ! load udffile (if chosen by veldf='user') on next
                      ! call to dshmudf.f
      udfearthload=.true. ! load udfearthfile (if chosen by veldfearth='user')
                          ! on next call to dshmudfearth.f

      isodfload=.true.   ! load isodf file on next call to dshmisotrnum.f

c...Three-dimensional velocity dispersion of the WIMPs in the halo
c...Note that in a simple isothermal sphere, the circular speed
c...(approx. solar speed v_sun) is sqrt(2/3)*vd_3d
      vd_3d = 270.0d0     ! WIMP 3D velocity dispersion, km/s
      v_sun = 220.0d0     ! circular velocity at the Sun location, km/s
      v_obs = v_sun       ! Velocity of observer
      vgalesc = 600.d0          ! galactic escape speed in km/s
c...Observer speed w.r.t. the halo, i.e. including Sun + Earth speed,
c...yearly average,
      vobs = 264.d0       ! observer speed w.r.t. the halo in km/s
      v_earth=29.78d0    ! Earth speed in the solar system in km/s

c a few options for the density profile:

c ... modified isothermal profile, smooth profile
      if (c.eq.'isosm'.or.c.eq.'default') then
        hclumpy=1
        r_0=8.5d0           ! sun galactocentric distance (kpc)
        rho0=0.3d0          ! local halo density (gev cm^-3)
        haloshape='spherical' !here you choose between spherical and axisymm.
        halotype='albega'  !this picks the alpha-beta-gamma profile, set by:
        alphah=2.d0        
        betah=2.d0
        gammah=0.d0
        ah=3.5d0             ! length scale (kpc)
        Rref=r_0
        rhoref=rho0         ! normalizing the profile to the local halo density
        rhcut=1.d-10        ! cut radius (kpc) there is no cut radius for this
                            ! case but this value avoids numerical problems
        haloid='isosm'      ! tag used to identify pbar and dbar files
        veldf='gauss'       ! tag used to identify velocity profile
        veldfearth='sdbest' ! tag to identify earth vel. profile

c ... modified isothermal profile, clumpy profile
      elseif (c.eq.'isocl') then
        hclumpy=2
        r_0=8.5d0           !sun galactocentric distance (kpc)
        rho0=0.3d0          !local halo density (gev cm^-3)  
        probshape='spherhalo' !here you choose which probability distribution
                              !you have for the small clumps, spherhalo is
                              !the option to have the same distribution as
                              !the spherical density profile, then you have
                              !to set this profile
        halotype='albega'   !this picks the alpha-beta-gamma profile, set by:
        alphah=2.d0        
        betah=2.d0
        gammah=0.d0
        ah=5.d0             ! length scale (kpc)
        Rref=r_0
        rhoref=rho0         ! normalizing the profile to the local halo density
        rhcut=1.d-10        ! cut radius (kpc) there is no cut radius for this
        haloid='isocl'      ! tag used to identify pbar and dbar files
        veldf='gauss'       ! tag used to identify velocity profile
        veldfearth='sdbest' ! tag to identify earth vel. profile

c ... navarro-frenk-white profile, smooth profile
      elseif (c.eq.'nfwsm') then
        hclumpy=1
        r_0=8.d0           ! sun galactocentric distance (kpc)
        rho0=0.3d0          ! local halo density (gev cm^-3)
        haloshape='spherical' !here you choose between spherical and axisymm.
        halotype='albega' !this picks the alpha-beta-gamma profile, set by:
        alphah=1.d0        
        betah=3.d0
        gammah=1.d0
        ah=20.d0             ! length scale (kpc)
        Rref=r_0
        rhoref=rho0         ! normalizing the profile to the local halo density
        rhcut=1.d-5         ! cut radius (kpc) 
        haloid='nfwsm'      ! tag used to identify pbar and dbar files
        veldf='gauss'       ! tag used to identify velocity profile
        veldfearth='sdbest' ! tag to identify earth vel. profile


c ... moore et al. profile, smooth profile
      elseif (c.eq.'moosm') then
        hclumpy=1
        r_0=8.d0           ! sun galactocentric distance (kpc)
        rho0=0.3d0          ! local halo density (gev cm^-3)
        haloshape='spherical' !here you choose between spherical and axisymm.
        halotype='albega'  !this picks the alpha-beta-gamma profile, set by:
        alphah=1.5d0        
        betah=3.d0
        gammah=1.5d0
        ah=28.d0             ! length scale (kpc)
        Rref=r_0
        rhoref=rho0         ! normalizing the profile to the local halo density
        rhcut=1.d-5         ! cut radius (kpc) 
        haloid='moosm'      ! tag used to identify pbar and dbar files
        veldf='gauss'       ! tag used to identify velocity profile
        veldfearth='sdbest' ! tag to identify earth vel. profile

c ... burkert et al. profile, smooth profile
      elseif (c.eq.'burksm') then
        hclumpy=1
        r_0=8.d0           ! sun galactocentric distance (kpc)
        haloshape='spherical' ! here you choose between spherical and axisymm.
        halotype='burkert'  ! this picks the burkert profile, set by:
c model with virial mass and virial radius:
c        Mvir=0.130000d+13
c        cvir=0.160000d+02
c this corresponds to:
        ah=11.6839499d0
        rho0=0.339382133d0
        Rref=r_0
        rhoref=rho0          ! normalizing the profile at the local radius
                             ! it can be anything else
        haloid='burksm'      ! tag used to identify pbar and dbar files
        rhcut=0.0d0
        veldf='numc'       ! tag used to identify velocity profile
        udfnumfile=dsroot
        call dscharadd(udfnumfile,'dat/vdf-burksm.dat') ! sun vel. prof.
        isodfnumfile=dsroot
        call dscharadd(isodfnumfile,'dat/df-burksm.dat') ! sun vel df (3D)
        isodf='num'         ! tag to use the above file
        veldfearth='sdbest' ! tag to identify earth vel. profile
        v_sun = 216.2856d0     ! circular velocity at the Sun location, km/s


c ... adiabatically contracted profile, smooth profile loaded from file
      elseif (c.eq.'adiabsm') then
        hclumpy=1
c model with virial mass and virial radius:
c        Mvir=0.180000d+13
c        cvir=0.120000d+02
        r_0=8.d0           ! sun galactocentric distance (kpc)
        haloshape='spherical' ! here you choose between spherical and axisymm.
        halotype='numerical'  ! this picks the profile from file:
        hmrhofile=dsroot
        call dscharadd(hmrhofile,'dat/rho-adiabsm.dat')
                        ! this must be a file with a set of pairs
                        ! dlog(radialcoord[kpc]) - dlog(rho[GeV cm^-3])
                        ! with the format (2(1x,e14.8))
        rho0=dshmaxirho(r_0,0.d0) ! set the local halo density
        haloid='adiabsm'      ! tag used to identify pbar and dbar files
        rhcut=0.0d0
        veldf='numc'   ! tag used to identify velocity profile
        udfnumfile=dsroot
        call dscharadd(udfnumfile,'dat/vdf-adiabsm.dat') ! sun vel. prof.
        isodfnumfile=dsroot
        call dscharadd(isodfnumfile,'dat/df-adiabsm.dat') ! sun vel df (3D)
        isodf='num'         ! tag to use the above file
        veldfearth='sdbest'  ! tag to identify earth vel. profile
        v_sun = 220.9832d0     ! circular velocity at the Sun location, km/s

c ... navarro-frenk-white adiab. profile, smooth profile based on
c.....hep-ph/0506204 
      elseif (c.eq.'nfwadiabsm') then
        hclumpy=1
        r_0=8.d0           ! sun galactocentric distance (kpc)
        rho0=0.3d0          ! local halo density (gev cm^-3)
        haloshape='spherical' !here you choose between spherical and axisymm.
        halotype='albega' !this picks the alpha-beta-gamma profile, set by:
        alphah=0.8d0        
        betah=2.7d0
        gammah=1.45d0
        ah=20.d0             ! length scale (kpc)
        Rref=r_0
        rhoref=rho0         ! normalizing the profile to the local halo density
        rhcut=1.d-5         ! cut radius (kpc) 
        haloid='nfw-adiabsm'      ! tag used to identify pbar and dbar files
        veldf='gauss'       ! tag used to identify velocity profile
        veldfearth='sdbest' ! tag to identify earth vel. profile

c ... moore et al. adib. profile, smooth profile based on
c.....hep-ph/0506204 
      elseif (c.eq.'mooadiabsm') then
        hclumpy=1
        r_0=8.d0           ! sun galactocentric distance (kpc)
        rho0=0.3d0          ! local halo density (gev cm^-3)
        haloshape='spherical' !here you choose between spherical and axisymm.
        halotype='albega'  !this picks the alpha-beta-gamma profile, set by:
        alphah=0.8d0        
        betah=2.7d0
        gammah=1.65d0
        ah=28.d0             ! length scale (kpc)
        Rref=r_0
        rhoref=rho0         ! normalizing the profile to the local halo density
        rhcut=1.d-5         ! cut radius (kpc) 
        haloid='moo-adiabsm'      ! tag used to identify pbar and dbar files
        veldf='gauss'       ! tag used to identify velocity profile
        veldfearth='sdbest' ! tag to identify earth vel. profile

      else
         write (*,*) 'dshmset: unrecognized option ',c
         stop
      endif


c...unrescaled local density
      rhox = rho0

      return
      end
