function  bsgamma_intp(mt, alphas)

!------------------------------
!reads Misiak's SM BR(B -> s \gamma) computation 
!table and interpolates result for any mtop  
!and alphas value
!author: Roberto Ruiz
!------------------------------


implicit none
include 'dssusy-f90.h'


real(8) :: bsgamma_intp

real(8) :: mt, alphas

real(8), parameter :: mt_min = 165.1
real(8), parameter :: mt_max = 177.7 
real(8), parameter :: alphas_min = 0.1116
real(8), parameter :: alphas_max = 0.1236

integer, parameter :: dim = 91
integer, parameter :: nq = 13
integer, parameter :: nr = 3
integer, parameter :: nw = 19



integer :: fl, l, m, n
real(8), dimension(dim) :: mta, alphasa, bsgamma
real(8) :: a(5,dim)


integer :: ier
integer :: lcell(3,3)
integer :: lnext(dim)
real(8) :: rmax
real(8) :: dmt, dalphas
real(8) :: mtmin, alphasmin
real(8) :: rsq(dim)

real(8) :: qs2val

character(160) :: filein
logical, save :: first = .true.


filein = trim(dsroot)//'src/interf/bsg-Misiak.dat'


if (first) then

   open(10,file=filein,status='old',err=111)

   read(10,*)
   read(10,*)
   
   do n = 1,dim
      read(10,*) mta(n), alphasa(n), bsgamma(n)
   enddo
   first = .false.
   
endif


   if(mt > mt_max) mt = mt_max
   if(mt < mt_min) mt = mt_min  
   if(alphas > alphas_max) alphas = alphas_max
   if(alphas < alphas_min) alphas = alphas_min 


   call qshep2 (dim, mta, alphasa, bsgamma, nq, nw, nr, lcell, lnext,&
     mtmin, alphasmin, dmt, dalphas, rmax, rsq, a, ier )


   bsgamma_intp = qs2val (mt, alphas, dim,  mta , alphasa, &
     bsgamma, nr, lcell, lnext, mtmin, alphasmin, dmt, dalphas, rmax, rsq, a )

   return


111 write(*,*) 'Error reading in the Misiak bsg file ', trim(filein)
    write(*,*) 'dsroot in bsgamma.f90 is ', trim(dsroot)
    stop


end function 
