subroutine ds(Flag, InP, InN, Outsoft, DSO)

!----------------------------------------------
! interfacing routine DarkSusy with MCMC  
! author: Roberto Ruiz
!---------------------------------------------

use parameters

implicit none  

include'dssusy-f90.h'  
include'dsidtag-f90.h' 
include'dssoftsusy-f90.h' 
 

Type(FLAGS), INTENT(IN) :: Flag
Type(Input_params), INTENT(IN) :: InP
Type(Nuisance_params), INTENT(IN) :: InN
Type(Softout), INTENT(IN)  :: Outsoft
Type(DS_Output), INTENT(INOUT) :: DSO


real (8) :: xf, dsrdomega                             ! relic density
real (8) :: sigsip, sigsin, sigsdp, sigsdn            ! direct detec.
real (8) :: nmusigmav                                 ! indirect detec
real (8) :: dshmj, dshmjave, dshrgacontdiff, dshrgacont, & 
            dshrpbardiff, dshrdbardiff, dsepdiff, dshrmuhalo, &
            dshrmudiff, dsepfr, dssigmav              ! ID functions
real (8) :: dsgm2muon                                 ! g-2 function
integer  :: softerr, unphys, hwarning, ierr, iwar, nfc, &
            acceptable, istat                         ! err or war flags
integer  :: ihiggs                                    ! HDECAY flag 
real(8)  :: bsg_SM, rbsg(2)                           ! bsg parameters
real (8) :: brr_SM, brr_MSSM    ! Higgs branching ratios part
real(8)  :: bsgamma_intp        ! new SM bsgamma value from Misiak et al 

logical  :: hmd = .false.

real (8), save :: jpsi_gamma(10) = 0.d0, jpsi_nt(10) = 0.d0
integer, save ::  time = 1

character(LEN=35) rateneut_units, drateneut_units

real (8) :: dsabsq ! useful function: sqrt(abs(x))


DSO%PrintOutParams=.false. !reseting


! The amount of output onto standard output can be controlled by the
! variable prtlevel at deep level and debug for a rough testing. 
! Setting prtlevel=0 suppresses all messages,
! except fatal errors that terminate the program. Setting prtlevel=1
! displays informational messages. Setting prtlevel to higher values
! displays more and more information. 
! Setting debug = .true. make a diagnosis of what is doing 
! at the level of the call to the routines which compute 
! the different observables.


prtlevel = 0 !to get avoid tags
debug = .false. 


! SM parameters are passed from Softsusy:

! EW coupling constants
alphem = InN%alphamz
alph3mz = InN%alphasmz


! strange quark hadronic elements
ftp_s = InN%ftp_s
ftn_s = InN%ftn_s


! bottom and top pole masses
mass(kb) = outsoft%run%mbpole 
mass(kt) = InN%mtop 


! leptons are pole ones
mass(ke)  =  outsoft.sm.mElectron
mass(kmu) = outsoft.sm.mMuon
mass(ktau)= outsoft.sm.mTau


! quarks are running ones 
mass(ku) = outsoft.sm.mUp 
mass(kd) = outsoft.sm.mDown
mass(ks) = outsoft.sm.mStrange
mcmc = outsoft.sm.mCharm 
mbmb = InN%mbottom


! This call initializes the DarkSUSY package. This call should
! be the first call in any program using DarkSUSY. This call initial
! some global variables and calls various other modules to initializ
! them with their default values. Check out src/ini/dsinit.f if you
! want to see what it does.


call dsinit  


! All the SUSY spectrum passed to DarkSUSY is computed by Softsusy. 
! The  supersymmetric parameters are: the gaugino masses m1, m2, m3; the
! Higgs pseudoscalar mass ma; the ratio of Higgs vacuum expectation
! values tanbe; the square of the squark and slepton mass parameters
! mass2q(i), mass2u(i), mass2d(i), mass2l(i), mass2e(i); the
! trilinear soft parameters asofte(i), asoftu(i), asoftd(i). Here
! i=1,2,3 is a generation index. They are defined at the 
! weak scale. Physical parameters are passed too.    


! SUSY parameters passed from Softsusy:

tanbe = InP%tanb

! Soft masses
mu = Outsoft%soft%mu
m1 = Outsoft%soft%m1
m2 = Outsoft%soft%m2

asofte = Outsoft%soft%asofte
asoftu = Outsoft%soft%asoftu
asoftd = Outsoft%soft%asoftd

mass2l = Outsoft%soft%mass2l
mass2e = Outsoft%soft%mass2e
mass2q = Outsoft%soft%mass2q
mass2u = Outsoft%soft%mass2u
mass2d = Outsoft%soft%mass2d


! Gluino mass
m3 = outsoft%mgluino


! Higgs sector
Mass(kh1)=outsoft%higg%mhh
Mass(kh2)=outsoft%higg%mlh
Mass(kh3) = outsoft%higg%ma
Mass(khc)=outsoft%higg%mch
alpha=outsoft%higg%halpha


! Sfermion sector
msu=outsoft%sfer%msu
msd=outsoft%sfer%msd
mse=outsoft%sfer%mse

msnu=outsoft%sfer%msnu

! Mixings
thetat = outsoft%sfer%thetat
thetab = outsoft%sfer%thetab
thetatau = outsoft%sfer%thetatau


! Neutralino sector
mchi=outsoft%neut%mneut
mixchi=outsoft%neut%mixchi


! Chargino sector
mcha=outsoft%charg%mcha

thetaL=outsoft%charg%thetaL
thetaR=outsoft%charg%thetaR


! Fermions third generation stuff 

! Running masses including SUSY threshold corr.
runmass(ktau) = outsoft%run%mtaumz
runmass(kb) = outsoft%run%mbmz
runmass(kt) =outsoft%run%mtmt


! Yukawa couplings at 2*mchi 
ylsp_tau = outsoft%run%ylsp_tau
ylsp_b = outsoft%run%ylsp_b
ylsp_t = outsoft%run%ylsp_top

! Init errors
DSO%err%unphys = 0
DSO%err%oh2 = 0

! Error flags from SoftSusy 

if(DSO%err%muSqWrongSign.eq.1 .or. DSO%err%b.eq.1 .or. &
     DSO%err%noRhoConvergence.eq.1 .or. DSO%err%tachyon.eq.1 .or. & 
     DSO%err%nonperturbative.eq.1 .or. DSO%err%noMuConvergence.eq.1 .or. &
     DSO%err%higgsUfb.eq.1 .or. DSO%err%irqfp.eq.1 .or. &
     DSO%err%badConvergence.eq.1) then

     softerr = 1

else
    call dssusy (unphys, hwarning) 
    DSO%err%unphys = unphys

    if( unphys /= 0 .and. debug)  then
      write ( * , * ) ' Error: The lightest neutralino is not the LSP'
   endif

endif


!For serious debugging
! writes the spectrum in dstest1.tmp
!        open (unit=30,file='dsspect.tmp')
!        write(30,*) ' '
!        call dswspectrum(30)
!        close(30)

! writes the vertex in dsvtx.tmp
!        open (unit=40,file='dsvtx.tmp')
!        call dswvertx(40)
!        close(40)

!
!


if ( (softerr.eq.0) .and. (unphys.eq.0) ) then  
   acceptable = 0  
else  
   acceptable = 1  
endif  

if (debug) write ( * , * ) 'acceptable =', acceptable, ' (0=OK, 1=not OK)'  

if (acceptable.ne.0) then  
   DSO%DM%oh2 = 1.0d+40  
!
   DSO%DD%sigsi_nuc = 1.0d+40
!
   DSO%Coll%mw =  1.0d+40
   DSO%Coll%sweff2 = 1.0d+40
   DSO%Coll%gm2amp = 1.0d+40  
   DSO%Coll%bsgamma = 1.0d+40 
   DSO%Coll%brbsmm = 1.0d+40
   DSO%Coll%brr = 1.0d+40
   return 
endif  



! We are now ready to calculate the relic density and different
! direct or indirect detection rates. We will start with the relic
! density. The relic density is calculated by the function dsrdomega
! The first argument determines if coannihilations should be include
! If the argument is 0, no coannihilations are included, if it is 1,
! all relevant coannihilations are included (charginos, neutralinos
! and sfermions if light enough). One can also choose to only
! include a partial set of coannihilations: argument=2, only chargin
! and neutralino coannihilaitons, argument=3, only sfermion
! coannihiltions.
! The second argument determines the
! accuracy, if set to 1 a faster calculation (e.g. only including
! coannihilations if the mass difference is less then 30%) is
! performed and if set to 0 a more accurate calculation is performed
! In practice, the fast option is more than adequate (better than
! 5% accuracy). The function returns the relic density, omega h^2,
! the freeze-out temperature, xf (x=m/T), one error flag, ierr and
! one warning flag, iwar, both of which are 0 if everything is OK.
! nfc is the number points in momentum where the cross section had
! to be calculated. Note that the omega calculation can be very
! time consuming, needing up to several minutes for tricky models
! (with many resonances, thresholds, coannihilations etc).
!

! Reseting error and warnings related with oh2 calculation.


if(debug) write ( *, *) '>>> DS Out'


 if(.not. Flag%ID%evar .or. (Flag%ID%evar .and. (DSO%ID_in%egam == DSO%ID_in%efluxes_i))) then 

 if (Flag%DM_predict) then 

  if (debug) write ( * ,  * ) '    >>> Calculating omega h^2 with & 
           coannihilations, ', 'please be patient...'

  DSO%DM%oh2 = dsrdomega (1, 1, xf, ierr, iwar, nfc)  

  if (debug) write ( * ,  * ) '         Oh2 = ', DSO%DM%oh2

! Gaugino fraction 
  DSO%DM%gf =  dsabsq (neunmx (1, 1) ) + dsabsq (neunmx (1, 2))


! error from oh2 calculation, Params are written ---- !
  DSO%err%oh2 = ierr 


 endif


! Direct Detection rates

 if (Flag%DD_predict) then

  if (debug) write ( * , * ) '    >>> Calculating scattering cross sections...'  

  call sheffddneunuc (sigsip, sigsin, sigsdp, sigsdn)


! Spin-independent nucleon: taked as the maximun of proton
! and neutron.    
  DSO%DD%sigsi_nuc = max(sigsip, sigsin) * 1.0d36

! Spin-dependent proton and neutron
  DSO%DD%sigsdp = sigsdp * 1.0d36 
  DSO%DD%sigsdn = sigsdn * 1.0d36 !in pb


  if (debug) then 
   write ( * ,  * ) '         sigsip (pb)   = ', sigsip * 1.0d36,  ' pb'  
   write ( * ,  * ) '         sigsin (pb)   = ', sigsin * 1.0d36,  ' pb'
   write ( * ,  * ) '         sigsi_nuc(pb) = ', DSO%DD%sigsi_nuc, ' pb'
   write ( * ,  * ) '         sigsdp (pb)   = ', DSO%DD%sigsdp,    ' pb'
   write ( * ,  * ) '         sigsdn (pb)   = ', DSO%DD%sigsdn,    ' pb'
  endif

 
  if(Flag%DD_rescal) then !oh2 < oh2_exp needs rescaling...
     stop 'Need to write the rescaling in dsinterface.f90'
  end if

 endif

endif

! Now we calculate the ID parameters

if (Flag%ID_predict) then

     if (debug) write ( * , * ) '    >>> Calculating ID parameters...' 

!
     if(Flag%ID%gadiff .or. Flag%ID%gac .or. Flag%ID%gam &
        .or. Flag%ID%muhadiff .or. Flag%ID%muha) hmd = .true.

! First of all, we decide on the density profile of the dark matter halo  

     if(Flag%ID%pb .or. Flag%ID%db) & 
      call dspbset(DSO%ID_in%pbmodel) !sets up antiproton and antideutron diffusion model (See theroutine for details)       
     if(Flag%ID%ep) & 
      call dsepset(DSO%ID_in%epmodel) !sets up positron diffusion model (See theroutine for details)
     if(Flag%ID%musu .or. Flag%ID%muea .or. Flag%ID%musudiff & 
       .or. Flag%ID%musudiff .or. Flag%ID%muha .or. Flag%ID%muhadiff ) & 
      call dsntset(DSO%ID_in%ntmodel) !sets up neutrino telescope parameters (See the routine for details)


     do IDmod = 1, DSO%ID_in%num_hm

! Now we initialize the spherically symmetric density profile
! provided with DarkSUSY (see dshmset.f for details).

       if(hmd) then 

        call dshmset(DSO%ID_in%hmodel(IDmod))

        if (debug) write(* ,'(A,G10.4)')  '       >>> halo model: ', DSO%ID_in%hmodel(IDmod)

        if (debug .and. DSO%ID_in%delta_gamma /= 0.d0 .and. time > 1) write(* ,'(A,G10.4)')  '        >>> jpsi for gamma rays: ',jpsi_gamma(IDmod)

        if (debug .and. DSO%ID_in%delta_nt /= 0.d0 .and. time > 1 ) write(* ,'(A,G10.4)')  '        >>> jpsi for neutrinos: ',jpsi_nt(IDmod)  


       endif

       if (time.eq.1) then 

        if(hmd) then


! Currently we take into  account that your
! detector has an angular resolution delta (in sr) and compute
! j averaged over that solid angle

         if((DSO%ID_in%delta_gamma == 0.d0) .and. (DSO%ID_in%delta_nt /= 0.d0)) then
          jpsi_gamma(IDmod) = dshmj(DSO%ID_in%cospsi0)
          jpsi_nt(IDmod) = dshmjave(DSO%ID_in%cospsi0,DSO%ID_in%delta_nt)

         else if((DSO%ID_in%delta_nt == 0.d0) .and. (DSO%ID_in%delta_gamma /= 0.d0)) then
          jpsi_nt(IDmod) = dshmj(DSO%ID_in%cospsi0)
          jpsi_gamma(IDmod) = dshmjave(DSO%ID_in%cospsi0,DSO%ID_in%delta_gamma)

         else if((DSO%ID_in%delta_gamma == 0.d0) .and. (DSO%ID_in%delta_gamma == 0.d0)) then 
          jpsi_nt(IDmod) = dshmj(DSO%ID_in%cospsi0)
          jpsi_gamma(IDmod)= jpsi_nt(IDmod)
         else
           jpsi_gamma(IDmod) = dshmjave(DSO%ID_in%cospsi0,DSO%ID_in%delta_gamma)

           if(DSO%ID_in%delta_gamma == DSO%ID_in%delta_nt) then
            jpsi_nt(IDmod)= jpsi_gamma(IDmod)
           else
            jpsi_nt(IDmod) = dshmjave(DSO%ID_in%cospsi0,DSO%ID_in%delta_nt)      
           endif   
         endif
     
        
         if (debug .and. DSO%ID_in%delta_gamma /= 0.d0) write(* ,'(A,G10.4)')  '        >>> jpsi for gamma rays: ',jpsi_gamma(IDmod)

         if (debug .and. DSO%ID_in%delta_nt /= 0.d0) write(* ,'(A,G10.4)')  '        >>> jpsi for neutrinos: ',jpsi_nt(IDmod)

        endif

        first_idmod = 1   

       endif


   if (Flag%ID%gadiff) then

! i) gamma-ray flux with continuum energy spectrum at a given energy
! egam (GeV)

    if(DSO%ID_in%delta_gamma .eq. 0.d0) then 

     DSO%ID%fluxgadiff(IDmod) = dshrgacontdiff(DSO%ID_in%egam, jpsi_gamma(IDmod), istat) !ph cm^-2 s^-1 GeV^-1 sr^-1

     if (debug) write (* ,'(A,F8.3,A,G10.4,A)') '        >>> Gamma ray fluxes with energy', DSO%ID_in%egam,  ' GeV: ', DSO%ID%fluxgadiff(IDmod), ' cm^-2 s^-1 GeV^-1 sr^-1'

    else

      DSO%ID%fluxgadiff(IDmod) = dshrgacontdiff(DSO%ID_in%egam,jpsi_gamma(IDmod)*DSO%ID_in%delta_gamma, istat) !ph cm^-2 s^-1 GeV^-1

      if (debug) write (* ,'(A,F8.3,A,G10.4,A)') '  dsinterface.f90      >>> Gamma ray fluxes with energy', DSO%ID_in%egam,  ' GeV: ', DSO%ID%fluxgadiff(IDmod), ' cm^-2 s^-1 GeV^-1' 

    endif


   endif


   if (Flag%ID%gac) then

! ii) gamma-ray flux with continuum energy spectrum integrated above
! some given threshold egath (GeV)


    if(DSO%ID_in%delta_gamma == 0.d0) then 

      DSO%ID%fluxgac(IDmod) = dshrgacont(DSO%ID_in%egath,jpsi_gamma(IDmod),istat) !ph cm^-2 s^-1

      if (debug)  write (* ,'(A,F8.3,A,G10.4,A)') '         >>> Gamma ray fluxes with threshold enery', &
        DSO%ID_in%egath, ' GeV: ', DSO%ID%fluxgac(IDmod), ' cm^-2 s^-1 sr^-1' 

     else

      DSO%ID%fluxgac(IDmod) = dshrgacont(DSO%ID_in%egath,jpsi_gamma(IDmod)*DSO%ID_in%delta_gamma,istat) !ph cm^-2 s^-1

      if (debug)  write (* ,'(A,F8.3,A,G10.4,A)') '        >>> Gamma ray fluxes with threshold enery', &
        DSO%ID_in%egath, ' GeV: ', DSO%ID%fluxgac(IDmod), ' cm^-2 s^-1 ' 


     endif



   endif

   if (Flag%ID%gam) then

! iii) monocromatic monochromatic gamma-ray flux induced by 
! 1-loop annihilationprocesses into a 2-body final state containing a photon. 
! There are two such final states: the 2 photon final state (with 
! energy of the photons equal to the neutralino mass mx) and
! the final state with a photon and a Z boson (with energy of 
! the photon equal to mx * (1 - mz**2 / (4 * mx**2)) and mz
! the mass of the Z boson)


    if(DSO%ID_in%delta_gamma == 0.d0) then 

     call dshrgaline(jpsi_gamma(IDmod),DSO%ID%fluxgaga(IDmod),DSO%ID%fluxgaz(IDmod))
  
   
     if (debug)  write (* ,'(A,F8.3,A,G10.4,A)') '        >>> Monocromatic gamma ray fluxes gamma with energy ', mass(lsp) , ' GeV: ', DSO%ID%fluxgaga(IDmod), ' cm^-2 s^-1 sr^-1'
 

     if (debug)  write (* ,'(A,F8.3,A,G10.4,A)') '        >>> Monocromatic gamma ray fluxes gamma with energy ', mass(lsp) * (1.d0 - mass(kz)**2.d0 / (4.d0 * mass(lsp)**2.d0)) , &
        ' GeV: ', DSO%ID%fluxgaz(IDmod), ' cm^-2 s^-1 sr^-1'

    else

     call dshrgaline(jpsi_gamma(IDmod)*DSO%ID_in%delta_gamma,DSO%ID%fluxgaga(IDmod),DSO%ID%fluxgaz(IDmod))
  
   
     if (debug)  write (* ,'(A,F8.3,A,G10.4,A)') '        >>> Monocromatic gamma ray fluxes gamma with energy ', mass(lsp) , ' GeV: ', DSO%ID%fluxgaga(IDmod), ' cm^-2 s^-1 '
 

     if (debug)  write (* ,'(A,F8.3,A,G10.4,A)') '        >>> Monocromatic gamma ray fluxes gamma with energy ', mass(lsp) * (1.d0 - mass(kz)**2.d0 / (4.d0 * mass(lsp)**2.d0)) , &
        ' GeV: ', DSO%ID%fluxgaz(IDmod), ' cm^-2 s^-1  '

    endif


   endif


   if (Flag%ID%pb) then

! Antiproton rates, interstellar, solarmodulated
! Now we come to the antiproton routines. We get the differential
! spectrum in units of GeV^-1 cm^-2 s^-1 sr^-1 by calling
! dshrpbardiff with the antiproton kinetic energy as the first
! argument. The second argument determines if the flux is solar
! modulated (a la Perko) or not (0=no solar modulation, 1=solar
! modulation). The third argument determines how the diffusion
! equations are solved. If the argument is
!          1 = they are solved for the requested energy only
!          2 = they are tabulated for a large range of energies for the first
!              call and the table is used for subsequent calls.
!          3 = as 2, but the table is also written to disk in the current
!              directory on the first call.
!          4 = the table is read from disk on first call, and is used for
!              that and subsequent calls.
! The tabulation can take several minutes (or hours), 
! so if you are only interested in a few models, don't tabulate,
! but if you want to calculate many models as for what is designed 
! the Markov Chain method, do tabulate 
! (i.e. using any of the options 2-4). Some standard tables are 
! included in the distribution and are available in files
! of the form dat/pbtd-*.dat.
! We include a boost factor.

    DSO%ID%phipb(IDmod) = DSO%ID_in%BF * dshrpbardiff(DSO%ID_in%epb, 1, 4) ! use tables, faster after slower startup 


    if (debug)  write (*, '(A,F7.3,A,G10.4,A,F7.3)') '        >>> Antiproton fluxes for energy', & 
          DSO%ID_in%epb, ' GeV: ', DSO%ID%phipb(IDmod), ' GeV^-1 cm^-2 s^-1 sr^-1 for a boost factor: ',  DSO%ID_in%BF 

   endif


   if (Flag%ID%db) then

! We can also calculate the flux of antideutrons. The call is very
! similar to that for pbar. The dbar's use the same diffusion model
! as the pbar and can thus be changed with a call to dspbset.

    DSO%ID%phidb(IDmod) = DSO%ID_in%BF * dshrdbardiff(DSO%ID_in%edb, 1, 4)   ! use tables, faster after slower startu


    if (debug) write(*, '(A,F7.3,A,G10.4,A,f7.3)')  '        >>> Antideutron fluxes for energy', &    
       DSO%ID_in%edb, ' GeV: ', DSO%ID%phidb(IDmod), ' GeV^-1 cm^-2 s^-1 sr^-1 for a boost factor: ',  DSO%ID_in%BF  


   endif


   if (Flag%ID%ep) then


! We now continue with the rates of positrons from neutralino
! halo annihilation. With the default propagation model 'esu04'
! (Baltz and Edsjo propgation with new parameters), you get
! the differential flux with a call like below. The first argument
! is the positron energy (GeV), the second one gives the 
! option of tabulating the differential spectrum of positrons
! for each model (1: computed directly, 2: tabulated). Useful 
! for when one wants to compute the fluxes for a set of positron 
! energies for each model. The last argument determines
! if the diffusion model should be read from disk or created on the
! spot. Default is to read from disk, and if the file does not 
! exist, it will recreate it. Some standard files are available in 
! dat/eptab-*.dat and dat/epgretab-*.dat.
! switch to old version of ep fluxes routines since there are 
! numerical problems with latest version in NFW and Moore-type 
! profiles.
! A boost factor BF is applied. 

    DSO%ID%phiep(IDmod) = DSO%ID_in%BF * dsepdiff(DSO%ID_in%eep, 1, 4) !GeV^-1 cm^-2 s^-1 sr^-2


    if (debug) write (*,'(A,F7.3,A,G10.4,A,F7.3)') '        >>> Positron fluxes for energy ',&  
        DSO%ID_in%eep, ' GeV: ', DSO%ID%phiep(IDmod), ' GeV^-1 cm^-2 s^-1 sr^-1 for a boost factor: ',  DSO%ID_in%BF   

   endif



! Positron fraction. The second argument is the boost factor
! which represents the effect of clumps in the galactic halo and 
! which is parameterised by a multiplicative factor called BF. 

   if (Flag%ID%epfrac) then  

    DSO%ID%phiep_frac(IDmod) = dsepfr(DSO%ID_in%eep, DSO%ID_in%BF)


    if (debug)  write (* ,'(A,G10.4,A,F7.3)') '        >>> Positron fluxes fraction ',&
       DSO%ID%phiep_frac(IDmod), ' for a boost factor: ',  DSO%ID_in%BF

   endif


 
! Neutrino telescopes rates
! To calculate the rates in neutrino telescopes we call dsntrates. 
! We can either calculate the flux of neutrinos, the conversion rate
! (per volume element) of neutrinos to muons or the muon flux. This
! is determined by the argument rtype. We can also choose the
! energy threshold and the maximal half-aperture angle from the
! center of the Earth/Sun we are interested in. The rates for both
! the earth and the sun are returned in units of km^-2 yr^-1 for
! rtype=1,3 and in units of km^-3 yr^-1 for rtype=2. If some
! warnings were issued, the flag istat is non-zero.
!
! The defualt calculation method is to use the full expressions by
! Gould and numerically integrate them over the velocity 
! distribution as specified by the halo profile, e.g. a gaussian 
! for an isothermal sphere. This numerical integration is rather 
! slow and there is thus an option to use tables (read from disk, 
! recreated if absent) instead. This is the default. 
! Changing to numerical integration directly is handled by a call 
! to dsntset (see dsntset.f for details). Also other (approximate) 
! formulae or the Damour Krauss population are available and can be 
! chosen by a call to dsntset.
! Note: For the earth, which captures from a population of WIMPs 
! bound in the solar system, a new estimate (Lundberg and Edsjo, 
! astro-ph/0401113) is used as a default.


   if (Flag%ID%musu .or. Flag%ID%muea) then 


    call dsntrates(DSO%ID_in%eth,DSO%ID_in%thmax,DSO%ID_in%rtype,DSO%ID%rateea(IDmod),DSO%ID%ratesu(IDmod),istat) ! km^-2 yr^-1


    if(DSO%ID_in%rtype == 2) then
      rateneut_units =  ' km^-3 yr^-1'
    else
      rateneut_units =  ' km^-2 yr^-1'
    endif


    if (Flag%ID%musu) then
     if (debug)  write (* ,'(A,F7.3,A,G10.4,A)') '        >>> Muon flux from the Sun with threshold energy',&
       DSO%ID_in%eth,' GeV: ', DSO%ID%ratesu(IDmod), rateneut_units
    endif
    
   

    if (Flag%ID%muea) then 
     if (debug)   write (* ,'(A,F7.3,A,G10.4,A)') '       >>> Muon flux from the Earth with threshold energy',&
       DSO%ID_in%eth,' GeV: ', DSO%ID%rateea(IDmod), rateneut_units
    endif
   endif


   if (Flag%ID%musudiff .or. Flag%ID%mueadiff) then

! If you want differential rates, you instead call dsntdiffrates. 
! The units of the rates are then km^-2 yr^-1 GeV^-1 degree^-1 for
! rtype=1 or rtype=3 and km^-3 yr^-1 GeV^-1 degree^-1 for
! rtype=2.


    call dsntdiffrates(DSO%ID_in%enu,DSO%ID_in%theta,DSO%ID_in%rtype,DSO%ID%rateeadiff(IDmod),DSO%ID%ratesudiff(IDmod),istat)  


     
    if(DSO%ID_in%rtype == 2) then
      drateneut_units =  ' km^-3 yr^-1 GeV^-1 degree^-1'
    else
      drateneut_units =  ' km^-2 yr^-1 GeV^-1 degree^-1'
    endif


    if (Flag%ID%musudiff) then
     if (debug)  write (* ,'(A,F7.3,A,G10.4,A)') '       >>> Diff muon flux from the Sun with energy',&
       DSO%ID_in%enu,' GeV: ', DSO%ID%ratesudiff(IDmod), drateneut_units   
    endif

    if (Flag%ID%mueadiff) then
     if (debug)   write (* ,'(A,F7.3,A,G10.4,A)') '       >>> Diff muon flux from the Earth with energy',&
       DSO%ID_in%enu,' GeV: ', DSO%ID%rateeadiff(IDmod), drateneut_units 
    endif


   endif


   if (Flag%ID%muha) then 

! It is also possible to calculate the neutrino-induced muon rates
! in neutrino telescopes that would occur from neutralino
! annihilations in our galactic halo. The neutrino-induced muon
! flux above a threshold Eth is given by


    if(DSO%ID_in%delta_nt == 0.d0) then 

     DSO%ID%phimuhalo(IDmod) = dshrmuhalo(jpsi_nt(IDmod),DSO%ID_in%eth,nmusigmav,istat)


     if (debug)  write(*,'(A,F7.3,A,G10.4,A)') '        >>> Muon fluxes from the halo with threshold energy', & 
      DSO%ID_in%eth, ' GeV: ', DSO%ID%phimuhalo(IDmod), ' km^-2 yr^-1 sr^-1'

    else

     DSO%ID%phimuhalo(IDmod) = dshrmuhalo(jpsi_nt(IDmod)*DSO%ID_in%delta_nt,DSO%ID_in%eth,nmusigmav,istat)


     if (debug)  write(*,'(A,F7.3,A,G10.4,A)') '       >>> Muon fluxes from the halo with threshold energy', & 
      DSO%ID_in%eth, ' GeV: ', DSO%ID%phimuhalo(IDmod), ' km^-2 yr^-1 '

    endif


   endif

  if (Flag%ID%muhadiff) then     

! If we instead wanted the differential flux, we would call

    if(DSO%ID_in%delta_nt == 0.d0) then 

     DSO%ID%phimuhalodiff(IDmod) = dshrmudiff(jpsi_nt(IDmod),DSO%ID_in%enu,nmusigmav,istat)

   
     if (debug)  write(*,'(A,F7.3,A,G10.4,A)') '        >>> Diff muon fluxes from the halo with energy', & 
      DSO%ID_in%enu, ' GeV: ', DSO%ID%phimuhalodiff(IDmod), ' km^-2 yr^-1 GeV^-1 sr^-1'

   else
     DSO%ID%phimuhalodiff(IDmod) = dshrmudiff(jpsi_nt(IDmod)*DSO%ID_in%delta_nt,DSO%ID_in%enu,nmusigmav,istat)

   
     if (debug)  write(*,'(A,F7.3,A,G10.4,A)') '        >>> Diff muon fluxes from the halo with energy', & 
      DSO%ID_in%enu, ' GeV: ', DSO%ID%phimuhalodiff(IDmod), ' km^-2 yr^-1 GeV^-1'

   endif


   endif


   if (Flag%ID%sigmav) then 

! Now we compute the annihilation cross section
! sigma v at p=0 for neutralino-neutralino annihilation.

     if(IDmod == 1) DSO%ID%sigmav = dssigmav(0) 

     if (debug)  write(*,'(A,G10.4,A)') '        >>> Neutralino annihilation cross section sigma v at p=0: ', DSO%ID%sigmav, ' cm^3 s^-1 '

   endif

   if (Flag%ID%evar) DSO%ID%e = DSO%ID_in%enu 

   if(debug)  write ( *, *)

  enddo

  time = time + 1 

  first_idmod = 0 !undecouples model dependent tabulation

endif

 
! Now we calculate the collider parameters

if(.not. Flag%ID%evar .or. (Flag%ID%evar .and. (DSO%ID_in%egam == DSO%ID_in%efluxes_i))) then 

 if (Flag%Collider_predict) then

  if (debug) write ( * , * ) '   >>> Calculating collider parameters...' 

  call EWPO(DSO%Coll%mw, DSO%Coll%sweff2) 

  if (debug) write ( * , * ) '       >>> MW: ', DSO%Coll%mw, ' GeV'
  if (debug) write ( * , * ) '       >>> sw2: ', DSO%Coll%sweff2

! BR(B -> Xs gamma) + BR(B -> Xs l^+ l^-) through 
! sign(Ampl(B -> Xs gamma)_tot/Ampl(B -> Xs gamma)_SM) 

  bsg_SM = bsgamma_intp(mass(kt), alph3mz) * 1.d-4

  call sheffbsgamma (rbsg, DSO%Coll%sgn_MSSMoSM, 1)

! We substract the old SM value from the MSSM one and add the new one.

  DSO%Coll%bsgamma = rbsg(2) - rbsg(1) + bsg_SM

  if (debug) write ( * , * ) '       >>> b->s gamma: ', DSO%Coll%bsgamma  

  if (debug) write ( * , * ) '       >>> sgn_MSSMoSM: ', DSO%Coll%sgn_MSSMoSM

! BR(Bs -> mu^+ mu^-)  

  call bsmm (DSO%Coll%brbsmm)  

  if (debug)   write ( * , * ) '       >>> b->mu+ mu-: ', DSO%Coll%brbsmm

! (g-2)_SUSY

  DSO%Coll%gm2amp = dsgm2muon ()  

  if (debug) write ( * , * ) '       >>> g-2 amplitude [a_mu = (g-2)/2 = ]: ', &
      DSO%Coll%gm2amp  

! [BR(h-> b bbar)/BR(h->WW*])_MSSM/[BR(h-> b bbar)/BR(h->WW*)]_SM
! as computed by HDECAY

  DSO%Coll%brr = 0.d0

  if (.false.) then !this part deactivated, use Feynhiggs instead
     ihiggs = 0  
     call dshdecay(ihiggs, brr_SM) !SM branching ratios
     ihiggs = 1  
     call dshdecay(ihiggs, brr_MSSM) !MSSM branching ratios
     DSO%Coll%brr=brr_MSSM/brr_SM
     if (debug)  write ( * , * ) '       >>> ratio branching ratios: ', & 
     DSO%Coll%brr 
  end if


 end if !collider predict
endif

! ------ WARNING FLAGS -------

if(iwar.ne.0.or.suwar.ne.0.or.aswar.ne.0.or.hdecwar.ne.0 ) then 
   DSO%PrintOutParams =.true.
   print*, ' Warning flags from DS interface: ' 
endif

if(iwar.ne.0) print*,' from relic density routines iwar    = ', iwar

if(suwar.ne.0) print*,' from Higgs widths comp. suwar   = ', suwar

if(aswar.ne.0) print*,' from annihilation rates routines aswar   = ', aswar

if(hdecwar.ne.0) print*,' from hdecay hdecwar = ', hdecwar


end subroutine ds

