*****************************************************************************
*** function phiith integrates dsmudydth over the angle theta.
*** it is the yield from particle 1 (which decays from m0)
*** that is calculated. particle one corresponds to channel ch.
*** units: 1.0e-30 m**-2 (annihilation)**-1
*****************************************************************************

      real*8 function dsmuyieldfth(e0,m0,mp1,mp2,emuthr,thmax,ch,wh,
     &  yieldk,yieldv,istat)
      implicit none
      include 'dsmucom.h'
      include 'dsidtag.h'
      include 'dsio.h'

c------------------------ variables ------------------------------------

      real*8 e0,m0,mp1,mp2,emuthr,thmax,m00,e00,thu
      integer ch,istat,yieldk,yieldv
      character*2 wh

      real*8 sum,emax,emin,e1cm,pi
      parameter (pi=3.1415926535)
      logical wb,chok

c------------------------ functions ------------------------------------

      external dsmudydth
      real*8 dsmudydth,dsmuyield_int

c-----------------------------------------------------------------------

      dsmuyieldfth=0.0
      wb=.true.
      m00=m0
      e00=e0
c...take care of slight numerical inaccuracy problems
      if (m00.lt.(mp1+mp2)) then
        if (m00.gt.0.99*(mp1+mp2)) then
          m00=(mp1+mp2)*1.0001
          if(e0.lt.m00) e00=m00
        else
          write(*,*) 'error in dsmuyieldfth: a particle with mass ',m0
          write(*,*) 'is let to decay to two particles with mass ',
     &      mp1,' and ',mp2
          write(*,*) 'which is not energetically allowed.'
          write(*,*) 'particle 1 has code ',ch
          write(*,*) 'the yield from this decay is put to 0.0'
          write(*,*) 'model: ',idtag
          dsmuyieldfth=0.0
          return
        endif
      endif
c...calculate maximum and minimum energy of particle 1.
      e1cm=(m00**2+mp1**2-mp2**2)/(2.0*m00)
      emin=e00/m00*(e1cm - sqrt(e00**2-m00**2)/e00*
     +  sqrt(e1cm**2-mp1**2))
      emax=e00/m00*(e1cm + sqrt(e00**2-m00**2)/e00*
     +  sqrt(e1cm**2-mp1**2))

      if (emax.gt.emuthr.and.emin.lt.emuthr) then
        thu = acos((emuthr*m00/e00-e1cm)/(sqrt(e00**2-m00**2)
     &  /e00*sqrt(e1cm**2-mp1**2)))
      else
        thu = pi
      endif
c...check if mneu within correct bounds
      chok = .true.
      if ((ch.eq.1.or.ch.eq.2.or.ch.eq.4).and.emax.lt.lb(ch).and.
     &  emuthr.lt.lb(ch).and.emax.gt.emuthr) chok=.false.
      if ((ch.eq.3.or.ch.eq.5.or.ch.eq.6).and.emax.lt.(0.99*lb(ch))
     &  .and.emuthr.lt.(0.99*lb(ch)).and.emax.gt.emuthr) chok=.false.

      if (.not.chok) then
        if (prtlevel.gt.0) then
          write(6,*)
          write(6,5000)
     +     'warning in dsmuyieldfth: neutralino masses of less than ',
     +      emax,' gev wants to be used,'
          write(6,5010) 'while the lower bound for channel ',ch,
     +      ' is ',lb(ch),' gev.'
          write(6,*) 'the yield is put to 0.0 for these too low masses.'
          write(6,*) 'the results can thus only be trusted as a',
     +      ' lower bound.'
        endif
        wb=.false.
        dsmuyieldfth=0.0d0
        istat=(istat/2)*2+1
      endif

      chok = .true.
      if ((ch.eq.1.or.ch.eq.2.or.ch.eq.4).and.emin.lt.lb(ch).and.
     &  emax.ge.lb(ch).and.emuthr.lt.lb(ch).and.
     &  emax.gt.emuthr) chok=.false.
      if ((ch.eq.3.or.ch.eq.5.or.ch.eq.6).and.emin.lt.(0.99*lb(ch))
     &  .and.emax.ge.(0.99*lb(ch)).and.emuthr.lt.(0.99*lb(ch)).and.
     &  emax.gt.emuthr) chok=.false.
      if (.not.chok) then
        if (prtlevel.gt.0) then
          write(6,*)
          write(6,5000)
     &      'warning in dsmuyieldfth: a neutralino mass down to ',
     +      emin,' gev wants to be used,'
          write(6,5010) 'while the lower bound for channel ',ch,
     +      ' is ',lb(ch),' gev.'
          write(6,*) 'the yield is put to 0.0 for these too low masses.'
          write(6,*) 'the results can thus only be trusted as a',
     +      ' lower bound.'
          endif
        istat=(istat/2)*2+1
      endif

      if (emax.gt.ub(ch)) then
        if (prtlevel.gt.0) then
          write(6,*)
          write(6,5000) 
     &      'warning in dsmuyieldfth: a neutralino mass up to ',
     +      emax,' gev wants to be used,'
          write(6,5010) 'while the upper bound for channel ',ch,
     +      ' is ',ub(ch),' gev.'
          write(6,5020) 'a neutralino mass of ',ub(ch),' gev is used',
     +      ' instead for these too high masses.'
          write(6,*) 'the results can thus only be trusted as a',
     +      ' lower bound.'
          endif
        istat=(istat/2)*2+1
      endif

      if (wb) then
c...set common block variables
        phich=ch
        if (wh.eq.'su'.or.wh.eq.'SU') then
          phiwh=1
        else
          phiwh=2
        endif
        phim0=m00
        phim1=mp1
        phim2=mp2
        phie0=e00
        phieth=emuthr
        phithm=thmax
        phifk=yieldk
        phifv=yieldv

        sum=dsmuyield_int(dsmudydth,0.0d0,thu)

c        eps=0.01
c        sum=0.0
c        call gadap(0.0,thu,dsmudydth,eps,sum)
c
c        if (sum.lt.1.0.and.sum.gt.0.0) then
c          write(6,*) 'sum less then 1.0 in gadap (phiith)'
c          eps=0.01*sum
c          eps0=eps
c  100     call gadap(0.0,thu,dsmudydth,eps,sum)
c          if ((eps0/sum).gt.0.01) then
c            eps0=eps0/2.0
c            eps=eps0
c          endif
c          if ((eps0*2/sum).gt.0.01) goto 100
c        endif

        dsmuyieldfth=sum*1.0d-15

      endif

 5000 format(' ',a,f8.2,a)
 5010 format(' ',a,i2,a,f8.2,a)
 5020 format(' ',a,f8.2,a,a)

      end
