c ***********************************************************************
       real*8 function dshonda(nu_type,e_nu,c_th)
c ***********************************************************************
c
c      gives atmospheric neutrino flux according to m. dshonda et al.,
c      phys. rev. d52 (1995) 4985, by interpolating their tables iv
c      and v in log(e_nu) and cos_theta.
c
c      variables:
c                 nu_type - type of neutrino:
c
c                 nu_type=1 muon neutrino
c                 nu_type=2 muon antineutrino
c                 nu_type=3 electron antineutrino (not yet implemented)
c                 nu_type=4 electron antineutrino (not yet implemented)
c
c
c                 e_nu - neutrino energy in gev
c                 c_th - cosine of zenith angle
c      output: dshonda returns neutrino differential flux dn_nu/de in units
c              of cm^(-2)sec^(-1)sr^(-1)gev^(-1).
c
c      allowed energy range: 1 gev to 3.9 tev (above 3.1 tev extrapolation
c      is made)
c
c      lars bergstrom 1996-09-02
c
c
c ***********************************************************************
       implicit none
       real*8 e_nu,c_th
       integer nu_type
       real*8 nu_table(1:2,1:37,1:11),e_log
       real*8 a_i,a_j,fijpl1,fipl1j,fij,fipl1jpl1,r_i,r_j
       integer i_call,i,j
       save i_call, nu_table
       data i_call/0/
       i_call=i_call+1
       if (i_call.eq.1) goto 500   ! initialize tables
 20    continue
       e_log=dlog10(e_nu)
       i=int(10.*e_log)+1
       j=int(10.*c_th)+1
       a_i=10.*e_log+1.
       a_j=10.*c_th+1.
       r_i=a_i-float(i)
       r_j=a_j-float(j)-0.5
       if (i.lt.1.or.i.gt.36) then
       write(*,*) 'dshonda.f: nu energy outside allowed range, e=',
     &  e_nu
       stop
       endif
       if (j.lt.1.or.j.gt.10) then
       write(*,*) 'dshonda.f: cos_theta outside allowed range, c_th=',
     &  c_th
       stop
       endif
       fij=nu_table(nu_type,i,j)
       fijpl1=nu_table(nu_type,i,j+1)
       fipl1j=nu_table(nu_type,i+1,j)
       fipl1jpl1=nu_table(nu_type,i+1,j+1)
       dshonda=(1.-r_i-r_j+r_i*r_j)*fij+r_j*(1.-r_i)*fijpl1+
     1        r_i*(1.-r_j)*fipl1j+r_i*r_j*fipl1jpl1
       dshonda=dshonda/e_nu**3*1.d-4 ! per cm**2
        return
 500    continue
c  the table was generated by this program:
c       program wrte
c       open(22,file='wrt.txt',status='unknown')
c       do 10 i=1,2
c       do 10 j=1,36
c       do 10 k=1,10
c       write(22,1000) i,j,k
c 1000  format(1x,'nu_table(',i2,',',i2,',',i2,')=')
c 10    continue
c       end
c
c
        nu_table( 1, 1, 1)=213.5
        nu_table( 1, 1, 2)=208.0
        nu_table( 1, 1, 3)=202.1
        nu_table( 1, 1, 4)=195.2
        nu_table( 1, 1, 5)=189.1
        nu_table( 1, 1, 6)=182.2
        nu_table( 1, 1, 7)=176.1
        nu_table( 1, 1, 8)=170.7
        nu_table( 1, 1, 9)=165.9
        nu_table( 1, 1,10)=161.9
        nu_table( 1, 2, 1)=245.1
        nu_table( 1, 2, 2)=237.3
        nu_table( 1, 2, 3)=229.2
        nu_table( 1, 2, 4)=221.0
        nu_table( 1, 2, 5)=212.6
        nu_table( 1, 2, 6)=204.3
        nu_table( 1, 2, 7)=196.9
        nu_table( 1, 2, 8)=190.5
        nu_table( 1, 2, 9)=185.0
        nu_table( 1, 2,10)=180.4
        nu_table( 1, 3, 1)=275.5
        nu_table( 1, 3, 2)=264.5
        nu_table( 1, 3, 3)=253.8
        nu_table( 1, 3, 4)=243.4
        nu_table( 1, 3, 5)=233.4
        nu_table( 1, 3, 6)=223.8
        nu_table( 1, 3, 7)=215.4
        nu_table( 1, 3, 8)=208.2
        nu_table( 1, 3, 9)=202.1
        nu_table( 1, 3,10)=197.0
        nu_table( 1, 4, 1)=302.8
        nu_table( 1, 4, 2)=288.0
        nu_table( 1, 4, 3)=274.2
        nu_table( 1, 4, 4)=261.5
        nu_table( 1, 4, 5)=249.8
        nu_table( 1, 4, 6)=239.0
        nu_table( 1, 4, 7)=229.7
        nu_table( 1, 4, 8)=221.9
        nu_table( 1, 4, 9)=215.4
        nu_table( 1, 4,10)=210.2
        nu_table( 1, 5, 1)=327.1
        nu_table( 1, 5, 2)=308.3
        nu_table( 1, 5, 3)=291.5
        nu_table( 1, 5, 4)=276.4
        nu_table( 1, 5, 5)=262.8
        nu_table( 1, 5, 6)=250.8
        nu_table( 1, 5, 7)=240.7
        nu_table( 1, 5, 8)=232.3
        nu_table( 1, 5, 9)=225.7
        nu_table( 1, 5,10)=220.5
        nu_table( 1, 6, 1)=349.6
        nu_table( 1, 6, 2)=327.1
        nu_table( 1, 6, 3)=307.3
        nu_table( 1, 6, 4)=289.9
        nu_table( 1, 6, 5)=274.6
        nu_table( 1, 6, 6)=261.2
        nu_table( 1, 6, 7)=250.2
        nu_table( 1, 6, 8)=241.4
        nu_table( 1, 6, 9)=234.6
        nu_table( 1, 6,10)=229.7
        nu_table( 1, 7, 1)=370.6
        nu_table( 1, 7, 2)=344.4
        nu_table( 1, 7, 3)=321.7
        nu_table( 1, 7, 4)=302.1
        nu_table( 1, 7, 5)=285.1
        nu_table( 1, 7, 6)=270.5
        nu_table( 1, 7, 7)=258.7
        nu_table( 1, 7, 8)=249.5
        nu_table( 1, 7, 9)=242.7
        nu_table( 1, 7,10)=238.0
        nu_table( 1, 8, 1)=390.6
        nu_table( 1, 8, 2)=360.4
        nu_table( 1, 8, 3)=334.7
        nu_table( 1, 8, 4)=312.9
        nu_table( 1, 8, 5)=294.5
        nu_table( 1, 8, 6)=279.2
        nu_table( 1, 8, 7)=266.9
        nu_table( 1, 8, 8)=257.4
        nu_table( 1, 8, 9)=250.4
        nu_table( 1, 8,10)=245.8
        nu_table( 1, 9, 1)=409.4
        nu_table( 1, 9, 2)=374.7
        nu_table( 1, 9, 3)=346.0
        nu_table( 1, 9, 4)=322.3
        nu_table( 1, 9, 5)=302.9
        nu_table( 1, 9, 6)=287.2
        nu_table( 1, 9, 7)=274.7
        nu_table( 1, 9, 8)=265.0
        nu_table( 1, 9, 9)=257.9
        nu_table( 1, 9,10)=253.1
        nu_table( 1,10, 1)=427.2
        nu_table( 1,10, 2)=387.7
        nu_table( 1,10, 3)=355.7
        nu_table( 1,10, 4)=330.2
        nu_table( 1,10, 5)=309.9
        nu_table( 1,10, 6)=294.2
        nu_table( 1,10, 7)=281.6
        nu_table( 1,10, 8)=271.8
        nu_table( 1,10, 9)=264.3
        nu_table( 1,10,10)=259.2
        nu_table( 1,11, 1)=444.5
        nu_table( 1,11, 2)=399.4
        nu_table( 1,11, 3)=363.9
        nu_table( 1,11, 4)=336.4
        nu_table( 1,11, 5)=315.4
        nu_table( 1,11, 6)=299.8
        nu_table( 1,11, 7)=287.2
        nu_table( 1,11, 8)=277.1
        nu_table( 1,11, 9)=269.2
        nu_table( 1,11,10)=263.5
        nu_table( 1,12, 1)=460.8
        nu_table( 1,12, 2)=409.7
        nu_table( 1,12, 3)=370.5
        nu_table( 1,12, 4)=340.9
        nu_table( 1,12, 5)=319.2
        nu_table( 1,12, 6)=303.6
        nu_table( 1,12, 7)=291.0
        nu_table( 1,12, 8)=280.6
        nu_table( 1,12, 9)=272.3
        nu_table( 1,12,10)=265.8
        nu_table( 1,13, 1)=475.6
        nu_table( 1,13, 2)=418.2
        nu_table( 1,13, 3)=375.2
        nu_table( 1,13, 4)=343.5
        nu_table( 1,13, 5)=321.0
        nu_table( 1,13, 6)=305.6
        nu_table( 1,13, 7)=293.0
        nu_table( 1,13, 8)=282.3
        nu_table( 1,13, 9)=273.3
        nu_table( 1,13,10)=266.0
        nu_table( 1,14, 1)=488.7
        nu_table( 1,14, 2)=424.9
        nu_table( 1,14, 3)=378.0
        nu_table( 1,14, 4)=344.3
        nu_table( 1,14, 5)=321.0
        nu_table( 1,14, 6)=305.7
        nu_table( 1,14, 7)=293.0
        nu_table( 1,14, 8)=281.9
        nu_table( 1,14, 9)=272.3
        nu_table( 1,14,10)=264.1
        nu_table( 1,15, 1)=500.0
        nu_table( 1,15, 2)=429.6
        nu_table( 1,15, 3)=378.9
        nu_table( 1,15, 4)=343.1
        nu_table( 1,15, 5)=319.1
        nu_table( 1,15, 6)=303.9
        nu_table( 1,15, 7)=291.1
        nu_table( 1,15, 8)=279.6
        nu_table( 1,15, 9)=269.3
        nu_table( 1,15,10)=260.2
        nu_table( 1,16, 1)=509.6
        nu_table( 1,16, 2)=433.1
        nu_table( 1,16, 3)=379.0
        nu_table( 1,16, 4)=341.4
        nu_table( 1,16, 5)=316.7
        nu_table( 1,16, 6)=301.5
        nu_table( 1,16, 7)=288.5
        nu_table( 1,16, 8)=276.6
        nu_table( 1,16, 9)=265.7
        nu_table( 1,16,10)=255.7
        nu_table( 1,17, 1)=517.6
        nu_table( 1,17, 2)=436.4
        nu_table( 1,17, 3)=379.5
        nu_table( 1,17, 4)=340.4
        nu_table( 1,17, 5)=315.0
        nu_table( 1,17, 6)=299.7
        nu_table( 1,17, 7)=286.5
        nu_table( 1,17, 8)=274.2
        nu_table( 1,17, 9)=262.7
        nu_table( 1,17,10)=252.1
        nu_table( 1,18, 1)=523.9
        nu_table( 1,18, 2)=439.4
        nu_table( 1,18, 3)=380.5
        nu_table( 1,18, 4)=340.2
 67     nu_table( 1,18, 5)=314.2
        nu_table( 1,18, 6)=298.4
        nu_table( 1,18, 7)=284.9
        nu_table( 1,18, 8)=272.3
        nu_table( 1,18, 9)=260.4
        nu_table( 1,18,10)=249.3
        nu_table( 1,19, 1)=528.4
        nu_table( 1,19, 2)=442.2
        nu_table( 1,19, 3)=382.0
        nu_table( 1,19, 4)=340.8
        nu_table( 1,19, 5)=314.1
        nu_table( 1,19, 6)=297.8
        nu_table( 1,19, 7)=283.9
        nu_table( 1,19, 8)=270.8
        nu_table( 1,19, 9)=258.7
        nu_table( 1,19,10)=247.4
        nu_table( 1,20, 1)=531.2
        nu_table( 1,20, 2)=444.6
        nu_table( 1,20, 3)=383.9
        nu_table( 1,20, 4)=342.1
        nu_table( 1,20, 5)=314.7
        nu_table( 1,20, 6)=297.8
        nu_table( 1,20, 7)=283.3
        nu_table( 1,20, 8)=270.0
        nu_table( 1,20, 9)=257.7
        nu_table( 1,20,10)=246.3
        nu_table( 1,21, 1)=532.1
        nu_table( 1,21, 2)=446.7
        nu_table( 1,21, 3)=383.3
        nu_table( 1,21, 4)=344.2
        nu_table( 1,21, 5)=316.2
        nu_table( 1,21, 6)=298.4
        nu_table( 1,21, 7)=283.2
        nu_table( 1,21, 8)=269.6
        nu_table( 1,21, 9)=257.2
        nu_table( 1,21,10)=246.1
        nu_table( 1,22, 1)=531.7
        nu_table( 1,22, 2)=448.0
        nu_table( 1,22, 3)=388.1
        nu_table( 1,22, 4)=345.9
        nu_table( 1,22, 5)=317.2
        nu_table( 1,22, 6)=298.4
        nu_table( 1,22, 7)=282.6
        nu_table( 1,22, 8)=268.6
        nu_table( 1,22, 9)=256.3
        nu_table( 1,22,10)=245.3
        nu_table( 1,23, 1)=530.4
        nu_table( 1,23, 2)=448.0
        nu_table( 1,23, 3)=388.5
        nu_table( 1,23, 4)=346.1
        nu_table( 1,23, 5)=316.6
        nu_table( 1,23, 6)=296.9
        nu_table( 1,23, 7)=280.4
        nu_table( 1,23, 8)=266.0
        nu_table( 1,23, 9)=253.6
        nu_table( 1,23,10)=242.9
        nu_table( 1,24, 1)=528.2
        nu_table( 1,24, 2)=446.8
        nu_table( 1,24, 3)=387.4
        nu_table( 1,24, 4)=344.6
        nu_table( 1,24, 5)=314.5
        nu_table( 1,24, 6)=293.7
        nu_table( 1,24, 7)=276.6
        nu_table( 1,24, 8)=261.9
        nu_table( 1,24, 9)=249.3
        nu_table( 1,24,10)=238.7
        nu_table( 1,25, 1)=525.2
        nu_table( 1,25, 2)=444.3
        nu_table( 1,25, 3)=384.9
        nu_table( 1,25, 4)=341.6
        nu_table( 1,25, 5)=310.8
        nu_table( 1,25, 6)=289.0
        nu_table( 1,25, 7)=271.2
        nu_table( 1,25, 8)=256.2
        nu_table( 1,25, 9)=243.5
        nu_table( 1,25,10)=233.0
        nu_table( 1,26, 1)=521.3
        nu_table( 1,26, 2)=440.6
        nu_table( 1,26, 3)=380.9
        nu_table( 1,26, 4)=337.1
        nu_table( 1,26, 5)=305.5
        nu_table( 1,26, 6)=282.9
        nu_table( 1,26, 7)=264.5
        nu_table( 1,26, 8)=249.0
        nu_table( 1,26, 9)=236.2
        nu_table( 1,26,10)=225.8
        nu_table( 1,27, 1)=516.4
        nu_table( 1,27, 2)=435.1
        nu_table( 1,27, 3)=374.9
        nu_table( 1,27, 4)=330.6
        nu_table( 1,27, 5)=298.4
        nu_table( 1,27, 6)=275.2
        nu_table( 1,27, 7)=256.3
        nu_table( 1,27, 8)=240.7
        nu_table( 1,27, 9)=227.8
        nu_table( 1,27,10)=217.4
        nu_table( 1,28, 1)=510.3
        nu_table( 1,28, 2)=427.2
        nu_table( 1,28, 3)=366.1
        nu_table( 1,28, 4)=321.4
        nu_table( 1,28, 5)=289.0
        nu_table( 1,28, 6)=265.7
        nu_table( 1,28, 7)=246.8
        nu_table( 1,28, 8)=231.3
        nu_table( 1,28, 9)=218.6
        nu_table( 1,28,10)=208.3
        nu_table( 1,29, 1)=503.0
        nu_table( 1,29, 2)=417.3
        nu_table( 1,29, 3)=355.0
        nu_table( 1,29, 4)=309.9
        nu_table( 1,29, 5)=277.6
        nu_table( 1,29, 6)=254.6
        nu_table( 1,29, 7)=236.1
        nu_table( 1,29, 8)=220.9
        nu_table( 1,29, 9)=208.5
        nu_table( 1,29,10)=198.5
        nu_table( 1,30, 1)=494.6
        nu_table( 1,30, 2)=405.4
        nu_table( 1,30, 3)=341.7
        nu_table( 1,30, 4)=296.4
        nu_table( 1,30, 5)=264.5
        nu_table( 1,30, 6)=242.3
        nu_table( 1,30, 7)=224.5
        nu_table( 1,30, 8)=209.8
        nu_table( 1,30, 9)=197.8
        nu_table( 1,30,10)=188.2
        nu_table( 1,31, 1)=485.3
        nu_table( 1,31, 2)=391.7
        nu_table( 1,31, 3)=326.5
        nu_table( 1,31, 4)=281.1
        nu_table( 1,31, 5)=250.0
        nu_table( 1,31, 6)=228.9
        nu_table( 1,31, 7)=212.0
        nu_table( 1,31, 8)=198.1
        nu_table( 1,31, 9)=186.7
        nu_table( 1,31,10)=177.4
        nu_table( 1,32, 1)=474.4
        nu_table( 1,32, 2)=375.7
        nu_table( 1,32, 3)=308.8
        nu_table( 1,32, 4)=263.5
        nu_table( 1,32, 5)=233.4
        nu_table( 1,32, 6)=213.7
        nu_table( 1,32, 7)=198.1
        nu_table( 1,32, 8)=185.1
        nu_table( 1,32, 9)=174.3
        nu_table( 1,32,10)=165.3
        nu_table( 1,33, 1)=461.6
        nu_table( 1,33, 2)=356.9
        nu_table( 1,33, 3)=288.2
        nu_table( 1,33, 4)=243.2
        nu_table( 1,33, 5)=214.3
        nu_table( 1,33, 6)=196.5
        nu_table( 1,33, 7)=182.2
        nu_table( 1,33, 8)=170.2
        nu_table( 1,33, 9)=160.0
        nu_table( 1,33,10)=151.5
        nu_table( 1,34, 1)=447.1
        nu_table( 1,34, 2)=335.8
        nu_table( 1,34, 3)=265.4
        nu_table( 1,34, 4)=220.9
        nu_table( 1,34, 5)=193.7
        nu_table( 1,34, 6)=177.8
        nu_table( 1,34, 7)=165.0
        nu_table( 1,34, 8)=154.1
        nu_table( 1,34, 9)=144.6
        nu_table( 1,34,10)=136.5
        nu_table( 1,35, 1)=431.1
        nu_table( 1,35, 2)=313.0
        nu_table( 1,35, 3)=241.2
        nu_table( 1,35, 4)=197.6
        nu_table( 1,35, 5)=172.2
        nu_table( 1,35, 6)=158.3
        nu_table( 1,35, 7)=147.1
        nu_table( 1,35, 8)=137.3
        nu_table( 1,35, 9)=128.6
        nu_table( 1,35,10)=120.9
        nu_table( 1,36, 1)=406.0
        nu_table( 1,36, 2)=296.5
        nu_table( 1,36, 3)=228.2
        nu_table( 1,36, 4)=185.2
        nu_table( 1,36, 5)=158.4
        nu_table( 1,36, 6)=142.0
        nu_table( 1,36, 7)=128.8
        nu_table( 1,36, 8)=117.5
        nu_table( 1,36, 9)=107.8
        nu_table( 1,36,10)=99.44

c   muon antineutrinos:

        nu_table( 2, 1, 1)=217.1
        nu_table( 2, 1, 2)=210.4
        nu_table( 2, 1, 3)=203.3
        nu_table( 2, 1, 4)=196.0
        nu_table( 2, 1, 5)=188.3
        nu_table( 2, 1, 6)=180.6
        nu_table( 2, 1, 7)=173.5
        nu_table( 2, 1, 8)=167.0
        nu_table( 2, 1, 9)=161.1
        nu_table( 2, 1,10)=155.7
        nu_table( 2, 2, 1)=246.6
        nu_table( 2, 2, 2)=237.4
        nu_table( 2, 2, 3)=228.0
        nu_table( 2, 2, 4)=218.4
        nu_table( 2, 2, 5)=208.7
        nu_table( 2, 2, 6)=199.1
        nu_table( 2, 2, 7)=190.4
        nu_table( 2, 2, 8)=182.6
        nu_table( 2, 2, 9)=175.7
        nu_table( 2, 2,10)=169.6
        nu_table( 2, 3, 1)=274.4
        nu_table( 2, 3, 2)=262.2
        nu_table( 2, 3, 3)=249.9
        nu_table( 2, 3, 4)=237.7
        nu_table( 2, 3, 5)=225.6
        nu_table( 2, 3, 6)=213.8
        nu_table( 2, 3, 7)=203.4
        nu_table( 2, 3, 8)=194.5
        nu_table( 2, 3, 9)=186.9
        nu_table( 2, 3,10)=180.5
        nu_table( 2, 4, 1)=299.6
        nu_table( 2, 4, 2)=283.5
        nu_table( 2, 4, 3)=267.9
        nu_table( 2, 4, 4)=252.8
        nu_table( 2, 4, 5)=238.4
        nu_table( 2, 4, 6)=224.3
        nu_table( 2, 4, 7)=212.4
        nu_table( 2, 4, 8)=202.6
        nu_table( 2, 4, 9)=194.6
        nu_table( 2, 4,10)=188.2
        nu_table( 2, 5, 1)=323.6
        nu_table( 2, 5, 2)=302.6
        nu_table( 2, 5, 3)=283.2
        nu_table( 2, 5, 4)=265.0
        nu_table( 2, 5, 5)=248.2
        nu_table( 2, 5, 6)=232.7
        nu_table( 2, 5, 7)=219.7
        nu_table( 2, 5, 8)=209.2
        nu_table( 2, 5, 9)=200.9
        nu_table( 2, 5,10)=194.6
        nu_table( 2, 6, 1)=348.0
        nu_table( 2, 6, 2)=321.3
        nu_table( 2, 6, 3)=297.5
        nu_table( 2, 6, 4)=276.5
        nu_table( 2, 6, 5)=257.7
        nu_table( 2, 6, 6)=241.1
        nu_table( 2, 6, 7)=227.5
        nu_table( 2, 6, 8)=216.4
        nu_table( 2, 6, 9)=207.7
        nu_table( 2, 6,10)=201.1
        nu_table( 2, 7, 1)=371.7
        nu_table( 2, 7, 2)=338.6
        nu_table( 2, 7, 3)=310.5
        nu_table( 2, 7, 4)=286.6
        nu_table( 2, 7, 5)=266.3
        nu_table( 2, 7, 6)=249.1
        nu_table( 2, 7, 7)=235.0
        nu_table( 2, 7, 8)=223.6
        nu_table( 2, 7, 9)=214.5
        nu_table( 2, 7,10)=207.5
        nu_table( 2, 8, 1)=393.4
        nu_table( 2, 8, 2)=353.9
        nu_table( 2, 8, 3)=321.5
        nu_table( 2, 8, 4)=295.0
        nu_table( 2, 8, 5)=273.4
        nu_table( 2, 8, 6)=255.9
        nu_table( 2, 8, 7)=241.6
        nu_table( 2, 8, 8)=229.9
        nu_table( 2, 8, 9)=220.5
        nu_table( 2, 8,10)=213.2
        nu_table( 2, 9, 1)=412.6
        nu_table( 2, 9, 2)=366.6
        nu_table( 2, 9, 3)=330.2
        nu_table( 2, 9, 4)=301.4
        nu_table( 2, 9, 5)=278.9
        nu_table( 2, 9, 6)=261.5
        nu_table( 2, 9, 7)=247.1
        nu_table( 2, 9, 8)=235.3
        nu_table( 2, 9, 9)=225.6
        nu_table( 2, 9,10)=218.0
        nu_table( 2,10, 1)=429.2
        nu_table( 2,10, 2)=377.0
        nu_table( 2,10, 3)=336.7
        nu_table( 2,10, 4)=305.9
        nu_table( 2,10, 5)=282.7
        nu_table( 2,10, 6)=265.4
        nu_table( 2,10, 7)=251.1
        nu_table( 2,10, 8)=239.2
        nu_table( 2,10, 9)=229.4
        nu_table( 2,10,10)=221.4
        nu_table( 2,11, 1)=443.5
        nu_table( 2,11, 2)=385.1
        nu_table( 2,11, 3)=341.2
        nu_table( 2,11, 4)=308.4
        nu_table( 2,11, 5)=284.5
        nu_table( 2,11, 6)=267.3
        nu_table( 2,11, 7)=253.2
        nu_table( 2,11, 8)=241.2
        nu_table( 2,11, 9)=231.2
        nu_table( 2,11,10)=223.0
        nu_table( 2,12, 1)=455.6
        nu_table( 2,12, 2)=391.0
        nu_table( 2,12, 3)=343.5
        nu_table( 2,12, 4)=308.9
        nu_table( 2,12, 5)=284.3
        nu_table( 2,12, 6)=267.3
        nu_table( 2,12, 7)=253.3
        nu_table( 2,12, 8)=241.3
        nu_table( 2,12, 9)=231.2
        nu_table( 2,12,10)=222.7
        nu_table( 2,13, 1)=465.5
        nu_table( 2,13, 2)=394.8
        nu_table( 2,13, 3)=343.7
        nu_table( 2,13, 4)=307.4
        nu_table( 2,13, 5)=282.3
        nu_table( 2,13, 6)=265.6
        nu_table( 2,13, 7)=251.7
        nu_table( 2,13, 8)=239.7
        nu_table( 2,13, 9)=229.4
        nu_table( 2,13,10)=220.5
        nu_table( 2,14, 1)=473.2
        nu_table( 2,14, 2)=396.2
        nu_table( 2,14, 3)=341.8
        nu_table( 2,14, 4)=303.9
        nu_table( 2,14, 5)=278.4
        nu_table( 2,14, 6)=262.1
        nu_table( 2,14, 7)=248.5
        nu_table( 2,14, 8)=236.5
        nu_table( 2,14, 9)=225.9
        nu_table( 2,14,10)=216.6
        nu_table( 2,15, 1)=478.4
        nu_table( 2,15, 2)=395.3
        nu_table( 2,15, 3)=337.8
        nu_table( 2,15, 4)=298.5
        nu_table( 2,15, 5)=272.8
        nu_table( 2,15, 6)=257.0
        nu_table( 2,15, 7)=243.6
        nu_table( 2,15, 8)=231.6
        nu_table( 2,15, 9)=220.8
        nu_table( 2,15,10)=211.1
        nu_table( 2,16, 1)=481.3
        nu_table( 2,16, 2)=393.0
        nu_table( 2,16, 3)=332.8
        nu_table( 2,16, 4)=292.5
        nu_table( 2,16, 5)=266.7
        nu_table( 2,16, 6)=251.3
        nu_table( 2,16, 7)=238.2
        nu_table( 2,16, 8)=226.3
        nu_table( 2,16, 9)=215.3
        nu_table( 2,16,10)=205.2
        nu_table( 2,17, 1)=482.1
        nu_table( 2,17, 2)=389.9
        nu_table( 2,17, 3)=328.0
        nu_table( 2,17, 4)=286.9
        nu_table( 2,17, 5)=261.2
        nu_table( 2,17, 6)=246.1
        nu_table( 2,17, 7)=233.3
        nu_table( 2,17, 8)=221.4
        nu_table( 2,17, 9)=210.3
        nu_table( 2,17,10)=200.0
        nu_table( 2,18, 1)=480.9
        nu_table( 2,18, 2)=386.3
        nu_table( 2,18, 3)=323.3
        nu_table( 2,18, 4)=281.9
        nu_table( 2,18, 5)=256.2
        nu_table( 2,18, 6)=241.5
        nu_table( 2,18, 7)=228.9
        nu_table( 2,18, 8)=217.1
        nu_table( 2,18, 9)=206.0
        nu_table( 2,18,10)=195.6
        nu_table( 2,19, 1)=477.6
        nu_table( 2,19, 2)=382.0
        nu_table( 2,19, 3)=318.7
        nu_table( 2,19, 4)=277.3
        nu_table( 2,19, 5)=251.8
        nu_table( 2,19, 6)=237.3
        nu_table( 2,19, 7)=224.9
        nu_table( 2,19, 8)=213.2
        nu_table( 2,19, 9)=202.2
        nu_table( 2,19,10)=191.8
        nu_table( 2,20, 1)=472.2
        nu_table( 2,20, 2)=377.1
        nu_table( 2,20, 3)=314.2
        nu_table( 2,20, 4)=273.2
        nu_table( 2,20, 5)=247.9
        nu_table( 2,20, 6)=233.5
        nu_table( 2,20, 7)=221.3
        nu_table( 2,20, 8)=209.8
        nu_table( 2,20, 9)=198.9
        nu_table( 2,20,10)=188.7
        nu_table( 2,21, 1)=464.8
        nu_table( 2,21, 2)=371.6
        nu_table( 2,21, 3)=309.8
        nu_table( 2,21, 4)=269.4
        nu_table( 2,21, 5)=244.4
        nu_table( 2,21, 6)=230.2
        nu_table( 2,21, 7)=218.1
        nu_table( 2,21, 8)=206.8
        nu_table( 2,21, 9)=196.2
        nu_table( 2,21,10)=186.2
        nu_table( 2,22, 1)=456.0
        nu_table( 2,22, 2)=365.2
        nu_table( 2,22, 3)=304.8
        nu_table( 2,22, 4)=265.2
        nu_table( 2,22, 5)=240.6
        nu_table( 2,22, 6)=226.5
        nu_table( 2,22, 7)=214.5
        nu_table( 2,22, 8)=203.4
        nu_table( 2,22, 9)=193.0
        nu_table( 2,22,10)=183.4
        nu_table( 2,23, 1)=446.2
        nu_table( 2,23, 2)=357.5
        nu_table( 2,23, 3)=298.4
        nu_table( 2,23, 4)=259.6
        nu_table( 2,23, 5)=235.4
        nu_table( 2,23, 6)=221.5
        nu_table( 2,23, 7)=209.7
        nu_table( 2,23, 8)=198.8
        nu_table( 2,23, 9)=188.7
        nu_table( 2,23,10)=179.3
        nu_table( 2,24, 1)=435.4
        nu_table( 2,24, 2)=348.6
        nu_table( 2,24, 3)=290.8
        nu_table( 2,24, 4)=252.8
        nu_table( 2,24, 5)=229.1
        nu_table( 2,24, 6)=215.4
        nu_table( 2,24, 7)=203.8
        nu_table( 2,24, 8)=193.1
        nu_table( 2,24, 9)=183.2
        nu_table( 2,24,10)=174.0
        nu_table( 2,25, 1)=423.8
        nu_table( 2,25, 2)=338.6
        nu_table( 2,25, 3)=282.0
        nu_table( 2,25, 4)=244.8
        nu_table( 2,25, 5)=221.6
        nu_table( 2,25, 6)=208.2
        nu_table( 2,25, 7)=196.9
        nu_table( 2,25, 8)=186.4
        nu_table( 2,25, 9)=176.6
        nu_table( 2,25,10)=167.6
        nu_table( 2,26, 1)=411.5
        nu_table( 2,26, 2)=327.7
        nu_table( 2,26, 3)=272.1
        nu_table( 2,26, 4)=235.7
        nu_table( 2,26, 5)=213.2
        nu_table( 2,26, 6)=200.1
        nu_table( 2,26, 7)=189.1
        nu_table( 2,26, 8)=178.8
        nu_table( 2,26, 9)=169.2
        nu_table( 2,26,10)=160.3
        nu_table( 2,27, 1)=398.7
        nu_table( 2,27, 2)=316.4
        nu_table( 2,27, 3)=262.0
        nu_table( 2,27, 4)=225.4
        nu_table( 2,27, 5)=204.3
        nu_table( 2,27, 6)=191.5
        nu_table( 2,27, 7)=180.6
        nu_table( 2,27, 8)=170.4
        nu_table( 2,27, 9)=160.9
        nu_table( 2,27,10)=152.0
        nu_table( 2,28, 1)=385.6
        nu_table( 2,28, 2)=305.5
        nu_table( 2,28, 3)=252.4
        nu_table( 2,28, 4)=217.5
        nu_table( 2,28, 5)=195.7
        nu_table( 2,28, 6)=182.7
        nu_table( 2,28, 7)=171.6
        nu_table( 2,28, 8)=161.3
        nu_table( 2,28, 9)=151.6
        nu_table( 2,28,10)=142.6
        nu_table( 2,29, 1)=372.3
        nu_table( 2,29, 2)=294.9
        nu_table( 2,29, 3)=243.2
        nu_table( 2,29, 4)=209.1
        nu_table( 2,29, 5)=187.2
        nu_table( 2,29, 6)=173.8
        nu_table( 2,29, 7)=162.3
        nu_table( 2,29, 8)=151.6
        nu_table( 2,29, 9)=141.7
        nu_table( 2,29,10)=132.5
        nu_table( 2,30, 1)=358.9
        nu_table( 2,30, 2)=284.6
        nu_table( 2,30, 3)=234.6
        nu_table( 2,30, 4)=201.0
        nu_table( 2,30, 5)=179.0
        nu_table( 2,30, 6)=164.9
        nu_table( 2,30, 7)=152.8
        nu_table( 2,30, 8)=141.6
        nu_table( 2,30, 9)=131.3
        nu_table( 2,30,10)=121.9
        nu_table( 2,31, 1)=345.5
        nu_table( 2,31, 2)=274.2
        nu_table( 2,31, 3)=226.4
        nu_table( 2,31, 4)=193.2
        nu_table( 2,31, 5)=170.9
        nu_table( 2,31, 6)=156.0
        nu_table( 2,31, 7)=143.1
        nu_table( 2,31, 8)=131.3
        nu_table( 2,31, 9)=120.7
        nu_table( 2,31,10)=110.9
        nu_table( 2,32, 1)=331.6
        nu_table( 2,32, 2)=264.6
        nu_table( 2,32, 3)=217.9
        nu_table( 2,32, 4)=185.2
        nu_table( 2,32, 5)=162.4
        nu_table( 2,32, 6)=146.5
        nu_table( 2,32, 7)=132.7
        nu_table( 2,32, 8)=120.4
        nu_table( 2,32, 9)=109.3
        nu_table( 2,32,10)=99.4
        nu_table( 2,33, 1)=317.1
        nu_table( 2,33, 2)=253.8
        nu_table( 2,33, 3)=208.7
        nu_table( 2,33, 4)=176.3
        nu_table( 2,33, 5)=153.0
        nu_table( 2,33, 6)=136.0
        nu_table( 2,33, 7)=121.4
        nu_table( 2,33, 8)=108.5
        nu_table( 2,33, 9)=97.2
        nu_table( 2,33,10)=87.1
        nu_table( 2,34, 1)=302.1
        nu_table( 2,34, 2)=242.4
        nu_table( 2,34, 3)=198.8
        nu_table( 2,34, 4)=166.6
        nu_table( 2,34, 5)=142.8
        nu_table( 2,34, 6)=124.8
        nu_table( 2,34, 7)=109.5
        nu_table( 2,34, 8)=96.2
        nu_table( 2,34, 9)=84.7
        nu_table( 2,34,10)=74.7
        nu_table( 2,35, 1)=286.7
        nu_table( 2,35, 2)=230.6
        nu_table( 2,35, 3)=188.2
        nu_table( 2,35, 4)=156.3
        nu_table( 2,35, 5)=132.0
        nu_table( 2,35, 6)=113.2
        nu_table( 2,35, 7)=97.4
        nu_table( 2,35, 8)=83.9
        nu_table( 2,35, 9)=72.4
        nu_table( 2,35,10)=62.6
        nu_table( 2,36, 1)=278.9
        nu_table( 2,36, 2)=210.9
        nu_table( 2,36, 3)=165.2
        nu_table( 2,36, 4)=134.3
        nu_table( 2,36, 5)=113.1
        nu_table( 2,36, 6)=98.3
        nu_table( 2,36, 7)=85.8
        nu_table( 2,36, 8)=74.9
        nu_table( 2,36, 9)=65.5
        nu_table( 2,36,10)=57.2
        do 50 i=1,2 ! extrapolate linearly one step outside range of table
        do 50 j=1,36
 50        nu_table(i,j,11)=2.*nu_table(i,j,10)-nu_table(i,j,9)
        do 60 i=1,2
        do 60 j=1,10
 60        nu_table(i,37,j)=2.*nu_table(i,36,j)-nu_table(i,35,j)
        nu_table(1,37,11)=72.
        nu_table(2,37,11)=44.
        goto 20
        end
