* Couplings.F
* evaluation of Higgs couplings
* this file is part of FeynHiggs
* last modified 15 Nov 06 th

#include "externals.h"
#include "debug.h"
#include "FHCouplings.h"


	subroutine FHCouplings(error,
     &    couplings_, couplingsms_, gammas_, gammasms_, excl)
	implicit none
	integer error, excl
	double complex couplings_(ncouplings)
	double complex couplingsms_(ncouplingsms)
	double precision gammas_(ngammas)
	double precision gammasms_(ngammasms)

#include "FH.h"
#define __SUBROUTINE__ "FHCouplings"

	integer i, lo, hi, mid

#include "ExclData-hbb.F"

	call CheckCouplings(error)
	if( error .ne. 0 ) return

	do i = 1, ncouplings
	  couplings_(i) = couplings(i)
	enddo
	do i = 1, ncouplingsms
	  couplingsms_(i) = couplingsms(i)
	enddo
	do i = 1, ngammas
	  gammas_(i) = gammas(i)
	enddo
	do i = 1, ngammasms
	  gammasms_(i) = gammasms(i)
	enddo

	excl = 0

	lo = 1
	hi = nhbb

	do while( hi - lo .gt. 1 )
	  mid = (lo + hi)/2
	  if( MHiggs(1) .gt. hbb(1,mid) ) then
	    lo = mid
	  else
	    hi = mid
	  endif
	enddo

	if( SBA**2*BR(H0VV(1,1)) .gt. hbb(2,hi) ) excl = 1
	end


************************************************************************

	subroutine CheckCouplings(error)
	implicit none
	integer error

#include "FH.h"

	integer i, h, h1, h2, hv, t, g, s1, s2, c1, c2, n1, n2, uz, Nc
	double precision tmp, resummedDeltaMB
	double precision Mfeff(4,3), cMT, asMH, asMC
	double complex DD(2,3,3)
	double complex cL(2), cR(2), chffL(2,0:1), chffR(2,0:1)
	double complex chhv(2,3), chhh(2,3,4,4), chss(2,3,2,2)
	double complex u1, u2, u3, u4, c
	double precision ga(4), gasm(4)

	double precision Kallen, KallenLR, HXVWidth
	double precision AlfasMH, AlphaS, cfkt2
	external Kallen, KallenLR, HXVWidth, AlfasMH, AlphaS, cfkt2

	double precision hgaga_M2(2,2,3)
	double complex hgaga_M(2,2,3)

	character*2 higgs(4)
	data higgs /"h0", "HH", "A0", "Hp"/

	double precision lambda, ratio
	common /KallenVars/ lambda, ratio

	error = 0
	if( coup_valid ) return

	call CheckHiggsCorr(error)
	if( error .ne. 0 ) return

	do i = 1, ncouplings
	  couplings(i) = 0
	enddo
	do i = 1, ncouplingsms
	  couplingsms(i) = 0
	enddo
	do i = 1, ngammas
	  gammas(i) = 0
	enddo
	do i = 1, ngammasms
	  gammasms(i) = 0
	enddo
	do i = 1, H0FF(3,4,3)
	  ratios(i) = 0
	enddo

* We first compute DD(1,2) = (U,Z)Higgs.Y, where
* Y = {{CA, -SA, 0}, {SA, CA, 0}, {0, 0, 1}}
* Note that SA & CA are the sine and cosine of alpha_*tree*.

* for checking with the MSSM.mod couplings
c#define XHiggs(i,j,uz) Delta(i,j)

	do uz = 1, 2
	  DD(uz,1,1) = CA*XHiggs(1,1,uz) + SA*XHiggs(1,2,uz)
	  DD(uz,2,1) = CA*XHiggs(2,1,uz) + SA*XHiggs(2,2,uz)
	  DD(uz,3,1) = CA*XHiggs(3,1,uz) + SA*XHiggs(3,2,uz)

	  DD(uz,1,2) = CA*XHiggs(1,2,uz) - SA*XHiggs(1,1,uz)
	  DD(uz,2,2) = CA*XHiggs(2,2,uz) - SA*XHiggs(2,1,uz)
	  DD(uz,3,2) = CA*XHiggs(3,2,uz) - SA*XHiggs(3,1,uz)

	  DD(uz,1,3) = XHiggs(1,3,uz)
	  DD(uz,2,3) = XHiggs(2,3,uz)
	  DD(uz,3,3) = XHiggs(3,3,uz)
	enddo

     
!testing
!        print*, DD(2,:,:)    

        
*** COUPLINGS AND DECAY WIDTHS ***

* For couplings where the order is essential, the following
* conventions apply:
* - A charged Higgs at pos. 1 is always the particle, i.e. H-,
*   this fixes all other particles by charge conservation.
* - For the H0ChaCha coupling, the first Chargino is the
*   particle, chi-, and the second one the antiparticle, chi+.

* Higgs -> Vector Vector

	do h = 1, 3

* Higgs -> gamma gamma

	  call hgagaSM(h, GammaSM(H0VV(h,1)),
     &      hgaga_M2(1,1,h), hgaga_M(1,1,h))
	  GammaSMTot(h) = GammaSMTot(h) + GammaSM(H0VV(h,1))

	  call hgaga(h, Gamma(H0VV(h,1)),
     &      hgaga_M2(1,1,h), hgaga_M(1,1,h))
	  GammaTot(h) = GammaTot(h) + Gamma(H0VV(h,1))

* Higgs -> gamma Z

* Higgs -> Z Z

	  do uz = 1, 2
	    cL(uz) = SB*DD(uz,h,1) + CB*DD(uz,h,2)
	  enddo

	  CouplingSM(H0VV(h,3)) = cI*EL*MW/(SW*CW2)
	  Coupling(H0VV(h,3)) = CouplingSM(H0VV(h,3))*cL(uzint)

	  GammaSM(H0VV(h,3)) =
     &      .5D0*HXVWidth(MHiggs(h), MZ, MZ, GammaZ, 1)
	  GammaSMTot(h) = GammaSMTot(h) + GammaSM(H0VV(h,3))

	  ratios(H0VV(h,3)) = Sq(cL(uzext))
	  Gamma(H0VV(h,3)) = GammaSM(H0VV(h,3))*ratios(H0VV(h,3))
	  GammaTot(h) = GammaTot(h) + Gamma(H0VV(h,3))

* Higgs -> W W

	  CouplingSM(H0VV(h,4)) = cI*EL*MW/SW
	  Coupling(H0VV(h,4)) = CouplingSM(H0VV(h,4))*cL(uzint)

	  GammaSM(H0VV(h,4)) =
     &      HXVWidth(MHiggs(h), MW, MW, GammaW, 1)
	  GammaSMTot(h) = GammaSMTot(h) + GammaSM(H0VV(h,4))

	  ratios(H0VV(h,4)) = Sq(cL(uzext))
	  Gamma(H0VV(h,4)) = GammaSM(H0VV(h,4))*ratios(H0VV(h,4))
	  GammaTot(h) = GammaTot(h) + Gamma(H0VV(h,4))

* Higgs -> gluon gluon

	  call hggSM(h, GammaSM(H0VV(h,5)))
	  GammaSMTot(h) = GammaSMTot(h) + GammaSM(H0VV(h,5))

	  call hgg(h, Gamma(H0VV(h,5)))
	  GammaTot(h) = GammaTot(h) + Gamma(H0VV(h,5))

	enddo

* Higgs -> Higgs Vector

	do uz = 1, 2
	  do hv = 1, 3
	    chhv(uz,hv) = CB*DD(uz,hv,1) - SB*DD(uz,hv,2)
	  enddo
	enddo

* Higgs -> Higgs Z

	do hv = 1, 3
	  do h = 1, 3
	    do uz = 1, 2
	      cL(uz) = DD(uz,h,3)*chhv(uz,hv) - DD(uz,hv,3)*chhv(uz,h)
	    enddo

	    Coupling(H0HV(h,hv)) = -EL/(2*SW*CW)*cL(uzint)

	    Gamma(H0HV(h,hv)) =
     &        HXVWidth(MHiggs(h), MHiggs(hv), MZ, GammaZ, 0)*
     &        Sq(cL(uzext))
	    GammaTot(h) = GammaTot(h) + Gamma(H0HV(h,hv))
	  enddo
	enddo

* Higgs -> Higgs W

	do hv = 1, 3
	  do uz = 1, 2
	    cL(uz) = chhv(uz,hv) + cI*DD(uz,hv,3)
	  enddo

	  Coupling(HpHV(hv)) = -cI*EL/(2*SW)*cL(uzint)

	  Gamma(HpHV(hv)) =
     &      HXVWidth(MHiggs(4), MHiggs(hv), MW, GammaW, 0)*
     &      Sq(cL(uzext))
	  GammaTot(4) = GammaTot(4) + Gamma(HpHV(hv))
	enddo

* Higgs -> Fermion Fermion

	resummedDeltaMB = 1/abs(1 + DeltaMBc(2))

	Mfeff(1,1) = 0
	Mfeff(1,2) = 0
	Mfeff(1,3) = 0
	Mfeff(2,1) = Mf(2,1)
	Mfeff(3,1) = Mf(3,1)
	Mfeff(4,1) = Mf(4,1)
	Mfeff(2,2) = Mf(2,2)
	Mfeff(2,3) = Mf(2,3)
	Mfeff(3,3) = Mf(3,3)
 
        

       	cMT = 1/cfkt2(MT)
	asMC = AlphaS(MC)

	do h = 1, 3
	  tmp = cMT*cfkt2(MHiggs(h))
	  asMH = AlfasMH(MHiggs(h))
	  Mfeff(3,2) = MCMT*tmp*
     &      sqrt(1 + asMH/pi*(4/3D0*9/4D0 + 8/3D0*(1 - asMC/asMH)))
	  Mfeff(4,2) = MSMT*tmp
	  Mfeff(4,3) = MBMT*tmp*
     &      sqrt(1 + asMH/pi*(4/3D0*9/4D0 + 8/3D0))

	  do uz = 1, 2
	    chffL(uz,0) = DD(uz,h,2)/CB + cI*DD(uz,h,3)*TB
	    chffR(uz,0) = DD(uz,h,2)/CB - cI*DD(uz,h,3)*TB
	    chffL(uz,1) = DD(uz,h,1)/SB + cI*DD(uz,h,3)/TB
	    chffR(uz,1) = DD(uz,h,1)/SB - cI*DD(uz,h,3)/TB
	  enddo

!ruiz
!          print*,h,DD(1,1,2), CB, DD(1,1,2)/CB, chffL(1,0), chffL(2,0) 


	  do t = 2, 4
	    Nc = ior(t - 1, 1)
	    do uz = 1, 2
	      cL(uz) = chffL(uz,iand(t, 1))
	      cR(uz) = chffR(uz,iand(t, 1))
	    enddo
	    do g = 1, 3
	      if( t .eq. 4 .and. g .eq. 3 ) then
	        do uz = 1, 2
	          cL(uz) = resummedDeltaMB*(cL(uz) +
     &               DBLE(DeltaMBc(2))*chffR(uz,1) +
     &              DIMAG(DeltaMBc(2))*chffL(uz,1))
	          cR(uz) = resummedDeltaMB*(cR(uz) +
     &               DBLE(DeltaMBc(2))*chffL(uz,1) -
     &              DIMAG(DeltaMBc(2))*chffR(uz,1))
!                print*,'g',uz,cL(uz),cR(uz), chffR(uz,1), chffL(uz,1)
	        enddo
!                stop
	      endif

       
	      tmp = -.5D0*EL/(SW*MW)*Mfeff(t,g)
	      LCouplingSM(H0FF(h,t,g)) = cI*tmp
	      RCouplingSM(H0FF(h,t,g)) = LCouplingSM(H0FF(h,t,g))
	      LCoupling(H0FF(h,t,g)) = LCouplingSM(H0FF(h,t,g))*cL(uzint)
	      RCoupling(H0FF(h,t,g)) = RCouplingSM(H0FF(h,t,g))*cR(uzint)

	      tmp = Nc/(8*pi)*MHiggs(h)*tmp**2
	      Gamma(H0FF(h,t,g)) = tmp*
     &          KallenLR(MHiggs(h), Mf(t,g), Mf(t,g), cL(uzext), cR(uzext))
	      GammaTot(h) = GammaTot(h) + Gamma(H0FF(h,t,g))

	      GammaSM(H0FF(h,t,g)) = tmp*lambda
	      GammaSMTot(h) = GammaSMTot(h) + GammaSM(H0FF(h,t,g))

            

	      ratios(H0FF(h,t,g)) = ratio
	    enddo
	  enddo
	enddo

c ruiz

	tmp = cMT*cfkt2(MHiggs(4))
	asMH = AlfasMH(MHiggs(4))
	Mfeff(3,2) = MCMT*tmp*
     &    sqrt(1 + asMH/pi*(4/3D0*9/4D0 + 8/3D0*(1 - asMC/asMH)))
	Mfeff(4,2) = MSMT*tmp
c	Mfeff(4,3) = MBMT*tmp*
c     &    sqrt(1 + asMH/pi*(4/3D0*9/4D0 + 8/3D0))
	Mfeff(4,3) = MBrun*resummedDeltaMB

	c = cI/sqrt2*EL/(SW*MW)

	do t = 2, 4, 2
	  Nc = ior(t - 1, 1)
	  do g = 1, 3
	    cL(1) = c*Mfeff(t,g)*TB
	    LCoupling(HpFF(t/2,g)) = cL(1)

	    cR(1) = c*Mfeff(t-1,g)/TB
	    RCoupling(HpFF(t/2,g)) = cR(1)

	    Gamma(HpFF(t/2,g)) = Nc/(8*pi)*MHiggs(4)*
     &        KallenLR(MHiggs(4), Mf(t-1,g), Mf(t,g), cL(1), cR(1))
	    GammaTot(4) = GammaTot(4) + Gamma(HpFF(t/2,g))
	  enddo
	enddo

* Higgs -> Higgs Higgs

	call H0HHCouplings(chhh, DD)

	do h2 = 1, 4
	  do h1 = 1, 4
	    do h = 1, 3
	      Coupling(H0HH(h,h1,h2)) = chhh(uzint,h,h1,h2)
	      Gamma(H0HH(h,h1,h2)) = 1/(32*pi*MHiggs(h))*
     &          Kallen(MHiggs(h), MHiggs(h1), MHiggs(h2))*
     &          Sq(chhh(uzext,h,h1,h2))
#if 0
	if( h1 .lt. 4 .and. h2 .lt. 4 ) then
	  call hhh(h, h1, h2, tmp)
	  PRINT *, "h,h1,h2=", h,h1,h2
	  PRINT *, "old,new=", Gamma(H0HH(h,h1,h2)), tmp
	endif
#endif
	      GammaTot(h) = GammaTot(h) + Gamma(H0HH(h,h1,h2))
	    enddo
	  enddo
	enddo

* Higgs -> Sfermion Sfermion

	do t = 2, 4
	  Nc = ior(t - 1, 1)
	  do g = 1, 3
	    call H0SfSfCouplings(chss, t, g, DD)
	    do s2 = 1, 2
	      do s1 = 1, 2
	        do h = 1, 3
	          Coupling(H0SfSf(h,s1,s2,t,g)) = chss(uzint,h,s1,s2)
	          Gamma(H0SfSf(h,s1,s2,t,g)) = Nc/(16*pi*MHiggs(h))*
     &              Kallen(MHiggs(h), MSf(s1,t,g), MSf(s2,t,g))*
     &              Sq(chss(uzext,h,s1,s2))
	          GammaTot(h) = GammaTot(h) + Gamma(H0SfSf(h,s1,s2,t,g))
	        enddo
	      enddo
	    enddo
	  enddo
	enddo

	do t = 2, 4, 2
	  Nc = ior(t - 1, 1)
	  do g = 1, 3
	    call HpSfSfCouplings(Coupling(HpSfSf(1,1,t/2,g)), t, g)
	    do s2 = 1, 2
	      do s1 = 1, 2
	        Gamma(HpSfSf(s1,s2,t/2,g)) = Nc/(16*pi*MHiggs(4))*
     &            Kallen(MHiggs(4), MSf(s1,t-1,g), MSf(s2,t,g))*
     &            Sq(Coupling(HpSfSf(s1,s2,t/2,g)))
	        GammaTot(4) = GammaTot(4) + Gamma(HpSfSf(s1,s2,t/2,g))
	      enddo
	    enddo
	  enddo
	enddo

* Higgs -> Chargino Chargino

	do c2 = 1, 2
          do c1 = 1, 2
	    u1 = UChaC(c1,2)*VChaC(c2,1)
	    u2 = UChaC(c1,1)*VChaC(c2,2)
	    u3 = VCha(c1,1)*UCha(c2,2)
	    u4 = VCha(c1,2)*UCha(c2,1)
	    do h = 1, 3
	      do uz = 1, 2
	        cL(uz) = 1/sqrt2*EL/SW*(
     &            DD(uz,h,3)*(-u1*SB - u2*CB) +
     &            DD(uz,h,2)*(-cI*u1) +
     &            DD(uz,h,1)*(-cI*u2) )
	        cR(uz) = 1/sqrt2*EL/SW*(
     &            DD(uz,h,3)*(u3*SB + u4*CB) +
     &            DD(uz,h,2)*(-cI*u3) +
     &            DD(uz,h,1)*(-cI*u4) )
	      enddo

	      LCoupling(H0ChaCha(h,c1,c2)) = cL(uzint)
	      RCoupling(H0ChaCha(h,c1,c2)) = cR(uzint)

	      Gamma(H0ChaCha(h,c1,c2)) = MHiggs(h)/(8*pi)*
     &          KallenLR(MHiggs(h), MCha(c1), MCha(c2), cL(uzext), cR(uzext))
	      GammaTot(h) = GammaTot(h) + Gamma(H0ChaCha(h,c1,c2))
	    enddo
	  enddo
	enddo

* Higgs -> Neutralino Neutralino

	do n2 = 1, 4
          do n1 = 1, 4
	    u1 = ZNeu(n1,3)*(CW*ZNeu(n2,2) - SW*ZNeu(n2,1)) +
     &           ZNeu(n2,3)*(CW*ZNeu(n1,2) - SW*ZNeu(n1,1))
	    u2 = ZNeu(n1,4)*(CW*ZNeu(n2,2) - SW*ZNeu(n2,1)) +
     &           ZNeu(n2,4)*(CW*ZNeu(n1,2) - SW*ZNeu(n1,1))
	    do h = 1, 3
	      do uz = 1, 2
	        cL(uz) = .5D0*EL/(CW*SW)*(
     &            DD(uz,h,3)*(-DCONJG(u1)*SB + DCONJG(u2)*CB) +
     &            DD(uz,h,2)*(-cI*DCONJG(u1)) +
     &            DD(uz,h,1)*( cI*DCONJG(u2)) )
	        cR(uz) = .5D0*EL/(CW*SW)*(
     &            DD(uz,h,3)*( u1*SB - u2*CB) +
     &            DD(uz,h,2)*(-cI*u1) +
     &            DD(uz,h,1)*( cI*u2) )
	      enddo

	      LCoupling(H0NeuNeu(h,n1,n2)) = cL(uzint)
	      RCoupling(H0NeuNeu(h,n1,n2)) = cR(uzint)

	      Gamma(H0NeuNeu(h,n1,n2)) = MHiggs(h)/(16*pi)*
     &          KallenLR(MHiggs(h), MNeu(n1), MNeu(n2), cL(uzext), cR(uzext))
	      GammaTot(h) = GammaTot(h) + Gamma(H0NeuNeu(h,n1,n2))
	      if( n1 .ne. n2 )
     &          Gamma(H0NeuNeu(h,n1,n2)) = 2*Gamma(H0NeuNeu(h,n1,n2))
	    enddo
	  enddo
	enddo

* Higgs -> Neutralino Chargino

	do c2 = 1, 2
	  do n1 = 1, 4
	    cL(1) = -cI*EL/SW*CB*(ZNeuC(n1,4)*VChaC(c2,1) +
     &        1/sqrt2*(ZNeuC(n1,2) + ZNeuC(n1,1)*SW/CW)*VChaC(c2,2))
	    cR(1) = -cI*EL/SW*SB*(ZNeu(n1,3)*UCha(c2,1) -
     &        1/sqrt2*(ZNeu(n1,2) + ZNeu(n1,1)*SW/CW)*UCha(c2,2))

	    LCoupling(HpNeuCha(n1,c2)) = cL(1)
	    RCoupling(HpNeuCha(n1,c2)) = cR(1)

	    Gamma(HpNeuCha(n1,c2)) = MHiggs(4)/(8*pi)*
     &        KallenLR(MHiggs(4), MNeu(n1), MCha(c2), cL(1), cR(1))
	    GammaTot(4) = GammaTot(4) + Gamma(HpNeuCha(n1,c2))
	  enddo
	enddo

*** BRANCHING RATIOS ***

	do h = 1, 4
	  if( GammaTot(h) .gt. 0 ) then
	    ga(h) = 1/GammaTot(h)
	  else
	    ga(h) = 0
	  endif
	  if( GammaSMTot(h) .gt. 0 ) then
	    gasm(h) = 1/GammaSMTot(h)
	  else
	    gasm(h) = 0
	  endif
	enddo

	h = 0

*#define Next(h) mod(h, 3) + 1
#define Next(h) (iand(h + 1, 3) + ishft(h + 1, -2))

	do i = H0VV(1,1), H0VV(3,5)
	  h = Next(h)
	  BR(i) = Gamma(i)*ga(h)
	  BRSM(i) = GammaSM(i)*gasm(h)
	enddo

	do i = H0FF(1,2,1), H0FF(3,4,3)
	  h = Next(h)
	  BR(i) = Gamma(i)*ga(h)
	  BRSM(i) = GammaSM(i)*gasm(h)
	enddo

	do i = H0ChaCha(1,1,1), H0ChaCha(3,2,2)
	  h = Next(h)
	  BR(i) = Gamma(i)*ga(h)
	enddo

	do i = H0NeuNeu(1,1,1), H0NeuNeu(3,4,4)
	  h = Next(h)
	  BR(i) = Gamma(i)*ga(h)
	enddo

	do i = H0HV(1,1), H0HV(3,3)
	  h = Next(h)
	  BR(i) = Gamma(i)*ga(h)
	enddo

	do i = H0HH(1,1,1), H0HH(3,4,4)
	  h = Next(h)
	  BR(i) = Gamma(i)*ga(h)
	enddo

	do i = H0SfSf(1,1,1,2,1), H0SfSf(3,2,2,4,3)
	  h = Next(h)
	  BR(i) = Gamma(i)*ga(h)
	enddo

	do i = HpFF(1,1), HpFF(2,3)
	  BR(i) = Gamma(i)*ga(4)
	enddo

	do i = HpNeuCha(1,1), HpNeuCha(4,2)
	  BR(i) = Gamma(i)*ga(4)
	enddo

	do i = HpHV(1), HpHV(3)
	  BR(i) = Gamma(i)*ga(4)
	enddo

	do i = HpSfSf(1,1,1,1), HpSfSf(2,2,2,3)
	  BR(i) = Gamma(i)*ga(4)
	enddo

	coup_valid = .TRUE.



    	if( debuglevel .lt. 1 ) return

#define Header(h) higgs(h), " decays    Width                    BR"
	DCOUP "============================" ENDL
	do h = 1, 3
	  DHEAD higgs(h), " total width =", GammaTot(h) ENDL
	  DHEAD Header(h) ENDL
	  DCOUP "gamma-gamma", Gamma(H0VV(h,1)), BR(H0VV(h,1)) ENDL
	  DCOUP "gamma-Z    ", Gamma(H0VV(h,2)), BR(H0VV(h,2)) ENDL
	  DCOUP "Z-Z        ", Gamma(H0VV(h,3)), BR(H0VV(h,3)) ENDL
	  DCOUP "W-W        ", Gamma(H0VV(h,4)), BR(H0VV(h,4)) ENDL
	  DCOUP "gluon-gluon", Gamma(H0VV(h,5)), BR(H0VV(h,5)) ENDL
	  DCOUP " " ENDL
	  DCOUP "h0-Z       ", Gamma(H0HV(h,1)), BR(H0HV(h,1)) ENDL
	  DCOUP "HH-Z       ", Gamma(H0HV(h,2)), BR(H0HV(h,2)) ENDL
	  DCOUP "A0-Z       ", Gamma(H0HV(h,3)), BR(H0HV(h,3)) ENDL
	  DCOUP " " ENDL
	  DCOUP "h0-h0      ", Gamma(H0HH(h,1,1)), BR(H0HH(h,1,1)) ENDL
	  DCOUP "Hp-Hp      ", Gamma(H0HH(h,4,4)), BR(H0HH(h,4,4)) ENDL
	  DCOUP " " ENDL
	  DCOUP "f-f    e   ", Gamma(H0FF(h,2,1)), BR(H0FF(h,2,1)) ENDL
	  DCOUP "      mu   ", Gamma(H0FF(h,2,2)), BR(H0FF(h,2,2)) ENDL
	  DCOUP "     tau   ", Gamma(H0FF(h,2,3)), BR(H0FF(h,2,3)) ENDL
	  DCOUP "       u   ", Gamma(H0FF(h,3,1)), BR(H0FF(h,3,1)) ENDL
	  DCOUP "       c   ", Gamma(H0FF(h,3,2)), BR(H0FF(h,3,2)) ENDL
	  DCOUP "       t   ", Gamma(H0FF(h,3,3)), BR(H0FF(h,3,3)) ENDL
	  DCOUP "       d   ", Gamma(H0FF(h,4,1)), BR(H0FF(h,4,1)) ENDL
	  DCOUP "       s   ", Gamma(H0FF(h,4,2)), BR(H0FF(h,4,2)) ENDL
	  DCOUP "       b   ", Gamma(H0FF(h,4,3)), BR(H0FF(h,4,3)) ENDL
	  DCOUP " " ENDL
	  DCOUP "Cha1-Cha1  ", Gamma(H0ChaCha(h,1,1)),
     &                            BR(H0ChaCha(h,1,1))	ENDL
	  DCOUP "Cha1-Cha2  ", Gamma(H0ChaCha(h,1,2)),
     &                            BR(H0ChaCha(h,1,2))	ENDL
	  DCOUP "Cha2-Cha1  ", Gamma(H0ChaCha(h,2,1)),
     &                            BR(H0ChaCha(h,2,1))	ENDL
	  DCOUP "Cha2-Cha2  ", Gamma(H0ChaCha(h,2,1)),
     &                            BR(H0ChaCha(h,2,2))	ENDL
	  DCOUP " " ENDL
	  DCOUP "Neu1-Neu1  ", Gamma(H0NeuNeu(h,1,1)),
     &                            BR(H0NeuNeu(h,1,1))	ENDL
	  DCOUP "Neu1-Neu2  ", Gamma(H0NeuNeu(h,1,2)),
     &                            BR(H0NeuNeu(h,1,2))	ENDL
	  DCOUP "Neu1-Neu3  ", Gamma(H0NeuNeu(h,1,3)),
     &                            BR(H0NeuNeu(h,1,3))	ENDL
	  DCOUP "Neu1-Neu4  ", Gamma(H0NeuNeu(h,1,4)),
     &                            BR(H0NeuNeu(h,1,4))	ENDL
	  DCOUP "Neu2-Neu2  ", Gamma(H0NeuNeu(h,2,2)),
     &                            BR(H0NeuNeu(h,2,2))	ENDL
	  DCOUP "Neu2-Neu3  ", Gamma(H0NeuNeu(h,2,3)),
     &                            BR(H0NeuNeu(h,2,3))	ENDL
	  DCOUP "Neu2-Neu4  ", Gamma(H0NeuNeu(h,2,4)),
     &                            BR(H0NeuNeu(h,2,4))	ENDL
	  DCOUP "Neu3-Neu3  ", Gamma(H0NeuNeu(h,3,3)),
     &                            BR(H0NeuNeu(h,3,3))	ENDL
	  DCOUP "Neu3-Neu4  ", Gamma(H0NeuNeu(h,3,4)),
     &                            BR(H0NeuNeu(h,3,4))	ENDL
	  DCOUP "Neu4-Neu4  ", Gamma(H0NeuNeu(h,4,4)),
     &                            BR(H0NeuNeu(h,4,4))	ENDL
	  DCOUP " " ENDL
	  DCOUP "sf-sf  e 11", Gamma(H0SfSf(h,1,1,2,1)),
     &                            BR(H0SfSf(h,1,1,2,1))	ENDL
	  DCOUP "         12", Gamma(H0SfSf(h,1,2,2,1)),
     &                            BR(H0SfSf(h,1,2,2,1))	ENDL
	  DCOUP "         22", Gamma(H0SfSf(h,2,2,2,1)),
     &                            BR(H0SfSf(h,2,2,2,1))	ENDL
	  DCOUP "      mu 11", Gamma(H0SfSf(h,1,1,2,2)),
     &                            BR(H0SfSf(h,1,1,2,2)) ENDL
	  DCOUP "         12", Gamma(H0SfSf(h,1,2,2,2)),
     &                            BR(H0SfSf(h,1,2,2,2))	ENDL
	  DCOUP "         22", Gamma(H0SfSf(h,2,2,2,2)),
     &                            BR(H0SfSf(h,2,2,2,2))	ENDL
	  DCOUP "     tau 11", Gamma(H0SfSf(h,1,1,2,3)),
     &                            BR(H0SfSf(h,1,1,2,3))	ENDL
	  DCOUP "         12", Gamma(H0SfSf(h,1,2,2,3)),
     &                            BR(H0SfSf(h,1,2,2,3))	ENDL
	  DCOUP "         22", Gamma(H0SfSf(h,2,2,2,3)),
     &                            BR(H0SfSf(h,2,2,2,3))	ENDL
	  DCOUP "       u 11", Gamma(H0SfSf(h,1,1,3,1)),
     &                            BR(H0SfSf(h,1,1,3,1))	ENDL
	  DCOUP "         12", Gamma(H0SfSf(h,1,2,3,1)),
     &                            BR(H0SfSf(h,1,2,3,1))	ENDL
	  DCOUP "         22", Gamma(H0SfSf(h,2,2,3,1)),
     &                            BR(H0SfSf(h,2,2,3,1))	ENDL
	  DCOUP "       c 11", Gamma(H0SfSf(h,1,1,3,2)),
     &                            BR(H0SfSf(h,1,1,3,2))	ENDL
	  DCOUP "         12", Gamma(H0SfSf(h,1,2,3,2)),
     &                            BR(H0SfSf(h,1,2,3,2))	ENDL
	  DCOUP "         22", Gamma(H0SfSf(h,2,2,3,2)),
     &                            BR(H0SfSf(h,2,2,3,2))	ENDL
	  DCOUP "       t 11", Gamma(H0SfSf(h,1,1,3,3)),
     &                            BR(H0SfSf(h,1,1,3,3))	ENDL
	  DCOUP "         12", Gamma(H0SfSf(h,1,2,3,3)),
     &                            BR(H0SfSf(h,1,2,3,3))	ENDL
	  DCOUP "         22", Gamma(H0SfSf(h,2,2,3,3)),
     &                            BR(H0SfSf(h,2,2,3,3))	ENDL
	  DCOUP "       d 11", Gamma(H0SfSf(h,1,1,4,1)),
     &                            BR(H0SfSf(h,1,1,4,1)) ENDL
	  DCOUP "         12", Gamma(H0SfSf(h,1,2,4,1)),
     &                            BR(H0SfSf(h,1,2,4,1)) ENDL
	  DCOUP "         22", Gamma(H0SfSf(h,2,2,4,1)),
     &                            BR(H0SfSf(h,2,2,4,1)) ENDL
	  DCOUP "       s 11", Gamma(H0SfSf(h,1,1,4,2)),
     &                            BR(H0SfSf(h,1,1,4,2)) ENDL
	  DCOUP "         12", Gamma(H0SfSf(h,1,2,4,2)),
     &                            BR(H0SfSf(h,1,2,4,2)) ENDL
	  DCOUP "         22", Gamma(H0SfSf(h,2,2,4,2)),
     &                            BR(H0SfSf(h,2,2,4,2)) ENDL
	  DCOUP "       b 11", Gamma(H0SfSf(h,1,1,4,3)),
     &                            BR(H0SfSf(h,1,1,4,3)) ENDL
	  DCOUP "         12", Gamma(H0SfSf(h,1,2,4,3)),
     &                            BR(H0SfSf(h,1,2,4,3)) ENDL
	  DCOUP "         22", Gamma(H0SfSf(h,2,2,4,3)),
     &                            BR(H0SfSf(h,2,2,4,3)) ENDL
	  DCOUP " " ENDL
	enddo

	DHEAD higgs(4), " total width =", GammaTot(4)
	DHEAD Header(4)
	DCOUP "h0-W       ", Gamma(HpHV(1)), BR(HpHV(1)) ENDL
	DCOUP "HH-W       ", Gamma(HpHV(2)), BR(HpHV(2)) ENDL
	DCOUP "A0-W       ", Gamma(HpHV(3)), BR(HpHV(3)) ENDL
	DCOUP " " ENDL
	DCOUP "f-f'   e   ", Gamma(HpFF(1,1)), BR(HpFF(1,1)) ENDL
	DCOUP "      mu   ", Gamma(HpFF(1,2)), BR(HpFF(1,2)) ENDL
	DCOUP "     tau   ", Gamma(HpFF(1,3)), BR(HpFF(1,3)) ENDL
	DCOUP "     u-d   ", Gamma(HpFF(2,1)), BR(HpFF(2,1)) ENDL
	DCOUP "     c-s   ", Gamma(HpFF(2,2)), BR(HpFF(2,2)) ENDL
	DCOUP "     t-b   ", Gamma(HpFF(2,3)), BR(HpFF(2,3)) ENDL
	DCOUP " " ENDL
	DCOUP "Neu1-Cha1  ", Gamma(HpNeuCha(1,1)),
     &                          BR(HpNeuCha(1,1)) ENDL
	DCOUP "Neu1-Cha2  ", Gamma(HpNeuCha(1,2)),
     &                          BR(HpNeuCha(1,2)) ENDL
	DCOUP "Neu2-Cha1  ", Gamma(HpNeuCha(2,1)),
     &                          BR(HpNeuCha(2,1)) ENDL
	DCOUP "Neu2-Cha2  ", Gamma(HpNeuCha(2,2)),
     &                          BR(HpNeuCha(2,2)) ENDL
	DCOUP "Neu3-Cha1  ", Gamma(HpNeuCha(3,1)),
     &                          BR(HpNeuCha(3,1)) ENDL
	DCOUP "Neu3-Cha2  ", Gamma(HpNeuCha(3,2)),
     &                          BR(HpNeuCha(3,2)) ENDL
	DCOUP "Neu4-Cha1  ", Gamma(HpNeuCha(4,1)),
     &                          BR(HpNeuCha(4,1)) ENDL
	DCOUP "Neu4-Cha2  ", Gamma(HpNeuCha(4,2)),
     &                          BR(HpNeuCha(4,2)) ENDL
	DCOUP " " ENDL
	DCOUP "sf-sf' e 11", Gamma(HpSfSf(1,1,1,1)),
     &                          BR(HpSfSf(1,1,1,1)) ENDL
	DCOUP "         12", Gamma(HpSfSf(1,2,1,1)),
     &                          BR(HpSfSf(1,2,1,1)) ENDL
	DCOUP "         22", Gamma(HpSfSf(2,2,1,1)),
     &                          BR(HpSfSf(2,2,1,1)) ENDL
	DCOUP "      mu 11", Gamma(HpSfSf(1,1,1,2)),
     &                          BR(HpSfSf(1,1,1,2)) ENDL
	DCOUP "         12", Gamma(HpSfSf(1,2,1,2)),
     &                          BR(HpSfSf(1,2,1,2)) ENDL
	DCOUP "         22", Gamma(HpSfSf(2,2,1,2)),
     &                          BR(HpSfSf(2,2,1,2)) ENDL
	DCOUP "     tau 11", Gamma(HpSfSf(1,1,1,3)),
     &                          BR(HpSfSf(1,1,1,3)) ENDL
	DCOUP "         12", Gamma(HpSfSf(1,2,1,3)),
     &                          BR(HpSfSf(1,2,1,3)) ENDL
	DCOUP "         22", Gamma(HpSfSf(2,2,1,3)),
     &                          BR(HpSfSf(2,2,1,3)) ENDL
	DCOUP "     u-d 11", Gamma(HpSfSf(1,1,2,1)),
     &                          BR(HpSfSf(1,1,2,1)) ENDL
	DCOUP "         12", Gamma(HpSfSf(1,2,2,1)),
     &                          BR(HpSfSf(1,2,2,1)) ENDL
	DCOUP "         22", Gamma(HpSfSf(2,2,2,1)),
     &                          BR(HpSfSf(2,2,2,1)) ENDL
	DCOUP "     c-s 11", Gamma(HpSfSf(1,1,2,2)),
     &                          BR(HpSfSf(1,1,2,2)) ENDL
	DCOUP "         12", Gamma(HpSfSf(1,2,2,2)),
     &                          BR(HpSfSf(1,2,2,2)) ENDL
	DCOUP "         22", Gamma(HpSfSf(2,2,2,2)),
     &                          BR(HpSfSf(2,2,2,2)) ENDL
	DCOUP "     t-b 11", Gamma(HpSfSf(1,1,2,3)),
     &                          BR(HpSfSf(1,1,2,3)) ENDL
	DCOUP "         12", Gamma(HpSfSf(1,2,2,3)),
     &                          BR(HpSfSf(1,2,2,3)) ENDL
	DCOUP "         22", Gamma(HpSfSf(2,2,2,3)),
     &                          BR(HpSfSf(2,2,2,3)) ENDL
	DCOUP "============================" ENDL
	end


************************************************************************

	subroutine H0HHCouplings(ghhh, DD)
	implicit none
	double complex ghhh(2,3,4,4), DD(2,3,3)

#include "FH.h"

	integer h, h1, h2, uz
	double complex omega_t
	double complex hc(6,6,6)
* 1 = Phi2, 2 = Phi1, 3 = A, 4 = H+-, 5 = G, 6 = G+-

#if 0
	double precision lambda, MQ2, MQ
	double precision rZT, rTQ, logTQ, logTQr
	double complex rXQ

	lambda = 1/8D0 - 1/3D0*SW2 + 4/9D0*SW2**2
	MQ2 = MSf(1,3,3)*MSf(2,3,3)
	MQ = sqrt(MQ2)

	rZT = MZ2/MT2
	rTQ = MT2/MQ2
	rXQ = Xf(3,3)**2/MQ2
	logTQ = log(rTQ)
	logTQr = log(MTrun**2/MQ2)
c --> This is the 'real' omega_t = Sigma_Phi2(0)
c	omega_t = GF*sqrt2/(8*pi**2*SB2*SB)*MT2**2/MZ2*(
c FIXME: formula correct for complex rXQ & omega_t?
	omega_t = GF*sqrt2/(8*pi**2*SB2   )*MT2**2    *(
     &    rZT*(11/10D0*rZT - 2) +
     &    (rZT*SB2*(8*rZT*SB2*lambda - 6) + 12)*logTQ +
     &    rXQ*(4*rZT + 6*rTQ - 12 +
     &      rXQ*(rTQ*(3*rTQ - 4) + 1 +
     &        rXQ*(rTQ*(rTQ*(2*rTQ - 12/5D0) + 3/5D0) +
     &          rXQ*rTQ**2*(rTQ*(3/2D0*rTQ - 12/7D0) + 3/7D0)))) +
     &    8*Alfas/pi*(MTrun/MT)**4 *
     &      (3*logTQr*(logTQr - rXQ - 2) - 6*Xf(3,3)/MQ + 3/4D0*rXQ**2) )
#else
	call CalcSelf(Mh02, cpeven, 0)
	omega_t = CA2*DBLE(seR(h0h0)) + SA2*DBLE(seR(HHHH)) +
     &    S2A*DBLE(seR(h0HH))
#endif

	hc(1,1,1) = -3*SB + 3/MZ2/SB*(omega_t +
     &    GF*sqrt2/(pi**2*SB2)*MT2**2)		! hep-ph/0108245, eq26
	hc(1,1,2) = CB 
	hc(1,2,2) = SB 
	hc(2,2,2) = -3*CB
	hc(1,3,3) = SB*(omega_t/(MZ2*TB2) - C2B)
	hc(2,3,3) = CB*C2B

	do h2 = 1, 3
	  do h1 = 1, 3
	    do h = 1, 3
	      do uz = 1, 2
	        ghhh(uz,h,h1,h2) = cI*EL*MZ/(2*CW*SW)*(
     &            hc(1,1,1)* DD(uz,h,1)*DD(uz,h1,1)*DD(uz,h2,1) +
     &            hc(1,1,2)*(DD(uz,h,1)*DD(uz,h1,1)*DD(uz,h2,2) +
     &                       DD(uz,h,1)*DD(uz,h1,2)*DD(uz,h2,1) +
     &                       DD(uz,h,2)*DD(uz,h1,1)*DD(uz,h2,1)) +
     &            hc(1,2,2)*(DD(uz,h,1)*DD(uz,h1,2)*DD(uz,h2,2) +
     &                       DD(uz,h,2)*DD(uz,h1,1)*DD(uz,h2,2) +
     &                       DD(uz,h,2)*DD(uz,h1,2)*DD(uz,h2,1)) +
     &            hc(2,2,2)* DD(uz,h,2)*DD(uz,h1,2)*DD(uz,h2,2) +
     &            hc(1,3,3)*(DD(uz,h,1)*DD(uz,h1,3)*DD(uz,h2,3) +
     &                       DD(uz,h,3)*DD(uz,h1,1)*DD(uz,h2,3) +
     &                       DD(uz,h,3)*DD(uz,h1,3)*DD(uz,h2,1)) +
     &            hc(2,3,3)*(DD(uz,h,2)*DD(uz,h1,3)*DD(uz,h2,3) +
     &                       DD(uz,h,3)*DD(uz,h1,2)*DD(uz,h2,3) +
     &                       DD(uz,h,3)*DD(uz,h1,3)*DD(uz,h2,2)) )
	      enddo
	    enddo
	  enddo
	enddo

* hep-ph/9612362, eq16
	hc(1,4,4) = SB*(C2B - 2*CW2 - omega_t/(MZ2*TB2))
	hc(2,4,4) = CB*(C2B - 2*CW2)
	hc(3,4,4) = 0

	do h = 1, 3
	  do uz = 1, 2
	    ghhh(uz,h,4,1) = 0
	    ghhh(uz,h,4,2) = 0
	    ghhh(uz,h,4,3) = 0
	    ghhh(uz,h,1,4) = 0
	    ghhh(uz,h,2,4) = 0
	    ghhh(uz,h,3,4) = 0
	    ghhh(uz,h,4,4) = cI*EL*MZ/(2*CW*SW)*(
     &        hc(1,4,4)*DD(uz,h,1) +
     &        hc(2,4,4)*DD(uz,h,2) +
     &        hc(3,4,4)*DD(uz,h,3) )
	  enddo
	enddo

c	hc(2,6,6) = -C2B*CB
c	hc(1,6,6) =  C2B*SB
c	hc(2,5,5) = -C2B*CB
c	hc(1,5,5) =  C2B*SB
c	hc(2,3,6) =  S2B*CB
c	hc(1,3,6) = -S2B*SB
c	hc(2,4,6) = -CW2*SB + S2B*CB
c	hc(1,4,6) =  CW2*CB - S2B*SB
c	hc(3,4,6) = CW2		! (-I) is missing for (AH+G-), (+I) for AH-G+
	end


************************************************************************

	subroutine H0SfSfCouplings(ghss, t, g, DD)
	implicit none
	double complex ghss(2,3,2,2), DD(2,3,3)
	integer t, g

#include "FH.h"

	integer h, uz
	double complex u1111, u1112, u1121, u1122, u1212
	double complex u1222, u2112, u2121, u2122, u2222
	double precision m11a, m22a, m11b, m22b, m
	double complex m12a, m12b, m3, c
	double complex hc(3,3)

	u1111 = USf(1,1,t,g)*USfC(1,1,t,g)
	u1112 = USf(1,1,t,g)*USfC(1,2,t,g)
	u1121 = USf(1,1,t,g)*USfC(2,1,t,g)
	u1122 = USf(1,1,t,g)*USfC(2,2,t,g)
	u1212 = USf(1,2,t,g)*USfC(1,2,t,g)
	u1222 = USf(1,2,t,g)*USfC(2,2,t,g)
	u2112 = USf(2,1,t,g)*USfC(1,2,t,g)
	u2121 = USf(2,1,t,g)*USfC(2,1,t,g)
	u2122 = USf(2,1,t,g)*USfC(2,2,t,g)
	u2222 = USf(2,2,t,g)*USfC(2,2,t,g)

	m11a = SB/C2B*DSf(1,t)
	m22a = SB/C2B*DSf(2,t)
	m11b = CB/C2B*DSf(1,t)
	m22b = CB/C2B*DSf(2,t)

	if( iand(t, 1) .eq. 1 ) then
	  m = -Mf(t,g)/SB
	  m11a = m11a + m*Mf(t,g)
	  m22a = m22a + m*Mf(t,g)
	  m12a = m*DCONJG(Af(t,g,g))
	  m12b = m*MUE
	  m3 = -m12a*CB - m12b*SB
	else
	  m = Mf(t,g)/CB
	  m11b = m11b + m*Mf(t,g)
	  m22b = m22b + m*Mf(t,g)
	  m12a = m*MUE
	  m12b = m*DCONJG(Af(t,g,g))
	  m3 = m12a*CB + m12b*SB
	endif

	hc(1,1) = m11a*u1111 + m22a*u1212 + DBLE(m12a*u1112)
	hc(2,1) = m11b*u1111 + m22b*u1212 + DBLE(m12b*u1112)
	hc(3,1) = DIMAG(m3*u1112)

	hc(1,2) = m11a*u2121 + m22a*u2222 + DBLE(m12a*u2122)
	hc(2,2) = m11b*u2121 + m22b*u2222 + DBLE(m12b*u2122)
	hc(3,2) = DIMAG(m3*u2122)

	hc(1,3) = m11a*u1121 + m22a*u1222 +
     &    .5D0*(m12a*u1122 + DCONJG(m12a*u2112))
	hc(2,3) = m11b*u1121 + m22b*u1222 +
     &    .5D0*(m12b*u1122 + DCONJG(m12b*u2112))
	hc(3,3) = -.5D0*cI*(m3*u1122 - DCONJG(m3*u2112))

	c = cI*EL/(MW*SW)
	do h = 1, 3
	  do uz = 1, 2
	    ghss(uz,h,1,1) = c*(DD(uz,h,1)*hc(1,1) -
     &                          DD(uz,h,2)*hc(2,1) -
     &                          DD(uz,h,3)*hc(3,1))
	    ghss(uz,h,2,2) = c*(DD(uz,h,1)*hc(1,2) -
     &                          DD(uz,h,2)*hc(2,2) -
     &                          DD(uz,h,3)*hc(3,2))
	    ghss(uz,h,2,1) = c*(DD(uz,h,1)*hc(1,3) -
     &                          DD(uz,h,2)*hc(2,3) -
     &                          DD(uz,h,3)*hc(3,3))
	    ghss(uz,h,1,2) = -DCONJG(ghss(uz,h,2,1))
	  enddo
	enddo
	end


************************************************************************

	subroutine HpSfSfCouplings(ghpss, t, g)
	implicit none
	double complex ghpss(2,2)
	integer t, g

#include "FH.h"

	double precision m11, m22
	double complex m12, m21
	integer s1, s2

	m11 = Mf2(t-1,g)/TB + Mf2(t,g)*TB - MW2*S2B
	m22 = Mf(t-1,g)*Mf(t,g)*(1/TB + TB)
	m12 = Mf(t,g)*(DCONJG(MUE) + Af(t,g,g)*TB)
	m21 = Mf(t-1,g)*(MUE + DCONJG(Af(t-1,g,g))/TB)

	do s1 = 1, 2
	  do s2 = 1, 2
	    ghpss(s1,s2) = cI/sqrt2*EL/(MW*SW)*(
     &        USfC(s1,1,t-1,g)*
     &          (m11*USf(s2,1,t,g) + m12*USf(s2,2,t,g)) +
     &        USfC(s1,2,t-1,g)*
     &          (m21*USf(s2,1,t,g) + m22*USf(s2,2,t,g)) )
	  enddo
	enddo
	end


************************************************************************
	
	double precision function Kallen(m, m1, m2)
	implicit none
	double precision m, m1, m2

	double precision km, kp

	if( m .gt. m1 + m2 ) then
	  km = (m1 - m2)/m
	  km = (1 - km)*(1 + km)
	  kp = (m1 + m2)/m
	  kp = (1 - kp)*(1 + kp)
	  Kallen = sqrt(km*kp)
	else
	  Kallen = 0
	endif
	end


************************************************************************
	
	double precision function KallenLR(m, m1, m2, cL, cR)
	implicit none
	double precision m, m1, m2
	double complex cL, cR

	double precision cm, cp, km, kp

	double precision lambda, ratio
	common /KallenVars/ lambda, ratio

	cm = .25D0*Sq(cL - cR)
	cp = .25D0*Sq(cL + cR)
* ratio = |cL|^2 + |cR|^2 - Re cL cR^*
	ratio = 3*cm + cp

	if( m .gt. m1 + m2 ) then
	  km = (m1 - m2)/m
	  km = (1 - km)*(1 + km)
	  kp = (m1 + m2)/m
	  kp = (1 - kp)*(1 + kp)
	  lambda = sqrt(km*kp)
	  KallenLR = lambda*(cm*km + cp*kp)
	else
	  lambda = 0
	  KallenLR = 0
	endif
	end


************************************************************************

	double precision function HXVWidth(mh, mx, mv, gammav, isvector)
	implicit none
	double precision mh, mx, mv, gammav
	integer isvector

#include "FH.h"

	double precision m, m1, g, c
	common /HXVVars/ m, m1, g, c

	double precision Kallen, Gauss, HXVIntegrand
	external Kallen, Gauss, HXVIntegrand

	double precision lambda

	if( mh .gt. mx + mv ) then
* real decay above threshold
	  lambda = Kallen(mh, mx, mv)
	  HXVWidth = lambda*(lambda**2 + isvector*12*(mv/mh)**4)
	else if( mh .gt. mx + 1 ) then
* effects from finite width of gauge boson
	  m = mh/mv
	  m1 = mx/mv
	  g = gammav/mv
	  c = isvector*12
	  HXVWidth = (isvector + 1)*Gauss(0D0, m - m1, HXVIntegrand, 16)
	else
	  HXVWidth = 0
	endif

	HXVWidth = GF/(8*sqrt2*pi)*mh**3*HXVWidth
	end


************************************************************************

	double precision function HXVIntegrand(x)
	implicit none
	double precision x

#include "const.h"

	double precision Kallen
	external Kallen

	double precision m, m1, g, c
	common /HXVVars/ m, m1, g, c

	double precision lambda

	lambda = Kallen(m, m1, x)

	HXVIntegrand = 2/pi*x*g*lambda*(lambda**2 + c*(x/m**2)**2)/
     &    ((x**2 - 1)**2 + g**2)
	end


************************************************************************

	double precision function cfkt2(Q)
	implicit none
	double precision Q

#include "FH.h"

	double precision x

	double precision AlphaS
	external AlphaS

	x = AlphaS(Q)/pi

	if( Q .le. MC ) then
	  cfkt2 = (9/2D0)**(4/9D0)*(1 + x*(0.895D0 + 1.371D0*x))
	else if( Q .le. MB ) then
	  cfkt2 = (25/6D0)**(12/25D0)*(1 + x*(1.014D0 + 1.389D0*x))
	else if( Q .le. MT ) then
	  cfkt2 = (23/6D0)**(12/23D0)*(1 + x*(1.175D0 + 1.501D0*x))
	else
	  cfkt2 = (7/2D0)**(4/7D0)*(1 + x*(1.398D0 + 1.793D0*x))
	endif
	end
