* HiggsCorr.F
* Subroutines for the calculation of higher-order corrections
* to the MSSM Higgs sector
* this file is part of FeynHiggs
* V1.0 by Markus Frank 21/08/2002
* last modified 5 Dec 06 th

#include "externals.h"
#include "debug.h"

#define safesqrt(x) sign(sqrt(abs(x)), x)
#define safesq(x) sign(x**2, x)


	subroutine FHHiggsCorr(error,
     &    MHiggs_, SAeff_, UHiggs_, ZHiggs_)
	implicit none

#include "FH.h"
#define __SUBROUTINE__ "FHHiggsCorr"

	integer error
	double precision MHiggs_(NHiggs)
	double complex SAeff_
	double complex UHiggs_(NNeutral,NNeutral)
	double complex ZHiggs_(NNeutral,NNeutral)

	integer i, j

	call CheckHiggsCorr(error)
	if( error .ne. 0 ) return

	do i = 1, NHiggs
	  MHiggs_(i) = MHiggs(i)
	enddo

	SAeff_ = SAeff

	do i = 1, NNeutral
	  do j = 1, NNeutral
	    UHiggs_(j,i) = UHiggs(j,i)
	    ZHiggs_(j,i) = ZHiggs(j,i)
	  enddo
	enddo
	end


************************************************************************
* main subroutine, calculates MHiggs, SAeff, and {U,Z}Higgs

	subroutine CheckHiggsCorr(error)
	implicit none
	integer error

#include "FH.h"

	integer i, j, k, l, p, minperm, hmix
	double precision m, d1, d2, test, mintest
	double precision gw(NNeutral,NNeutral)
	double complex c, Zh(NNeutral,NNeutral,NNeutral)

	double precision EVFunc, GWFunc
	external EVFunc, GWFunc

	double complex Ch(0:NNeutral,0:NNeutral,NNeutral)
	common /evmix/ Ch

	integer perm(NNeutral,6), nperm(NNeutral)
	data perm /1,2,3, 2,1,3, 1,3,2, 2,3,1, 3,1,2, 3,2,1/
	data nperm /0, 2, 6/

	error = 0

	if( higgs_valid ) return

	if( .not. flags_valid )
     &    Error("must call FHSetFlags before")
	if( .not. para_valid )
     &    Error("must call FHSetPara before")

	coup_valid = .FALSE.

	entry CalcHiggsCorr(error)

	if( error .ne. 0 ) return

	call CalcCTs

	do i = 1, NNeutral
	  do j = 1, NNeutral
	    UHiggs(j,i) = 0
	    ZHiggs(j,i) = 0
	  enddo
	  UHiggs(i,i) = 1
	  ZHiggs(i,i) = 1
	enddo

	do i = 1, NHiggs
	  MHiggs2(i) = Msq(i)
	enddo

	hmix = higgsmix - ishft(higgsmix, -2)

	if( looplevel .ne. 0 ) then
	  call CalcUHiggs

       
	  if( debuglevel .lt. 0 .and. .not. higgs_valid ) then
	    do i = 1, NHiggs
	      do m = MHp - 75, MHp + 20 - .1D0, .1D0
c	      do m = 0, 60, .1D0
	        d1 = EVFunc(i, m)
	        if( i .eq. 4 ) then
	          d2 = 0
	        else
	          d2 = GWFunc(i, m, Zh(1,i,1), 1)
	        endif
	        write(70+i,*) m, d1, d2
	      enddo
	    enddo
	    stop
	  endif

	  do i = 1, NHiggs
	    if( debuglevel .ge. 3 )
     &        DHIGGS "searching Mh"//Digit(i) ENDL

	    m = safesqrt(MHiggs2(i))


	    if( i .ne. inputmass ) then
	      call FindRoot(i, m)
	      MHiggs2(i) = safesq(m)
	    endif

   

	    if( i .le. NNeutral ) then
	      k = ishft(hmix - i, -2)
	      do j = 1 + iand(k, i - 1),
     &               hmix + iand(k, i - hmix)
	        gw(i,j) = GWFunc(j, m, Zh(1,i,j), 1)
	      enddo
	    endif
	  enddo

   

	  if( debuglevel .ge. 3 ) then
	    DHIGGS "------------------------"	ENDL
	    DHIGGS "testing permutations:"	ENDL
	  endif

	  mintest = 1D300
	  minperm = 1
	  l = min(2, hmix)
	  do i = 1, nperm(hmix)
	    test = 0
	    do j = 1, hmix
	      p = perm(j,i)
	      d1 = 0
	      d2 = 0
	      do k = 1, hmix
	        d1 = d1 + abs(Zh(k,j,p) - Ch(j,k,l))
	        d2 = d2 + abs(Zh(k,j,p) + Ch(j,k,l))
	      enddo
	      test = test + abs(gw(j,p)) + min(d1, d2)
	    enddo
	    if( test .lt. mintest ) then
	      mintest = test
	      minperm = i
	    endif
	    if( debuglevel .ge. 3 )
     &        DHIGGS "test("//Digit(perm(1,i))//
     &                        Digit(perm(2,i))//
     &                        Digit(perm(3,i))//") =", test ENDL
	  enddo
	  if( debuglevel .ge. 3 )
     &      DHIGGS "minperm = "//Digit(perm(1,minperm))//
     &                           Digit(perm(2,minperm))//
     &                           Digit(perm(3,minperm)) ENDL
	  if( .not. higgs_valid .and. mintest .gt. 10 )
     &      Warning("ZHiggs possibly unreliable")

	  do i = 1, NNeutral
	    p = perm(i,minperm)
	    do j = 1, NNeutral
	      ZHiggs(i,j) = Zh(j,i,p)
	    enddo
	  enddo
	endif

* sort the masses
	do i = 1, hmix - 1
	  p = i
	  m = MHiggs2(i)
	  do j = i + 1, hmix
	    if( MHiggs2(j) .lt. m ) then
	      m = MHiggs2(j)
	      p = j
	    endif
	  enddo

	  if( p .ne. i ) then
	    MHiggs2(p) = MHiggs2(i)
	    MHiggs2(i) = m
	    do j = 1, hmix
	      c = ZHiggs(i,j)
	      ZHiggs(i,j) = ZHiggs(p,j)
	      ZHiggs(p,j) = c
	    enddo
	  endif
	enddo

	do i = 1, NHiggs
	  MHiggs(i) = safesqrt(MHiggs2(i))
	  if( MHiggs2(i) .lt. 0 ) then
* suppress the error message during FHUncertainties:
	    if( higgs_valid ) then
	      error = __LINE__
	    else
	      Error("error computing Mh"//Digit(i))
	    endif
	  endif
	enddo

	if( hmix .le. 2 ) then
	  SAeff = SA*UHiggs(1,1) - CA*UHiggs(1,2)
	else
	  SAeff = -2
	endif

	higgs_valid = error .eq. 0

	if( debuglevel .lt. 2 ) return

	DHIGGS "------------------------"	ENDL
	DHIGGS "final values:"			ENDL
	DHIGGS "MHiggs(1) =", MHiggs(1)		ENDL
	DHIGGS "MHiggs(2) =", MHiggs(2)		ENDL
	DHIGGS "MHiggs(3) =", MHiggs(3)		ENDL
	DHIGGS "MHiggs(4) =", MHiggs(4)		ENDL
	DHIGGS "------------------------"	ENDL
	DHIGGS "UHiggs(1,1) =", UHiggs(1,1)	ENDL
	DHIGGS "UHiggs(1,2) =", UHiggs(1,2)	ENDL
	DHIGGS "UHiggs(1,3) =", UHiggs(1,3)	ENDL
	DHIGGS "UHiggs(2,1) =", UHiggs(2,1)	ENDL
	DHIGGS "UHiggs(2,2) =", UHiggs(2,2)	ENDL
	DHIGGS "UHiggs(2,3) =", UHiggs(2,3)	ENDL
	DHIGGS "UHiggs(3,1) =", UHiggs(3,1)	ENDL
	DHIGGS "UHiggs(3,2) =", UHiggs(3,2)	ENDL
	DHIGGS "UHiggs(3,3) =", UHiggs(3,3)	ENDL
	DHIGGS "------------------------"	ENDL
	DHIGGS "ZHiggs(1,1) =", ZHiggs(1,1)	ENDL
	DHIGGS "ZHiggs(1,2) =", ZHiggs(1,2)	ENDL
	DHIGGS "ZHiggs(1,3) =", ZHiggs(1,3)	ENDL
	DHIGGS "ZHiggs(2,1) =", ZHiggs(2,1)	ENDL
	DHIGGS "ZHiggs(2,2) =", ZHiggs(2,2)	ENDL
	DHIGGS "ZHiggs(2,3) =", ZHiggs(2,3)	ENDL
	DHIGGS "ZHiggs(3,1) =", ZHiggs(3,1)	ENDL
	DHIGGS "ZHiggs(3,2) =", ZHiggs(3,2)	ENDL
	DHIGGS "ZHiggs(3,3) =", ZHiggs(3,3)	ENDL
	DHIGGS "------------------------"	ENDL
	DHIGGS "SAeff =", SAeff			ENDL

        
  
	end



************************************************************************
* CalcUHiggs: Calculate UHiggs and initial values for MHiggs2 at k^2 = 0

	subroutine CalcUHiggs
	implicit none

#include "FH.h"


	integer key, lo
	double complex M(0:NNeutral,0:NNeutral)

* must cut off imaginary parts, otherwise UHiggs is not unitary
#define seU(i) DBLE(seR(i))

#define Need(i) -ibits(i-1-higgsmix,4,1)

	key = Key(h0h0) + Key(HHHH) +
     &    iand(not(Key(inputmass)), Key(A0A0)) +
     &    iand(Need(2), Key(h0HH)) +
     &    iand(Need(3), Key(h0A0) + Key(HHA0)) +
     &    iand(Need(4), goldstones)

* calculate masses^2 & mixing angles at k^2 = 0 Is thus true ?

	call CalcSelf(-1D0, key, key)


	M(1,1) = Msq(h0h0) - seU(h0h0)
	M(2,2) = Msq(HHHH) - seU(HHHH)
	M(3,3) = Msq(A0A0) - seU(A0A0)
	M(1,2) = -seU(h0HH)
	M(1,3) = -seU(h0A0)
	M(2,3) = -seU(HHA0)

	M(0,0) = Msq(G0G0) - seU(G0G0)
	M(0,1) = -seU(h0G0)
	M(0,2) = -seU(HHG0)
	M(0,3) = -seU(A0G0)

	if( debuglevel .ge. 4 ) then
	  DHIGGS "CalcUHiggs:"		ENDL
	  DHIGGS "M(1,1) =", M(1,1)	ENDL
	  DHIGGS "M(1,2) =", M(1,2)	ENDL
	  DHIGGS "M(2,2) =", M(2,2)	ENDL
	  DHIGGS "M(1,3) =", M(1,3)	ENDL
	  DHIGGS "M(2,3) =", M(2,3)	ENDL
	  DHIGGS "M(3,3) =", M(3,3)	ENDL
	  DHIGGS "M(0,0) =", M(0,0)	ENDL
	  DHIGGS "M(0,1) =", M(0,1)	ENDL
	  DHIGGS "M(0,2) =", M(0,2)	ENDL
	  DHIGGS "M(0,3) =", M(0,3)	ENDL
	endif

	lo = ieor(ishft(higgsmix, -2), 1)
	call HEigensystem(higgsmix, M(lo,lo),NNeutral+1, MHiggs2(lo),
     &    UHiggs(lo,lo),NNeutral+1, 1)

	if( debuglevel .lt. 3 ) return

	DHIGGS "========================"		ENDL
	DHIGGS "initial values:"			ENDL
	DHIGGS "MHiggs(0) =", safesqrt(MHiggs2(0))	ENDL
	DHIGGS "MHiggs(1) =", safesqrt(MHiggs2(1))	ENDL
	DHIGGS "MHiggs(2) =", safesqrt(MHiggs2(2))	ENDL
	DHIGGS "MHiggs(3) =", safesqrt(MHiggs2(3))	ENDL
	DHIGGS "------------------------"		ENDL
	end


************************************************************************
* EVFunc: the function whose roots are the exact poles of the
* Higgs propagators

	double precision function EVFunc(i, p)
	implicit none
	integer i
	double precision p

#include "FH.h"

	double complex Ch(0:NNeutral,0:NNeutral,NNeutral)
	common /evmix/ Ch

	integer key, lo
	double complex M(0:NNeutral,0:NNeutral), ev(0:NNeutral)
	double precision p2, mh

#define seC(h) (seR(h) - cI*DIMAG(seR(i))*dseR(h))
*#define seC(h) seR(h)

	p2 = safesq(p)

	if( higgsmix - ishft(higgsmix, -2) .lt. max(2, i) ) then
	  key = iand(not(Key(inputmass)), Key(i))
	  call CalcSelf(p2, key, key)
	  mh = Msq(i) - seC(i)
	else
	  key = cpeven +
     &      iand(Need(3), iand(not(Key(inputmass)), cpodd)) +
     &      iand(Need(4), goldstones)
	  call CalcSelf(p2, key, key)

	  M(1,1) = Msq(h0h0) - seC(h0h0)
	  M(2,2) = Msq(HHHH) - seC(HHHH)
	  M(3,3) = Msq(A0A0) - seC(A0A0)
	  M(1,2) = -seC(h0HH)
	  M(1,3) = -seC(h0A0)
	  M(2,3) = -seC(HHA0)

	  M(0,0) = Msq(G0G0) - seC(G0G0)
	  M(0,1) = -seC(h0G0)
	  M(0,2) = -seC(HHG0)
	  M(0,3) = -seC(A0G0)

	  if( debuglevel .ge. 4 ) then
	    DHIGGS "EVFunc:"		ENDL
	    DHIGGS "M(1,1) =", M(1,1)	ENDL
	    DHIGGS "M(1,2) =", M(1,2)	ENDL
	    DHIGGS "M(2,2) =", M(2,2)	ENDL
	    DHIGGS "M(1,3) =", M(1,3)	ENDL
	    DHIGGS "M(2,3) =", M(2,3)	ENDL
	    DHIGGS "M(3,3) =", M(3,3)	ENDL
	    DHIGGS "M(0,0) =", M(0,0)	ENDL
	    DHIGGS "M(0,1) =", M(0,1)	ENDL
	    DHIGGS "M(0,2) =", M(0,2)	ENDL
	    DHIGGS "M(0,3) =", M(0,3)	ENDL
	  endif

	  lo = ieor(ishft(higgsmix, -2), 1)
	  call SEigensystem(higgsmix, M(lo,lo),NNeutral+1, ev(lo),
     &      Ch(lo,lo,i),NNeutral+1, 1)
	  mh = DBLE(ev(i))

	  if( debuglevel .ge. 4 ) then
	    DHIGGS "ev(0) =", sqrt(ev(0))	ENDL
	    DHIGGS "ev(1) =", sqrt(ev(1))	ENDL
	    DHIGGS "ev(2) =", sqrt(ev(2))	ENDL
	    DHIGGS "ev(3) =", sqrt(ev(3))	ENDL
	  endif
	endif

	mh = safesqrt(mh)
	EVFunc = mh - p

	if( debuglevel .ge. 3 )
     &    DHIGGS "p,mh_ev("//Digit(i)//") =", p, mh ENDL
	end


************************************************************************
* GWFunc: another function whose roots are the exact poles of the
* neutral Higgs propagators

	double precision function GWFunc(i, p, Z, need_se)
	implicit none

#include "FH.h"

	integer i, need_se
	double precision p
	double complex Z(NNeutral)

	integer j, k, ij, ik, jk
	integer needi, needj, needk, key
	double precision p2, mh, norm
	double complex Dj, Dk, den, Zij, Zik, se_eff
	double complex dDj, dDk, dden, dZij, dZik, dse_eff

	integer se_ijk(NNeutral-1+NNeutral,NNeutral)
	data se_ijk /
     &    HHHH,A0A0, h0HH,h0A0,HHA0,
     &    h0h0,A0A0, h0HH,HHA0,h0A0,
     &    h0h0,HHHH, h0A0,HHA0,h0HH /



	p2 = safesq(p)

	j = se_ijk(1,i)
	k = se_ijk(2,i)
	ij = se_ijk(3,i)
	ik = se_ijk(4,i)
	jk = se_ijk(5,i)

	if( need_se .eq. 1 ) then
	  needi = Need(i)
	  needj = Need(j)
	  needk = Need(k)
	  key = iand( not(Key(inputmass)),
     &      Key(i) +
     &      iand(needj, Key(j) +
     &                  iand(needi, Key(ij))) +
     &      iand(needk, Key(k) +
     &                  iand(needi, Key(ik)) +
     &                  iand(needj, Key(jk))) )
	  call CalcSelf(p2, key, key)
	endif



	Dj = p2 - Msq(j) + seR(j)
	Dk = p2 - Msq(k) + seR(k)
	den = Dj*Dk - seR(jk)**2
	if( den .eq. 0 ) then
* either j or k = inputmass and this boson does not mix
* (typically A0 and 2x2 mixing)
	  GWFunc = 1D100
	  return
	endif
	den = 1/den

	Zij = den*(seR(ik)*seR(jk) - seR(ij)*Dk)
	Zik = den*(seR(ij)*seR(jk) - seR(ik)*Dj)

	dDj = 1 + dseR(j)
	dDk = 1 + dseR(k)
	dden = dDj*Dk + Dj*dDk - 2*seR(jk)*dseR(jk)

	dZij = dseR(ik)*seR(jk) + seR(ik)*dseR(jk) -
     &    dseR(ij)*Dk - seR(ij)*dDk
	dZik = dseR(ij)*seR(jk) + seR(ij)*dseR(jk) -
     &    dseR(ik)*Dj - seR(ik)*dDj

	se_eff = seR(i) + seR(ij)*Zij + seR(ik)*Zik
	dse_eff = dseR(i) + dseR(ij)*Zij + dseR(ik)*Zik +
     &    den*(seR(ij)*(dZij - dden*Zij) +
     &         seR(ik)*(dZik - dden*Zik))

	if( debuglevel .ge. 4 ) then
	  DHIGGS "se_eff("//Digit(i)//") =", se_eff	ENDL
	  DHIGGS "dse_eff("//Digit(i)//") =", dse_eff	ENDL
	  DHIGGS "Z"//Digit(i)//Digit(j)//" =", Zij	ENDL
	  DHIGGS "Z"//Digit(i)//Digit(k)//" =", Zik	ENDL
	endif

	norm = 1/(1 + DBLE(dse_eff))
	Z(i) = sqrt(DCMPLX(norm))
	Z(j) = Z(i)*Zij
	Z(k) = Z(i)*Zik

	norm = norm*DIMAG(se_eff)
	mh = Msq(i) - DBLE(se_eff) - norm*DIMAG(dse_eff)
	mh = safesqrt(mh)

	GWFunc = mh - p

	if( debuglevel .ge. 3 ) then
	  DHIGGS "  p,mh_gw("//Digit(i)//") =", p, mh ENDL
	  DHIGGS "  width =", safesqrt(norm) ENDL
	endif
	end


************************************************************************

	subroutine FindRoot(i, p)
	implicit none
	integer i
	double precision p

	double precision lim(2,0:1), f1, f2, f, s
	integer try, j

	double precision EVFunc
	external EVFunc

	double precision eps, inf, maxstep
	parameter (eps = 1D0/2**16)
	parameter (inf = 2D0**1000)
	parameter (maxstep = 10)

* use Newton's method as long as the interval shrinks
	lim(1,0) = inf
	lim(1,1) = -inf
	do try = 1, 20
	  f1 = EVFunc(i, p - eps)
	  s = sign(1D0, f1)
	  j = ishft(int(s) + 1, -1)
	  if( s*(lim(1,j) - p) .gt. .1D0 ) goto 1
	  f2 = EVFunc(i, p + eps)
	  f = .5D0*(f1 + f2)
	  lim(1,j) = p
	  lim(2,j) = f
	  s = 2*eps*f/(f2 - f1)
	  p = p - s
	  if( abs(s) .lt. eps ) return
	enddo

1	if( abs(lim(1,1-j)) .eq. inf ) then
* find two points straddling the root
2	  s = 1.2D0*lim(2,j)
	  if( abs(s) .gt. maxstep ) s = sign(maxstep, s)
	  j = 1 - j
	  lim(1,j) = lim(1,1-j) + s
	  lim(2,j) = EVFunc(i, lim(1,j))
	  if( lim(2,j)*lim(2,1-j) .gt. 0 ) goto 2
	endif

* find the exact root using bisection
3	p = .5D0*(lim(1,0) + lim(1,1))
	if( abs(lim(1,1) - lim(1,0)) .lt. eps ) return
	f = EVFunc(i, p)
	if( abs(f) .lt. eps ) return
	j = ishft(int(sign(1D0, f*lim(2,1))) + 1, -1)
	lim(1,j) = p
	lim(2,j) = f
	goto 3
	end

