* SetSLHA.F
* initialize parameters from the SLHA data structure
* this file is part of FeynHiggs
* last modified 12 Oct 06 th

#include "externals.h"
#include "debug.h"


	subroutine FHSetSLHA(error, slhadata, scalefactor)

	implicit none
	integer error
	double complex slhadata(*)
	double precision scalefactor

#include "SLHA.h"
#include "FH.h"
#define __SUBROUTINE__ "FHSetSLHA"

#define Missing(p) Error("cannot determine "//p//" from SLHA data")
#define Origin(p) if(debuglevel.ge.1) DSLHA p ENDL

	double complex M_1_, M_2_, M_3_, MUE_, Af_(2:4)
	double precision TB_, MA0_, QA0_, MSS2_(5), Q_(2:4)
	integer g, t, n

	character*4 mssname(5)
	data mssname /"MSL(", "MSE(", "MSQ(", "MSU(", "MSD("/

	if( int(ModSel_Content) .gt. 0 )
     &    Warning("SLHA record indicates model other than MSSM")

!	if( SMInputs_Mb .eq. invalid ) Missing("MB")

!	if( ExtPar_TB .ne. invalid ) then
!	  TB_ = ExtPar_TB
!	  Origin("TB from ExtPar_TB")
!	else if( HMix_TB .ne. invalid ) then
!	  TB_ = HMix_TB
!	  Origin("TB from HMix_TB")
!	else if( MinPar_TB .ne. invalid ) then
!	  TB_ = MinPar_TB
!	  Origin("TB from MinPar_TB")
!	else
!	  TB_ = -1
!	  Missing("TB")
!	endif


       
	TB_ = HMix_TB
	Origin("TB from HMix_TB")
        


!	if( ExtPar_M3 .ne. invalid ) then
!	  M_3_ = ExtPar_M3
!	  Origin("M_3 from ExtPar_M3")
!	else if( Mass_MGl .ne. invalid ) then
!	  M_3_ = Mass_MGl
!	  Origin("M_3 from Mass_MGl")
!	else if( MSoft_M3 .ne. invalid ) then
!	  M_3_ = MSoft_M3
!	  Origin("M_3 from MSoft_M3")
!	else
!	  M_3_ = 0
!	  Missing("M_3")
!	endif

        
        M_3_ = MSoft_M3        
        Origin("M_3 from MSoft_M3")



	call BeginPara(error, scalefactor,
     &    SMInputs_invAlfaMZ, SMInputs_AlfasMZ, SMInputs_GF,
     &    -1D0, SMInputs_Mt, SMInputs_Mb, -1D0, SMInputs_MZ,
     &    TB_, M_3_)

       

* Higgs sector

c	if( ExtPar_MA02 .ne. invalid ) then
c	  MA0_ = sqrt(ExtPar_MA02)
c	  QA0_ = ExtPar_Q
c	  Origin("MA0 from ExtPar_MA02")
c	else if( Mass_MA0 .ne. invalid ) then
c	  MA0_ = Mass_MA0
c	  QA0_ = 0
c	  Origin("MA0 from Mass_MA0")
c	else
c	  MA0_ = -1
c	  Missing("MA0")
c	endif

	MA0_ = Mass_MA0
	QA0_ = 0
	Origin("MA0 from Mass_MA0")

	call HiggsMasses(error, MA0_, Mass_MHp, QA0_)

* Gaugino sector

c	if( ExtPar_MUE .ne. invalid ) then
c	  MUE_ = ExtPar_MUE
c	  Origin("MUE from ExtPar_MUE")
c	else if( HMix_MUE .ne. invalid ) then
c	  MUE_ = HMix_MUE
c	  Origin("MUE from HMix_MUE")
c	else
c	  MUE_ = invalid
c	endif

	MUE_ = HMix_MUE
	Origin("MUE from HMix_MUE")        

c	if( ExtPar_M2 .ne. invalid ) then
c	  M_2_ = ExtPar_M2
c	  Origin("M_2 from ExtPar_M2")
c	else if( MSoft_M2 .ne. invalid ) then
c	  M_2_ = MSoft_M2
c	  Origin("M_2 from MSoft_M2")
c	else
c	  M_2_ = invalid
c	endif

	M_2_ = MSoft_M2
	Origin("M_2 from MSoft_M2")

c	if( MUE_ .ne. invalid .and. M_2_ .ne. invalid ) then
c	  call CharginoMasses(error, M_2_, MUE_)
c	else if( SLHAValid(Mass_MCha(1), 2) .and.
c     &           SLHAValid(UMix_UCha(1,1), 4) .and.
c     &           SLHAValid(VMix_VCha(1,1), 4) ) then
c	  call CharginoPara(error,
c     &      Mass_MCha(1), UMix_UCha(1,1), VMix_VCha(1,1))
c	else
c	  Missing("M_2 and MUE")
c	endif


	call CharginoMasses(error, M_2_, MUE_)

	M_1_ = MSoft_M1
	Origin("M_1 from MSoft_M1")

c	if( ExtPar_M1 .ne. invalid ) then
c	  M_1_ = ExtPar_M1
c	  Origin("M_1 from ExtPar_M1")
c	else if( MSoft_M1 .ne. invalid ) then
c	  M_1_ = MSoft_M1
c	  Origin("M_1 from MSoft_M1")
c	else
c	  M_1_ = invalid
c	endif



c	if( M_1_ .ne. invalid ) then
c	  call NeutralinoMasses(error, M_1_)
c	else if( SLHAValid(Mass_MNeu(1), 4) .and.
c     &           SLHAValid(NMix_ZNeu(1,1), 16) ) then
c	  call NeutralinoPara(error,
c     &      Mass_MNeu(1), NMix_ZNeu(1,1))
c	else
c	  Missing("M_1")
c	endif

	call NeutralinoMasses(error, M_1_)
* Sfermion sector

	do g = 3, 1, -1
	  do t = 2, 4
c              Af_(t) = Af_Af(3,3,t)
              Af_(t) = Af_Af(g,g,t)
	      Q_(t) = Af_Q(t)
	      Origin("Af("//Digit(t)//",3) from Af_Af")
           enddo

           
c	    if( ExtPar_Af(t) .ne. invalid ) then
c	      Af_(t) = ExtPar_Af(t)
c	      Q_(t) = ExtPar_Q
c	      Origin("Af("//Digit(t)//",3) from ExtPar_Af")
c	    else if( Af_Af(3,3,t) .ne. invalid ) then
c	      Af_(t) = Af_Af(3,3,t)
c	      Q_(t) = Af_Q(t)
c	      Origin("Af("//Digit(t)//",3) from Af_Af")
c	    else
c	      if( g .eq. 3 ) then
c	        Af_(t) = invalid
c	      else
c	        Af_(t) = Af(t,g+1,g+1)
c	      endif
c	    endif
c	  enddo

c         do g = 3, 1, -1
	  do n = 1, 5
c	    if( n .gt. 2 .and. MSS2_MSS2(g,g,n) .ne. invalid ) then
c	      MSS2_(n) = DBLE(MSS2_MSS2(g,g,n))
c	      Origin(mssname(n)//Digit(g)//") from MSS_MSS2")
c	    else if( ExtPar_MSS(g,n) .ne. invalid ) then
c	      MSS2_(n) = DBLE(ExtPar_MSS(g,n))**2
c	      Origin(mssname(n)//Digit(g)//") from ExtPar_MSS")
c	    else if( MSoft_MSS(g,n) .ne. invalid ) then
	      MSS2_(n) = DBLE(MSoft_MSS(g,n))**2
	      Origin(mssname(n)//Digit(g)//") from MSoft_MSS")
c	    else
c	      MSS2_(n) = invalid
c	    endif
	  enddo

c	  if( MSS2_(1) .ne. invalid .and.
c     &        MSS2_(2) .ne. invalid .and.
c     &        MSS2_(3) .ne. invalid .and.
c     &        MSS2_(4) .ne. invalid .and.
c     &        MSS2_(5) .ne. invalid .and.
c     &        Af_(2) .ne. invalid .and.
c     &        Af_(3) .ne. invalid .and.
c     &        Af_(4) .ne. invalid ) then

	    call SfermionMasses(error, g,
     &        MSS2_(1), MSS2_(2), MSS2_(3), MSS2_(4), MSS2_(5),
     &        Af_(2), Af_(3), Af_(4), Q_(2), Q_(3), Q_(4))
c	  else if( SLHAValid(Mass_MSf(1,2,g), 3*2) .and.
c     &             SLHAValid(SfMix_USf(1,1,2), 3*4) ) then
c	    call SfermionPara(error, g,
c     &        Mass_MSf(1,2,g), Mass_MSf(1,3,g), Mass_MSf(1,4,g),
c     &        StauMix_USf(1,1), StopMix_USf(1,1), SbotMix_USf(1,1))
c	  else
c	    Missing("MSS and Af")
c	  endif

	enddo



	call EndPara(error)

        goto 100

	if( SLHAValid(BlockVCKM(1), LengthVCKM) ) then
	  call SetCKM(VCKM_theta12, VCKM_theta23,
     &      VCKM_theta13, VCKM_delta13)
	else if( SLHAValid(BlockVCKMInputs(1), LengthVCKMInputs) ) then
	  call SetCKM(VCKMInputs_theta12, VCKMInputs_theta23,
     &      VCKMInputs_theta13, VCKMInputs_delta13)
	endif

	if( SLHAValid(BlockMSQ2(1), LengthMSQ2) .or.
     &      SLHAValid(BlockMSU2(1), LengthMSU2) .or.
     &      SLHAValid(BlockMSD2(1), LengthMSD2) ) then
	  call SetNMFVSLHA(error, 3,
     &      MSQ2_MSQ2(1,1), MSU2_MSU2(1,1), Au_Af(1,1))
	  call SetNMFVSLHA(error, 4,
     &      MSQ2_MSQ2(1,1), MSD2_MSD2(1,1), Ad_Af(1,1))
	else if( SLHAValid(BlockMSQ2In(1), LengthMSQ2In) .or.
     &           SLHAValid(BlockMSU2In(1), LengthMSU2In) .or.
     &           SLHAValid(BlockMSD2In(1), LengthMSD2In) ) then
	  call SetNMFVSLHA(error, 3,
     &      MSQ2In_MSQ2(1,1), MSU2In_MSU2(1,1), Au_Af(1,1))
	  call SetNMFVSLHA(error, 4,
     &      MSQ2In_MSQ2(1,1), MSD2In_MSD2(1,1), Ad_Af(1,1))
	endif

 100	continue
	end
