* SLHAMode.F
* the FeynHiggs front end for SLHA input files
* this file is part of FeynHiggs
* last modified 13 Oct 06 th

#include "FeynHiggs.h"
#include "PDG.h"



	subroutine SLHAMode(error, scalefactor, slhadata, gammas, 
     &                      gammasms, prodxs)
	implicit none
	integer error
	double precision scalefactor

#include "SLHA.h"

	integer parent
	integer i, j, h, t, g, vv, c1, c2, n1, n2, s1, s2, hv, excl
	double complex slhadata(*)

	double precision MHiggs(4)
	double complex SAeff, DeltaSAeff, UHiggs(3,3), ZHiggs(3,3)

#if SLHAPARA
	integer nmfv
	double precision MSf(2,4,3), MASf(6,3:4)
	double precision MCha(2), MNeu(4)
	double complex UASf(6*6,3:4), DeltaMB
#endif

	double complex couplings(ncouplings), couplingsms(ncouplingsms)
	double precision gammas(ngammas), gammasms(ngammasms)
	double precision prodxs(nprodxs)

	character*256 outputfile

	integer lnblnk
	external lnblnk

	integer higgs(4), v_v(2,5), ferm(4,3), sferm(2,4,3)
	integer cha(2), neu(4)

* Important: unlike the charged (s)fermions, PDG_Hp, PDG_W, and 
* PDG_chargino{1,2} refer to *positive* particles.  This differs
* from the FeynArts conventions adopted in FeynHiggs where
* particles are generally negative and antiparticles positive.

	data higgs / PDG_h0, PDG_HH, PDG_A0, -PDG_Hp /

	data v_v /
     &    PDG_photon, PDG_photon,
     &    PDG_photon, PDG_Z,
     &    PDG_Z, PDG_Z,
     &    PDG_W, -PDG_W,
     &    PDG_gluon, PDG_gluon /

	data ferm /
     &    PDG_nu_e, PDG_electron, PDG_up, PDG_down,
     &    PDG_nu_mu, PDG_muon, PDG_charm, PDG_strange,
     &    PDG_nu_tau, PDG_tau, PDG_top, PDG_bottom /

	data sferm /
     &    PDG_snu_e1, PDG_snu_e2, PDG_selectron1, PDG_selectron2,
     &    PDG_sup1, PDG_sup2, PDG_sdown1, PDG_sdown2,
     &    PDG_snu_mu1, PDG_snu_mu2, PDG_smuon1, PDG_smuon2,
     &    PDG_scharm1, PDG_scharm2, PDG_sstrange1, PDG_sstrange2,
     &    PDG_snu_tau1, PDG_snu_tau2, PDG_stau1, PDG_stau2,
     &    PDG_stop1, PDG_stop2, PDG_sbottom1, PDG_sbottom2 /

	data cha / -PDG_chargino1, -PDG_chargino2 /

	data neu / PDG_neutralino1, PDG_neutralino2,
     &    PDG_neutralino3, PDG_neutralino4 /


      

	call FHSetSLHA(error, slhadata, scalefactor)

	if( error .ne. 0 ) goto 999

#if SLHAPARA
	call FHGetPara(error, nmfv,
     &    MSf, SfMix_USfFlat(1,1), MASf, UASf,
     &    MCha, UMix_UChaFlat(1), VMix_VChaFlat(1),
     &    MNeu, NMix_ZNeuFlat(1),
     &    DeltaMB)
	if( error .ne. 0 ) goto 999

	do t = 2, 2*(nmfv + 1)
	  Mass_MSf(1,t,1) = MSf(1,t,1)
	  Mass_MSf(2,t,1) = MSf(2,t,1)
	  Mass_MSf(1,t,2) = MSf(1,t,2)
	  Mass_MSf(2,t,2) = MSf(2,t,2)
	  Mass_MSf(1,t,3) = MSf(1,t,3)
	  Mass_MSf(2,t,3) = MSf(2,t,3)
	enddo

	if( nmfv .ne. 0 ) then
	  do t = 3, 4
	    Mass_MSf(1,t,1) = MASf(1,t)
	    Mass_MSf(2,t,1) = MASf(2,t)
	    Mass_MSf(1,t,2) = MASf(3,t)
	    Mass_MSf(2,t,2) = MASf(4,t)
	    Mass_MSf(1,t,3) = MASf(5,t)
	    Mass_MSf(2,t,3) = MASf(6,t)
	  enddo
	  do i = 1, 6*6
	    USqMix_UASfFlat(i) = UASf(i,3)
	    DSqMix_UASfFlat(i) = UASf(i,4)
	  enddo
	endif

	Mass_MCha(1) = MCha(1)
	Mass_MCha(2) = MCha(2)

	do i = 1, 4
	  Mass_MNeu(i) = MNeu(i)
	  if( DIMAG(NMix_ZNeu(i,1)) .ne. 0 ) then
	    Mass_MNeu(i) = -MNeu(i)
	    do j = 1, 4
	      NMix_ZNeu(i,j) = (0,-1)*NMix_ZNeu(i,j)
	    enddo
	  endif
	enddo
#endif

c	call FHHiggsCorr(error, MHiggs, SAeff, UHiggs, ZHiggs)
c	if( error .ne. 0 ) goto 999

c	Mass_Mh0 = MHiggs(1)
c	Mass_MHH = MHiggs(2)
c	Mass_MA0 = MHiggs(3)
c	Mass_MHp = MHiggs(4)

c	if( SAeff .eq. -2 ) then
c	  Alpha_Alpha = invalid
c	else
c	  Alpha_Alpha = asin(DBLE(SAeff))
c	endif

c	do i = 1, 3
c	  do j = 1, 3
c	    CVHMix_UH(i,j) = UHiggs(i,j)
c	  enddo
c	enddo

c	call FHUncertainties(error,
c     &    MHiggs, DeltaSAeff, UHiggs, ZHiggs)
c	if( error .ne. 0 ) goto 999

c	DMass_DeltaMh0 = MHiggs(1)
c	DMass_DeltaMHH = MHiggs(2)
c	DMass_DeltaMA0 = MHiggs(3)
c	DMass_DeltaMHp = MHiggs(4)

c	if( DeltaSAeff .eq. -2 ) then
c	  DAlpha_DeltaAlpha = invalid
c	else
c	  DAlpha_DeltaAlpha = .5D0*abs(
c     &      asin(min(DBLE(SAeff) + DBLE(DeltaSAeff), 1D0)) -
c     &      asin(max(DBLE(SAeff) - DBLE(DeltaSAeff), -1D0)) )
c	endif

	call FHCouplings(error,
     &    couplings, couplingsms, gammas, gammasms, excl)
	if( error .ne. 0 ) goto 999

	call FHHiggsProd(error, prodxs)
	if( error .ne. 0 ) goto 999

999     continue


	end

