* SingularValues.F
* singular value decomposition of an m-by-n matrix
* this file is part of FeynHiggs
* last modified 7 Jul 06 th

#include "externals.h"
#include "Util.h"


************************************************************************
** SingularValues performs a singular value decomposition.
** Input: m, n, A = m-by-n matrix (m > n).
** Output: d = n-vector of singular values,
** U = n-by-m left transformation matrix,
** V = n-by-n right transformation matrix,
** these fulfill diag(d) = U^* A V^+.

	subroutine SingularValues(m, n, A,ldA, d, U,ldU, V,ldV, sort)
	implicit none
	integer m, n, ldA, ldU, ldV, sort
	double complex A(ldA,*), U(ldU,*), V(ldV,*)
	double precision d(*)

	integer i, j, k, l, nz, iz(MAXMAT)
	logical valid(MAXMAT)
	double complex s
	double precision norm

	double complex AdagA(MAXMAT,MAXMAT)

	double precision eps
	parameter (eps = 5D-16)

* The singular values are the roots of the eigenvalues of A^+ A

	do k = 1, n
	  do i = 1, k
	    s = 0
	    do j = 1, m
	      s = s + DCONJG(A(j,i))*A(j,k)
	    enddo
	    AdagA(i,k) = s
	  enddo
	enddo
	call Eigensystem(n, AdagA,MAXMAT, d, V,ldV, sort)

	nz = 0
	do k = 1, n
	  valid(k) = d(k) .gt. eps
	  if( valid(k) ) then
	    d(k) = sqrt(d(k))
	    norm = 1/d(k)
	    do i = 1, m
	      s = 0
	      do j = 1, n
	        s = s + A(i,j)*DCONJG(V(k,j))
	      enddo
	      U(k,i) = s*norm
	    enddo
	  else
	    nz = nz + 1
	    iz(nz) = k
	  endif
	enddo

* The vectors corresponding to the zero singular values can be chosen
* arbitrarily.  We choose u_i = delta_ij and use Gram-Schmidt to
* orthonormalize these.

#define Delta(i,j) ibits(ieor(i,j)-1,15,1)

	l = m
	do i = 1, nz
	  norm = 0

1	  do j = 1, m
	    s = 0
	    do k = 1, n
	      if( valid(k) ) s = s + U(k,l)*DCONJG(U(k,j))
	    enddo
	    s = Delta(l, j) - s
	    U(iz(i),j) = s
	    norm = norm + DBLE(s)**2 + DIMAG(s)**2
	  enddo
	  valid(iz(i)) = .TRUE.
	  l = l - 1
	  if( norm .lt. eps ) goto 1

	  norm = 1/sqrt(norm)
	  k = iz(i)
	  do j = 1, m
	    U(k,j) = norm*U(k,j)
	  enddo
	enddo
	end

