/*
	CFeynHiggs.h
		C/C++ wrapper functions for the FH subroutines
		this file is part of FeynHiggs
		last modified 3 Nov 06 th
*/


#ifndef __CFeynHiggs_h__
#define __CFeynHiggs_h__

#include "FHCouplings.h"

#define couplingS(c) couplings[c-1]
#define couplingsmS(c) couplingsms[c-1]
#define gammaS(c) gammas[c-1]
#define gammasmS(c) gammasms[c-1]
#define prodXS(c) prodxs[c-1]

#if 1
#define fhsetflags	fhsetflags_
#define fhsetpara	fhsetpara_
#define fhsetslha	fhsetslha_
#define fhsetckm	fhsetckm_
#define fhsetnmfv	fhsetnmfv_
#define fhsetdebug	fhsetdebug_
#define fhgetpara	fhgetpara_
#define fhhiggscorr	fhhiggscorr_
#define fhuncertainties	fhuncertainties_
#define fhcouplings	fhcouplings_
#define fhselectuz	fhselectuz_
#define fhconstraints	fhconstraints_
#define fhgetself	fhgetself_
#define fhhiggsprod	fhhiggsprod_
#endif


void fhsetflags(int *error,
  const int *mssmpart, const int *fieldren, const int *tanbren,
  const int *higgsmix, const int *p2approx, const int *looplevel,
  const int *tl_running_mt, const int *tl_bot_resum);

static inline void FHSetFlags(int *error,
  const int mssmpart, const int fieldren, const int tanbren,
  const int higgsmix, const int p2approx, const int looplevel,
  const int tl_running_mt, const int tl_bot_resum)
{
  fhsetflags(error, &mssmpart, &fieldren, &tanbren,
    &higgsmix, &p2approx, &looplevel,
    &tl_running_mt, &tl_bot_resum);
}


void fhsetpara(int *error, const double *scalefactor,
  const double *AlfasMZ, const double *MC,
  const double *MT, const double *MB,
  const double *MW, const double *MZ,
  const double *TB, const double *MA0, const double *MHp,
  const double *M3SL, const double *M3SE,
  const double *M3SQ, const double *M3SU, const double *M3SD,
  const double *M2SL, const double *M2SE,
  const double *M2SQ, const double *M2SU, const double *M2SD,
  const double *M1SL, const double *M1SE,
  const double *M1SQ, const double *M1SU, const double *M1SD,
  const dcomplex *MUE,
  const dcomplex *Atau, const dcomplex *At, const dcomplex *Ab,
  const dcomplex *Amu, const dcomplex *Ac, const dcomplex *As,
  const dcomplex *Ae, const dcomplex *Au, const dcomplex *Ad,
  const dcomplex *M_1, const dcomplex *M_2, const dcomplex *M_3,
  const double *Qtau, const double *Qt, const double *Qb);

static inline void FHSetPara(int *error, const double scalefactor,
  const double AlfasMZ, const double MC,
  const double MT, const double MB,
  const double MW, const double MZ,
  const double TB, const double MA0, const double MHp,
  const double M3SL, const double M3SE,
  const double M3SQ, const double M3SU, const double M3SD,
  const double M2SL, const double M2SE,
  const double M2SQ, const double M2SU, const double M2SD,
  const double M1SL, const double M1SE,
  const double M1SQ, const double M1SU, const double M1SD,
  const dcomplex MUE,
  const dcomplex Atau, const dcomplex At, const dcomplex Ab,
  const dcomplex Amu, const dcomplex Ac, const dcomplex As,
  const dcomplex Ae, const dcomplex Au, const dcomplex Ad,
  const dcomplex M_1, const dcomplex M_2, const dcomplex M_3,
  const double Qtau, const double Qt, const double Qb)
{
  fhsetpara(error, &scalefactor,
    &AlfasMZ, &MC, &MT, &MB, &MW, &MZ,
    &TB, &MA0, &MHp,
    &M3SL, &M3SE, &M3SQ, &M3SU, &M3SD,
    &M2SL, &M2SE, &M2SQ, &M2SU, &M2SD,
    &M1SL, &M1SE, &M1SQ, &M1SU, &M1SD,
    &MUE,
    &Atau, &At, &Ab,
    &Amu, &Ac, &As,
    &Ae, &Au, &Ad,
    &M_1, &M_2, &M_3,
    &Qtau, &Qt, &Qb);
}


void fhsetslha(int *error,
  const dcomplex *slhadata, const double *scalefactor);

static inline void FHSetSLHA(int *error,
  const dcomplex *slhadata, const double scalefactor)
{
  fhsetslha(error, slhadata, &scalefactor);
}


void fhsetckm(int *error,
  const double *theta12, const double *theta23, const double *theta13,
  const double *delta13);

static inline void FHSetCKM(int *error,
  const double theta12, const double theta23, const double theta13,
  const double delta13)
{
  fhsetckm(error, &theta12, &theta23, &theta13, &delta13);
}


void fhsetnmfv(int *error,
  const dcomplex *deltaLLuc, const dcomplex *deltaLRuc,
  const dcomplex *deltaRLuc, const dcomplex *deltaRRuc,
  const dcomplex *deltaLLct, const dcomplex *deltaLRct,
  const dcomplex *deltaRLct, const dcomplex *deltaRRct,
  const dcomplex *deltaLLut, const dcomplex *deltaLRut,
  const dcomplex *deltaRLut, const dcomplex *deltaRRut,
  const dcomplex *deltaLLds, const dcomplex *deltaLRds,
  const dcomplex *deltaRLds, const dcomplex *deltaRRds,
  const dcomplex *deltaLLsb, const dcomplex *deltaLRsb,
  const dcomplex *deltaRLsb, const dcomplex *deltaRRsb,
  const dcomplex *deltaLLdb, const dcomplex *deltaLRdb,
  const dcomplex *deltaRLdb, const dcomplex *deltaRRdb);

void FHSetNMFV(int *error,
  const dcomplex deltaLLuc, const dcomplex deltaLRuc,
  const dcomplex deltaRLuc, const dcomplex deltaRRuc,
  const dcomplex deltaLLct, const dcomplex deltaLRct,
  const dcomplex deltaRLct, const dcomplex deltaRRct,
  const dcomplex deltaLLut, const dcomplex deltaLRut,
  const dcomplex deltaRLut, const dcomplex deltaRRut,
  const dcomplex deltaLLds, const dcomplex deltaLRds,
  const dcomplex deltaRLds, const dcomplex deltaRRds,
  const dcomplex deltaLLsb, const dcomplex deltaLRsb,
  const dcomplex deltaRLsb, const dcomplex deltaRRsb,
  const dcomplex deltaLLdb, const dcomplex deltaLRdb,
  const dcomplex deltaRLdb, const dcomplex deltaRRdb)
{
  fhsetnmfv(error,
    &deltaLLuc, &deltaLRuc, &deltaRLuc, &deltaRRuc,
    &deltaLLct, &deltaLRct, &deltaRLct, &deltaRRct,
    &deltaLLut, &deltaLRut, &deltaRLut, &deltaRRut,
    &deltaLLds, &deltaLRds, &deltaRLds, &deltaRRds,
    &deltaLLsb, &deltaLRsb, &deltaRLsb, &deltaRRsb,
    &deltaLLdb, &deltaLRdb, &deltaRLdb, &deltaRRdb);
}


void fhsetdebug(const int *debuglevel);

static inline void FHSetDebug(const int debuglevel)
{
  fhsetdebug(&debuglevel);
}


void fhgetpara(int *error,
  int *nmfv, double *MSf, dcomplex *USf, double *MASf, dcomplex *UASf,
  double *MCha, dcomplex *UCha, dcomplex *VCha,
  double *MNeu, dcomplex *ZNeu,
  dcomplex *DeltaMB);

#define FHGetPara fhgetpara


void fhhiggscorr(int *error, double *MHiggs,
  dcomplex *SAeff, dcomplex *UHiggs, dcomplex *ZHiggs);

#define FHHiggsCorr fhhiggscorr


void fhuncertainties(int *error, double *DeltaMHiggs,
  dcomplex *DeltaSAeff, dcomplex *DeltaUHiggs, dcomplex *DeltaZHiggs);

#define FHUncertainties fhuncertainties


void fhcouplings(int *error,
  dcomplex *couplings, dcomplex *couplingsms,
  double *gammas, double *gammasms, int *excl);

#define FHCouplings fhcouplings


void fhselectuz(int *error, const int *uzint, const int *uzext);

static inline void FHSelectUZ(int *error, const int uzint, const int uzext)
{
  fhselectuz(error, &uzint, &uzext);
}


void fhconstraints(int *error,
  double *gm2, double *deltarho,
  double *MWMSSM, double *MWSM, double *SW2MSSM, double *SW2SM,
  double *edmeTh, double *edmn, double *edmHg,
  double *bsgMSSM, double *bsgSM);

#define FHConstraints fhconstraints


void fhhiggsprod(int *error, double *prodxs);

#define FHHiggsProd fhhiggsprod


void fhgetself(int *error, const double *k2,
  const int *key, dcomplex *sig, const int *dkey, dcomplex *dsig);

static inline void FHGetSelf(int *error, const double k2,
  const int key, dcomplex *sig, const int dkey, dcomplex *dsig)
{
  fhgetself(error, &k2, &key, sig, &dkey, dsig);
}

#define Key(se) (1 << (se-1))

#endif

