! This program is based on the GetDist routine of CosmoMC  by Anthony Lewis & Sarah Bridle (cosmologist.info/cosmomc)
! modified to allow the computation of new derived observables and to read in param labels from .info file

!Output files are largely as in the original GetDist (refer to cosmologist.info/cosmomc for 
! details)

! * file_root.margestats - the mean, limits and stddev from the 1D marginalized distributions
! * file_root.likestats  - the best fit, and limits from the N-D confidence region
! * file_root_thin.txt   - thinned merged versions of input files
! * file_root_single.txt - File of weight-1 single samples generated by probabilistic importance sampling
! * file_root.covmat     - covariance matrix for parameters
! * file_root.corr       - covariance matrix for parameters normalized to have unit variance
! * file_root.converge   - Various statistics to help assess chain convergence

! Data for plots are exported to the plot_data folder
! stats test in module mcsamples.f9
! customize matlab plots (2D and scatter 3D) in module matlab
! customize SM plots (1D) below 
! SuperBayes Package - by Roberto Ruiz de Austri (rruiz@delta.ft.uam.es) and Roberto Trotta (rxt@astro.ox.ac.uk)
! This version May 2007 !

! Revision 1.13 2008/01/12
! Bug-fix regarding to produce 1D plots using the SM script. Producing 
! profile likelihood plots rather than Bayesian posterior ones.
! At lines 566 and 568: mx -> y 


program GetPlots

        use IniFile
        use MCSamples
        use matlab
        implicit none

        real  bincounts(-1000:1000),binlikes(-1000:1000),binmaxlikes(-1000:1000)
    
        character(LEN=80) InputFile, numstr, InfoFile
        character(LEN=300) fname
        character(LEN=30000) InLine, CLine  ! ,LastL,OutLine
        real invars(max_cols)
        real*8 :: highprec(max_cols)
        integer ix, ix1,i,ix2,ix3, nbins, ignorerows, wx
   
        logical bad, adjust_priors

        real  amax, amin
        character(LEN=120) filename, infile, in_root,out_root, sm_file
        character(LEN=120) matlab_col, InS1,InS2,gfmt
        integer plot_row, plot_col, chain_num, first_chain,chain_ix, plot_num
       
        integer x,y,j,j2, num_2D_plots, num_cust2D_plots
        integer num_3D_plots
        character(LEN=80) plot_3D(20)
        integer plot_x, plot_y, thin_factor, outliers
        real limmin(max_cols),limmax(max_cols), maxbin
        integer plot_2D_param, j2min
        real try_b, try_t, binweight
        real cont_lines(max_cols,2,2), dist, distweight, limfrac 
        integer bestfit_ix
        integer chain_exclude(max_chains), num_exclude
        logical map_params
        logical :: all_2D_plots = .false.
        real counts
        integer :: countres, tot_plots, failures = 0
   
        real cool
        integer plotparams_num, plotparams(max_cols)
        logical Done2D(max_cols,max_cols)

        real thin_cool
        logical no_tests

 
        InputFile = GetParam(1)
        if (InputFile == '') stop 'No parameter input file'

        !Gets chains filename
        call Ini_Open(InputFile, 1, bad, .false.)
        if (bad) stop 'Error opening parameter file'
        Ini_fail_on_not_found = .true.
        in_root = Ini_Read_String('file_root')
        rootname = ExtractFileName(in_root)
        call Ini_Close
        
        !Opens .info file to read lables from it
        !ncols set automatically
        InfoFile = 'chains/'//trim(rootname)//'.info'
        write(*,*) 'Opening .info file: ', trim(InfoFile)
        call Ini_Open(InfoFile, 1, bad, .false.)
        if (bad) then
           write(*,*) 'Stop - problem opening file: ', trim(InfoFile)
           stop
        end if
        !NumKeys is the number of lines in the .info file
        write(*,*) 'Number of parameters in MCMC chain:' , NumKeys
        ncols = NumKeys
        do ix=3, ncols
           write (numstr,*) ix-2
           lables(ix) = Ini_Read_String('lab'//trim(adjustl(numstr)), .true.)
        end do
        call Ini_Close

        lables(1) = 'mult'
        lables(2) = 'likelihood'
        Ini_fail_on_not_found = .false.

        !reopens .ini file for rest of parameters
        call Ini_Open(InputFile, 1, bad, .true.)
        if (bad) stop 'Error opening parameter file'
        chain_num = Ini_Read_Int('chain_num')
        nmapped = Ini_Read_Int('add_columns',0)
        !nmapped is the number of extra mapped params you want
        norig = ncols
        ncols = ncols + nmapped
        if (nmapped > 0) then
           write(*,*) 'Warning - adding ', nmapped, ' new params combinations'
           !reads in name of extra params
           do ix=1,nmapped
              write (numstr,*) ix
              lables(norig+ix) = Ini_Read_String('labA'//trim(adjustl(numstr)), .true.)
           end do
        end if



        if (ncols > max_cols) stop 'Increase max_cols'

        write(*,*) 'Allocation'
        allocate(coldata(ncols,0:max_rows))


        !if you want 1 or 2 tails confidence limits
        do ix=3, ncols
           if (ix <= norig) then
              write (numstr,*) ix-2
              InLine = Ini_Read_String('tails'//trim(adjustl(numstr)), .false.)
           else
              write (numstr,*) ix-norig
              InLine = Ini_Read_String('tailsA'//trim(adjustl(numstr)), .false.)
           end if
           if (InLine /= '') then
              read (InLine,*) tails(ix) 
           end if
        end do

        nbins = Ini_Read_Int('num_bins')
        skip_bin = Ini_Read_Int('skip_bin', 0)
        ignorerows = Ini_Read_Int('ignore_rows',0)
        adjust_priors = Ini_Read_Logical('adjust_priors',.false.)

        all_2D_plots = Ini_Read_Logical('all_2D_plots',.false.)
        no_tests = Ini_Read_Logical('no_tests',.false.)

        thin_factor = Ini_Read_Int('thin_factor',0)
        thin_cool = Ini_read_Real('thin_cool',1.)

        first_chain = Ini_Read_Int('first_chain',1)

        make_single_samples = Ini_Read_logical('do_3D_plots',.false.)
        single_thin = Ini_Read_Int('single_thin',1)
        cool = Ini_Read_Real('cool',1.)

        Num_ComparePlots = Ini_Read_Int('compare_num',0)
        do ix = 1, Num_ComparePlots
           ComparePlots(ix) = ExtractFileName(Ini_Read_String(numcat('compare',ix), .true.))
        end do

        has_limits_top = .false.
        has_limits_bot = .false.

        !limits (only for plotting limits) 
        do ix=3, ncols
           if (ix <= norig+2) then
              write (numstr,*) ix-2
              InLine = Ini_Read_String('limits'//trim(adjustl(numstr)))
           else
              write (numstr,*) ix-norig
              InLine = Ini_Read_String('limitsA'//trim(adjustl(numstr)))
           end if
           if (InLine /= '') then
              read (InLine,*) InS1, InS2 
              if (trim(adjustl(InS1)) /= 'N') then
               has_limits_bot(ix) = .true.
               read(InS1,*) limmin(ix)
              end if
              if (trim(adjustl(InS2)) /= 'N') then
                has_limits_top(ix) = .true.
                read(InS2,*) limmax(ix)
              end if
           end if
        end do
        has_limits = has_limits_top .or. has_limits_bot

        plot_2D_param = Ini_Read_Int('plot_2D_param',0)
        if (plot_2D_param /= 0) then
           plot_2D_param = plot_2D_param + 2
           num_cust2D_plots = 0
        else
           !Use custom array of specific plots
           num_cust2D_plots = Ini_Read_Int('plot_2D_num',0)
           do ix = 1, num_cust2D_plots
              InLine = Ini_Read_String_A(numcat('plot',ix), norig, .true.)
              read (InLine, *) ix1,ix2
              cust2Dplots(ix) = ix1+2 + (ix2+2)*1000 
           end do
        end if

        InS1 =Ini_Read_String('exclude_chain')
        num_exclude = 0
        chain_exclude = 0
        read (InS1, *, end =20) chain_exclude
        
20      do while (chain_exclude(num_exclude+1)/=0) 
           num_exclude = num_exclude + 1
        end do
    

        map_params = Ini_Read_Logical('map_params',.false.)
        if (map_params) write (*,*) 'WARNING: Mapping params - .covmat file is new params.'

        smoothing = Ini_Read_Logical('smoothing',.true.)

        out_root = Ini_Read_String('out_root')
        if (out_root /= '') then
           rootname = out_root
           write (*,*) 'producing files with root '//trim(out_root)
        end if
        sm_file = trim(outn)//trim(rootname) // '.sm'

        contours(1) = Ini_Read_Real('contour1',0.)
        contours(2) = Ini_Read_Real('contour2',0.)
        num_contours = count(contours/= 0)
        if (contours(1)==0) contours(1) = contours(2)

        force_twotail = Ini_Read_Logical('force_twotail',.false.)  
        if (force_twotail) then
           write (*,*) 'Computing two tail limits'
           tails(:) = 2
        end if
        
        covmat_dimension = Ini_Read_Int('cov_matrix_dimension',0)

        plot_NDcontours = Ini_Read_Logical('plot_NDcontours',.false.)

        plotparams_num = Ini_Read_Int('plotparams_num',0)
        if (plotparams_num /= 0) then
           InLine = Ini_Read_String_A('plotparams', norig)
           read(InLine,*,end=500) plotparams(1:plotparams_num)
        end if

        
        num_3D_plots = Ini_Read_Int('num_3D_plots',0)
        do ix =1, num_3D_plots
           plot_3D(ix) = Ini_Read_String_A(numcat('3D_plot',ix), norig, .true.)
        end do
        if (num_3D_plots > 0) make_single_samples = .true. 

        do_shading = Ini_Read_Logical('do_shading',.true.)
        call Ini_Close 

        !allocation
        allocate(bins2D(-1000:1000,-1000:1000),bin2Dlikes(-1000:1000,-1000:1000),bin2Dmax(-1000:1000,-1000:1000))
 

        !Read in the chains     
        nrows = 0
        num_chains_used =0

        do chain_ix = first_chain, first_chain-1 + max(1,chain_num)

           do ix = 1, num_exclude
              if (chain_exclude(ix) == chain_ix) goto 30
           end do
      
           if (chain_num == 0) then
              infile = trim(in_root) // '.txt'
           else
              write (numstr,*) chain_ix
              infile = trim(in_root) //'_'//trim(adjustl(numstr))// '.txt'
           end if

           write (*,*) 'reading ' // trim(infile)
           failures = 0
           open(unit=50,file=infile,form='formatted',status='old', err=28)

           do ix = 1, ignorerows
              read (50,'(a)',end=1) InLine
           end do

           num_chains_used = num_chains_used + 1
           if (num_chains_used > max_chains) stop 'Increase max_chains in GetDist'
           chain_indices(num_chains_used) = nrows
           chain_numbers(num_chains_used) = chain_ix
           do
12            read (50,'(a)',end=1) InLine
              read(InLine,*,end=1, err=2) highprec(1:ncols-nmapped)
              
              invars(1:ncols-nmapped) = real(highprec(1:ncols-nmapped))
              !write(*,*) 'Mapping params'
              if (map_params) call MapParameters(invars)
              if (nmapped > 0) call AddParameters(invars)
              
              coldata(1:ncols, nrows) = invars(1:ncols)
              nrows = nrows + 1
              if (nrows > max_rows) stop 'need to increase max_rows'
              
           end do
2          write (*,*) 'error reading line ', nrows + ignorerows, ' - trying to continue...'
           write(*,*) 'Offending line ', trim(InLine)
           failures = failures+1
           nrows = nrows + 1
           if (failures<max_failures  ) then
              goto 12
           else
              write(*,*) 'Too many errors, skipping the rest of the file'   
           end if
1          close(50)
           goto 30
28         write (*,'(" chain ",1I4," missing")') chain_ix
30      end do
        
        if (nrows == 0) stop 'No un-ignored rows!'
        

        if (cool /= 1) call CoolChain(cool)
        !Adjust weights if requested
        if (adjust_priors) then
           call AdjustPriors
        end if
        
        call GetUsedCols !See which parameters are fixed
        

        mean_mult = sum(coldata(1,0:nrows-1))/nrows
        max_mult = (mean_mult*nrows)/min(nrows/2,500)
        outliers = count(coldata(1,0:nrows-1) > max_mult)
        if (outliers /=0) write (*,*) 'outlier fraction ', real(outliers)/nrows
        
        maxmult = maxval(coldata(1,0:nrows-1))
        numsamp = sum(coldata(1,0:nrows-1))
        
        if (.not. no_tests) call DoConvergeTests
        if (adjust_priors) call DeleteZeros  
        write (*,*) 'mean input multiplicity = ',mean_mult
        
        !Output thinned data if requested
        !Must do this with unsorted output  
        if (thin_factor /= 0) then 
           call ThinData(thin_factor)
           call WriteThinData(trim(rootname)//'_thin.txt',thin_cool)
        end if
        
        !Produce file of weight-1 samples if requested
        if (make_single_samples) then
           write(*,*) 'single thin'
           single_thin = max(1,nint(numsamp/maxmult)/2000)
        end if
        if (make_single_samples) call MakeSingleSamples(single_thin)
        
        !Sort data in order of likelihood of points
        call SortColData(2)
        numsamp = sum(coldata(1,0:nrows-1))
        
        !Get ND confidence region (index into sorted coldata)
        counts = 0
        ND_cont1=-1; ND_cont2=-1
        do j=0, nrows -1
           counts = counts + coldata(1,j)
           if (counts > numsamp*contours(1) .and. ND_cont1==-1) then
              ND_cont1 = j
              if (ND_cont2 /= -1) exit
           end if
           if (counts > numsamp*contours(2) .and. ND_cont2==-1) then
              ND_cont2 = j
           end if
        end do
        
       !Only use variables whose lable's are not empty (and in list of plotparams if plotparams_num /= 0)
        num_vars = 0
        do ix = 3,ncols
           if (lables(ix) /= '' .and. isused(ix)) then
              if (plotparams_num == 0 .or. any(plotparams(1:plotparams_num)==ix-2)) then
                 num_vars = num_vars + 1
                 colix(num_vars) = ix
                 mean(num_vars) = sum(coldata(1,0:nrows-1)*coldata(ix,0:nrows-1))/numsamp
                 sddev(num_vars)  = sqrt(sum(coldata(1,0:nrows-1)*(coldata(ix,0:nrows-1) &
                      -mean(num_vars))**2)/numsamp)
              end if
           end if
        end do
        
        all_2D_plots = all_2D_plots .and. (num_vars > 1)
        write (*,*) 'using ',nrows,' rows, processing ',num_vars,' parameters'
        write(*,*) 'All 2D plots status = ', all_2D_plots
        if (indep_thin/=0) then
           write (*,*) 'Approx indep samples: ', nint(numsamp/indep_thin)
        else
           write (*,*) 'effective number of samples (assuming indep): ', nint(numsamp/maxmult)
        end if
        !Get covariance matrix and correlation matrix
        call GetCovMatrix
        
        !Find best fit, and mean likelihood     
        bestfit_ix = 0 !Since we have sorted the lines
        maxlike = coldata(2,bestfit_ix)
        write (*,*) 'Best fit -Ln(like) = ', maxlike
        
        if (coldata(2,nrows-1) - maxlike < 30) then
           meanlike = log(sum(exp((coldata(2,0:nrows-1) -maxlike))*coldata(1,0:nrows-1)) & 
                / numsamp) + maxlike
           write (*,*) 'Ln(mean 1/like) = ', meanlike
        end if
        
        meanlike = sum(coldata(2,0:nrows-1)*coldata(1,0:nrows-1)) / numsamp 
        write (*,*) 'mean(-Ln(like)) = ', meanlike
        
        meanlike = -log(sum(exp(-(coldata(2,0:nrows-1) -maxlike))*coldata(1,0:nrows-1)) & 
             / numsamp) + maxlike
        write (*,*) '-Ln(mean like)  = ', meanlike
        
        !Output files for 1D plots
        plot_col =  nint(sqrt(num_vars/1.4))
        plot_row = (num_vars +plot_col-1)/plot_col 
        !So we don't forget to run sm, delete current .ps file
        call DeleteFile(trim(rootname)//'.ps')
        
        
        open(unit=50,file=sm_file,form='formatted',status='replace')
        !SuperMongo .sm file for 1D plots
        write (50,*) 'DEFINE TeX_strings 1'
        write (50,*) 'device postfile '//trim(rootname)//'.ps'
        write (50,*) 'DEFINE default_font rm'
        if (num_vars > 6) then
           write (50,*) 'expand 0.8'
           write (50, *)'define x_gutter 0.6'
           write (50,*) 'define y_gutter 0.8'
        else
           write (50,*) 'expand 1.1'
        endif
        write (50,*) 'location 2500 31000 3500 31000'
        write (50,*) 'lweight 3'
        cont_lines = 0

          
!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                                    !Do 1D bins
!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
          write(*,'(A,I6,A,I6)') 'Total number of variables = ', colix(num_vars), ' Used variables =', num_vars

          do j = 1,num_vars
             ix = colix(j)
             write(*,'(A,I4,A,I4,A,I2,2A)') 'Now doing 1D var j=', j, ' ix = ', ix,  ' tails = ', tails(ix),  ' name = ', trim(lables(ix))
             bincounts = 0
             binlikes = 0
             binmaxlikes = 0
             
             do ix1 = 1, num_contours
                limfrac = (1-contours(ix1))
                !              if (.not. has_limits(ix) .or. force_twotail) then
                !                limfrac = limfrac/2 !two tail
                !              end if
                if (tails(ix) .eq. 2) then
                   limfrac = limfrac/2 !two tail
                end if
                cont_lines(j,2,ix1) = ConfidVal(ix,limfrac,.true.)
                cont_lines(j,1,ix1) = ConfidVal(ix,limfrac,.false.)
             end do !contour lines

             if (has_limits_bot(ix)) then
                amin = limmin(ix)
             else
                amin = ConfidVal(ix,0.0001,.false.)
             end if
             
             if (has_limits_top(ix)) then
                amax = limmax(ix)
             else
                amax = ConfidVal(ix,0.0001,.true.)
             end if

             width(j) = (amax-amin)/(nbins+1)
             !write(*,*) 'width = ', width(j), amax-amin
             if (width(j)==0) cycle
             if (amin <= 0) then 
               center(j) = amax
             else
               center(j) = amin
             end if

             ix_min(j) = nint((amin - center(j))/width(j))
             ix_max(j) = nint((amax - center(j))/width(j))

             if (smoothing  .and. amin > 0) ix_min(j) = ix_min(j)-1
             if (smoothing) ix_max(j) = ix_max(j)+1

             do i = 0, nrows-1
                ix2=nint((coldata(ix,i)-center(j))/width(j))
                !puts all points above limits into upper bin                                                                                                                                 
                if (ix2 > ix_max(j)) ix2 = ix_max(j)
                !correspondingly for points below lower limit                                                                                                                                
                if (ix2 < ix_min(j)) ix2 = ix_min(j)

                if (smoothing .and. ix_min(j) /= ix_max(j)) then
                   do wx = max(ix_min(j),ix2-2),min(ix_max(j),ix2+2)
                      dist = coldata(colix(j),i) - (center(j) + wx*width(j))
                      distweight = exp(- dist**2/width(j)**2/2)
                      binweight = coldata(1,i)*distweight
                      bincounts(wx) = bincounts(wx)  + binweight
                      binlikes(wx) = binlikes(wx) + binweight*exp(meanlike-coldata(2,i))
                      binmaxlikes(wx) = max(binmaxlikes(wx), &
                           real(exp(maxlike-coldata(2,i))*distweight))
                   end do
                else
                   bincounts(ix2) = bincounts(ix2) + coldata(1,i)
                   binlikes(ix2) = binlikes(ix2) + coldata(1,i)*exp(meanlike-coldata(2,i))
                   binmaxlikes(ix2) = max(binmaxlikes(ix2), real(exp(maxlike-coldata(2,i))))
                end if
             end do

             
             do i=ix_min(j),ix_max(j)
                !put to zero isolated points, with less than 1 per mille of samples                    
                !if (bincounts(i) < 0.001*numsamp) then
                !   bincounts(i) = 0
                !   binlikes(i) = 0
                !end if
                if (bincounts(i) >0) binlikes(i) = binlikes(i)/bincounts(i)
             end do

             !data files for plotting (1D)
             fname =trim(trim(rootname)//trim(numcat('_p',colix(j)-2)))
             filename = 'plot_data/'// trim(fname)
             open(unit=49,file=trim(filename)//'.dat',form='formatted',status='replace')
             write(*,*) 'Opening for writing: ', trim(filename)

             maxbin = maxval(bincounts(ix_min(j)+skip_bin:ix_max(j)-skip_bin))
             do i = ix_min(j)+skip_bin, ix_max(j)-skip_bin
                write (49,'(3E15.5)') center(j) + (i+0.5)*width(j), bincounts(i)/maxbin, binmaxlikes(i)
             end do
             if (ix_min(j) == ix_max(j)) write (49,'(1E15.5,'' 0'')') center(j) + ix_min*width(j)
             close(49)

             if (plot_meanlikes) then
                open(unit=49,file=trim(filename)//'.likes',form='formatted',status='replace')       
                maxbin = maxval(binlikes(ix_min(j)+skip_bin:ix_max(j)-skip_bin))
                do i = ix_min(j)+skip_bin, ix_max(j)-skip_bin
                   write (49,'(5E15.5)') center(j) + (i+0.5)*width(j), binlikes(i)/maxbin, binlikes(i), bincounts(i), maxbin
                end do
                close(49)
             end if

             !----------------------------------------------------
             !SM 1D plot file
             !----------------------------------------------------
             plot_x = mod(j+plot_col-1,plot_col)+1
             plot_y = plot_row - (j+plot_col-1)/plot_col + 1 
             write (50,'(''window'',4I4)') plot_col,plot_row, plot_x, plot_y
             write (50,*) 'data "../' // trim(filename) //'.dat"'
             write (50,*) 'read { x 1 y 2 mx 3}'
             if (ix_min(j) == ix_max(j)) then
                write (50,'(''limits '',2E15.5,'' y'')') center(j) - abs(center(j))/2-0.1, &
                     center(j) + abs(center(j))/2+0.1
             else
                if (has_limits(ix)) then
                   write (50,'(''limits '',2E15.5,'' 0 1.1'')') amin, amax
                else
                   write (50,*) 'limits x 0 1.1'
                end if
             end if
             write (50,*) 'box 1 3 3 3'
             if (smoothing) then
                write (50,*) 'connect x y '
             else
                write (50,*) 'histogram x y '
             end if
             if (plot_meanlikes) then
                write (50,*) 'data "../' // trim(filename)//'.likes"'
                write (50,*) 'read { x 1 y 2 }'
                write (50,*) 'ltype 1'
                if (smoothing) then 
                   write (50,*) 'connect x y'
                   else
                   write (50,*) 'histogram x y'
                end if
             end if
         
             !         write (50,*) 'ltype 1'
             !         do ix1 = 1, num_contours
             !         do i2 =1,2
             !         write (numstr,*) cont_lines(j,i2,ix1)
             !         write (50,*) 'set x = {'//trim(numstr)//' '//trim(numstr)//'}'
             !         write (50,*) 'set y = { 0 1.1}'
             !!         write (50,*) 'connect x y'
             !uncomment this if you want dooted lines on 1D plots for condifence regions
             !Note 1D confidence regions are computed from full samples using different definition to
             !2D confidence regions
             !         end do
             !         end do
             write (50,*) 'ltype 0'
             write (50,*) 'xlabel '// trim(lables(ix))
             write (50,*)
             write (50,*) '#Next...'
          end do
          write (50,*) 'quit'
          close(50)
          
                    
!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                                    !Do 2D bins
!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

          if (plot_2D_param == 0 .and. num_cust2D_plots==0) then
             !In this case output the most correlated variable combinations
             write (*,*) 'doing 2D plots for most correlated variables'
             try_t = 1e5
             x=0;y=0;  
             num_cust2D_plots = 12
              do j = 1, num_cust2D_plots
               try_b = -1e5
               do ix1 =1 ,num_vars
                do ix2 = ix1+1,num_vars 
                   if (abs(corrmatrix(colix(ix1)-2,colix(ix2)-2)) < try_t .and. &
                        abs(corrmatrix(colix(ix1)-2,colix(ix2)-2)) > try_b) then
                      try_b = abs(corrmatrix(colix(ix1)-2,colix(ix2)-2))
                      x = ix1; y = ix2;
                    end if
                end do
               end do
               if (try_b == -1e5) then
                  num_cust2D_plots = j-1
                  exit
               end if
               try_t = try_b

             cust2Dplots(j) = colix(x) + colix(y)*1000
             end do
          end if !most correlated

          !no custom 2D plots specified, ie plot_2D_num == 0
          if (num_cust2D_plots == 0) then
             num_2D_plots = 0
             !does 2D plots for all variables with labels and not are not fixed
             do j= 1, num_vars
                if (ix_min(j) /= ix_max(j)) then
                   do j2 = j+1, num_vars
                      if (ix_min(j2) /= ix_max(j2)) then
                         if (plot_2D_param==0 .or. plot_2D_param == colix(j) .or. plot_2D_param == colix(j2)) &
                              num_2D_plots = num_2D_plots + 1
                      end if
                   end do
                end if
             end do
          else
             num_2D_plots = num_cust2D_plots
          end if
          done2D= .false.


          if (num_2D_plots > 0) then
             write (*,*) 'Producing ',num_2D_plots,' 2D plots'
             !---------------------------------------------
             !Matlab 2D file
             !--------------------------------------------
             filename = trim(outn)//trim(rootname)//'_2D.m'
             open(unit=50,file=filename,form='formatted',status='replace')
             !here all the header stuff you might need
             call WriteMatLabInit(50)
             plot_num = 0         
             do j= 1, num_vars
                if (ix_min(j) /= ix_max(j)) then
                   if (plot_2D_param/=0 .or. num_cust2D_plots /= 0) then
                      !looks for the corresponding column in file
                      if (colix(j) == plot_2D_param) cycle
                      j2min = 1
                   else
                      j2min = j+1
                   end if
                   do j2 = j2min, num_vars
                      if (ix_min(j2) /= ix_max(j2)) then
                         !write(*,*) 'some info for you'
                         !write(*,*) 'num_cust2D_plots = ', num_cust2D_plots
                         !countres = count(cust2Dplots(1:num_cust2D_plots) == colix(j)*1000 + colix(j2))
                         if (plot_2D_param/=0 .and. colix(j2) /= plot_2D_param) cycle
                         if (num_cust2D_plots /= 0 .and.  &
                              !the line below returns .false. if this is the plot you want
                              count(cust2Dplots(1:num_cust2D_plots) == colix(j)*1000 + colix(j2))==0) cycle
                         plot_num = plot_num + 1
                         write(*,'(A,I4,A,3A)') 'Doing 2D plot', plot_num, ' for params ', trim(lables(colix(j))), ' vs ', trim(lables(colix(j2)))
                         done2D(j,j2) = .true.
                         call Get2DPlotData(j,j2)
                         !marginal distribution
                         call Write2DPlotMATLAB(50,j,j2,.false.)
                         write(50, *) 'hold off;'
                         write (50,*) 'print -dpsc2 ../'//trim(outn)//trim(rootname)//'_2D_'//trim(IntToStr(plot_num))//'.ps;'
                         !likelihood
                         call Write2DPlotMATLAB(50,j,j2,.true.)
                         write(50, *) 'hold off;'
                         write (50,*) 'print -dpsc2 ../'//trim(outn)//trim(rootname)//'_2D_like_'//trim(IntToStr(plot_num))//'.ps;'
                      end if
                   end do
                end if
             end do
             if (plot_num < num_cust2D_plots) then
                write(*,'(A,I4,A)') 'WARNIG - you have requested', num_cust2D_plots, ' 2D plot(s) '
                write(*,'(A,I4,A)') '         but only', plot_num, ' have been produced'
                write(*,'(A)')      '         You might want to check your choice for plotparams'
             end if
              
             close(50)

          end if !num_2D_plots > 0

!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                                    !Do ALL 2D plots (all_2D_plots = T)
!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
          if (all_2D_plots) then
             plot_num = 0
             tot_plots = (num_vars**2)/2-num_vars
             do j2 = 1, num_vars
                do j = j2+1, num_vars                
                   !invert axis x <-> y 
                   plot_num = plot_num+1
                   if (mod(plot_num, 100) == 0) write(*,*) 'Done 2D triangle plot ', plot_num , ' over ', tot_plots
                   if (.not. Done2D(j2,j)) call Get2DPlotData(j2,j)
                end do
             end do
          end if
         
!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
!                 DO 3D plots (.m file only, everything is in a single file already)
!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
         !Do 3D plots (i.e. 2D scatter plots with coloured points)

         if (num_3D_plots /=0) then
            write (*,*) 'producing ',num_3D_plots, '3D scatter plots'
            filename = trim(outn)//trim(rootname)//'_3D.m'
            open(unit=50,file=filename,form='formatted',status='replace')
            write(50,*)  'load(''../colormaps/Redmap'',''redmap'');'
            write(50,*) 'colormap(''jet'');'
            write(50,'(A)') 'lab_fontsize = 24; axes_fontsize = 20; names_fontsize = 16;  textbox_fontsize = 30;  twolines_fontsize = 20;'
            if (mod(num_3D_plots,2)==0 .and. num_3D_plots < 11) then
               plot_col = num_3D_plots/2
            else
               plot_col =  nint(sqrt(1.*num_3D_plots))
            end if
            plot_row = (num_3D_plots +plot_col-1)/plot_col 
            
            write(50,*) 'pts = load('''//trim(rootname)//'_single.txt'');'             

            do j=1, num_3D_plots
               read(plot_3D(j),*) ix1, ix2, ix3  !x, y, color
               if (ix3<1) ix3 = MostCorrelated2D(ix1,ix2,ix3)
               
               write (50,*) 'figure'
               write (50,*) 'set(gcf, ''NextPlot'', ''replacechildren'');'
               write (50,*) 'axes(''position'', [0.3 0.3 0.5 0.5]   );'
               write (50,*) 'box on; axis square; hold on;'

               write (50,*) '%Do params ',ix1,ix2,ix3
               write (50,*) 'scatter(pts(:,',2+ix1,'),pts(:,',2+ix2,'),3,pts(:,',2+ix3,'));'
               gfmt = ''',''FontSize'',lab_fontsize);'
               !xlabel, slightly more sophisticated
               write (50,'(A)') ' text(''String'','''//trim(lables(ix1+2))//''',''FontSize'',lab_fontsize, ''Units'', ''normalized'', ''Position'', [0.5 -0.18], ''HorizontalAlignment'', ''center'');'
               !write (50,*) 'xlabel('''//trim(lables(ix1+2))//trim(gfmt)
               !ylabel
               write (50,*) 'ylabel('''//trim(lables(ix2+2))//trim(gfmt)
               write (50,'(A)') 'text(''String'', ''\it{SuperBayeS v 1.0}'',''FontSize'',names_fontsize, ''Units'', ''normalized'', ''Position'', [1.0 1.05], ''HorizontalAlignment'', ''right'');'
               write (50,*) 'set(gca,''FontSize'',axes_fontsize,  ''LineWidth'', lwaxes); pa = gca;'
               !colorbar
               write(50,*) 'cname  = '''//trim(lables(ix3+2))//''';'
               write (50,*) 'hbar = colorbar(''horiz'');axes(hbar); colormap(redmap);'
               write(50,*) 'hpos = get(hbar, ''Position'');'
               write(50,'(A)') 'text(''String'', cname,''FontSize'',axes_fontsize, ''Units'', ''normalized'', ''Position'', [0.5 2.5], ''HorizontalAlignment'', ''center'');'
                write(50,*) 'set(gca,''FontSize'',axes_fontsize);'
                write(50,*) 'set(hbar, ''Position'',[hpos(1) hpos(2)*0.3 hpos(3) hpos(4)/2]);'

               write (50,*)  'set(gcf, ''PaperUnits'',''centimeters'');'
               dummystring =trim(IntToStr(j))
               write (50,*) 'set(gcf, ''PaperPosition'',[ 0 0 20 20]);' 
               write (50,*) 'set(gcf, ''PaperType'', ''A4'');'
               write (50,*) 'set(gcf, ''PaperOrientation'', ''portrait''); '       
               write (50,*) 'print -dpsc2 '//trim(rootname)//'_3D_'//trim(dummystring)//'.ps;'
            end do
           
            close(50)
         end if
    
!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
!                output of statistics
!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

         write(*,*) 'writing margestats file... please wait'
         open(unit=50,file=trim(outn)//trim(rootname)//'.margestats',form='formatted',status='replace')
         write(50,*) 'param  mean        sddev       lower1      upper1      lower2      upper2 tails'
         do j=1, num_vars
            write(50,'(1I5,6E15.6,1I7"   '//trim(lables(colix(j)))//'")') colix(j)-2, mean(j), sddev(j), &
                 cont_lines(j,1:2,1),cont_lines(j,1:2,2),tails(colix(j))
         end do
         close(50)

         write(*,*) 'writing likestats file... please wait'         
         !Limits from global likelihood
         open(unit=50,file=trim(outn)//trim(rootname)//'.likestats',form='formatted',status='replace')
         write (50,*) 'Best fit sample -log(Like) = ',coldata(2,bestfit_ix)
         write (50,*) ''
         write(50,*) 'param  bestfit     lower1      upper1      lower2      upper2 tails'
         
         do j=1, num_vars
            
            write(50,'(1I5,5E19.10,1I7"   '//trim(lables(colix(j)))//'")') colix(j)-2, coldata(colix(j),bestfit_ix),&
                 minval(coldata(colix(j),0:ND_cont1)), &
                 maxval(coldata(colix(j),0:ND_cont1)), &
                 minval(coldata(colix(j),0:ND_cont2)), &
                 maxval(coldata(colix(j),0:ND_cont2)), tails(colix(j))
            
         end do
         close(50)

         stop

500      stop 'error reading plotparams list in .ini file - does not match plotparams_num'
         


       end program GetPlots

