!This module contains the routines to deal with the Postprocessing
!SuperBayes Package - by Roberto Ruiz de Austri (rruiz@delta.ft.uam.es) and Roberto Trotta (rxt@astro.ox.ac.uk)
!This version October 2007 

module MCMC_theory
 use Random
 use settings
 use ParamDef
 use calclike
 use likedata 

 implicit none
  

 contains


 subroutine GetTheoryInfo(MSSM, NuisP, DSO, SUSYSpec, OutSoft, BDecay, Prodxbr, error)


  Type(Input_Params):: MSSM
  Type(Nuisance_Params):: NuisP
  Type(Susy_spec):: SUSYSpec
  Type(DS_Output) :: DSO
  Type(SoftOut) :: OutSoft 
  Type(Bdecay_Out) :: Bdecay
  Type(Prodxbr_Out) :: Prodxbr

  integer :: error


  error = 0

 !computing SUSY spectrum 
  if(.not.GFlags%ID%evar .or. (GFlags%ID%evar .and. (GIDin%egam == GIDin%efluxes_i))) then

   call GetSoftSusySpectrum(MSSM, NuisP, SUSYSpec, OutSoft, DSO%Err)
 
   if (Feedback > 2) Call DebugSoftErrorOutput(DSO%Err)   
   !calls DS and Feynhiggs to predict the quantities of interest
  endif


  if (.not.  ErrorsPresent(DSO%Err)) then

   !sets up ID inputs
   if(GFlags%ID_predict) then
    if (.not. allocated(DSO%ID_in%hmodel)) allocate (DSO%ID_in%hmodel(num_hm))
    DSO%ID_in = GIDin          
   end if
   !Prodxbr will contain on output the predictions by Feynhiggs
 
  
   call GetPredictions(MSSM, NuisP, OutSoft, DSO, Prodxbr, BDecay)
  
  else
   error = 1

  endif



  end subroutine GetTheoryInfo 


 subroutine FlagsChains(in_unit)

  integer :: in_unit
  

  write(in_unit, *) 'compute_DM = ', GFlags%DM_predict 
  write(in_unit, *) 'compute_Direct_Detection = ', GFlags%DD_predict
  write(in_unit, *) 'compute_Indirect_Detection = ', GFlags%ID_predict
  write(in_unit, *) 'compute_Collider_Predictions = ', GFlags%Collider_predict
  write(in_unit, *) 'compute_BD = ', GFlags%BD_predict
  write(in_unit, *) 'compute_FH = ', GFlags%FH_predict

  if (GFlags%ID_predict)  then
    write(in_unit, *) 'num_hm =', GIDin%num_hm 
    write(in_unit, *) 'compute_ID_gadiff =', GFlags%ID%gadiff
    write(in_unit, *) 'compute_ID_gacont =', GFlags%ID%gac
    write(in_unit, *) 'compute_ID_gamonoc = ', GFlags%ID%gam
    write(in_unit, *) 'compute_ID_antiprot = ', GFlags%ID%pb
    write(in_unit, *) 'compute_ID_antideut = ', GFlags%ID%db
    write(in_unit, *) 'compute_ID_posit = ', GFlags%ID%ep
    write(in_unit, *) 'compute_ID_positfrac = ', GFlags%ID%epfrac
    write(in_unit, *) 'compute_ID_muonsun = ', GFlags%ID%musu
    write(in_unit, *) 'compute_ID_muonearth = ', GFlags%ID%muea
    write(in_unit, *) 'compute_ID_muonsundiff = ', GFlags%ID%musudiff
    write(in_unit, *) 'compute_ID_muonearthdiff = ', GFlags%ID%mueadiff
    write(in_unit, *) 'compute_ID_muonhalo = ', GFlags%ID%muha
    write(in_unit, *) 'compute_ID_muonhalodiff = ', GFlags%ID%muhadiff
    write(in_unit, *) 'compute_ID_sigmav = ', GFlags%ID%sigmav
  endif


 close(in_unit)

 end  subroutine FlagsChains



 subroutine  CheckChains(InputFile)

  logical bad
  character(LEN=80) InputFile


  call Ini_Open(InputFile, ppfile_unit, bad, .false.)


  GFlags_old%DM_predict = Ini_Read_Logical('compute_DM')
  GFlags_old%DD_predict = Ini_Read_Logical('compute_Direct_Detection')
  GFlags_old%ID_predict = Ini_Read_Logical('compute_Indirect_Detection') 
  GFlags_old%Collider_predict = Ini_Read_Logical('compute_Collider_Predictions')
  GFlags_old%BD_predict = Ini_Read_Logical('compute_BD')
  GFlags_old%FH_predict = Ini_Read_Logical('compute_FH')

  if(GFlags_old%ID_predict) then
    GIDin_old%num_hm = Ini_Read_Int('num_hm')
    GFlags_old%ID%gadiff = Ini_Read_Logical('compute_ID_gadiff') 
    GFlags_old%ID%gac = Ini_Read_Logical('compute_ID_gacont')
    GFlags_old%ID%gam = Ini_Read_Logical('compute_ID_gamonoc')
    GFlags_old%ID%pb  = Ini_Read_Logical('compute_ID_antiprot') 
    GFlags_old%ID%db  = Ini_Read_Logical('compute_ID_antideut') 
    GFlags_old%ID%ep  = Ini_Read_Logical('compute_ID_posit') 
    GFlags_old%ID%epfrac = Ini_Read_Logical('compute_ID_positfrac')
    GFlags_old%ID%musu = Ini_Read_Logical('compute_ID_muonsun')
    GFlags_old%ID%muea = Ini_Read_Logical('compute_ID_muonearth')
    GFlags_old%ID%musudiff = Ini_Read_Logical('compute_ID_muonsundiff')
    GFlags_old%ID%mueadiff = Ini_Read_Logical('compute_ID_muonearthdiff')
    GFlags_old%ID%muha = Ini_Read_Logical('compute_ID_muonhalo')
    GFlags_old%ID%muhadiff = Ini_Read_Logical('compute_ID_muonhalodiff')
    GFlags_old%ID%sigmav = Ini_Read_Logical('compute_ID_sigmav')
  endif

  close(ppfile_unit)

 end subroutine  CheckChains



 subroutine ReadChains(in_unit, ChainDSP, ChainSusySp, ChainBdecay, ChainReducedProdxbr, Xt, Xb, mu, tan_2thetat, tan_2thetab)

  Type(Susy_spec)::  ChainSusySp
  Type(DS_Output) :: ChainDSP
  Type(Bdecay_Out) :: ChainBdecay
  Type(Prodxbr_Out) :: ChainReducedProdxbr

  integer :: in_unit, num_hm
  real(8) :: Xt, Xb, mu, tan_2thetat, tan_2thetab
  character(LEN = 200) :: fmt_idhmd_old

  !init for formatting
  num_hm =  GIDin_old%num_hm

  !Old format for ID output
  fmt_idhmd_old = '('//trim(numcat(' ',num_hm))//'E15.5)'


  If(GFlags_old%DM_predict) read(in_unit, fmt_dm, ADVANCE='NO', EOR = 66) ChainDSP%dm
  if(GFlags_old%DD_predict) read(in_unit, fmt_dd, ADVANCE='NO', EOR = 66) ChainDSP%dd
  if(GFlags_old%Collider_predict) read(in_unit, fmt_coll, ADVANCE='NO', EOR = 66) ChainDSP%coll
  if(GFlags_old%BD_predict) read(in_unit, fmt_bdecay, ADVANCE='NO', EOR = 66) ChainBdecay
  read(in_unit, fmt_susysp, ADVANCE='NO') ChainSusySp
  read(in_unit, fmt_xtxbmu, ADVANCE='NO') Xt, Xb, mu
  read(in_unit, fmt_tantheta, ADVANCE='NO') tan_2thetat, tan_2thetab


  if(.not.redo_theory) then
   if (GFlags_old%FH_predict) &
    read(in_unit, fmt_prodxbr, ADVANCE='NO', EOR = 66) ChainReducedProdxbr
   if (GFlags_old%ID_predict)  then
     if (GFlags_old%ID%gadiff) read(in_unit, fmt_idhmd_old, ADVANCE='NO', EOR = 66) ChainDSP%id%fluxgadiff(1:num_hm)
     if (GFlags_old%ID%gac) read(in_unit, fmt_idhmd_old, ADVANCE='NO',EOR = 66) ChainDSP%id%fluxgac(1:num_hm)
     if (GFlags_old%ID%gam) then 
      read(in_unit, fmt_idhmd_old, ADVANCE='NO', EOR = 66) ChainDSP%id%fluxgaga(1:num_hm)
      read(in_unit, fmt_idhmd_old, ADVANCE='NO', EOR = 66) ChainDSP%id%fluxgaz(1:num_hm)
     endif 
     if (GFlags_old%ID%pb) read(in_unit, fmt_idhmd_old, ADVANCE='NO', EOR = 66) ChainDSP%id%phipb(1:num_hm) 
     if (GFlags_old%ID%db) read(in_unit, fmt_idhmd_old, ADVANCE='NO', EOR = 66) ChainDSP%id%phidb(1:num_hm)
     if (GFlags_old%ID%ep) read(in_unit, fmt_idhmd_old, ADVANCE='NO', EOR = 66) ChainDSP%id%phiep(1:num_hm)
     if (GFlags_old%ID%epfrac) read(in_unit, fmt_idhmd_old, ADVANCE='NO', EOR = 66) ChainDSP%id%phiep_frac(1:num_hm)
     if (GFlags_old%ID%musu) read(in_unit, fmt_idhmd_old, ADVANCE='NO', EOR = 66) ChainDSP%id%ratesu(1:num_hm)
     if (GFlags_old%ID%muea) read(in_unit, fmt_idhmd_old, ADVANCE='NO', EOR = 66) ChainDSP%id%rateea(1:num_hm)
     if (GFlags_old%ID%musudiff) read(in_unit, fmt_idhmd_old, ADVANCE='NO', EOR = 66) ChainDSP%id%ratesudiff(1:num_hm)
     if (GFlags_old%ID%mueadiff) read(in_unit, fmt_idhmd_old, ADVANCE='NO', EOR = 66) ChainDSP%id%rateeadiff(1:num_hm)
     if (GFlags_old%ID%muha) read(in_unit, fmt_idhmd_old, ADVANCE='NO', EOR = 66) ChainDSP%id%phimuhalo(1:num_hm)
     if (GFlags_old%ID%muhadiff) read(in_unit, fmt_idhmd_old, ADVANCE='NO', EOR = 66) ChainDSP%id%phimuhalodiff(1:num_hm)
     if (GFlags_old%ID%sigmav) read(in_unit, fmt_idhmid, ADVANCE='NO', EOR = 66) ChainDSP%id%sigmav
   endif
 endif

66 continue


 end subroutine ReadChains

end module MCMC_theory
