! SuperBayes Package - by Roberto Ruiz de Austri (rruiz@delta.ft.uam.es) and Roberto Trotta (rxt@astro.ox.ac.uk)
! This version October 2007 !
! Main file
program SuperBayes

        use IniFile
        use mc_mssm      !Monte Carlo routines to implement SUSY params & like
        use postprocess  !Post Processing of chains
        use ParamDef     !defined in paramdef.f90
	use settings     !in settings.f90
        use likedata     !numbers for the likelihood function

        implicit none
      
        character(LEN=80) InputFile, LogFile

        logical bad
        integer numsets, i, action
        character(LEN=80) rootname, out_rootname, filename, numstr, infoname, ppname
        Type(ParamSet) Params, EstParams
	integer num_points
        integer status
        real delta_loglike
        character(LEN = 5000) InLine
        real :: tmult, tlike
        double precision intime
        integer ierror, myrank, rc   

        integer  lines_file_unit, lines

        !default values for non-MPI runs (single chain)
        myrank = 0
        nprocs = 1
        instance = 0

#ifdef MPI
        call mpi_init(ierror)
        if (ierror/=MPI_SUCCESS) then
           print *,'Error starting MPI program. Terminating.'
           call MPI_ABORT(MPI_COMM_WORLD, rc, ierror)
        end if


        call MPI_COMM_RANK(MPI_COMM_WORLD, myrank,  ierror)
        call MPI_COMM_SIZE(MPI_COMM_WORLD, nprocs, ierror)
        if (Feedback >0) write(*,*) 'Number of processors or tasks=', &
             & nprocs,' My rank= ',myrank
#endif
        !The master process gets the file (MPI or not)
        if (myrank .eq. 0) then
           write(*,*) 'This is SuperBayeS '//trim(version)
	   InputFile = GetParam(1)
           if (InputFile == '') call DoStop('No parameter input file')
           if (Feedback >0) write(*,*)  'InputFile: ',InputFile
        end if

#ifdef MPI
        call MPI_BCAST(InputFile,79,MPI_CHARACTER,0,MPI_COMM_WORLD,ierror)
#endif        

        !every processor reads in the InputFile
        call Ini_Open(InputFile, 1, bad, .false.)
        if (bad)  call DoStop('Error opening parameter file: '//trim(InputFile))

#ifdef MPI 
        instance = myrank +1 !start at 1 for chains
        write (numstr,*) instance
        MPI_StartTime = MPI_WTime()
#endif
        
       Ini_fail_on_not_found = .true.
       rootname = Ini_Read_String('file_root')
     
       ppname = rootname 

!-----------------------------------
! Reading in of settings
!-----------------------------------


       ! ---- Feedback Level and what to compute ----
       FeedBack = Ini_Read_Int('feedback',0)
       write(*,*) 'Feedback is ', Feedback

       ! ---- Action type ----
       action = Ini_Read_Int('action',0)
       propose_grid = .false. 
       if(action == 4) propose_grid = .true.

        

       !for a restart, the continuation is always appended
       !to the previous files
       if (action .ne. 1) then
          restart = Ini_Read_Logical('restart_and_continue')
          numtoget = Ini_Read_Int('samples')
          out_rootname = rootname
          infoname = rootname
       else
          !action .eq. 1
          postproc = .true.
          !post processing: do you want to recompute the theory?
          redo_theory =  Ini_Read_Logical('redo_theory')
          !post processing: do you want to recompute the likelihood?
          redo_like = Ini_Read_Logical('redo_like')
          if (redo_theory) redo_like = .true.
          if (.not. redo_theory .and. .not. redo_like) call DoStop('Nothing to do in postprocessing!')
          if(.not. redo_theory .and. redo_like) & 
!          call system ('echo Warning: if you only recompute the likelihood the compute flags must match with the ones used for the old chains "(" ie. look at'//trim(rootname)//'.pp file")" >& 2')
          write(*, *) 'Warning: if you only recompute the likelihood the compute flags must match with the ones used for the old chains ( ie. look at'//trim(rootname)//'.pp file)'       
          !only changing likelihoods  not weights
          redo_change_like_only =  Ini_Read_Logical('redo_change_like_only',.false.)
          if(redo_change_like_only) &
           write (*,*) 'Warning: only changing likelihoods not weights'
          if (redo_theory) redo_like = .true.
          !you'd better recompute the like if you changed your predictions!
          out_rootname = Ini_Read_String('out_root')
          skip_lines = Ini_Read_Int('skip_lines', 0)
          infoname = out_rootname
       end if

       ! ---- MPI file names ----
       if (instance /= 0) then
          rootname = trim(rootname)//'_'//trim(adjustl(numstr))
          if (out_rootname .ne. '') out_rootname = trim(out_rootname)//'_'//trim(adjustl(numstr))
       end if
       if (feedback > 0) then
          if(action .ne. 1) then
             write(*,*) 'This is chain ', instance
             write(*,*) 'Writing to file: ', trim(rootname)
          else
             write(*,*) 'Post-processing chain: ', trim(rootname)
             write(*,*) 'Writing to file: ', trim(out_rootname)
          end if
       end if !feedback if
    

       ! ---- what we need to compute ----
       GFlags%DM_predict = Ini_Read_Logical('compute_DM')
       GFlags%DD_predict = Ini_Read_Logical('compute_Direct_Detection')
       GFlags%ID_predict = Ini_Read_Logical('compute_Indirect_Detection') 
       GFlags%Collider_predict = Ini_Read_Logical('compute_Collider_Predictions')
       GFlags%BD_predict = Ini_Read_Logical('compute_BD')
       GFlags%FH_predict = Ini_Read_Logical('compute_FH')

        !---- adding ID parameters ---
        if(GFlags%ID_predict) then 
           num_hm = Ini_Read_Int('num_hm')
           if(num_hm>10) &
              call DoStop('Not permitted more than 10 halo models at once')
           allocate(GIDin%hmodel(num_hm)) 
           do i = 1, num_hm
              write(numstr,*) i 
              GIDin%hmodel(i) = Ini_Read_String('model'//adjustl(numstr))
           enddo
           if(num_hm == 0) then 
              GIDin%num_hm = 1
           else
              GIDin%num_hm = num_hm
           endif
           GIDin%pbmodel = Ini_Read_String('pbmodel')
           GIDin%epmodel = Ini_Read_String('epmodel')
           GIDin%ntmodel = Ini_Read_String('ntmodel')
           GIDin%cospsi0 = Ini_Read_Double('cospsi0',1.d0)
           GIDin%delta_gamma   = Ini_Read_Double('delta_gamma',1.d-5)
           GIDin%egam    = Ini_Read_Double('egam')
           GIDin%egath   = Ini_Read_Double('egath')
           GIDin%BF      = Ini_Read_Double('BF',1.d0)
           GIDin%epb     = Ini_Read_Double('epb',1.76d0)
           GIDin%edb     = Ini_Read_Double('edb')
           GIDin%eep     = Ini_Read_Double('eep')
           GIDin%eth     = Ini_Read_Double('eth',25.d0)
           GIDin%thmax   = Ini_Read_Double('thmax',30.d0)
           GIDin%rtype   = Ini_Read_Int('rtype',3)
           GIDin%enu     = Ini_Read_Double('enu')
           GIDin%theta   = Ini_Read_Double('theta',30.d0)
           GIDin%delta_nt = Ini_Read_Double('delta_nt',1.d-5)
           GIDin%efluxes_i = Ini_Read_Double('ei')
           GIDin%efluxes_f = Ini_Read_Double('ef')   
           GIDin%nbins   = Ini_Read_Int('nbins')

           GFlags%ID%gadiff = Ini_Read_Logical('compute_ID_gadiff') 
           if(GFlags%ID%gadiff) num_ID = num_ID + 1
           GFlags%ID%gac = Ini_Read_Logical('compute_ID_gacont')
           if(GFlags%ID%gac) num_ID = num_ID + 1
           GFlags%ID%gam = Ini_Read_Logical('compute_ID_gamonoc')
           if(GFlags%ID%gam .and. (.not.GFlags%ID%gac .and. .not.GFlags%ID%gadiff)) &
           call DoStop('You cannot compute monocromatic fluxes without computing the continous diff. or total ones.')
           if(GFlags%ID%gam) num_ID = num_ID + 2
           GFlags%ID%pb  = Ini_Read_Logical('compute_ID_antiprot') 
           if(GFlags%ID%pb) num_ID = num_ID + 1
           GFlags%ID%db  = Ini_Read_Logical('compute_ID_antideut') 
           if(GFlags%ID%db) num_ID = num_ID + 1
           GFlags%ID%ep  = Ini_Read_Logical('compute_ID_posit') 
           if(GFlags%ID%ep) num_ID = num_ID + 1
           GFlags%ID%epfrac = Ini_Read_Logical('compute_ID_positfrac')
           if(GFlags%ID%epfrac) num_ID = num_ID + 1
           GFlags%ID%musu = Ini_Read_Logical('compute_ID_muonsun')
           if(GFlags%ID%musu) num_ID = num_ID + 1
           GFlags%ID%muea = Ini_Read_Logical('compute_ID_muonearth')
           if(GFlags%ID%muea) num_ID = num_ID + 1
           GFlags%ID%musudiff = Ini_Read_Logical('compute_ID_muonsundiff')
           if(GFlags%ID%musudiff) num_ID = num_ID + 1
           GFlags%ID%mueadiff = Ini_Read_Logical('compute_ID_muonearthdiff')
           if(GFlags%ID%mueadiff) num_ID = num_ID + 1
           GFlags%ID%muha = Ini_Read_Logical('compute_ID_muonhalo')
           if(GFlags%ID%muha) num_ID = num_ID + 1
           GFlags%ID%muhadiff = Ini_Read_Logical('compute_ID_muonhalodiff')
           if(GFlags%ID%muhadiff) num_ID = num_ID + 1
           GFlags%ID%sigmav = Ini_Read_Logical('compute_ID_sigmav')
           if(GFlags%ID%sigmav) then 
             num_ID = num_ID + 1
             num_hmnd = num_hmnd + 1 
           endif
           GFlags%ID%evar = Ini_Read_Logical('compute_ID_efluxes')
           !ID fluxes spectrum only permitted in the PP  
           if(action /= 1) GFlags%ID%evar = .false.
           if(num_hm == 0 .or. &
              (num_hm > 0 .and. num_ID-num_hmnd == 0)) then 
              GIDin%num_hm = 1
           else
              GIDin%num_hm = num_hm
           endif

           if(num_ID-num_hmnd > 0 .and. num_hm == 0) &
               call DoStop('You cannot try to compute halo model dependent parameters with num_hm = 0. Pls check this in the .ini file.')

        endif
      

       ! --- CDM made only of LSP ---
       CDM_purely_LSP = Ini_Read_Logical('CDM_purely_LSP', .true.)
       GFlags%DD_rescal = .not.(CDM_purely_LSP) 
       !The case where LSP is not the full CDM needs checking
       if (action == 0 .or. action == 4 .or. action .eq. 1) then
          LogFile = trim(out_rootname)//'.log'
          if (LogFile /= '') then
             logfile_unit = 49
             if (.not. restart) then
                call CreateTxtFile(LogFile,logfile_unit)
             else
                call OpenFileToAppend(LogFile, logfile_unit,'formatted')
                write(logfile_unit, '(A)') "----- Chain restarted -----"
                call FlushFile(logfile_unit)
             end if
          else
             logfile_unit = 0
          end if
          
          if (.not. restart) then
             if (action .eq. 1) call OpenTxtFile(trim(rootname)//'.txt',readin_file_unit)
             !creates new .txt files
             call CreateTxtFile(trim(out_rootname)//'.txt', outfile_unit)
          else
             !saves the last line of previous file in RestartLine
             !then used in paramdef.f90, Initialize routine
             if (action .eq. 1) call DoStop('restarting while post-processing not yet supported!')
             InLine = ''
             call OpenTxtFile(trim(rootname)//'.txt',tmp_file_unit)
             do
                read(tmp_file_unit,'(a)', iostat=ierror) InLine
                if (ierror < 0) exit
                read(InLine, *) tmult, tlike
                if (tlike < MaxLike) MaxLike = tlike
                num = num + tmult
                num_accept = num_accept+1
             end do
             close(tmp_file_unit)

             RestartLine = InLine
             num_accept = num_accept+1
             if (num_accept .ge. numtoget) call DoStop('No restart - sampling completed!')
             num_at_restart = num !used for timing purposes
             !open the file for further writing
             call OpenFileToAppend(trim(rootname)//'.txt', outfile_unit,'formatted')
             if (Feedback > 0) then
                write(*,*) 'Restarting from files ', trim(rootname)
                write(*,*) 'Resuming from:'
                write(*,*) '  num        = ', num
                write(*,*) '  num_accept = ', num_accept
                write(*,*) '  MaxLike    = ', real(MaxLike)
             end if
          end if
          !creates new .info file, regardless if you have restarted or not
          call CreateTxtFile(trim(infoname)//'.info', infofile_unit)
          call SetFormat

          !creates new .pp file
          if(action /= 1) then 
           call CreateTxtFile(trim(ppname)//'.pp', ppfile_unit)
           call FlagsChains(ppfile_unit)
          endif

          Ini_fail_on_not_found = .false.
          burn_in = Ini_Read_Int('burn_in',0)	
          
       end if

       numstr = Ini_Read_String('rand_seed')
       if (numstr /= '') then
	  read(numstr,*) i
	  call InitRandom(i)
       else
	  call InitRandom()
       end if


       ! --- Sampling method ---- 
       if (action .eq. 0) then
          use_nuisance_splitting = Ini_Read_Logical('Use_nuisance_splitting', .false.)
          oversample_nuisance = Ini_Read_Int('oversample_nuisance',1)
          sampling_method = Ini_Read_Int('sampling_method', 1)
          Temperature = Ini_Read_Real('temperature',1.)
          
          if (sampling_method > 1) then
             !this needs testing !
             slice_sampling = .true.
             procedure = Ini_Read_Int('slicing_procedure')
          end if
       end if
       ! --- Parameterization choice ---- 
       !whether to use log scale on m_0 and m_1/2
       use_log = Ini_Read_Logical('use_log')
       Use_Nuisance = Ini_Read_Logical('Use_Nuisance',.true.)
       

       ! ---- Data to include ------
       Use_CDM = Ini_Read_Logical('Use_CDM',.false.)
       Use_LEP = Ini_Read_Logical('Use_LEP',.false.)
       Use_Anomalous_Mu = Ini_Read_Logical('Use_Anomalous_Mu',.false.)
       Use_bsgamma = Ini_Read_Logical('Use_bsgamma',.false.)
       Use_Bsmumu = Ini_Read_Logical('Use_Bsmumu',.false.)
       Use_DD = Ini_Read_Logical('Use_Direct_Detection',.false.) 
       Use_ID = Ini_Read_Logical('Use_Indirect_Detection',.false.)       	
       Use_Mass_W = Ini_Read_Logical('Use_Mass_W',.false.)
       Use_Weak_Mixing = Ini_Read_Logical('Use_Weak_Mixing_Angle',.false.)      
       Use_Bdecay = Ini_Read_Logical('Use_Bdecay',.false.)  

       use_data = Ini_Read_Int('use_data')
    
       if (use_data .eq. synthetic_data) write(*,*) 'Warning - you are using synthetic data'

       if (Feedback > 1) then

          if (Use_CDM) write(*,*) 'Including CDM abundance data'        
          if (Use_LEP) write(*,*) 'Using LEP bounds on Higgs and Chargino Masses'
          if (Use_Anomalous_Mu) write(*,*) 'Including Mu Anomalous magnetic moment data'
          if (Use_bsgamma) write(*,*) 'Including b -> s gamma decay data'
          if (Use_Bsmumu) write(*,*) 'Including B_s -> mu+mu- BR bound'
          if (Use_Mass_W) write(*,*) 'Including M_W data'
          if (Use_Weak_Mixing) write(*,*) 'Including weak mixing angle data'          
          if (Use_Bdecay) write(*,*) 'Including B_s - bar(B)_s osillations'
          
       end if

       !----- Some checks for consistency of requests ----
        If (Use_CDM .and. (.not. GFlags%DM_predict)) then
           call DoStop('You cannot compare CDM abundance if you dont compute it!')
        end If
        
        if ((Use_bsgamma .or. Use_Bsmumu .or. Use_Anomalous_Mu) .and. (.not. GFlags%Collider_predict)) then
           call DoStop('You cannot use collider data if you dont set compute_Collider_Predictions = T')
           if (Use_DD .and. (.not. GFlags%DD_predict)) then
              call DoStop('You cannot use Direct Detection bound without computing the predictions!')
           end if
        end if
        
        if (Use_Bdecay .and. (.not. GFlags%BD_predict)) then
           call DoStop('You cannot use B oscillations without computing BDecay!')
        end if

        !---- Miscellaneous settings ----- 
        delta_loglike = Ini_Read_Real('delta_loglike',2.)
        
	num_threads = Ini_Read_Int('num_threads',0)
	!$ if (num_threads /=0) call OMP_SET_NUM_THREADS(num_threads)

	Ini_fail_on_not_found = .true.

	call Initialize(Params)
        
        call InitializeDataSets

        call Ini_Close
	num_points = 0


         
	 if (action == 0 .or. action .eq. 4) then
            !-----------------------------------
            ! Starts MCMC or grid
            !-----------------------------------	
            if (Feedback > 0) then
               if (action .eq. 0) then
                  write (*,*) 'starting Monte-Carlo'
                  write (*,*) 'Sampling method : ', sampling_method
                  if (slice_sampling) write(*,*) 'Slicing procedure: ', procedure
               end if
               if (action .eq. 4) then
                  write(*,*) 'starting with grid scan - all physical points will be saved'
               end if
            end if
            
            call MCMCSample(Params, numtoget)
            
            write (*,*)'Finished'
            write(*,*) 'Chain ', instance, ': Fct  evalutations: ', num
            
            if (logfile_unit /=0) close(logfile_unit)
            if (indepfile_unit /=0) close(indepfile_unit)
            close(outfile_unit)
            
         else if (action==1) then
            !-----------------------------------
            ! Starts post-processing
            !-----------------------------------
            
            !computes the number of samples in the chains
            lines_file_unit = 38	
            call CreateTxtFile(trim(rootname)//'_lines', lines_file_unit)
            call system ('wc -l '//trim(rootname)//'.txt | awk'//" '{ print $1 }' >"//trim(rootname)//"_lines")
            call OpenTxtFile(trim(rootname)//'_lines',lines_file_unit)
            read(lines_file_unit,*) lines
            close(lines_file_unit)
            call system('rm '//trim(rootname)//'_lines')
            
            if (Feedback > 0) write (*,*) 'starting post processing'
            call PostProcChains(ppname, readin_file_unit, lines)
            close(outfile_unit)
            close(readin_file_unit)
            write (*,*) 'Post-processing finished (chain ', instance, ')'
            
         else
            call DoStop('undefined action')
         end if
         !-----------------------------------
         ! Wrapping up loose ends
         !-----------------------------------
         write(*,*) 'I am chain', instance, ' and I have finished my run'
         call DoStop

       end program SUPERBAYES
