!This module contains the definition of some of the likelihood data
!(more numbers in calclike.f90)
!update this module with new observational data or with future data
!SuperBayes Package - by Roberto Ruiz de Austri (rruiz@delta.ft.uam.es) and Roberto Trotta (rxt@astro.ox.ac.uk)
!This version August 2007
!Visit www.superbayes.org for more info and updates

module likedata
  use ParamDef

  implicit none

  Type(LikeDatum) :: Mtop, Mbottom, alpha_em_inv, alpha_S
  Type(LikeDatum) :: CDM, AnomalousMu, Bsgamma 
  Type(LikeDatum) :: MW, SinEff, BdecayData 
  Type(LikeDatum) :: Bsmumu, MHiggs

contains

subroutine InitializeDataSets

if (use_data .eq. current_data) then
   !Those value from Table 2 in Spergel et al (2006),
   !(WMAP3 only), checked Nov 2006 RT
   CDM%datum_type = Gaussian
   CDM%mu=0.1037
   CDM%sigma=0.009
   CDM%tau=0.10
   CDM%tau_percent = .true.


   Mtop%datum_type = Gaussian
   Mtop%tau = 0.0
   Mtop%tau_percent = .false.
   !Updated Jun 2007 
   Mtop%mu = 170.9
   Mtop%sigma = 1.8

   Mbottom%datum_type = Gaussian
   Mbottom%tau = 0.0
   Mbottom%tau_percent = .false.
   !Updated Nov 2006
   Mbottom%mu = 4.20
   Mbottom%sigma = 0.07

   alpha_em_inv%datum_type = Gaussian
   alpha_em_inv%tau = 0.0
   alpha_em_inv%tau_percent = .false.
   !hep-ph/0611102, Updated Nov 2006
   alpha_em_inv%mu =  127.955
   alpha_em_inv%sigma = 0.030

   alpha_S%datum_type = Gaussian
   alpha_S%tau = 0.0
   alpha_S%tau_percent = .false.
   !Updated Nov 2006
   alpha_S%mu =  0.1176
   alpha_S%sigma = 0.002

   AnomalousMu%datum_type = Gaussian
   !all numbers in units of 10^10
   AnomalousMu%tau = 1.0
   AnomalousMu%tau_percent = .false.
   !Updated Nov 06, hep-ph/0611102 
   AnomalousMu%mu =  28.0
   AnomalousMu%sigma = 8.1

   Bsgamma%datum_type = Gaussian
   !all numbers in units of 10^4
   Bsgamma%tau = 0.21
   Bsgamma%tau_percent = .false.
   !Nov 06 - changed to new SM computation by Misiak (hep-ph/0609241)
   !Updated experimental value, May 06, from HFAG
   Bsgamma%mu =  3.55
   Bsgamma%sigma = 0.26

   MW%datum_type = Gaussian
   MW%tau = 0.015
   MW%tau_percent = .false.
   !Updated June 2007
   MW%mu =  80.398
   MW%sigma =  0.025

   SinEff%datum_type = Gaussian
   SinEff%tau =  15.0*1E-5 
   SinEff%tau_percent = .false.
   !Updated Nov 2006, hep-ex/0509008
   SinEff%mu =  0.23153
   SinEff%sigma =  16.0*1E-5 

   BdecayData%datum_type = Gaussian
   BdecayData%tau =  4.8 
   BdecayData%tau_percent = .false.
   !updated June 07
   BdecayData%mu =  17.77 
   BdecayData%sigma =  0.12 

   Bsmumu%datum_type = UpperLimit
   Bsmumu%tau =  0.14
   Bsmumu%tau_percent = .true.
   !updated June 07, data in units of 10^7
   Bsmumu%mu =  1.00 !this is the value of the limit
   Bsmumu%sigma =  0.0 

else if (use_data .eq. synthetic_data) then
!  Those numbers for an hypothetical future data set
!  with reduced errors around the best fit model with params and NO NOISE for the moment
!  m_0        m_1/2      A_0        tan(beta)       sign(mu)
!  1192.      273.4      1871.       55.07             1
!
!  M_top      mb_mb      1/alpha      alpha_S
!  170.4      4.295     0.7814E-02    0.11720  

   !Planck-type exp error
   CDM%datum_type = Gaussian
   CDM%mu=0.11081  
   CDM%sigma=0.001 !factor of 10
   CDM%tau=0.02    !factor of 5
   CDM%tau_percent = .true.


   Mtop%datum_type = Gaussian
   Mtop%tau = 0.0
   Mtop%tau_percent = .false.
   Mtop%mu = 170.4372 
   Mtop%sigma = 0.9 !factor of 2

   Mbottom%datum_type = Gaussian
   Mbottom%tau = 0.0
   Mbottom%tau_percent = .false.
   Mbottom%mu = 4.2948
   Mbottom%sigma = 0.03 !factor of 2

   alpha_em_inv%datum_type = Gaussian
   alpha_em_inv%tau = 0.0
   alpha_em_inv%tau_percent = .false.
   alpha_em_inv%mu =  127.97959
   alpha_em_inv%sigma = 0.015 !factor of 2

   alpha_S%datum_type = Gaussian
   alpha_S%tau = 0.0
   alpha_S%tau_percent = .false.
   !Updated Nov 2006
   alpha_S%mu =  0.117201
   alpha_S%sigma = 0.001 !factor of 2

   AnomalousMu%datum_type = Gaussian
   !all numbers in units of 10^10
   AnomalousMu%tau = 1.0 !the same
   AnomalousMu%tau_percent = .false.
   AnomalousMu%mu =  12.606
   AnomalousMu%sigma = 2.0 !factor of 4

   Bsgamma%datum_type = Gaussian
   !all numbers in units of 10^4
   Bsgamma%tau = 0.10 !factor of 2
   Bsgamma%tau_percent = .false.
   Bsgamma%mu =  3.172
   Bsgamma%sigma = 0.10 !factor of 3 almost

   MW%datum_type = Gaussian
   MW%tau = 0.005 !factor of 3
   MW%tau_percent = .false.
   MW%mu =  80.3571
   MW%sigma =  0.01! factor of 2.5

   SinEff%datum_type = Gaussian
   SinEff%tau =  5.0*1E-5 !factor of 3
   SinEff%tau_percent = .false.
   SinEff%mu =  0.2315950
   SinEff%sigma =  4.0*1E-5  !factor of 4

   BdecayData%datum_type = Gaussian
   BdecayData%tau =  1.0 !factor of 5
   BdecayData%tau_percent = .false.
   BdecayData%mu =  16.409
   BdecayData%sigma =  0.12 !same 

   Bsmumu%datum_type = UpperLimit !the same
   Bsmumu%tau =  0.05 !factor of 3
   Bsmumu%tau_percent = .true.
   !data in units of 10^7
   Bsmumu%mu =  1.00 !this is the value of the limit
   Bsmumu%sigma =  0.0 

   !imaginary Higgs detection
   MHiggs%datum_type = Gaussian
   MHiggs%tau =  1.5 !factor of 2
   MHiggs%tau_percent = .false.
   MHiggs%mu =  106.97
   MHiggs%sigma =  2.0 !made up

end if

end subroutine InitializeDataSets

end module likedata
