! Module containing plotting routines for matlab (2D and 3D plots)
! based on GetDist by Anthony Lewis & Sarah Bridle (cosmologist.info/cosmomc)
! SuperBayes Package - by Roberto Ruiz de Austri (rruiz@delta.ft.uam.es) and Roberto Trotta (rxt@astro.ox.ac.uk)
! This version May 2007 !


module matlab

  use mcsamples
  implicit none  
  contains

    subroutine WriteMatLabInit(unit)
      integer, intent(in) :: unit
      
      write(unit, '(A)') 'lab_fontsize = 28; axes_fontsize = 24; names_fontsize = 18;  cbar_fontsize = 14;  textbox_fontsize = 40; twolines_fontsize = 30;'
      !format for contours 
      write(unit, '(A)') ' l1=''-b''; lw1=1.2; lwaxes = 2.0;'
      !format for contours for compared models
      write(unit, '(A)') ' l2=''.r''; lw1=1.2; '
      
    end subroutine WriteMatLabInit


    function PlotContMATLAB(aunit,aroot,j,j2, DoShades, DoMeanLike)
      character(LEN=*), intent(in) :: aroot
      integer, intent(in) :: j, j2,aunit
      logical, intent(in) :: DoShades, DoMeanLike
      character(LEN=120) fname,numstr,plotfile
      logical PlotContMATLAB
      
      PlotContMATLAB= .false. 
      plotfile = numcat(trim(numcat(trim(aroot)//'_2D_',colix(j)-2))//'_',colix(j2)-2)
      if (.not. FileExists('plot_data/'//trim(plotfile))) then
         write(*,*) 'trying to plot', trim('plot_data/'//trim(plotfile))
         return !nothing to plot
      end if
      PlotContMATLAB= .true.
      write (aunit,'(a)') 'pts=load (fullfile(''../plot_data'',''' // trim(plotfile) //'''));'
      fname =trim(trim(aroot)//trim(numcat('_p',colix(j2)-2)))
      write (aunit,'(a)') 'tmp = load (fullfile(''../plot_data'',''' // trim(fname)//  '.dat''));'
      !xaxis
      write (aunit,*) 'x1 = tmp(:,1);'
      fname =trim(trim(aroot)//trim(numcat('_p',colix(j)-2)))
      write (aunit,'(a)') 'tmp = load (fullfile(''../plot_data'',''' // trim(fname)//  '.dat''));'
      !yaxis
      write (aunit,*) 'x2 = tmp(:,1);'
      
      !              gfmt = trim(numcat('(''x1 = ['',',ix_max(j2)-ix_min(j2)+1))//'E15.5,''];'')'
      !              write (aunit,gfmt) (/(I, I=ix_min(j2), ix_max(j2), 1)/)*width(j2)+center(j2)
      
      !             gfmt = trim(numcat('(''x2 = ['',',ix_max(j)-ix_min(j)+1))//'E15.5,''];'')'
      !             write (aunit,gfmt) (/(I, I=ix_min(j), ix_max(j), 1)/)*width(j)+center(j)
      if (DoShades) then
         if (DoMeanLike) then
            !shades meanlike
            write (aunit,'(a)') 'load (fullfile(''../plot_data'',''' // trim(plotfile) //'_likes''));'
            write (aunit,'(a)') 'ah=imagesc(x1,x2,'//trim(plotfile)//'_likes);'
         else 
            !shades marginal distribution
            write (aunit,'(a)') 'contourf(''v6'',x1,x2,pts,64);'
         end if
      end if
      write (aunit,'(a)') 'shading flat; hold on;'
      
      
      if (num_contours /= 0) then
         
         !puts contours for marg distribution
         write (aunit,'(a)') 'cnt = load (fullfile(''../plot_data'',''' // trim(plotfile) //'_cont''));'
         
         if (plot_NDcontours) then
            write (aunit,'(a)') 'load (fullfile(''../plot_data'',''' // trim(plotfile) //'_confid''));'
            write(numstr,*) 1-contours(1:num_contours)  
            if (num_contours==1) numstr = trim(numstr)//' '//trim(numstr)
            write (aunit,'(a)') 'contour(''v6'',x1,x2,'//trim(plotfile)//'_confid,['//trim(numstr)//'],''r'');'
            
         end if
      end if
                      
    end function PlotContMATLAB


    subroutine Write2DPlotMATLAB(aunit,j,j2, like)
      integer, intent(in) :: aunit,j,j2
      logical, intent(in) :: like
      character(LEN=120) gfmt
      integer i
      
      write (aunit, '(A)') 'figure'
      write (aunit, '(A)') 'set(gcf, ''NextPlot'', ''replacechildren'');'
      write (aunit, '(A)') 'axes(''position'', [0.3 0.3 0.5 0.5]   );'
      write (aunit, '(A)') 'box on; axis square; hold on;'
      !If anything to plot
      if (PlotContMATLAB(aunit,rootname,j,j2,.true.,like) .and. &
           num_contours /= 0) then
         !adds contours
         write (aunit,'(a)') '[C h] = contour(''v6'',x1,x2,pts,cnt,l1);'
         write (aunit,'(a)') 'set(h,''LineWidth'',lw1);'
         !compare with other chains (contours only)
         do i = 1, Num_ComparePlots
            if (PlotContMATLAB(aunit,ComparePlots(i),j,j2,.false., .false.)) &
                 write (aunit,'(a)') '[C h] = contour(''v6'',x1,x2,pts,cnt,l2);'
            write (aunit,'(a)') 'set(h,''LineWidth'',lw2);'
         end do
      end if
      write(aunit,'(A)') 'text(''String'', ''SuperBayeS v1.0'',''FontSize'',names_fontsize, ''Units'', ''normalized'', ''Position'', [1.0 1.05], ''HorizontalAlignment'', ''right'');'
      write (aunit,'(a)') 'set(gca,''Layer'',''top'',''FontSize'',axes_fontsize);'
      gfmt = ''',''FontSize'',lab_fontsize);'
      !labels
      write (aunit,'(a)') 'xlabel('''//trim(lables(colix(j2)))//trim(gfmt)
      write (aunit,'(a)') 'ylabel('''//trim(lables(colix(j)))//trim(gfmt)
      write (aunit,'(a)') 'axis tight;'              
      !colorbar
      write (aunit,'(A)') 'hbar = colorbar(''horiz'');axes(hbar);'
      if (.not. like) then
         !shaded marg probability
         write(50,'(A)') 'text(''String'', ''Relative probability density'',''FontSize'',cbar_fontsize, ''Units'', ''normalized'', ''Position'', [0.5 2.5], ''HorizontalAlignment'', ''center'');'
         write (aunit,'(A)') 'load(''../colormaps/ModInverseHot'',''modrhot'');'
         write (aunit,'(A)') 'colormap(modrhot);'
      else
         !shaded mean likelihood
         write(50,'(A)') 'text(''String'', ''Mean likelihood'',''FontSize'',cbar_fontsize, ''Units'', ''normalized'', ''Position'', [0.5 2.5], ''HorizontalAlignment'', ''center'');'
         write (aunit,'(A)') 'load(''../colormaps/greenmap'',''likegree'');'
         write (aunit,'(A)') 'colormap(likegree);'
      end if
      
      write(aunit,'(A)') 'set(gca,''FontSize'',axes_fontsize);'
      !write(aunit,'(A)') 'hpos = get(hbar, ''Position'');'
      write(aunit,'(A)') 'set(hbar, ''Position'',[0.3 0.05 0.5 0.02], ''Units'', ''normalized'');'
      write(aunit,'(A)') 'axis off;'              
      write (aunit,'(A)') 'set(gcf, ''PaperUnits'',''centimeters'');'
      write (aunit,'(A)') 'set(gcf, ''PaperPosition'',[ 0 0 20 20]);' 
      write (aunit,'(A)') 'set(gcf, ''PaperType'', ''A4'');'
      write (aunit,'(A)') 'set(gcf, ''PaperOrientation'', ''portrait''); '       
      
    end subroutine  Write2DPlotMATLAB



    
  end module matlab
