!module to perform post-processing of chains
!useful to add data/recompute predictions/add new observables
!SuperBayes Package - by Roberto Ruiz de Austri (rruiz@delta.ft.uam.es) and Roberto Trotta (rxt@astro.ox.ac.uk)
!This version October 2007 
module postprocess

  use calclike
  use MCMC_theory

  implicit none

  contains 

  subroutine PostProcChains(file, in_unit, lines)
    integer, intent(IN) :: in_unit, lines
 
   !outfile_unit is already open and globally defined
    integer :: i, ierr, line, rejected, l, error
    real :: mult, like, NewLike, weight
    real(8) :: Xt, Xb, mu, tan_2thetat, tan_2thetab, E, step
    character (LEN=80) :: file, InLine 
    logical :: dm_comp, dd_comp, coll_comp, bdecay_comp

    Type(ParamSet) :: P
    Type(Input_params) :: ChainMSSM
    Type(Nuisance_params) :: ChainNuisP
    Type(DS_Output) :: ChainDSP
    Type(Susy_spec) :: ChainSusySp
    Type(SoftOut) :: ChainSoftOut
    Type(Bdecay_Out) :: ChainBdecay
    Type(Prodxbr_Out) :: ChainProdxbr

 
    file = trim(file)//'.pp' 


    call  CheckChains(file)

    
    line = 0
    rejected = 0
    if (skip_lines > 0) then
       if (Feedback > 1) write(*,*) 'Skipping ', skip_lines, ' lines'
       do l=1,skip_lines
          read(in_unit, *, iostat=ierr) mult
          if (ierr < 0) call DoStop('Skipped to many lines while post-processing!')
       end do
    end if

   
    do !loops over lines in file 


       read (in_unit, fmt_params, ADVANCE='NO', END = 77) mult, like, P%P
       

       line = line + 1
       if (Feedback > 1) write(*,*) 'reading in line ', line
       if (Feedback > 2) write(*,*) 'params: ', P%P

 
       call ReadChains(in_unit, ChainDSP, ChainSusySp, ChainBdecay, ChainProdxbr, Xt, Xb, mu, tan_2thetat, tan_2thetab)            

       !This advances to the next line 
       read(in_unit, '(A)', ADVANCE='YES', END = 77) InLine

      
       !need to convert back some of the quantities
       call ConvertFromReading(P, ChainDSP)

       call ParamsToMSSMParams(P%P, ChainMSSM, ChainNuisP)
 

       if(redo_theory) then

        if(GFlags%ID_predict .and. GFlags%ID%evar) then

         E = GIDin%efluxes_i 

         !all the energies are set to be common 
         GIDin%egam = E
         GIDin%epb  = E
         GIDin%edb  = E
         GIDin%eep  = E
         GIDin%enu  = E
         
        endif
        

        call GetTheoryInfo(ChainMSSM, ChainNuisP, CurrentOutputP, CurrentSusySp,CurrentSoftOut, CurrentBdecay, CurrentProdxbr, error)


       else

        CurrentSoftOut%soft%mu = mu 
        CurrentSoftOut%soft%asoftu(3) = Xt + mu/P%P(4)
        CurrentSoftOut%soft%asoftd(3) = Xb + mu/P%P(4)       
        CurrentSoftOut%sfer%thetat = atan(tan_2thetat)/2.0
        CurrentSoftOut%sfer%thetab = atan(tan_2thetab)/2.0     
      
        CurrentOutputP = ChainDSP
        CurrentSusySp  = ChainSusySp
        CurrentBdecay  = ChainBDecay         
        CurrentProdxbr = ChainProdxbr

        error = 0
         
       endif


       PreviousOutputP = CurrentOutputP
       PreviousSusySp  = CurrentSusySp
       PreviousSoftOut = CurrentSoftOut
       PreviousProdxbr = CurrentProdxbr
       PreviousBdecay  = CurrentBdecay


       if (error == 0) then
 
        if (redo_like) then
             
         NewLike = GetLogLikePost(CurrentSusySp, ChainNuisP, CurrentOutputP, ChainMSSM, CurrentSoftOut, CurrentBDecay)
         
         if (NewLike == logZero) then
          weight = 0
         else
          weight = exp(Like-NewLike)
         endif
         if (.not. redo_change_like_only)  mult = mult*weight

         if (Feedback > 0 .and. NewLike < logZero) then
          write(*,*) ' ---- Changes in Like and Multiplicity  -----'
          write(*,'(A,F20.5,A,F20.5)') 'NewLike  = ', NewLike, ' DeltaLike = ', Like-NewLike
          if(.not. redo_change_like_only) & 
           write(*,'(A,F20.5,A,F20.5)') 'New mult = ', mult, ' Old mult  = ', mult/weight   

          if (redo_theory) then 
           write(*,*) ' ---- Changes in some of the observable values  -----'
           write(*,'(A,F10.6)') 'Percent Omega_CDM = ',  1d0 - PreviousOutputP%dm%oh2/ChainDSP%dm%oh2
           write(*,'(A,F10.6)') 'Percent g-2       = ',  1d0 - PreviousOutputP%coll%gm2amp/ChainDSP%coll%gm2amp 
           write(*,'(A,F10.6)') 'Percent Bsgamma   = ',  1d0 - PreviousOutputP%coll%bsgamma/ChainDSP%coll%bsgamma 
           write(*,'(A,F10.6)') 'Percent M_W       = ',  1d0 - PreviousOutputP%coll%mW/ChainDSP%coll%mW
           write(*,'(A,F10.6)') 'Percent sin_eff   = ',  1d0 - PreviousOutputP%coll%sweff2/ChainDSP%coll%sweff2
           write(*,'(A,F10.6)') 'Percent Delta Bs   = ', 1d0 - PreviousBdecay%delMB/ChainBDecay%delMB
          end if
         endif
        else
         NewLike = like
         weight = 1
        end if 

        if(NewLike < logZero) then
         !write the point only if its allowed under the new like
         call WriteParams(P, mult, NewLike)
        else
         rejected = rejected + 1
         if (Feedback >1 ) write (*,*) 'New like = ', NewLike
         cycle
        end if
       else
         rejected = rejected + 1
         if (Feedback >1 ) write (*,*) 'Error in spectrum...'
         cycle
       endif

   
       !here loop better after all like analysis
       if(redo_theory .and. GFlags%ID_predict .and. GFlags%ID%evar) then 

        !bins in a log10 scale
        step = log10(GIDin%efluxes_f/GIDin%efluxes_i)/GIDin%nbins
       
        do i = 2, GIDin%nbins+1 !loops over the fluxes energies required

         E = GIDin%efluxes_i * 10.d0**((i-1) * step)


         !all the energies are set to be common 
         GIDin%egam = E
         GIDin%epb  = E
         GIDin%edb  = E
         GIDin%eep  = E
         GIDin%enu  = E


         call GetTheoryInfo(ChainMSSM, ChainNuisP, CurrentOutputP, CurrentSusySp,CurrentSoftOut, CurrentBdecay, CurrentProdxbr, error)


         PreviousOutputP = CurrentOutputP
         PreviousSusySp  = CurrentSusySp
         PreviousSoftOut = CurrentSoftOut
         PreviousProdxbr = CurrentProdxbr
         PreviousBdecay  = CurrentBdecay      

         call WriteParams(P, mult, NewLike)

        enddo
       endif

    end do

77  continue

    write(*,*)  'Points rejected under new like:', rejected 
    write(*,*)  'Chain ', instance, ': post-processed models: ', line
    
    if(line+skip_lines /=  lines) &
     write(*,*) 'Warning: Not all the samples required have been PP !'
   
  end subroutine PostProcChains

end module postprocess
