! SuperBayes Package - by Roberto Ruiz de Austri (rruiz@delta.ft.uam.es) and Roberto Trotta (rxt@astro.ox.ac.uk)
! This version August 2007 !
! Tester file
! Useful to produce single models, or to check an existing chain point by point etc

  program tester

    use parameters
    use ParamDef
    use calclike

    implicit none

    
    Type(FLAGS) ::  Flag
    Type(Input_params) :: InP
    Type(ParamSet) :: P
    Type(Nuisance_params) :: InN  
    Type(Susy_spec) :: Spect
    Type(Prodxbr_Out) :: Prodxbr

    Type(BDecay_out) :: BDecay
    Type(DS_Output) :: DSO
    Type(Softout) :: Outsoft 

    !Debugging at local level (ie write output to a file if 
    !a single point is tested)
    logical :: Debug = .true.
    logical :: getlike = .true.

    !switch this to .true. if you want to read in the elements
    !of an existing chains one by one
    logical, parameter :: test_chain = .true.

    integer :: ierr, line
    real :: like, NewLike, mult
    real :: alpha_inv_difference
    
    real (8) :: beta, alpha, sbma2_th, DeltM, oh2_exp, mlsp, sigip_bound 
    real (8) :: sbma2_exp, mchar_exp,  sbma2_intp, d1mach
    real (8) ::  SinBmASq  

    write(*,*) 'This is SuperBayeS tester '//trim(version)

    !Debugging the code
    !Feedback level (>2: debug; 1: minimal 0: almost none
    Feedback = 4


    !Flags to switch on/off the calculation of parameters
    Flag%Collider_predict = .true.
    Flag%DM_predict  = .true. 
    Flag%DD_predict  = .true.
    Flag%ID_predict  = .false.
    Flag%FH_predict  = .false. ! Higgs detection prospects part
    Flag%BD_predict  = .true.

    !Flags for likelihood computation
    Use_data = current_data
    Use_CDM = .true.
    Use_Nuisance = .true.
    Use_LEP = .true.
    Use_Anomalous_Mu = .true.
    Use_bsgamma = .true.
    Use_Bsmumu = .true.
    Use_Mass_W = .true.
    Use_Weak_Mixing = .true.
    Use_Bdecay = .true.

    Use_DD = .false.
    Use_ID = .false.

    call InitializeDataSets

    if(Flag%ID_predict) then

     Flag%ID%gadiff = .true.
     Flag%ID%gac = .true.
     Flag%ID%gam = .true.
     Flag%ID%pb = .true.
     Flag%ID%db = .true.
     Flag%ID%ep = .true.
     Flag%ID%epfrac = .true.
     Flag%ID%musu = .true.
     Flag%ID%muea = .true.
     Flag%ID%musudiff = .true.
     Flag%ID%mueadiff = .true.
     Flag%ID%muha = .false.
     Flag%ID%muhadiff = .false.
     Flag%ID%sigmav = .true.


     !num the halo profiles
     DSO%ID_in%num_hm = 2
     num_hm = 2

     if(DSO%ID_in%num_hm > 10) &
      call DoStop('Not permitted more than 10 halo models at once')

      !Available halo models are: isothermal (isosm), modified isothermal (isocl),
      !Navarro-Frenk-White (nfwsm), Moore (moosm), Burket (burksm), 
      !adiabatic contracted (adiabsm), Navarro-Frenk-White adiab. (nfwadiabsm), 
      !Moore adiab. (mooadiabsm)
      !enumerate the ones desired
 
      allocate(DSO%ID_in%hmodel(DSO%ID_in%num_hm))
      DSO%ID_in%hmodel(1) = 'isosm'
      DSO%ID_in%hmodel(2) = 'nfwsm'
 

      !Antiprotons propagation model 
      DSO%ID_in%pbmodel = 'beu04'
      !Positron diffusion model
      DSO%ID_in%epmodel = 'esu04'
      !Neutrino parameters
      DSO%ID_in%ntmodel = 'tab'

      !ID parameters:
      !For the fluxes of gamma-rays and neutrinos with the chosen halo
      !profile, we calculate, once and for all, the line of sight
      !integration factor j in the direction of observation, which we
      !define as the direction which forms an angle psi0 with respect to
      !the direction of the galactic centre (e.g. cospsi0 = 1 is the
      !galactic center direction, cospsi0 = -1 the antigalactic centre) 
      DSO%ID_in%cospsi0 = 1.d0
      !There are two options: you can take into account that your 
      !detector has an angular resolution delta_gamma (in sr) and compute
      !j averaged over that solid angle or set  delta_gamma = 0 and just 
      !compute j without averaging
      DSO%ID_in%delta_gamma = 1.d-5 
      !gamma-ray flux with continuum energy spectrum at a given energy egam (GeV)
      DSO%ID_in%egam = 20.d0
      !Gamma-ray flux with continuum energy spectrum integrated above
      !some given threshold egath (GeV) (ie HESS = 60.d0)
      DSO%ID_in%egath = 1.d0
      !Boost Factor for antimatter fluxes
      DSO%ID_in%BF = 1.d0
      !Differential flux of antiprotons for antiproton kinetic energy epb (Gev)
      DSO%ID_in%epb = 1.76d0
      !Differential flux of antideutron for antideutron kinetic energy edb (Gev)
      DSO%ID_in%edb = 1.d0
      !Differential flux of antideutron for positron kinetic energy eep (Gev)
      DSO%ID_in%eep = 1.0
      !Neutrino telescopes: 
      !Energy threshold eth (GeV)
      DSO%ID_in%eth = 50.d0
      !The maximum half-aperture angle, degrees
      DSO%ID_in%thmax = 30.d0
      !Energy enu (GeV) 
      DSO%ID_in%enu = 1.d0
      !Angle from center of Earth/Sun, degrees
      DSO%ID_in%theta = 30.d0
      !type of fluxes: 1= neutrino flux, 2= neutrino to muon conversion, 
      !3= muon flux
      DSO%ID_in%rtype = 3
      ! There are two options: you can take into account that your 
      ! detector has an angular resolution delta_nt (in sr) and compute
      ! j averaged over that solid angle or set  delta_nt = 0 and just 
      ! compute j without averaging
      DSO%ID_in%delta_nt = 0.d0

      GIDin = DSO%ID_in

     endif


     if (.not. test_chain) then
      !---------------------------------------------------------
      ! CMSSM parameters - set them by hand to the value you want
      !---------------------------------------------------------
      InP%m0 = 0.14796E+04 
      InP%mhalf =0.12076E+04
      InP%a0 =  0.80307E+03
      InP%tanb =0.52366E+02 
      !fixed 
      InP%sgnmu0 = 1
      InP%scaledep = 1.0d0
      !Nuisance parameters.
      InN%mbottom = 0.41672E+01
      InN%mtop = 0.17312E+03
      InN%alphasmz =0.12039E+00 

      !this is the number dumped in the chains
      alpha_inv_difference =  -0.16327E-01
      InN%alphamz = 1.d0/(alpha_inv_mean + (alpha_inv_difference)) 

      !alpha_em_inv = alpha_EM^-1 - 127.958

      !Fixed Nuisance params
      !those are fixed for the moment
      InN%ftp_s = 0.118 
      InN%ftn_s = 0.118  

      goto 111


     else
      !---------------------------------------------------------
      !chain reprocessing - useful for all sorts of checks
      !---------------------------------------------------------
      write(*,*) ' Opening chain'
      !put here by hand the chain's filename
      open(33, file='chains/test_1.txt', status='old', iostat=ierr)  
      if (ierr /= 0) stop 'problem opening test chain'
      line = 0
      !put here the output file name
      open(outfile_unit, file="chains/out.tester", status='replace')
      do 
        read(33, *, iostat=ierr) mult, like, P%P
        if(ierr /= 0) exit
        line = line + 1
        write(*,*) 'reading in line ', line
        use_log = .false.
        !converting the value of alpha_EM_inv, which has been read in
        call ConvertFromReading(P)

       
 111    if (.not. test_chain) then
          !converting back to Params
          call  MSSMParamsToParams(InP, P%P, InN)
        end if

      
        !------------------------------------------
        !computes likelihood
        !-----------------------------------------


        !set scales
        Scales%PMin = -1E4
        Scales%PMax = 1E4
        use_data = current_data           
        GFlags = Flag 

        !Gflags are used in getloglike
        write(*,*) 'Getting Likelihood...'
        NewLike = GetLogLike(P)
        write(*,'(A,E15.5)') ' New   -lnlike: ', NewLike

        if (test_chain) then
          write(*,'(A,E15.5)') ' Old   -lnlike: ', like
          write(*,'(A,E15.5)') ' Delta -lnlike: ', NewLike - like
        end if

        Like = NewLike

        PreviousOutputP = CurrentOutputP
        PreviousSusySp  = CurrentSusySp
        PreviousSoftOut = CurrentSoftOut
        PreviousProdxbr = CurrentProdxbr
        PreviousBdecay  = CurrentBdecay

        if (test_chain) then
         call SetFormat

         call WriteParams(P, mult, like)
        endif      

        if (Debug  .and. (.not. test_chain)) then

         call softsusy(Flag, InP, InN, Spect, Outsoft, Leshouches, DSO%err)

         if (Flag%DM_Predict .or. Flag%DD_Predict .or. Flag%ID_Predict .or. Flag%Collider_Predict) then         
           call ds(Flag, InP, InN, Outsoft, DSO)
         endif

         if (Flag%BD_Predict) then
           call bd(Flag, InP, InN, Outsoft, BDecay, DSO%err)
         end if


         if (Flag%FH_Predict) then
           call FeynHiggs(Flag, DSO%err, Leshouches, Prodxbr)
         end if

         write(*,*) 'Writing physical spectrum in tester.out'
         call WriteDebug(Flag, InP, InN, Spect, DSO, Prodxbr, BDecay, Outsoft)

        endif      



        if (.not. test_chain) then
           stop !one model only and stop
        end if

      end do !loops over chain elements
     end if
     close (33)
     close (outfile_unit)

  end program tester

